@@ -135,7 +135,7 @@ class IntegrationGenericTests(unittest.TestCase):
         task_data = textwrap.dedent(
             '''\
                 result:
-                    true
+                    success
                 '''
         )
         self.assert_create_work_request(
