@@ -166,7 +166,7 @@ class IntegrationTaskAPTMirrorTests(
         httpd_thread.start()
         self.addCleanup(httpd_thread.join)
         self.addCleanup(httpd.shutdown)
-        host, port = httpd.server_address
+        host, port, *_ = httpd.server_address
         assert isinstance(host, str)
         return host, port
 
