@@ -8,9 +8,9 @@
 # contained in the LICENSE file.
 """Waiter utility: waits until success/failure and fails after timeout."""
 
-import time
+import datetime as dt
+import time as tm
 from collections.abc import Callable
-from datetime import datetime, timedelta
 from functools import partialmethod
 from typing import Any
 
@@ -30,13 +30,15 @@ class Waiter:
         :param func: function that gets executed
         :return: True if it succeeded, False if it timed out
         """
-        start_at = datetime.now()
+        start_at = dt.datetime.now(tz=dt.UTC)
 
         while func() != expected:
-            if datetime.now() - start_at > timedelta(seconds=timeout):
+            if dt.datetime.now(tz=dt.UTC) - start_at > dt.timedelta(
+                seconds=timeout
+            ):
                 return False
 
-            time.sleep(0.2)
+            tm.sleep(0.2)
 
         return True
 
