@@ -12,39 +12,139 @@ Local artifact's representation.
 
 See docs/design/ontology.html for the semantics of the Artifacts.
 """
+
 import abc
 import json
 import re
 from collections.abc import Iterable, Sequence
 from json import JSONDecodeError
 from pathlib import Path, PurePath
-from typing import Any, ClassVar, Generic, Self, TypeVar, cast, overload
+from typing import Any, ClassVar, Self, assert_never, overload
 
-from debian import deb822, debfile
+import pydantic
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic  # type: ignore
+# https://github.com/pydantic/pydantic/issues/12720
+from pydantic._internal._generics import get_origin
 
 import debusine.artifacts.models as data_models
+from debian import deb822, debfile
 from debusine import utils
 from debusine.artifacts.artifact_utils import files_in_meta_file_match_files
 from debusine.assets import KeyPurpose
 from debusine.client.models import FileRequest
 from debusine.utils import extract_generic_type_arguments
 
-AD = TypeVar("AD", bound=data_models.ArtifactData)
+
+class NewLocalFile(pydantic.BaseModel):
+    """A new local file to be added to the artifact."""
+
+    #: File in the local file system to be added to the artifact.
+    file: Path
+    #: Base directory of the artifact.  Must be an absolute path.  If it's
+    #: None, the file is added in the root of the artifact; otherwise, it is
+    #: added with the path of the file relative to `artifact_base_dir`.
+    artifact_base_dir: Path | None = None
+    #: Name to use for the file in the artifact instead of `file.name`.
+    override_name: str | None = None
+    #: Content-Type to use when serving the file instead of guessing.
+    content_type: str | None = None
 
 
-class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
+class NewRemoteFile(pydantic.BaseModel):
+    """
+    A new remote file to be added to the artifact.
+
+    This may be used when a file does not exist locally, but can be expected
+    to already exist on the server when uploading the rest of the artifact.
+    """
+
+    #: Path of the file in the artifact.
+    path_in_artifact: str
+    #: Description of the remote file.
+    file_request: FileRequest
+
+
+class NascentLocalArtifact(pydantic.BaseModel):
+    """Information about files that will be added to a new local artifact."""
+
+    local_files: dict[str, Path] = pydantic.Field(default_factory=dict)
+    remote_files: dict[str, FileRequest] = pydantic.Field(default_factory=dict)
+    content_types: dict[str, str] = pydantic.Field(default_factory=dict)
+
+    def _check_existing_file(
+        self, path_in_artifact: str, new_file: Path | FileRequest
+    ) -> None:
+        """Check whether the nascent artifact already contains a given file."""
+        all_files = self.local_files | self.remote_files
+        if path_in_artifact in all_files:
+            raise ValueError(
+                f"File with the same path ({path_in_artifact}) "
+                f"is already in the artifact "
+                f'("{all_files[path_in_artifact]}" and "{new_file}")'
+            )
+
+    def add_local_file(self, new_file: NewLocalFile) -> None:
+        """
+        Prepare a local file for addition to the artifact.
+
+        :param new_file: information about the new local file.
+        :raises ValueError: artifact_base_dir is not absolute or is not
+          a directory; file does not exist; the path in the artifact already
+          had a file.
+        """
+        file = new_file.file
+        if new_file.artifact_base_dir is not None:
+            if not new_file.artifact_base_dir.is_absolute():
+                raise ValueError(
+                    f'"{new_file.artifact_base_dir}" must be absolute'
+                )
+            if not new_file.artifact_base_dir.is_dir():
+                raise ValueError(
+                    f'"{new_file.artifact_base_dir}" does not exist or '
+                    f'is not a directory'
+                )
+
+            if not file.is_absolute():
+                file = new_file.artifact_base_dir.joinpath(file)
+
+            path_in_artifact = file.relative_to(
+                new_file.artifact_base_dir
+            ).as_posix()
+        else:
+            path_in_artifact = file.name
+
+        if new_file.override_name is not None:
+            path_in_artifact = new_file.override_name
+
+        if not file.exists():
+            raise ValueError(f'"{file}" does not exist')
+
+        if not file.is_file():
+            raise ValueError(f'"{file}" is not a file')
+
+        file_absolute = file.absolute()
+
+        self._check_existing_file(path_in_artifact, file_absolute)
+        self.local_files[path_in_artifact] = file_absolute
+        if new_file.content_type is not None:
+            self.content_types[path_in_artifact] = new_file.content_type
+
+    def add_remote_file(self, new_file: NewRemoteFile) -> None:
+        """
+        Prepare a remote file for addition to the artifact.
+
+        :param new_file: information about the new remote file.
+        """
+        self._check_existing_file(
+            new_file.path_in_artifact, new_file.file_request
+        )
+        self.remote_files[new_file.path_in_artifact] = new_file.file_request
+
+
+class LocalArtifact[AD: data_models.ArtifactData](pydantic.BaseModel, abc.ABC):
     """Represent an artifact locally."""
 
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
-        extra = "forbid"
+    model_config = pydantic.ConfigDict(validate_assignment=True, extra="forbid")
 
     #: Artifact type
     category: data_models.ArtifactCategory
@@ -70,18 +170,10 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
     # data_type is marked as PrivateAttr to make mypy happy. Setting
     # underscore_attrs_are_private to True does not seem to be enough
 
-    _local_artifacts_category_to_class: dict[
-        str, type["LocalArtifact['Any']"]
+    _local_artifacts_category_to_class: ClassVar[
+        dict[str, type["LocalArtifact['Any']"]]
     ] = {}
 
-    def __init__(self, **kwargs: Any) -> None:
-        """Set category to _category by default."""
-        kwargs.setdefault("category", self._category)
-        if "data" in kwargs:
-            if isinstance(d := kwargs["data"], dict):
-                kwargs["data"] = self.create_data(d)
-        super().__init__(**kwargs)
-
     def __init_subclass__(cls, **kwargs: Any) -> None:
         """
         Register subclass into LocalArtifact._local_artifacts_category_to_class.
@@ -90,25 +182,64 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         """
         super().__init_subclass__(**kwargs)
 
-        # The task data type, computed by introspecting the type argument
-        # used to specialize this generic class.
-        [cls._data_type] = extract_generic_type_arguments(cls, LocalArtifact)
+        if get_origin(cls) is not None:
+            # The task data type, computed by introspecting the type argument
+            # used to specialize this generic class.
+            [cls._data_type] = extract_generic_type_arguments(
+                cls, LocalArtifact
+            )
 
-        LocalArtifact._local_artifacts_category_to_class[cls._category] = cls
+            LocalArtifact._local_artifacts_category_to_class[
+                # This should just be cls._category, but if we try to use
+                # that in mypy we get:
+                # TypeError: unhashable type: 'ModelPrivateAttr'
+                cls._category.default  # type: ignore[attr-defined]
+            ] = cls
 
     @classmethod
     def create_data(cls, data_dict: dict[str, Any]) -> AD:
         """Instantiate a data model from a dict."""
-        return cast(AD, cls._data_type(**data_dict))
+        return cls._data_type(**data_dict)
+
+    @classmethod
+    def _create(
+        cls,
+        data: dict[str, Any] | AD,
+        files: Sequence[NewLocalFile | NewRemoteFile] | None = None,
+    ) -> Self:
+        """
+        Create a new instance of this model.
+
+        If possible, use the higher-level ``create`` method of the
+        appropriate subclass instead.
+        """
+        if isinstance(data, dict):
+            data = cls.create_data(data)
+        nascent = NascentLocalArtifact()
+        for file in files or ():
+            match file:
+                case NewLocalFile():
+                    nascent.add_local_file(file)
+                case NewRemoteFile():
+                    nascent.add_remote_file(file)
+                case _ as unreachable:
+                    assert_never(unreachable)
+        return cls(
+            category=cls._category,
+            data=data,
+            files=nascent.local_files,
+            remote_files=nascent.remote_files,
+            content_types=nascent.content_types,
+        )
 
     @staticmethod
     def artifact_categories() -> list[str]:
         """Return list of artifact categories."""
         return sorted(LocalArtifact._local_artifacts_category_to_class.keys())
 
-    @pydantic.validator("category", pre=True)
+    @pydantic.field_validator("category", mode="before")
     @classmethod
-    def _validate_category(cls, category: str) -> str:
+    def _validate_category(cls, category: Any) -> Any:
         """Validate that the category is known."""
         if category not in cls._local_artifacts_category_to_class:
             raise ValueError(
@@ -123,102 +254,6 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         category = LocalArtifact._validate_category(category)
         return LocalArtifact._local_artifacts_category_to_class[category]
 
-    def _check_existing_file(
-        self, path_in_artifact: str, new_file: Path | FileRequest
-    ) -> None:
-        """Check whether the artifact already contains a given file."""
-        all_files = self.files | self.remote_files
-        if path_in_artifact in all_files:
-            raise ValueError(
-                f"File with the same path ({path_in_artifact}) "
-                f"is already in the artifact "
-                f'("{all_files[path_in_artifact]}" and "{new_file}")'
-            )
-
-    def add_local_file(
-        self,
-        file: Path,
-        *,
-        artifact_base_dir: Path | None = None,
-        override_name: str | None = None,
-        content_type: str | None = None,
-    ) -> None:
-        """
-        Add a local file in the artifact.
-
-        :param file: file in the local file system that is added
-           to the artifact
-        :param artifact_base_dir: base directory of the artifact. Must be
-           an absolute path.
-           If it's None: file is added in the root of the artifact.
-           If it's not None: file is added with the relative path of the
-           file with the artifact_base_dir. E.g.
-           file=/tmp/artifact/dir1/file1
-           artifact_base_dir=/tmp/artifact
-           Path of this file in the artifact: dir1/file1
-        :param override_name: if not None: use it instead of file.name
-        :param content_type: if not None: record this as the
-          ``Content-Type`` to be used when serving the file, rather than
-          guessing
-        :raises ValueError: artifact_base_dir is not absolute or is not
-          a directory; file does not exist; the path in the artifact already
-          had a file.
-        """
-        if artifact_base_dir is not None:
-            if not artifact_base_dir.is_absolute():
-                raise ValueError(f'"{artifact_base_dir}" must be absolute')
-            if not artifact_base_dir.is_dir():
-                raise ValueError(
-                    f'"{artifact_base_dir}" does not exist or '
-                    f'is not a directory'
-                )
-
-            if not file.is_absolute():
-                file = artifact_base_dir.joinpath(file)
-
-            path_in_artifact = file.relative_to(artifact_base_dir).as_posix()
-        else:
-            path_in_artifact = file.name
-
-        if override_name is not None:
-            path_in_artifact = override_name
-
-        if not file.exists():
-            raise ValueError(f'"{file}" does not exist')
-
-        if not file.is_file():
-            raise ValueError(f'"{file}" is not a file')
-
-        file_absolute = file.absolute()
-
-        self._check_existing_file(path_in_artifact, file_absolute)
-        self.files[path_in_artifact] = file_absolute
-        if content_type is not None:
-            self.content_types[path_in_artifact] = content_type
-
-    def add_remote_file(
-        self, path_in_artifact: str, file_request: FileRequest
-    ) -> None:
-        """
-        Add a remote file to the artifact.
-
-        This may be used when a file does not exist locally, but can be
-        expected to already exist on the server when uploading the rest of
-        the artifact.
-
-        :param path_in_artifact: path of the file in the artifact
-        :param file_request: description of the remote file
-        """
-        self._check_existing_file(path_in_artifact, file_request)
-        self.remote_files[path_in_artifact] = file_request
-
-    def validate_model(self) -> None:
-        """Raise ValueError with an error if the model is not valid."""
-        *_, error = pydantic.validate_model(self.__class__, self.__dict__)
-
-        if error is not None:
-            raise ValueError(f"Model validation failed: {error}")
-
     @classmethod
     def _validate_files_length(
         cls, files: dict[str, Path], number_of_files: int
@@ -245,9 +280,9 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         return files
 
     @classmethod
-    def _validate_exactly_one_file_ends_in(
-        cls, files: dict[str, Path], suffix: str
-    ) -> dict[str, Path]:
+    def _validate_exactly_one_file_ends_in[T: Any](
+        cls, files: dict[str, T], suffix: str
+    ) -> dict[str, T]:
         """Raise ValueError if files doesn't have exactly 1 file with suffix."""
         changes_files = sum(1 for file in files if file.endswith(suffix))
         if changes_files != 1:
@@ -273,14 +308,15 @@ class WorkRequestDebugLogs(LocalArtifact[data_models.EmptyArtifactData]):
     @classmethod
     def create(cls, *, files: Iterable[Path]) -> Self:
         """Return a WorkRequestDebugLogs."""
-        artifact = cls(category=cls._category)
-
-        for file in files:
-            artifact.add_local_file(
-                file, content_type="text/plain; charset=utf-8"
-            )
-
-        return artifact
+        return cls._create(
+            data={},
+            files=[
+                NewLocalFile(
+                    file=file, content_type="text/plain; charset=utf-8"
+                )
+                for file in files
+            ],
+        )
 
 
 class DebusineTest(LocalArtifact[data_models.EmptyArtifactData]):
@@ -308,8 +344,7 @@ class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
         bd_uninstallable: data_models.DoseDistCheck | None = None,
     ) -> Self:
         """Return a PackageBuildLog."""
-        artifact = cls(
-            category=cls._category,
+        return cls._create(
             data=data_models.DebianPackageBuildLog(
                 source=source,
                 version=version,
@@ -317,13 +352,14 @@ class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
                 filename=file.name,
                 bd_uninstallable=bd_uninstallable,
             ),
+            files=[
+                NewLocalFile(
+                    file=file, content_type="text/plain; charset=utf-8"
+                )
+            ],
         )
 
-        artifact.add_local_file(file, content_type="text/plain; charset=utf-8")
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def validate_files_length_is_one(
         cls, files: dict[str, Path]
@@ -331,7 +367,7 @@ class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
         """Validate that artifact has only one file."""
         return super()._validate_files_length(files, 1)
 
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def file_must_end_in_build(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Raise ValueError if the file does not end in .build."""
@@ -380,19 +416,11 @@ class Upload(LocalArtifact[data_models.DebianUpload]):
     _category = data_models.ArtifactCategory.UPLOAD
 
     @classmethod
-    def create(
-        cls,
-        *,
-        changes_file: Path,
-        exclude_files: frozenset[Path] | set[Path] = frozenset(),
-        allow_remote: bool = False,
-    ) -> Self:
+    def create(cls, *, changes_file: Path, allow_remote: bool = False) -> Self:
         """
         Return a Upload. Add the changes_files and files listed in it.
 
         :param changes_file: a .changes file. Parsed by deb822.Changes.
-        :param exclude_files: do not add them in files even if listed in the
-          Files section in the changes_file.
         :param allow_remote: if True, allow files referenced by the .changes
           file to be missing locally; uploading this artifact will require
           them to already exist on the server.
@@ -403,47 +431,43 @@ class Upload(LocalArtifact[data_models.DebianUpload]):
                 changes_fields=deb822dict_to_dict(deb822.Changes(changes_obj)),
             )
 
-        artifact = cls.construct(category=cls._category, data=data)
-
-        artifact.add_local_file(
-            changes_file, content_type="text/plain; charset=utf-8"
+        files: list[NewLocalFile | NewRemoteFile] = []
+        files.append(
+            NewLocalFile(
+                file=changes_file, content_type="text/plain; charset=utf-8"
+            )
         )
 
-        # Add any files referenced by .changes (excluding the exclude_files)
+        # Add any files referenced by .changes.
         base_directory = changes_file.parent
         for file in data.changes_fields.get("Checksums-Sha256", []):
             path = base_directory / file["name"]
-
-            if path not in exclude_files:
-                if allow_remote and not path.exists():
-                    artifact.add_remote_file(
-                        file["name"],
-                        FileRequest(
+            if allow_remote and not path.exists():
+                files.append(
+                    NewRemoteFile(
+                        path_in_artifact=file["name"],
+                        file_request=FileRequest(
                             size=file["size"],
                             checksums={"sha256": file["sha256"]},
                             type="file",
                         ),
                     )
-                else:
-                    artifact.add_local_file(path)
+                )
+            else:
+                files.append(NewLocalFile(file=path))
 
-        return artifact
+        return cls._create(data=data, files=files)
 
-    @pydantic.root_validator(pre=True, allow_reuse=True)
-    @classmethod
-    def files_contain_changes(cls, values: dict[str, Any]) -> dict[str, Any]:
+    @pydantic.model_validator(mode="after")
+    def files_contain_changes(self) -> Self:
         """Raise ValueError when files does not have exactly 1 .changes file."""
-        cls._validate_exactly_one_file_ends_in(
-            values.get("files", {}) | values.get("remote_files", {}),
-            ".changes",
+        self._validate_exactly_one_file_ends_in(
+            {**self.files, **self.remote_files}, ".changes"
         )
-        return values
+        return self
 
-    @pydantic.root_validator(pre=True, allow_reuse=True)
-    @classmethod
-    def files_contains_files_in_changes(
-        cls, values: dict[str, Any]
-    ) -> dict[str, Any]:
+    @pydantic.model_validator(mode="after")
+    def files_contains_files_in_changes(self) -> Self:
         """
         Validate that set(files) == set(files_in_changes_file).
 
@@ -451,11 +475,9 @@ class Upload(LocalArtifact[data_models.DebianUpload]):
         file.
         """
         files_in_meta_file_match_files(
-            ".changes",
-            deb822.Changes,
-            values.get("files", {}) | values.get("remote_files", {}),
+            ".changes", deb822.Changes, {**self.files, **self.remote_files}
         )
-        return values
+        return self
 
 
 class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
@@ -481,20 +503,17 @@ class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
             dsc_fields=dsc_fields,
         )
 
-        artifact = cls(category=cls._category, data=data)
+        return cls._create(
+            data=data, files=[NewLocalFile(file=file) for file in files]
+        )
 
-        for file in files:
-            artifact.add_local_file(file)
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def files_contain_one_dsc(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Raise ValueError when files does not have exactly 1 .dsc file."""
         return cls._validate_exactly_one_file_ends_in(files, ".dsc")
 
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def files_contains_files_in_dsc(
         cls, files: dict[str, Path]
@@ -539,13 +558,9 @@ class BinaryPackage(LocalArtifact[data_models.DebianBinaryPackage]):
             deb_control_files=control_files,
         )
 
-        artifact = cls(category=cls._category, data=data)
+        return cls._create(data=data, files=[NewLocalFile(file=file)])
 
-        artifact.add_local_file(file)
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def files_must_end_in_deb_or_udeb(
         cls, files: dict[str, Path]
@@ -553,7 +568,7 @@ class BinaryPackage(LocalArtifact[data_models.DebianBinaryPackage]):
         """Raise ValueError if a file does not end in .deb or .udeb."""
         return super()._validate_files_end_in(files, [".deb", ".udeb"])
 
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def files_exactly_one(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Raise ValueError if len(files) != 1."""
@@ -590,14 +605,11 @@ class BinaryPackages(LocalArtifact[data_models.DebianBinaryPackages]):
             packages=[path.name.split("_", 1)[0] for path in files],
         )
 
-        artifact = cls(category=cls._category, data=data)
+        return cls._create(
+            data=data, files=[NewLocalFile(file=file) for file in files]
+        )
 
-        for file in files:
-            artifact.add_local_file(file)
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def files_must_end_in_deb_or_udeb(
         cls, files: dict[str, Path]
@@ -605,7 +617,7 @@ class BinaryPackages(LocalArtifact[data_models.DebianBinaryPackages]):
         """Raise ValueError if a file does not end in .deb or .udeb."""
         return super()._validate_files_end_in(files, [".deb", ".udeb"])
 
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def files_more_than_zero(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Raise ValueError if len(files) == 0."""
@@ -633,14 +645,15 @@ class LintianArtifact(LocalArtifact[data_models.DebianLintian]):
             architecture=architecture, summary=summary
         )
 
-        artifact = cls(category=cls._category, data=data)
+        return cls._create(
+            data=data,
+            files=[
+                NewLocalFile(file=analysis, override_name="analysis.json"),
+                NewLocalFile(file=lintian_output, override_name="lintian.txt"),
+            ],
+        )
 
-        artifact.add_local_file(analysis, override_name="analysis.json")
-        artifact.add_local_file(lintian_output, override_name="lintian.txt")
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def _validate_required_files(
         cls, files: dict[str, Path]
@@ -668,7 +681,7 @@ class LintianArtifact(LocalArtifact[data_models.DebianLintian]):
             except JSONDecodeError as exc:
                 raise ValueError(f"{file_name} is not valid JSON: {exc}")
 
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def _validate_file_analysis_is_json(
         cls, files: dict[str, Path]
@@ -688,7 +701,7 @@ class AutopkgtestArtifact(LocalArtifact[data_models.DebianAutopkgtest]):
         cls, artifact_directory: Path, data: data_models.DebianAutopkgtest
     ) -> Self:
         """Return AutopkgtestArtifact with the files and data set."""
-        artifact = cls(category=cls._category, data=data)
+        files = []
 
         for file in artifact_directory.rglob("*"):
             if not file.is_file():
@@ -699,9 +712,11 @@ class AutopkgtestArtifact(LocalArtifact[data_models.DebianAutopkgtest]):
                 # Skip binaries/
                 continue
 
-            artifact.add_local_file(file, artifact_base_dir=artifact_directory)
+            files.append(
+                NewLocalFile(file=file, artifact_base_dir=artifact_directory)
+            )
 
-        return artifact
+        return cls._create(data=data, files=files)
 
 
 class PiupartsArtifact(LocalArtifact[data_models.DebianPiuparts]):
@@ -714,11 +729,12 @@ class PiupartsArtifact(LocalArtifact[data_models.DebianPiuparts]):
         cls, *, piuparts_output: Path, data: data_models.DebianPiuparts
     ) -> Self:
         """Return a ``PiupartsArtifact`` with the files and data set."""
-        artifact = cls(category=cls._category, data=data)
-
-        artifact.add_local_file(piuparts_output, override_name="piuparts.txt")
-
-        return artifact
+        return cls._create(
+            data=data,
+            files=[
+                NewLocalFile(file=piuparts_output, override_name="piuparts.txt")
+            ],
+        )
 
 
 class DebianSystemTarballArtifact(
@@ -737,15 +753,12 @@ class DebianSystemTarballArtifact(
         """Return a DebianSystemTarballArtifact with the tarball file."""
         data = data.copy()
         data["filename"] = tarball.name
-        artifact = cls(
-            category=cls._category, data=data_models.DebianSystemTarball(**data)
+        return cls._create(
+            data=data_models.DebianSystemTarball(**data),
+            files=[NewLocalFile(file=tarball)],
         )
 
-        artifact.add_local_file(tarball)
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def _validate_file_name_ends_in_tar_xz(
         cls, files: dict[str, Path]
@@ -774,16 +787,16 @@ class BlhcArtifact(LocalArtifact[data_models.DebianBlhc]):
         build_log: data_models.DebianPackageBuildLog | None,
     ) -> Self:
         """Return a BlhcArtifact with the files set."""
-        data = data_models.DebianBlhc(
-            source=None if build_log is None else build_log.source,
-            version=None if build_log is None else build_log.version,
-            architecture=None if build_log is None else build_log.architecture,
+        return cls._create(
+            data=data_models.DebianBlhc(
+                source=None if build_log is None else build_log.source,
+                version=None if build_log is None else build_log.version,
+                architecture=(
+                    None if build_log is None else build_log.architecture
+                ),
+            ),
+            files=[NewLocalFile(file=blhc_output, override_name="blhc.txt")],
         )
-        artifact = cls(category=cls._category, data=data)
-
-        artifact.add_local_file(blhc_output, override_name="blhc.txt")
-
-        return artifact
 
 
 class DebDiffArtifact(LocalArtifact[data_models.DebDiff]):
@@ -792,15 +805,21 @@ class DebDiffArtifact(LocalArtifact[data_models.DebDiff]):
     _category = data_models.ArtifactCategory.DEBDIFF
 
     @classmethod
-    def create(cls, debdiff_output: Path, original: str, new: str) -> Self:
+    def create(
+        cls,
+        debdiff_output: Path,
+        original: str,
+        new: str,
+        *,
+        label: str | None = None,
+    ) -> Self:
         """Return a DebDiffArtifact with the files set."""
-        data = data_models.DebDiff(original=original, new=new)
-
-        artifact = cls(category=cls._category, data=data)
-
-        artifact.add_local_file(debdiff_output, override_name="debdiff.txt")
-
-        return artifact
+        return cls._create(
+            data=data_models.DebDiff(original=original, new=new, label=label),
+            files=[
+                NewLocalFile(file=debdiff_output, override_name="debdiff.txt")
+            ],
+        )
 
 
 class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
@@ -817,15 +836,12 @@ class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
         """Return a DebianSystemImageArtifact with the image file."""
         data = data.copy()
         data["filename"] = image.name
-        artifact = cls(
-            category=cls._category, data=data_models.DebianSystemImage(**data)
+        return cls._create(
+            data=data_models.DebianSystemImage(**data),
+            files=[NewLocalFile(file=image)],
         )
 
-        artifact.add_local_file(image)
-
-        return artifact
-
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def _validate_files(cls, files: dict[str, Path]) -> dict[str, Path]:
         """Check if the artifact contains only one file and it's ending."""
@@ -863,19 +879,22 @@ class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
             ),
             binary_package_name=binary_package_name,
         )
-        artifact = cls(category=cls._category, data=data)
-        for file in files:
-            artifact.add_local_file(
-                file,
-                artifact_base_dir=base_dir,
-                # The content-type here doesn't matter, but we don't want to
-                # involve libmagic in a potentially-sensitive signing
-                # workflow.
-                content_type="application/octet-stream",
-            )
-        return artifact
+        return cls._create(
+            data=data,
+            files=[
+                NewLocalFile(
+                    file=file,
+                    artifact_base_dir=base_dir,
+                    # The content-type here doesn't matter, but we don't
+                    # want to involve libmagic in a potentially-sensitive
+                    # signing workflow.
+                    content_type="application/octet-stream",
+                )
+                for file in files
+            ],
+        )
 
-    @pydantic.validator("files")
+    @pydantic.field_validator("files", mode="after")
     @classmethod
     def validate_at_least_one_file(
         cls, files: dict[str, Path]
@@ -908,17 +927,20 @@ class SigningOutputArtifact(LocalArtifact[data_models.DebusineSigningOutput]):
             results=list(results),
             binary_package_name=binary_package_name,
         )
-        artifact = cls(category=cls._category, data=data)
-        for file in files:
-            artifact.add_local_file(
-                file,
-                artifact_base_dir=base_dir,
-                # The content-type here doesn't matter, but we don't want to
-                # involve libmagic in a potentially-sensitive signing
-                # workflow.
-                content_type="application/octet-stream",
-            )
-        return artifact
+        return cls._create(
+            data=data,
+            files=[
+                NewLocalFile(
+                    file=file,
+                    artifact_base_dir=base_dir,
+                    # The content-type here doesn't matter, but we don't
+                    # want to involve libmagic in a potentially-sensitive
+                    # signing workflow.
+                    content_type="application/octet-stream",
+                )
+                for file in files
+            ],
+        )
 
 
 class RepositoryIndex(LocalArtifact[data_models.DebianRepositoryIndex]):
@@ -932,8 +954,13 @@ class RepositoryIndex(LocalArtifact[data_models.DebianRepositoryIndex]):
     ) -> Self:
         """Return a RepositoryIndex."""
         data = data_models.DebianRepositoryIndex(path=path)
-        artifact = cls(category=cls._category, data=data)
-        artifact.add_local_file(
-            file, override_name=PurePath(path).name, content_type=content_type
+        return cls._create(
+            data=data,
+            files=[
+                NewLocalFile(
+                    file=file,
+                    override_name=PurePath(path).name,
+                    content_type=content_type,
+                )
+            ],
         )
-        return artifact
