@@ -638,7 +638,7 @@ class BinaryPackageTests(TestCase):
             control_file_names=["md5sums"],
         )
 
-        package = BinaryPackage.create(file=file)
+        artifact = BinaryPackage.create(file=file)
 
         expected_data = data_models.DebianBinaryPackage(
             srcpkg_name=srcpkg_name,
@@ -655,9 +655,9 @@ class BinaryPackageTests(TestCase):
         )
         expected_files = {file.name: file}
 
-        self.assertEqual(package.category, BinaryPackage._category)
-        self.assertEqual(package.data, expected_data)
-        self.assertEqual(package.files, expected_files)
+        self.assertEqual(artifact.category, BinaryPackage._category)
+        self.assertEqual(artifact.data, expected_data)
+        self.assertEqual(artifact.files, expected_files)
 
     def test_files_must_end_in_deb_or_udeb(self) -> None:
         """files_must_end_in_deb_or_udeb() returns the files."""
@@ -731,7 +731,7 @@ class BinaryPackagesTests(TestCase):
         for file in files:
             file.touch()
 
-        package = BinaryPackages.create(
+        artifact = BinaryPackages.create(
             srcpkg_name=srcpkg_name,
             srcpkg_version=srcpkg_version,
             version=version,
@@ -751,9 +751,9 @@ class BinaryPackagesTests(TestCase):
             files[1].name: files[1],
         }
 
-        self.assertEqual(package.category, BinaryPackages._category)
-        self.assertEqual(package.data, expected_data)
-        self.assertEqual(package.files, expected_files)
+        self.assertEqual(artifact.category, BinaryPackages._category)
+        self.assertEqual(artifact.data, expected_data)
+        self.assertEqual(artifact.files, expected_files)
 
     def test_files_must_end_in_deb_or_udeb(self) -> None:
         """files_must_end_in_deb_or_udeb() return the files."""
@@ -869,13 +869,13 @@ class SourcePackageTests(TestCase):
             dsc_fields=dsc_fields,
         )
 
-        package = SourcePackage.create(
+        artifact = SourcePackage.create(
             name=name, version=version, files=[tar_file, dsc_file]
         )
 
-        self.assertEqual(package.category, SourcePackage._category)
-        self.assertEqual(package.files, expected_files)
-        self.assertEqual(package.data, expected_data)
+        self.assertEqual(artifact.category, SourcePackage._category)
+        self.assertEqual(artifact.files, expected_files)
+        self.assertEqual(artifact.data, expected_data)
 
     def test_create_invalid_dsc_raise_value_error(self) -> None:
         """Raise ValueError: invalid .dsc file."""
@@ -1337,17 +1337,18 @@ class DebDiffArtifactTests(TestCase):
             suffix=".txt", contents=self.output_content
         )
 
-        package = DebDiffArtifact.create(
-            debdiff_output=output, original="original", new="new"
+        artifact = DebDiffArtifact.create(
+            debdiff_output=output, original="original", new="new", label="O → N"
         )
 
-        self.assertIsInstance(package, DebDiffArtifact)
+        self.assertIsInstance(artifact, DebDiffArtifact)
         self.assertEqual(
-            package.data, data_models.DebDiff(original="original", new="new")
+            artifact.data,
+            data_models.DebDiff(original="original", new="new", label="O → N"),
         )
 
         self.assertEqual(
-            package.files,
+            artifact.files,
             {
                 "debdiff.txt": output,
             },
