@@ -8,14 +8,10 @@
 # contained in the LICENSE file.
 
 """Unit tests for classes in models.py."""
+
 from typing import Any
 from unittest import TestCase
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
 import debusine.artifacts.models as data_models
 from debusine.assets.models import KeyPurpose
 
@@ -52,11 +48,9 @@ class DebusineHistoricalTaskRunTests(TestCase):
         data_models.DebusineHistoricalTaskRun(
             task_type=data_models.TaskTypes.WORKER, **kwargs
         )
-        with self.assertRaisesRegex(
-            ValueError, "is not a valid enumeration member"
-        ):
+        with self.assertRaisesRegex(ValueError, r"Input should be.*type=enum"):
             data_models.DebusineHistoricalTaskRun(
-                task_type="does-not-exist", **kwargs  # type: ignore[arg-type]
+                task_type="does-not-exist", **kwargs
             )
 
 
@@ -94,11 +88,9 @@ class DebusineTaskConfigurationTests(TestCase):
         data_models.DebusineTaskConfiguration(
             task_type=data_models.TaskTypes.WORKER, **kwargs
         )
-        with self.assertRaisesRegex(
-            ValueError, "is not a valid enumeration member"
-        ):
+        with self.assertRaisesRegex(ValueError, r"Input should be.*type=enum"):
             data_models.DebusineTaskConfiguration(
-                task_type="does-not-exist", **kwargs  # type: ignore[arg-type]
+                task_type="does-not-exist", **kwargs
             )
         with self.assertRaisesRegex(ValueError, "should be set"):
             data_models.DebusineTaskConfiguration(task_type=None, **kwargs)
@@ -182,7 +174,7 @@ class DebusineTaskConfigurationTests(TestCase):
                 self.assertEqual(f(*args), expected)
 
 
-class EmptyArtifactDataTest(TestCase):
+class EmptyArtifactDataTests(TestCase):
     """Test the EmptyArtifactData model."""
 
     def test_get_label(self) -> None:
@@ -192,7 +184,7 @@ class EmptyArtifactDataTest(TestCase):
         self.assertIsNone(label)
 
 
-class DebianPackageBuildLogTest(TestCase):
+class DebianPackageBuildLogTests(TestCase):
     """Test the DebianPackageBuildLog model."""
 
     def test_get_label(self) -> None:
@@ -226,7 +218,7 @@ class DebianSourcePackageTests(TestCase):
 
     def test_dsc_fields_missing(self) -> None:
         """Test checking that dsc_fields is present."""
-        error_msg = r"dsc_fields\s+field required \(type=value_error\.missing\)"
+        error_msg = r"dsc_fields\s+Field required.*type=missing"
 
         with self.assertRaisesRegex(Exception, error_msg):
             data_models.DebianSourcePackage(  # type: ignore[call-arg]
@@ -436,7 +428,7 @@ class DebianUploadTests(TestCase):
         )
 
 
-class DebianBinaryPackageTest(TestCase):
+class DebianBinaryPackageTests(TestCase):
     """Test the DebianBinaryPackage model."""
 
     def test_get_label(self) -> None:
@@ -454,7 +446,7 @@ class DebianBinaryPackageTest(TestCase):
         self.assertEqual(data.get_label(), "test-bin_1.0-1+b1_amd64")
 
 
-class DebianBinaryPackagesTest(TestCase):
+class DebianBinaryPackagesTests(TestCase):
     """Test the DebianBinaryPackages model."""
 
     def test_get_label(self) -> None:
@@ -469,7 +461,7 @@ class DebianBinaryPackagesTest(TestCase):
         self.assertEqual(data.get_label(), "test_1.0-1")
 
 
-class DebianSystemTarballTest(TestCase):
+class DebianSystemTarballTests(TestCase):
     """Test the DebianSystemTarball model."""
 
     def test_get_label(self) -> None:
@@ -479,9 +471,7 @@ class DebianSystemTarballTest(TestCase):
             vendor="debian",
             codename="bookworm",
             components=["main"],
-            mirror=pydantic.parse_obj_as(
-                pydantic.AnyUrl, "https://deb.debian.org"
-            ),
+            mirror="https://deb.debian.org",
             variant=None,
             pkglist={},
             architecture="amd64",
@@ -491,7 +481,7 @@ class DebianSystemTarballTest(TestCase):
         self.assertEqual(data.get_label(), "bookworm.tar.gz")
 
 
-class DebianSystemImageTest(TestCase):
+class DebianSystemImageTests(TestCase):
     """Test the DebianSystemImage model."""
 
     def test_get_label(self) -> None:
@@ -501,9 +491,7 @@ class DebianSystemImageTest(TestCase):
             vendor="debian",
             codename="bookworm",
             components=["main"],
-            mirror=pydantic.parse_obj_as(
-                pydantic.AnyUrl, "https://deb.debian.org"
-            ),
+            mirror="https://deb.debian.org",
             variant=None,
             pkglist={},
             architecture="amd64",
@@ -517,7 +505,7 @@ class DebianSystemImageTest(TestCase):
         self.assertEqual(data.get_label(), "bookworm.qcow2")
 
 
-class DebianLintianTest(TestCase):
+class DebianLintianTests(TestCase):
     """Test the DebianLintian model."""
 
     def test_get_label(self) -> None:
@@ -548,7 +536,7 @@ class DebianLintianTest(TestCase):
         )
 
 
-class DebianAutopkgtestTest(TestCase):
+class DebianAutopkgtestTests(TestCase):
     """Test the DebianAutopkgtest model."""
 
     def test_get_label(self) -> None:
@@ -559,9 +547,7 @@ class DebianAutopkgtestTest(TestCase):
             source_package=data_models.DebianAutopkgtestSource(
                 name="hello",
                 version="1.0-1",
-                url=pydantic.parse_obj_as(
-                    pydantic.AnyUrl, "https://deb.debian.org/pool/h/hello"
-                ),
+                url="https://deb.debian.org/pool/h/hello",
             ),
             architecture="amd64",
             distribution="bookworm",
@@ -569,7 +555,7 @@ class DebianAutopkgtestTest(TestCase):
         self.assertEqual(data.get_label(), "hello")
 
 
-class DebianPiupartsTest(TestCase):
+class DebianPiupartsTests(TestCase):
     """Test the DebianPiuparts model."""
 
     def test_get_label(self) -> None:
@@ -582,7 +568,7 @@ class DebianPiupartsTest(TestCase):
         self.assertEqual(data.get_label(), "piuparts: hello, hello-doc")
 
 
-class DebusineSigningInputTest(TestCase):
+class DebusineSigningInputTests(TestCase):
     """Test the DebusineSigningInput model."""
 
     def test_get_label(self) -> None:
@@ -613,10 +599,10 @@ class DebusinePromiseTests(TestCase):
             promise_work_request_id=1,
             promise_workflow_id=100,
             promise_category="debian:binary-package",
-            extra_field="extra_value",  # type: ignore
+            extra_field="extra_value",  # type: ignore[call-arg]
         )
 
-        self.assertEqual(data.extra_field, "extra_value")  # type: ignore
+        self.assertEqual(data.extra_field, "extra_value")  # type: ignore[attr-defined]
 
     def test_forbidden_promise_prefix_in_extra_field(self) -> None:
         r"""Test that extra fields with 'promise\\_' are not allowed."""
@@ -628,7 +614,7 @@ class DebusinePromiseTests(TestCase):
                 promise_work_request_id=1,
                 promise_workflow_id=100,
                 promise_category="debian:binary-package",
-                promise_extra="not_allowed",  # type: ignore
+                promise_extra="not_allowed",  # type: ignore[call-arg]
             )
 
 
@@ -662,7 +648,7 @@ class SigningResultTests(TestCase):
             data_models.SigningResult(file="file")
 
 
-class DebusineSigningOutputTest(TestCase):
+class DebusineSigningOutputTests(TestCase):
     """Test the DebusineSigningOutput model."""
 
     def test_get_label(self) -> None:
@@ -690,9 +676,17 @@ class DebianBlhcTests(TestCase):
         self.assertEqual(data.get_label(), "blhc analysis for hello_1.0_amd64")
 
 
-class DebDiffTest(TestCase):
+class DebDiffTests(TestCase):
     """Test the DebDiff model."""
 
+    def test_get_label_override(self) -> None:
+        data = data_models.DebDiff(
+            original="original-package",
+            new="new-package",
+            label="A → B",
+        )
+        self.assertEqual(data.get_label(), "A → B")
+
     def test_get_label(self) -> None:
         """get_label returns the debdiff command."""
         data = data_models.DebDiff(
@@ -914,8 +908,8 @@ class GetBinaryPackageVersionTests(TestCase):
             )
 
 
-class GetArchitectureTests(TestCase):
-    """Tests for function get_architecture."""
+class GetArchitecturesTests(TestCase):
+    """Tests for function get_architectures."""
 
     def test_DebianBinaryPackages(self) -> None:
         artifact_data = data_models.DebianBinaryPackage(
@@ -929,7 +923,9 @@ class GetArchitectureTests(TestCase):
             deb_control_files=[],
         )
 
-        self.assertEqual(data_models.get_architecture(artifact_data), "amd64")
+        self.assertEqual(
+            data_models.get_architectures(artifact_data), {"amd64"}
+        )
 
     def test_DebianPackageBuildLog(self) -> None:
         data = data_models.DebianPackageBuildLog(
@@ -938,7 +934,7 @@ class GetArchitectureTests(TestCase):
             architecture="amd64",
             filename="test_1.0-1_amd64.buildlog",
         )
-        self.assertEqual(data_models.get_architecture(data), "amd64")
+        self.assertEqual(data_models.get_architectures(data), {"amd64"})
 
     def test_DebianBinaryPackage(self) -> None:
         data = data_models.DebianBinaryPackage(
@@ -951,7 +947,7 @@ class GetArchitectureTests(TestCase):
             },
             deb_control_files=[],
         )
-        self.assertEqual(data_models.get_architecture(data), "amd64")
+        self.assertEqual(data_models.get_architectures(data), {"amd64"})
 
     def test_DebianUpload(self) -> None:
         data = data_models.DebianUpload(
@@ -962,7 +958,22 @@ class GetArchitectureTests(TestCase):
                 "Files": [{"name": "hello_1.0_amd64.deb"}],
             },
         )
-        self.assertEqual(data_models.get_architecture(data), "amd64")
+        self.assertEqual(data_models.get_architectures(data), {"amd64"})
+
+    def test_mixed_DebianUpload(self) -> None:
+        data = data_models.DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Source": "hello",
+                "Architecture": "amd64 all source",
+                "Files": [
+                    {"name": "hello_1.0.dsc"},
+                    {"name": "hello_1.0_amd64.deb"},
+                    {"name": "hello-doc_1.0_all.deb"},
+                ],
+            },
+        )
+        self.assertEqual(data_models.get_architectures(data), {"amd64", "all"})
 
     def test_invalid_type(self) -> None:
         data = data_models.EmptyArtifactData()
@@ -970,4 +981,4 @@ class GetArchitectureTests(TestCase):
         with self.assertRaisesRegex(
             TypeError, "^Unexpected type: EmptyArtifactData$"
         ):
-            data_models.get_architecture(data)  # type: ignore[arg-type]
+            data_models.get_architectures(data)  # type: ignore[arg-type]
