@@ -10,11 +10,14 @@
 """Models used for asset data."""
 
 from debusine.assets.models import (
+    APTAuthenticationData,
     AWSProviderAccountConfiguration,
     AWSProviderAccountCredentials,
     AWSProviderAccountData,
     AssetCategory,
     BaseAssetDataModel,
+    BasicAPTAuthenticationData,
+    BasicAPTCredentials,
     CloudProviderAccountData,
     CloudProvidersType,
     DummyProviderAccountData,
@@ -28,11 +31,14 @@ from debusine.assets.models import (
 )
 
 __all__ = [
+    "APTAuthenticationData",
     "AWSProviderAccountConfiguration",
     "AWSProviderAccountCredentials",
     "AWSProviderAccountData",
     "AssetCategory",
     "BaseAssetDataModel",
+    "BasicAPTAuthenticationData",
+    "BasicAPTCredentials",
     "CloudProviderAccountData",
     "CloudProvidersType",
     "DummyProviderAccountData",
