@@ -10,17 +10,12 @@
 """Models used for asset data."""
 
 import abc
-import base64
-import binascii
 from enum import StrEnum
 from typing import Annotated, Any, Literal, Union
 
-from debusine.utils import DjangoChoicesEnum
+import pydantic
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
+from debusine.utils import Base64BytesAsStr, DjangoChoicesEnum
 
 
 class AssetCategory(DjangoChoicesEnum):
@@ -28,16 +23,13 @@ class AssetCategory(DjangoChoicesEnum):
 
     CLOUD_PROVIDER_ACCOUNT = "debusine:cloud-provider-account"
     SIGNING_KEY = "debusine:signing-key"
+    APT_AUTHENTICATION = "debian:apt-authentication"
 
 
 class BaseAssetDataModel(pydantic.BaseModel, abc.ABC):
     """Base pydantic model for asset data and their components."""
 
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
-        extra = "forbid"
+    model_config = pydantic.ConfigDict(validate_assignment=True, extra="forbid")
 
 
 class ModelCategory(BaseAssetDataModel):
@@ -67,7 +59,7 @@ class BaseRegisteredAssetDataModel(BaseAssetDataModel, abc.ABC):
         assert modelcategory.discriminator == discriminator
         discriminator_value: str | None = None
         if discriminator:
-            discriminator_value = cls.__fields__[discriminator].default
+            discriminator_value = getattr(cls, discriminator)
         modelcategory.models[discriminator_value] = cls
 
 
@@ -95,7 +87,7 @@ def asset_data_model(
                 f"{modelcategory.discriminator}={discriminator_value}."
             )
         model = modelcategory.models[discriminator_value]
-    return model.parse_obj(data)
+    return model(**data)
 
 
 def asset_categories() -> list[AssetCategory]:
@@ -124,20 +116,9 @@ class SigningKeyData(
 
     purpose: KeyPurpose
     fingerprint: str
-    public_key: str  # base64-encoded
+    public_key: Base64BytesAsStr
     description: str | None
 
-    # TODO: We could use Base64Str in Pydantic 2.
-    @pydantic.validator("public_key")
-    @classmethod
-    def check_public_key_base64(cls, data: str) -> str:
-        """Validate that ``public_key`` is base64-encoded."""
-        try:
-            base64.b64decode(data)
-        except binascii.Error:
-            raise ValueError("data must be base64-encoded")
-        return data
-
 
 class CloudProvidersType(StrEnum):
     """Choices for CloudProviderAccountData.provider_type."""
@@ -222,3 +203,35 @@ CloudProviderAccountData = Annotated[
     ],
     pydantic.Field(discriminator="provider_type"),
 ]
+
+
+class APTAuthenticationScheme(StrEnum):
+    """Choices for APTAuthentication.auth_scheme."""
+
+    BASIC = "basic"
+
+
+class BasicAPTCredentials(BaseAssetDataModel):
+    """APT credentials for use with HTTP Basic Auth."""
+
+    username: str
+    password: str
+
+
+class BasicAPTAuthenticationData(
+    BaseRegisteredAssetDataModel,
+    category=AssetCategory.APT_AUTHENTICATION,
+    discriminator="auth_scheme",
+):
+    """Data for a debian:apt-authentication asset for HTTP Basic Auth."""
+
+    auth_scheme: Literal[APTAuthenticationScheme.BASIC] = (
+        APTAuthenticationScheme.BASIC
+    )
+    name: str
+    credentials: BasicAPTCredentials
+
+
+APTAuthenticationData = Annotated[
+    BasicAPTAuthenticationData, pydantic.Field(discriminator="provider_type")
+]
