@@ -143,6 +143,10 @@ class Command(abc.ABC):
                 + ', '.join(repr(x) for x in cls.commands_by_group.keys())
             )
         for command_cls in cls.commands_by_group[group]:
+            # Skip deprecated commands during shell completion
+            if command_cls.deprecated and "_ARGCOMPLETE" in os.environ:
+                continue
+
             description = _get_help(command_cls)
             kwargs: dict[str, Any] = {"help": description}
             if command_cls.deprecated:
@@ -534,7 +538,6 @@ class InputDataCommand(Command, abc.ABC):
                 # Empty stdin defaults to {}
                 if not data or data.isspace():
                     return {}
-                pass  # Extra statement to attempt to fix coverage reporting
             case _:
                 try:
                     with open(file) as fd:
