@@ -17,6 +17,7 @@ from collections.abc import Iterable
 from functools import partial
 from typing import Any, NoReturn
 from unittest import mock
+from unittest.mock import patch
 
 import rich.console
 import yaml
@@ -823,3 +824,21 @@ class InputDataCommandTests(BaseCliTests):
         ):
             self.assertEqual(command.read_input_data("-"), {})
         console_print.assert_called()
+
+    def test_deprecated_commands_skipped_during_completion(self) -> None:
+        """Deprecated commands should not be registered during argcomplete."""
+        # Simulate argcomplete shell-completion mode
+        with patch.dict("os.environ", {"_ARGCOMPLETE": "1"}):
+            parser = argparse.ArgumentParser()
+            subparsers = parser.add_subparsers(dest="cmd")
+
+            # Register commands in completion mode
+            Command.add_parsers(subparsers)
+
+            registered = set(subparsers._name_parser_map.keys())
+
+            # Deprecated commands must not be offered as completion candidates
+            self.assertNotIn("list-work-requests", registered)
+
+            # Non-deprecated commands should still be registered
+            self.assertGreater(len(registered), 0)
