@@ -10,17 +10,25 @@
 """Debusine command line interface, work request management commands."""
 
 import argparse
+import datetime as dt
 import os
 import shutil
 import subprocess
 import tempfile
+from collections.abc import Iterable
 from itertools import islice
 from pathlib import Path
-from typing import Any
+from typing import override
+
+import rich
+import rich.json
+from rich.table import Table
 
 from debusine.artifacts import Upload
+from debusine.artifacts.models import WorkRequestResults
 from debusine.client.commands.base import (
     DebusineCommand,
+    ModelCommand,
     RequiredInputDataCommand,
     WorkspaceCommand,
 )
@@ -29,11 +37,116 @@ from debusine.client.models import (
     WorkRequestExternalDebsignRequest,
     WorkRequestRequest,
     WorkRequestResponse,
-    model_to_json_serializable_dict,
 )
 
 
-class List(DebusineCommand, group="work-request"):
+class WorkRequestModelCommand(
+    ModelCommand[WorkRequestResponse], DebusineCommand
+):
+    """Output WorkRequestResponse objects."""
+
+    @override
+    def _list_rich(self, instances: Iterable[WorkRequestResponse]) -> None:
+        table = Table(box=rich.box.MINIMAL_DOUBLE_HEAD)
+        table.add_column("ID")
+        table.add_column("Task")
+        table.add_column("Status")
+        table.add_column("Progress")
+        table.add_column("Worker")
+        table.add_column("Task data")
+        for wr in instances:
+            url = self._url(wr)
+            status = [wr.status]
+            if wr.result != WorkRequestResults.NONE:
+                status.append(wr.result)
+            progress = [f"Created: {self._format_timestamp(wr.created_at)}"]
+            if wr.started_at:
+                progress.append(
+                    f"Started: {self._format_timestamp(wr.started_at)}"
+                )
+            if wr.completed_at:
+                progress.append(
+                    f"Completed: {self._format_timestamp(wr.completed_at)}"
+                )
+            table.add_row(
+                f"[link={url}]#{wr.id}[/]",
+                f"{wr.task_type} {wr.task_name}",
+                " ".join(status),
+                "\n".join(progress),
+                # TODO: show the worker name, if we can extend the API to get
+                # that information
+                self._format_int(wr.worker),
+                rich.json.JSON.from_data(wr.task_data),
+            )
+        rich.print(table)
+
+    def _url(self, instance: WorkRequestResponse) -> str:
+        return self.debusine.webui_url(instance.get_link_url("webui_self"))
+
+    def _format_timestamp(self, ts: dt.datetime | None) -> str | None:
+        if ts is None:
+            return None
+        return ts.strftime("%Y-%m-%d %H:%M:%S")
+
+    def _format_int(self, value: int | None) -> str | None:
+        if value is None:
+            return None
+        return str(value)
+
+    def _format_float(self, value: float) -> str | None:
+        return f"{value:.2f}"
+
+    @override
+    def _show_rich(self, instance: WorkRequestResponse) -> None:
+        url = self._url(instance)
+        table = Table(box=rich.box.SIMPLE, show_header=False)
+        table.add_column("Name", justify="right")
+        table.add_row("ID:", f"#{instance.id} [link={url}]{url}[/]")
+        table.add_row("Task:", f"{instance.task_type} {instance.task_name}")
+        table.add_row("Workspace:", f"{instance.scope}/{instance.workspace}")
+        table.add_row(
+            "Created at:", self._format_timestamp(instance.created_at)
+        )
+        if instance.started_at is not None:
+            table.add_row(
+                "Started at:", self._format_timestamp(instance.started_at)
+            )
+        if instance.completed_at is not None:
+            table.add_row(
+                "Completed at:", self._format_timestamp(instance.completed_at)
+            )
+        if instance.duration is not None:
+            table.add_row("Duration:", self._format_float(instance.duration))
+        if instance.result == WorkRequestResults.NONE:
+            table.add_row("Status:", instance.status)
+        else:
+            table.add_row("Status:", f"{instance.status} {instance.result}")
+        if instance.worker is not None:
+            # TODO: show the worker name, if we can extend the API to get that
+            # information
+            table.add_row("Worker:", self._format_int(instance.worker))
+        if instance.priority_base != 0 or instance.priority_adjustment != 0:
+            table.add_row(
+                "Priority:",
+                f"{instance.priority_base} + {instance.priority_adjustment}",
+            )
+        table.add_row(
+            "Task data:", rich.json.JSON.from_data(instance.task_data)
+        )
+        if instance.dynamic_task_data:
+            table.add_row(
+                "Dynamic task data:",
+                rich.json.JSON.from_data(instance.dynamic_task_data),
+            )
+        if instance.artifacts:
+            table.add_row(
+                "Artifacts:",
+                rich.json.JSON.from_data(instance.artifacts),
+            )
+        rich.print(table)
+
+
+class List(WorkRequestModelCommand, DebusineCommand, group="work-request"):
     """List up to limit most recent work requests."""
 
     @classmethod
@@ -54,10 +167,7 @@ class List(DebusineCommand, group="work-request"):
             work_requests = list(
                 islice(self.debusine.work_request_iter(), self.args.limit)
             )
-        result = {
-            "results": [work_request.dict() for work_request in work_requests]
-        }
-        self.print_yaml_output(result)
+        self.list(work_requests)
 
 
 class LegacyList(
@@ -91,10 +201,10 @@ class Show(DebusineCommand, group="work-request"):
             with self._api_call_or_fail():
                 artifact_information = self.debusine.artifact_get(artifact_id)
             artifacts_information.append(
-                model_to_json_serializable_dict(artifact_information)
+                artifact_information.model_dump(mode="json")
             )
 
-        result = work_request.dict()
+        result = work_request.model_dump()
         result["artifacts"] = artifacts_information
 
         self.print_yaml_output(result)
@@ -106,7 +216,12 @@ class LegacyShow(
     """Print the status of a work request."""
 
 
-class Create(RequiredInputDataCommand, WorkspaceCommand, group="work-request"):
+class Create(
+    WorkRequestModelCommand,
+    RequiredInputDataCommand,
+    WorkspaceCommand,
+    group="work-request",
+):
     """
     Create a work request and schedule the execution.
 
@@ -151,12 +266,8 @@ class Create(RequiredInputDataCommand, WorkspaceCommand, group="work-request"):
                 work_request
             )
 
-        output: dict[str, Any] = {
-            'result': 'success',
-            'message': (f"Work request registered: {work_request_created.url}"),
-            'work_request_id': work_request_created.id,
-        }
-        self.print_yaml_output(output)
+        self.feedback("Work request started:")
+        self.show(work_request_created)
 
 
 class LegacyCreate(
@@ -348,11 +459,7 @@ class Sign(DebusineCommand, group="work-request"):
                 path = tmp / name
                 assert path.resolve().is_relative_to(tmp)
                 assert "/" not in name
-                if (
-                    name.endswith(".changes")
-                    or name.endswith(".dsc")
-                    or name.endswith(".buildinfo")
-                ):
+                if name.endswith((".changes", ".dsc", ".buildinfo")):
                     if local_file:
                         self._fetch_local_file(
                             local_file.parent / name, path, file_response
