@@ -14,6 +14,7 @@ Debusine: Interacts with debusine server.
 
 Debusine fetches information, submit work requests and other operations.
 """
+
 import asyncio
 import datetime as dt
 import functools
@@ -32,12 +33,17 @@ from urllib.parse import quote, urlencode, urlparse, urlunparse
 
 import aiohttp
 import magic
+import pydantic
 import requests
 import tenacity
 from requests_toolbelt.downloadutils.stream import stream_response_to_file
 
 from debusine.artifacts import LocalArtifact
-from debusine.artifacts.models import ArtifactData, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactData,
+    CollectionCategory,
+    WorkRequestResults,
+)
 from debusine.assets import AssetCategory, BaseAssetDataModel
 from debusine.client.debusine_http_client import DebusineHttpClient
 from debusine.client.exceptions import TokenDisabledError
@@ -52,6 +58,7 @@ from debusine.client.models import (
     CollectionData,
     CollectionDataNew,
     CreateWorkflowRequest,
+    EmitMetricRequest,
     FileRequest,
     FilesRequestType,
     LookupChildType,
@@ -75,8 +82,8 @@ from debusine.client.models import (
     WorkflowTemplateData,
     WorkflowTemplateDataNew,
     WorkspaceInheritanceChain,
-    model_to_json_serializable_dict,
 )
+from debusine.utils.urls import URLPath
 
 if TYPE_CHECKING:
     from debusine.client.task_configuration import (
@@ -136,22 +143,20 @@ def serialize_local_artifact(
     for artifact_path, file_request in artifact.remote_files.items():
         files[artifact_path] = file_request
 
-    artifact.validate_model()
+    artifact.model_validate(artifact)
 
-    serialized = model_to_json_serializable_dict(
-        ArtifactCreateRequest(
-            workspace=workspace,
-            category=artifact.category,
-            files=files,
-            data=(
-                artifact.data.dict()
-                if isinstance(artifact.data, ArtifactData)
-                else artifact.data
-            ),
-            work_request=work_request,
-            expire_at=expire_at,
-        )
-    )
+    serialized = ArtifactCreateRequest(
+        workspace=workspace,
+        category=artifact.category,
+        files=files,
+        data=(
+            artifact.data.model_dump()
+            if isinstance(artifact.data, ArtifactData)
+            else artifact.data
+        ),
+        work_request=work_request,
+        expire_at=expire_at,
+    ).model_dump(mode="json")
 
     # If the workspace was not specified: do not send it to the API.
     # The server will assign it.
@@ -184,7 +189,8 @@ class Debusine:
         self.token: str | None = api_token
         self.api_url = base_api_url.rstrip("/") + "/" + self.API_VERSION
 
-        # TODO: Currently Workers require the use of an unscoped client.
+        # TODO: Currently Workers require the use of an unscoped client for
+        # their websocket connection.
         # Ideally 'scope' will become a constructor parameter, where
         # self.scope can't be None, simplifying/hardening the code.
         self.scope: str | None = scope
@@ -198,10 +204,11 @@ class Debusine:
     def webui_url(self, path: str) -> str:
         """Generate a URL to the web UI given a full path."""
         parsed = urlparse(self.base_api_url)
-        new_path = (
-            os.path.dirname(parsed.path).rstrip("/") + "/" + path.lstrip('/')
-        )
-        parsed = parsed._replace(path=new_path)
+        parsed_path = URLPath(parsed.path)
+        if parsed_path and parsed_path[-1] == "api":
+            parsed_path.pop()
+        parsed_path += path
+        parsed = parsed._replace(path=str(parsed_path))
         return urlunparse(parsed)
 
     def work_request_iter(self) -> Iterable[WorkRequestResponse]:
@@ -263,15 +270,18 @@ class Debusine:
         return f"/work-request/{work_request_id}/completed/"
 
     def work_request_completed_update(
-        self, work_request_id: int, result: str, output_data: "OutputData"
+        self,
+        work_request_id: int,
+        result: WorkRequestResults,
+        output_data: "OutputData",
     ) -> None:
         """Update work_request_id as completed with result."""
         return self._debusine_http_client.put(
             path=self.work_request_completed_path(work_request_id),
             data={
-                "result": result,
-                "output_data": model_to_json_serializable_dict(
-                    output_data, exclude_unset=True
+                "result": str(result),
+                "output_data": output_data.model_dump(
+                    mode="json", exclude_unset=True
                 ),
             },
             expected_class=None,
@@ -286,7 +296,7 @@ class Debusine:
         :return: WorkRequest returned by the server.
         :raises: see _api_request method documentation.
         """
-        data = work_request.dict()
+        data = work_request.model_dump()
 
         # If workspace is None: the client does not want to specify
         # a workspace. Do not send it to the server: the server will
@@ -315,7 +325,7 @@ class Debusine:
         external_debsign_request: WorkRequestExternalDebsignRequest,
     ) -> WorkRequestResponse:
         """Provide external `debsign` data to a work request."""
-        data = external_debsign_request.dict()
+        data = external_debsign_request.model_dump()
 
         return self._debusine_http_client.post(
             f"/work-request/{work_request_id}/external-debsign/",
@@ -335,7 +345,7 @@ class Debusine:
         return self._debusine_http_client.post(
             f"/workflow-template/{quote(workspace)}/",
             WorkflowTemplateData,
-            data=workflow_template.dict(),
+            data=workflow_template.model_dump(),
             expected_statuses=[requests.codes.created],
         )
 
@@ -363,7 +373,7 @@ class Debusine:
         """Set information for a workflow template."""
         return self._debusine_http_client.patch(
             f"/workflow-template/{quote(workspace)}/{data.id}/",
-            data=model_to_json_serializable_dict(data),
+            data=data.model_dump(mode="json"),
             expected_class=WorkflowTemplateData,
         )
 
@@ -376,7 +386,7 @@ class Debusine:
         :return: Workflow WorkRequest returned by the server.
         :raises: see _api_request method documentation.
         """
-        data = create_workflow.dict()
+        data = create_workflow.model_dump()
 
         # If workspace is None: the client does not want to specify a
         # workspace. Do not send it to the server: the server will assign
@@ -391,16 +401,19 @@ class Debusine:
             expected_statuses=[requests.codes.created],
         )
 
-    def artifact_get(self, artifact_id: int) -> ArtifactResponse:
+    def artifact_get(
+        self, artifact_id: int, workspace: str | None = None
+    ) -> ArtifactResponse:
         """
         Get artifact information.
 
         Use download_artifact() to download the artifact.
         """
-        return self._debusine_http_client.get(
-            f"/artifact/{artifact_id}/",
-            ArtifactResponse,
-        )
+        if workspace:
+            path = f"/artifact/{workspace}/{artifact_id}/"
+        else:
+            path = f"/artifact/{artifact_id}/"
+        return self._debusine_http_client.get(path, ArtifactResponse)
 
     def artifact_create(
         self,
@@ -449,8 +462,11 @@ class Debusine:
         # assertion is optimized out then the file upload will fail
         # harmlessly; this just helps with type-checking.
         assert self.token is not None
+        assert self.scope is not None
 
-        file_uploader = FileUploader(self.token, logger=self._logger)
+        file_uploader = FileUploader(
+            self.token, scope=self.scope, logger=self._logger
+        )
 
         for artifact_path, local_path in upload_files.items():
             if not local_path.is_absolute():
@@ -521,7 +537,9 @@ class Debusine:
         return self._debusine_http_client.post(
             "/artifact-relation/",
             RelationResponse,
-            data=relation_request.dict(include={"artifact", "target", "type"}),
+            data=relation_request.model_dump(
+                include={"artifact", "target", "type"}
+            ),
             expected_statuses=[requests.codes.ok, requests.codes.created],
         )
 
@@ -570,13 +588,10 @@ class Debusine:
                 return False
         return total_size < 100 * 1024 * 1024
 
-    def _streaming_download(
-        self,
-        url: str,
-    ) -> requests.Response:
+    def _streaming_download(self, url: pydantic.AnyUrl) -> requests.Response:
         """Streaming download of url."""
         headers = {"Token": self.token}
-        response = requests.get(url, headers=headers, stream=True)
+        response = requests.get(str(url), headers=headers, stream=True)
         response.raise_for_status()
         return response
 
@@ -586,6 +601,7 @@ class Debusine:
         destination: Path,
         *,
         tarball: bool = False,
+        workspace: str | None = None,
     ) -> ArtifactResponse:
         """
         Download artifact_id into destination directory.
@@ -595,10 +611,7 @@ class Debusine:
         :param tarball: True to only download the tarball (artifact-id.tar.gz),
           False to uncompress it
         """
-        artifact = self._debusine_http_client.get(
-            f"/artifact/{artifact_id}",
-            ArtifactResponse,
-        )
+        artifact = self.artifact_get(artifact_id, workspace=workspace)
 
         if not tarball and not self._should_download_as_tarball(artifact):
             self._logger.info("Downloading artifact files into %s", destination)
@@ -639,7 +652,7 @@ class Debusine:
         self,
         raw_response: io.IOBase,
         destination: Path,
-        url: str,
+        url: pydantic.AnyUrl,
     ) -> None:
         with (
             tempfile.TemporaryFile() as stderr_file,
@@ -678,6 +691,7 @@ class Debusine:
         artifact_id: int | ArtifactResponse,
         path_in_artifact: str,
         destination: Path,
+        workspace: str | None = None,
     ) -> ArtifactResponse:
         """
         Download a single file from artifact into destination directory.
@@ -691,10 +705,7 @@ class Debusine:
         if isinstance(artifact_id, ArtifactResponse):
             artifact = artifact_id
         else:
-            artifact = self._debusine_http_client.get(
-                f"/artifact/{artifact_id}",
-                ArtifactResponse,
-            )
+            artifact = self.artifact_get(artifact_id, workspace=workspace)
 
         if path_in_artifact not in artifact.files:
             raise ValueError(
@@ -719,14 +730,12 @@ class Debusine:
         return self._debusine_http_client.post(
             "/asset/",
             AssetResponse,
-            data=json.loads(
-                AssetCreateRequest(
-                    category=category,
-                    workspace=workspace,
-                    work_request=work_request,
-                    data=data,
-                ).json()
-            ),
+            data=AssetCreateRequest(
+                category=category,
+                workspace=workspace,
+                work_request=work_request,
+                data=data.model_dump(),
+            ).model_dump(),
             expected_statuses=[requests.codes.created],
         )
 
@@ -820,7 +829,7 @@ class Debusine:
             default_category=default_category,
         )
         return self._debusine_http_client.post(
-            "/lookup/single/", LookupSingleResponse, data=request.dict()
+            "/lookup/single/", LookupSingleResponse, data=request.model_dump()
         )
 
     @overload
@@ -859,13 +868,15 @@ class Debusine:
     ) -> LookupMultipleResponse[LookupSingleResponse]:
         """Look up multiple collection items."""
         request = LookupMultipleRequest(
-            lookup=lookup.dict()["__root__"],
+            lookup=lookup.model_dump(),
             work_request=work_request,
             expect_type=expect_type,
             default_category=default_category,
         )
         return self._debusine_http_client.post(
-            "/lookup/multiple/", LookupMultipleResponse, data=request.dict()
+            "/lookup/multiple/",
+            LookupMultipleResponse,
+            data=request.model_dump(),
         )
 
     def fetch_task_configuration_collection(
@@ -899,8 +910,8 @@ class Debusine:
             raise InvalidRepository("repository has no manifest")
 
         data = {
-            "collection": repo.manifest.collection.dict(),
-            "items": [i.item.dict() for i in repo.entries.values()],
+            "collection": repo.manifest.collection.model_dump(),
+            "items": [i.item.model_dump() for i in repo.entries.values()],
             "dry_run": dry_run,
         }
 
@@ -926,7 +937,7 @@ class Debusine:
         """Set a workspace inheritance chain."""
         return self._debusine_http_client.post(
             f"/workspace/{quote(workspace)}/inheritance/",
-            data=model_to_json_serializable_dict(chain),
+            data=chain.model_dump(mode="json"),
             expected_class=WorkspaceInheritanceChain,
         )
 
@@ -947,7 +958,7 @@ class Debusine:
         """Create a new collection."""
         return self._debusine_http_client.post(
             f"/collection/{quote(workspace)}/",
-            data=model_to_json_serializable_dict(data),
+            data=data.model_dump(mode="json"),
             expected_class=CollectionData,
             expected_statuses=[requests.codes.created],
         )
@@ -958,7 +969,7 @@ class Debusine:
         """Set information for a collection."""
         return self._debusine_http_client.patch(
             f"/collection/{quote(workspace)}/{data.id}/",
-            data=model_to_json_serializable_dict(data),
+            data=data.model_dump(mode="json"),
             expected_class=CollectionData,
         )
 
@@ -976,6 +987,24 @@ class Debusine:
             CollectionData,
         )
 
+    def emit_metric(
+        self,
+        *,
+        metric_type: Literal["counter", "gauge", "summary", "histogram"],
+        name: str,
+        labels: dict[str, str],
+        value: float,
+    ) -> None:
+        """Ask the server to emit a metric on behalf of this client."""
+        self._debusine_http_client.post(
+            "/open-metrics/emit/",
+            data=EmitMetricRequest(
+                metric_type=metric_type, name=name, labels=labels, value=value
+            ).model_dump(mode="json"),
+            expected_class=None,
+            expected_statuses=[requests.codes.no_content],
+        )
+
     def _log_tenacity_exception(
         self, retry_state: tenacity.RetryCallState
     ) -> None:
@@ -993,7 +1022,7 @@ class Debusine:
         last_completed_at: Path | None,
         msg_content: dict[str, Any],
     ) -> None:
-        on_work_request_completed = OnWorkRequestCompleted.parse_obj(
+        on_work_request_completed = OnWorkRequestCompleted.model_validate(
             msg_content
         )
 
@@ -1041,7 +1070,9 @@ class Debusine:
 
             self._logger.info("Requesting %s", url)
 
-            def connected(msg_content: dict[Any, Any]) -> None:  # noqa: U100
+            def connected(
+                msg_content: dict[Any, Any],  # noqa: ARG001, U100
+            ) -> None:
                 self._logger.info("Connected!")
 
             async with session.ws_connect(
