@@ -14,7 +14,7 @@ from typing import Any, overload
 
 import requests
 from pydantic import BaseModel, ValidationError
-from requests.compat import json  # type: ignore
+from requests.compat import json  # type: ignore[attr-defined]
 from urllib3.util import Retry
 
 from debusine.client import exceptions
