@@ -51,7 +51,7 @@ def choose_workflow(changes: Changes, profile: dict[str, Any]) -> str:
 def check_workflow(
     changes: Changes,
     profile: dict[str, Any],
-    interface: AbstractInterface,  # noqa: U100
+    interface: AbstractInterface,  # noqa: ARG001, U100
 ) -> None:
     """Check that we know what debusine workflow to create."""
     choose_workflow(changes, profile)
@@ -60,7 +60,7 @@ def check_workflow(
 def create_workflow(
     changes: Changes,
     profile: dict[str, Any],
-    interface: AbstractInterface,  # noqa: U100
+    interface: AbstractInterface,  # noqa: ARG001, U100
 ) -> None:
     """Create a workflow based on an upload to debusine."""
     scope = profile["debusine_scope"]
