@@ -10,31 +10,24 @@
 """Models used by debusine client."""
 
 import datetime as dt
-import json
 import re
 from collections.abc import Collection, Iterator, Sequence
 from enum import StrEnum
 from pathlib import Path
-from typing import Any, Generic, Literal, NewType, Self, TypeVar
+from typing import Annotated, Any, Literal, NewType, Self, TypeVar
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
+import pydantic
 import yaml
 
 from debusine.artifacts.models import DebusineTaskConfiguration
-from debusine.assets import AssetCategory, BaseAssetDataModel, asset_data_model
+from debusine.assets import AssetCategory, asset_data_model
 from debusine.utils import calculate_hash
 
 
 class StrictBaseModel(pydantic.BaseModel):
     """Stricter pydantic configuration."""
 
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
+    model_config = pydantic.ConfigDict(validate_assignment=True)
 
 
 class Link(StrictBaseModel):
@@ -73,9 +66,9 @@ class ServerObject(StrictBaseModel):
 class PaginatedResponse(StrictBaseModel):
     """Paginated response from the API."""
 
-    count: int | None
-    next: pydantic.AnyUrl | None
-    previous: pydantic.AnyUrl | None
+    count: int | None = None
+    next: pydantic.AnyUrl | None = None
+    previous: pydantic.AnyUrl | None = None
     results: list[dict[str, Any]]
 
 
@@ -96,19 +89,21 @@ class WorkRequestResponse(ServerObject):
     created_at: dt.datetime
     started_at: dt.datetime | None = None
     completed_at: dt.datetime | None = None
-    duration: int | None = None
+    duration: float | None = None
     status: str
     result: str
     worker: int | None = None
     task_type: str
     task_name: str
     task_data: dict[str, Any]
-    dynamic_task_data: dict[str, Any] | None
+    dynamic_task_data: dict[str, Any] | None = None
     priority_base: int
     priority_adjustment: int
     artifacts: list[int]
     scope: str | None = None  # Server >= 0.12 specifies scope
     workspace: str
+    scheduler_tags_provided: list[str] = []
+    scheduler_tags_required: list[str] = []
 
     def __str__(self) -> str:
         """Return representation of the object."""
@@ -174,20 +169,11 @@ class CreateWorkflowRequest(StrictBaseModel):
     task_data: dict[str, Any]
 
 
-# With pydantic >= 2.1.0, use Annotated[str, pydantic.Field(max_length=255)]
-# instead.  Unfortunately, while pydantic v1 seems to accept this, it
-# silently ignores the max_length annotation.
-class StrMaxLength255(pydantic.ConstrainedStr):
-    """A string with a maximum length of 255 characters."""
-
-    max_length = 255
-
-
 class FileRequest(StrictBaseModel):
     """Declare a FileRequest: client sends it to the server."""
 
-    size: int = pydantic.Field(ge=0)
-    checksums: dict[str, StrMaxLength255]
+    size: pydantic.NonNegativeInt
+    checksums: dict[str, Annotated[str, pydantic.Field(max_length=255)]]
     type: Literal["file"]
     #: A media type, suitable for an HTTP ``Content-Type`` header.
     content_type: str | None = None
@@ -199,11 +185,7 @@ class FileRequest(StrictBaseModel):
         """Return a FileRequest for the file path."""
         return FileRequest(
             size=path.stat().st_size,
-            checksums={
-                "sha256": pydantic.parse_obj_as(
-                    StrMaxLength255, calculate_hash(path, "sha256").hex()
-                )
-            },
+            checksums={"sha256": calculate_hash(path, "sha256").hex()},
             type="file",
             content_type=content_type,
         )
@@ -212,8 +194,8 @@ class FileRequest(StrictBaseModel):
 class FileResponse(ServerObject):
     """Declare a FileResponse: server sends it to the client."""
 
-    size: int = pydantic.Field(ge=0)
-    checksums: dict[str, StrMaxLength255]
+    size: pydantic.NonNegativeInt
+    checksums: dict[str, Annotated[str, pydantic.Field(max_length=255)]]
     type: Literal["file"]
     url: pydantic.AnyUrl
     content_type: str | None = None
@@ -262,7 +244,7 @@ class AssetCreateRequest(StrictBaseModel):
     """Request for the Asset creation API."""
 
     category: AssetCategory
-    data: BaseAssetDataModel
+    data: dict[str, Any]
     work_request: int | None = None
     workspace: str
 
@@ -272,30 +254,27 @@ class AssetResponse(ServerObject):
 
     id: int
     category: AssetCategory
-    data: BaseAssetDataModel | dict[str, Any]
+    data: dict[str, Any]
     work_request: int | None = None
     workspace: str
 
-    @pydantic.root_validator
-    @classmethod
-    def parse_data(cls, values: dict[str, Any]) -> dict[str, Any]:
-        """Parse data using the correct data model."""
-        if isinstance(values.get("data"), dict):
-            category = values.get("category")
-            if category:  # pragma: no cover
-                values["data"] = asset_data_model(category, values["data"])
-        return values
+    @pydantic.model_validator(mode="after")
+    def validate_data(self) -> Self:
+        """Validate data using the correct data model."""
+        asset_data_model(self.category, self.data)
+        return self
 
 
-class AssetsResponse(StrictBaseModel):
+class AssetsResponse(
+    pydantic.RootModel[Sequence[AssetResponse]], StrictBaseModel
+):
     """A response from the server with multiple AssetResponse objects."""
 
-    # This model parses a list, not an object/dict.
-    __root__: Sequence[AssetResponse]
+    model_config = pydantic.ConfigDict(validate_assignment=True)
 
     def __iter__(self) -> Iterator[AssetResponse]:  # type: ignore[override]
         """Iterate over individual asset responses."""
-        return iter(self.__root__)
+        return iter(self.root)
 
 
 class AssetPermissionCheckResponse(ServerObject):
@@ -332,15 +311,14 @@ class RelationResponse(ServerObject):
     type: RelationType
 
 
-class RelationsResponse(StrictBaseModel):
+class RelationsResponse(
+    pydantic.RootModel[Sequence[RelationResponse]], StrictBaseModel
+):
     """A response from the server with multiple RelationResponse objects."""
 
-    # This model parses a list, not an object/dict.
-    __root__: Sequence[RelationResponse]
-
     def __iter__(self) -> Iterator[RelationResponse]:  # type: ignore[override]
         """Iterate over individual relation responses."""
-        return iter(self.__root__)
+        return iter(self.root)
 
 
 class LookupChildType(StrEnum):
@@ -410,18 +388,20 @@ class LookupSingleResponseCollection(LookupSingleResponse):
     collection: int
 
 
+# TODO: We should use PEP 695 type parameter declarations here instead, but
+# that causes "Overloaded function implementation cannot produce return
+# type" errors on Debusine.lookup_multiple.
 LSR = TypeVar("LSR", bound=LookupSingleResponse, covariant=True)
 
 
-class LookupMultipleResponse(StrictBaseModel, Generic[LSR]):
+class LookupMultipleResponse(
+    pydantic.RootModel[Sequence[LSR]], StrictBaseModel
+):
     """A response from the server with multiple lookup results."""
 
-    # This model parses a list, not an object/dict.
-    __root__: Sequence[LSR]
-
     def __iter__(self) -> Iterator[LSR]:  # type: ignore[override]
         """Iterate over individual results."""
-        return iter(self.__root__)
+        return iter(self.root)
 
 
 re_nonce = re.compile(r"^[A-Za-z0-9_-]{8,64}$")
@@ -431,38 +411,18 @@ re_challenge = re.compile(r"^\w{4,10}(?: \w{4,10}){2,7}$", re.ASCII)
 class EnrollPayload(pydantic.BaseModel):
     """Client-provided enrollment payload."""
 
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
-        extra = "forbid"
+    model_config = pydantic.ConfigDict(validate_assignment=True, extra="forbid")
 
     #: Nonce identifying the client enrollment
-    nonce: str
+    nonce: Annotated[str, pydantic.Field(pattern=re_nonce)]
     #: Human-readable challenge to verify the client in the web UI
-    challenge: str
+    challenge: Annotated[str, pydantic.Field(pattern=re_challenge)]
     #: Scope. Informational, the confirmation page will warn if invalid
     scope: str
     #: Hostname. Informational, shown in the confirmation page and in the
     #: generated token comment
     hostname: str
 
-    @pydantic.validator("nonce", pre=True)
-    @classmethod
-    def _validate_nonce(cls, nonce: str) -> str:
-        """Validate that the nonce is well formed."""
-        if not re_nonce.match(nonce):
-            raise ValueError("Nonce is malformed")
-        return nonce
-
-    @pydantic.validator("challenge", pre=True)
-    @classmethod
-    def _validate_challenge(cls, challenge: str) -> str:
-        """Validate that the challenge is well formed."""
-        if not re_challenge.match(challenge):
-            raise ValueError("Challenge is malformed")
-        return challenge
-
 
 re_token = re.compile(r"^[A-Za-z0-9]{8,64}$")
 
@@ -477,22 +437,10 @@ class EnrollOutcome(StrEnum):
 class EnrollConfirmPayload(pydantic.BaseModel):
     """Enrollment response from the server."""
 
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
-        extra = "forbid"
+    model_config = pydantic.ConfigDict(validate_assignment=True, extra="forbid")
 
     outcome: EnrollOutcome
-    token: str | None = None
-
-    @pydantic.validator("token", pre=True)
-    @classmethod
-    def _validate_token(cls, token: str | None) -> str | None:
-        """Validate that the token is well formed."""
-        if token is not None and not re_token.match(token):
-            raise ValueError("Token is malformed")
-        return token
+    token: Annotated[str, pydantic.Field(pattern=re_token)] | None = None
 
 
 class TaskConfigurationCollection(ServerObject):
@@ -526,13 +474,12 @@ class WorkspaceInheritanceChainElement(StrictBaseModel):
     scope: str | None = None
     workspace: str | None = None
 
-    @pydantic.root_validator
-    @classmethod
-    def not_empty(cls, values: dict[str, Any]) -> dict[str, Any]:
+    @pydantic.model_validator(mode="after")
+    def not_empty(self) -> Self:
         """Ensure at least one of id or workspace is set."""
-        if values.get("id") is None and values.get("workspace") is None:
+        if self.id is None and self.workspace is None:
             raise ValueError("at least id or workspace need to be set")
-        return values
+        return self
 
     def matches(self, element: "WorkspaceInheritanceChainElement") -> bool:
         """Check if the components of element that are set match this one."""
@@ -619,31 +566,6 @@ class EmitMetricRequest(StrictBaseModel):
     value: float
 
 
-def model_to_json_serializable_dict(
-    model: pydantic.BaseModel, exclude_unset: bool = False
-) -> dict[Any, Any]:
-    """
-    Similar to model.dict() but the returned dictionary is JSON serializable.
-
-    For example, a datetime() is not JSON serializable. Using this method will
-    return a dictionary with a string instead of a datetime object.
-
-    Replace with model_dump() in Pydantic 2.
-    """
-    serializable = json.loads(model.json(exclude_unset=exclude_unset))
-    assert isinstance(serializable, dict)
-    return serializable
-
-
-def model_to_json_serializable_list(
-    model: pydantic.BaseModel, exclude_unset: bool = False
-) -> list[dict[Any, Any]]:
-    """Similar to model_to_json_serializable_dict, but for a list response."""
-    serializable = json.loads(model.json(exclude_unset=exclude_unset))
-    assert isinstance(serializable, list)
-    return serializable
-
-
 # Workarounds for https://github.com/yaml/pyyaml/issues/722
 yaml.SafeDumper.add_multi_representer(
     StrEnum,
