@@ -54,7 +54,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
 
             await ws.send_json({"text": "connected"})
 
-            for loop in range(0, self.ON_COMPLETED_WORK_REQUESTS):
+            for loop in range(self.ON_COMPLETED_WORK_REQUESTS):
                 await ws.send_json(
                     {
                         "text": "work_request_completed",
