@@ -31,6 +31,7 @@ from urllib.parse import urlencode
 
 import aiohttp
 import pydantic
+import pydantic_core
 import requests
 import responses
 
@@ -266,13 +267,15 @@ class SerializeLocalArtifactTests(TestCase):
 
     def test_with_expire_at(self) -> None:
         """``serialize_local_artifact`` includes ``expire_at``."""
-        expire_at = dt.datetime.now() + dt.timedelta(days=1)
+        expire_at = dt.datetime.now(tz=dt.UTC) + dt.timedelta(days=1)
 
         serialized = serialize_local_artifact(
             self.local_artifact, workspace="workspace", expire_at=expire_at
         )
 
-        self.assertEqual(serialized["expire_at"], expire_at.isoformat())
+        self.assertEqual(
+            serialized["expire_at"], pydantic_core.to_jsonable_python(expire_at)
+        )
 
 
 class DebusineTests(TestCase):
@@ -746,7 +749,7 @@ class DebusineTests(TestCase):
 
         workspace = "test"
         work_request = 5
-        expire_at = dt.datetime.now() + dt.timedelta(days=1)
+        expire_at = dt.datetime.now(tz=dt.UTC) + dt.timedelta(days=1)
 
         artifact_response = create_artifact_response(
             id=2,
@@ -1063,11 +1066,9 @@ class DebusineTests(TestCase):
         :return: tuple [tarFile, Path of a file]
         """
         compressed = io.BytesIO()
-        tar_file = tarfile.open(fileobj=compressed, mode="w|gz")
-
-        file = self.create_temporary_file()
-        tar_file.add(file, arcname=file.name)
-        tar_file.close()
+        with tarfile.open(fileobj=compressed, mode="w|gz") as tar_file:
+            file = self.create_temporary_file()
+            tar_file.add(file, arcname=file.name)
 
         compressed.seek(0)
 
@@ -2009,7 +2010,7 @@ class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
         expected_in_file = ""
         expected_log_executing_calls = []
 
-        for number in range(0, on_completed_work_requests):
+        for number in range(on_completed_work_requests):
             expected_in_file += (
                 f"{server.DebusineAioHTTPTestCase.WORK_REQUEST_ID + number} "
                 f"{server.DebusineAioHTTPTestCase.WORK_REQUEST_RESULT}\n"
