@@ -662,7 +662,15 @@ class GitTaskConfigurationRepositoryTests(
         cmd = ["git", "-C", self.root.as_posix()]
         cmd.extend(args)
         return subprocess.run(
-            cmd, cwd=self.root, capture_output=True, text=True, check=True
+            cmd,
+            cwd=self.root,
+            capture_output=True,
+            text=True,
+            check=True,
+            env={
+                "GIT_CONFIG_GLOBAL": "/dev/null",
+                "GIT_CONFIG_SYSTEM": "/dev/null",
+            },
         )
 
     def commit(self, message: str = "Commit message") -> None:
