@@ -12,7 +12,7 @@
 import datetime as dt
 from functools import partial
 from pathlib import Path
-from typing import Any, Optional, TYPE_CHECKING
+from typing import Any, Optional, TYPE_CHECKING, TypedDict
 
 from django.conf import settings
 from django.core.exceptions import PermissionDenied, ValidationError
@@ -37,10 +37,16 @@ from debusine.utils.typing_utils import copy_signature_from
 
 if TYPE_CHECKING:
     from django.http import HttpRequest
+    from django_stubs_ext import WithAnnotations
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.work_requests import WorkRequest
     from debusine.web.views.ui.artifacts import ArtifactUI, FileInArtifactUI
+
+    ArtifactWithComplete = WithAnnotations["Artifact", "CompleteDict"]
+
+    WithAnnotations  # fake usage for vulture
+
 else:
     TypedModelMeta = object
 
@@ -79,6 +85,12 @@ ARTIFACT_CATEGORY_SHORT_NAMES = {
 }
 
 
+class CompleteDict(TypedDict):
+    """Additional fields returned by ArtifactQuerySet.annotate_complete."""
+
+    complete: bool
+
+
 class ArtifactQuerySet[A](QuerySet["Artifact", A]):
     """Custom QuerySet for Artifact."""
 
@@ -116,10 +128,9 @@ class ArtifactQuerySet[A](QuerySet["Artifact", A]):
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
-    # mypy_django_plugin generates an
-    # Artifact@AnnotatedWith[TypedDict({"complete": Any})] model subclass
-    # for this, but we don't have a way to refer to it here.
-    def annotate_complete(self) -> "ArtifactQuerySet[Any]":
+    def annotate_complete(
+        self,
+    ) -> "ArtifactQuerySet[ArtifactWithComplete]":
         """Annotate artifacts with whether all their files are complete."""
         return self.annotate(
             complete=~Exists(
@@ -138,16 +149,16 @@ class ArtifactQuerySet[A](QuerySet["Artifact", A]):
           expire_at is after the given datetime.
         """
         return self.annotate(
-            effective_expiration_delay=Coalesce(
+            _effective_expiration_delay=Coalesce(
                 "expiration_delay",
                 "workspace__default_expiration_delay",
             )
         ).filter(
-            Q(effective_expiration_delay=dt.timedelta(0))
+            Q(_effective_expiration_delay=dt.timedelta(0))
             | Q(
                 # https://github.com/typeddjango/django-stubs/issues/1548
                 created_at__gt=(
-                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                    at - F("_effective_expiration_delay")  # type: ignore[operator] # noqa: E501
                 )
             )
         )
@@ -161,16 +172,16 @@ class ArtifactQuerySet[A](QuerySet["Artifact", A]):
         """
         return (
             self.annotate(
-                effective_expiration_delay=Coalesce(
+                _effective_expiration_delay=Coalesce(
                     "expiration_delay",
                     "workspace__default_expiration_delay",
                 )
             )
-            .exclude(effective_expiration_delay=dt.timedelta(0))
+            .exclude(_effective_expiration_delay=dt.timedelta(0))
             .filter(
                 # https://github.com/typeddjango/django-stubs/issues/1548
                 created_at__lte=(
-                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                    at - F("_effective_expiration_delay")  # type: ignore[operator] # noqa: E501
                 )
             )
         )
