@@ -21,6 +21,7 @@ from django.db.models.constraints import CheckConstraint, UniqueConstraint
 from debusine.assets import (
     AssetCategory,
     BaseAssetDataModel,
+    BasicAPTAuthenticationData,
     SigningKeyData,
     asset_data_model,
 )
@@ -138,6 +139,11 @@ class AssetUsageRoles(permissions.Roles, AssetUsageRoleBase, enum.ReprEnum):
         label="Repository signer",
         implied_by=[SIGNER, Workspace.Roles.OWNER],
     )
+    APT_AUTHENTICATOR = Role(
+        "apt_authenticator",
+        label="Can use APT authentication",
+        implied_by=[Workspace.Roles.OWNER],
+    )
 
 
 AssetUsageRoles.setup()
@@ -163,6 +169,7 @@ class AssetQuerySet[A](models.QuerySet["Asset", A]):
         self, user: PermissionUser
     ) -> "AssetQuerySet[A]":
         """Filter to Assets that can be managed by user."""
+        assert user is not None and user.is_authenticated
         return self.filter(
             roles__group__users=user, roles__role=AssetRoles.OWNER
         )
@@ -179,7 +186,9 @@ class AssetManager(models.Manager["Asset"]):
         """Use the custom QuerySet."""
         return AssetQuerySet(self.model, using=self._db)
 
-    def get_by_slug(self, category: str, slug: str) -> "Asset":
+    def get_by_slug(
+        self, category: str, slug: str, workspace: Workspace | None = None
+    ) -> "Asset":
         """Return an asset with a matching slug."""
         match category:
             case AssetCategory.SIGNING_KEY:
@@ -189,6 +198,18 @@ class AssetManager(models.Manager["Asset"]):
                     data__purpose=purpose,
                     data__fingerprint=fingerprint,
                 )
+            case AssetCategory.APT_AUTHENTICATION:
+                assert workspace is not None
+                assets = self.filter(category=category)
+                if ":" in slug:
+                    workspace_name, name = slug.split(":", 1)
+                    return assets.get(
+                        workspace__scope=workspace.scope,
+                        workspace__name=workspace_name,
+                        data__name=name,
+                    )
+                else:
+                    return assets.get(workspace=workspace, data__name=slug)
             case _:
                 raise ValueError(f"No slug defined for category '{category}'")
 
@@ -216,6 +237,13 @@ class AssetUsageQuerySet[A](models.QuerySet["AssetUsage", A]):
         """Keep only AssetUsages that the user can sign a repository with."""
         return self.with_role(user, AssetUsageRoles.REPOSITORY_SIGNER)
 
+    @permission_filter(workers=Allow.PASS)
+    def can_use_apt_authentication_with(
+        self, user: PermissionUser
+    ) -> "AssetUsageQuerySet[A]":
+        """Keep only AssetUsages that the user can use for APT auth."""
+        return self.with_role(user, AssetUsageRoles.APT_AUTHENTICATOR)
+
 
 class AssetUsageManager(models.Manager["AssetUsage"]):
     """Manager for the AssetUsage model."""
@@ -261,10 +289,28 @@ class Asset(models.Model):
                 nulls_distinct=False,
                 name="%(app_label)s_%(class)s_unique_signing_key_fingerprints",
             ),
-            # Some categories of asset can have null workspaces, but not
-            # signing keys
             CheckConstraint(
-                check=~models.Q(category=AssetCategory.SIGNING_KEY)
+                check=(
+                    ~models.Q(category=AssetCategory.APT_AUTHENTICATION)
+                    | models.Q(data__name__regex=r"^[A-Za-z][A-Za-z0-9+._-]*$")
+                ),
+                name="%(app_label)s_%(class)s_apt_auth_name",
+            ),
+            JsonDataUniqueConstraint(
+                fields=["workspace", "data->>'name'"],
+                condition=models.Q(category=AssetCategory.APT_AUTHENTICATION),
+                nulls_distinct=False,
+                name="%(app_label)s_%(class)s_unique_apt_auth_workspace_name",
+            ),
+            # Some categories of asset can have null workspaces, but not
+            # signing keys or APT authentication.
+            CheckConstraint(
+                check=~models.Q(
+                    category__in={
+                        AssetCategory.SIGNING_KEY,
+                        AssetCategory.APT_AUTHENTICATION,
+                    }
+                )
                 | models.Q(workspace__isnull=False),
                 name="%(app_label)s_%(class)s_workspace_not_null",
             ),
@@ -308,6 +354,11 @@ class Asset(models.Model):
                 data_model = self.data_model
                 assert isinstance(data_model, SigningKeyData)
                 return f"{data_model.purpose}:{data_model.fingerprint}"
+            case AssetCategory.APT_AUTHENTICATION:
+                assert self.workspace is not None
+                data_model = self.data_model
+                assert isinstance(data_model, BasicAPTAuthenticationData)
+                return f"{self.workspace.name}:{data_model.name}"
             case _:
                 raise NotImplementedError(
                     f"No slug defined for category '{self.category}'"
@@ -433,6 +484,15 @@ class AssetUsage(models.Model):
         assert user is not None and user.is_authenticated
         return self.has_role(user, AssetUsageRoles.REPOSITORY_SIGNER)
 
+    @permission_check(
+        "{user} cannot use {resource} for APT authentication",
+        workers=Allow.PASS,
+    )
+    def can_use_apt_authentication_with(self, user: PermissionUser) -> bool:
+        """Check if the user can use APT authentication with this resource."""
+        assert user is not None and user.is_authenticated
+        return self.has_role(user, AssetUsageRoles.APT_AUTHENTICATOR)
+
 
 class AssetUsageRole(models.Model):
     """Role assignment for assets within a workspace."""
