@@ -15,7 +15,7 @@ database access.
 """
 
 from collections.abc import Collection as AbcCollection
-from typing import overload
+from typing import overload, override
 
 from django.conf import settings
 
@@ -24,7 +24,10 @@ from debusine.client.models import LookupChildType, RelationType
 from debusine.db.models.artifacts import Artifact
 from debusine.db.models.collections import Collection
 from debusine.db.models.work_requests import WorkRequest
-from debusine.tasks.models import LookupMultiple, LookupSingle
+from debusine.tasks.models import (
+    LookupMultiple,
+    LookupSingle,
+)
 from debusine.tasks.server import (
     ArtifactInfo,
     CollectionInfo,
@@ -75,96 +78,50 @@ class TaskDatabase(TaskDatabaseInterface):
         default_category: CollectionCategory | None = None,
     ) -> None: ...
 
+    @override
     def lookup_single_artifact(
         self,
         lookup: LookupSingle | None,
         default_category: CollectionCategory | None = None,
     ) -> ArtifactInfo | None:
-        """
-        Look up a single artifact.
-
-        :param lookup: A :ref:`lookup-single`.
-        :param default_category: If the first segment of a string lookup
-          (which normally identifies a collection) does not specify a
-          category, use this as the default category.
-        :return: Information about the artifact, or None if the provided
-          lookup is None (for convenience in some call sites).
-        :raises KeyError: if the lookup does not resolve to an item.
-        :raises LookupError: if the lookup is invalid in some way, or does
-          not resolve to an artifact.
-        """
-        # Import here to prevent circular imports
-        from debusine.server.collections.lookup import lookup_single
-
         return (
             None
             if lookup is None
             else self._make_artifact_info(
-                lookup_single(
+                self.work_request.lookup_single(
                     lookup=lookup,
-                    workspace=self.work_request.workspace,
-                    user=self.work_request.created_by,
                     default_category=default_category,
-                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact
             )
         )
 
+    @override
     def lookup_multiple_artifacts(
         self,
         lookup: LookupMultiple | None,
         default_category: CollectionCategory | None = None,
     ) -> MultipleArtifactInfo:
-        """
-        Look up multiple artifacts.
-
-        :param lookup: A :ref:`lookup-multiple`.
-        :param default_category: If the first segment of a string lookup
-          (which normally identifies a collection) does not specify a
-          category, use this as the default category.
-        :return: Information about each artifact.
-        :raises KeyError: if any of the lookups does not resolve to an item.
-        :raises LookupError: if any of the lookups is invalid in some way,
-          or does not resolve to an artifact.
-        """
-        # Import here to prevent circular imports
-        from debusine.server.collections.lookup import lookup_multiple
-
         return MultipleArtifactInfo(
             []
             if lookup is None
             else [
                 self._make_artifact_info(result.artifact)
-                for result in lookup_multiple(
+                for result in self.work_request.lookup_multiple(
                     lookup=lookup,
-                    workspace=self.work_request.workspace,
-                    user=self.work_request.created_by,
                     default_category=default_category,
-                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 )
             ]
         )
 
+    @override
     def find_related_artifacts(
         self,
         artifact_ids: AbcCollection[int],
         target_category: ArtifactCategory,
         relation_type: RelationType = RelationType.RELATES_TO,
     ) -> MultipleArtifactInfo:
-        """
-        Find artifacts via relations.
-
-        For each of the artifacts in ``artifact_ids``, find all artifacts of
-        ``target_category`` that are related to the initial artifacts with a
-        relation of type ``relation_type``.
-
-        :param artifact_ids: The IDs of artifacts to start from.
-        :param target_category: The category of artifacts to find.
-        :param relation_type: The type of relation to follow.
-        :return: Information about each artifact.
-        """
         return MultipleArtifactInfo(
             [
                 self._make_artifact_info(artifact)
@@ -190,50 +147,32 @@ class TaskDatabase(TaskDatabaseInterface):
         default_category: CollectionCategory | None = None,
     ) -> None: ...
 
+    @override
     def lookup_single_collection(
         self,
         lookup: LookupSingle | None,
         default_category: CollectionCategory | None = None,
     ) -> CollectionInfo | None:
-        """
-        Look up a single collection.
-
-        :param lookup: A :ref:`lookup-single`.
-        :param default_category: If the first segment of a string lookup
-          (which normally identifies a collection) does not specify a
-          category, use this as the default category.
-        :return: Information about the collection, or None if the provided
-          lookup is None (for convenience in some call sites).
-        :raises KeyError: if the lookup does not resolve to an item.
-        :raises LookupError: if the lookup is invalid in some way, or does
-          not resolve to a collection.
-        """
-        # Import here to prevent circular imports
-        from debusine.server.collections.lookup import lookup_single
-
         return (
             None
             if lookup is None
             else self._make_collection_info(
-                lookup_single(
+                self.work_request.lookup_single(
                     lookup=lookup,
-                    workspace=self.work_request.workspace,
-                    user=self.work_request.created_by,
                     default_category=default_category,
-                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.COLLECTION,
                 ).collection
             )
         )
 
+    @override
     def get_server_setting(self, setting: str) -> str:
-        """Look up a Django setting (strings only)."""
         value = getattr(settings, setting)
         assert isinstance(value, str)
         return value
 
+    @override
     def export_suite_signing_keys(self, suite_id: int) -> str | None:
-        """Export signing keys for a suite as an ASCII-armored keyring."""
         # Import here to prevent circular imports
         from debusine.server.collections import DebianSuiteManager
 
