@@ -9,13 +9,14 @@
 
 """Unit tests for the models."""
 
-from datetime import timedelta
+import datetime as dt
 from pathlib import Path
 
 import django.db.utils
 from django.conf import settings
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import PermissionDenied, ValidationError
+from django.test import RequestFactory
 from django.utils import timezone
 
 from debusine.artifacts.local_artifact import WorkRequestDebugLogs
@@ -64,7 +65,7 @@ class ArtifactManagerTests(TestCase):
 
         self.assertEqual(artifact.category, local_artifact.category)
         self.assertEqual(artifact.workspace, self.scenario.workspace)
-        self.assertEqual(artifact.data, local_artifact.data)
+        self.assertEqual(artifact.data, local_artifact.data.model_dump())
         self.assertEqual(artifact.created_by_work_request, work_request)
         self.assertEqual(artifact.fileinartifact_set.count(), 2)
         for file_in_artifact, path in zip(
@@ -81,16 +82,16 @@ class ArtifactManagerTests(TestCase):
         """not_expired() return only the not expired artifacts."""
         artifact_1 = Artifact.objects.create(
             workspace=self.scenario.workspace,
-            expiration_delay=timedelta(days=1),
+            expiration_delay=dt.timedelta(days=1),
         )
-        artifact_1.created_at = timezone.now() - timedelta(days=2)
+        artifact_1.created_at = timezone.now() - dt.timedelta(days=2)
         artifact_1.save()
         artifact_2 = Artifact.objects.create(
             workspace=self.scenario.workspace,
-            expiration_delay=timedelta(days=1),
+            expiration_delay=dt.timedelta(days=1),
         )
         artifact_3 = Artifact.objects.create(
-            workspace=self.scenario.workspace, expiration_delay=timedelta(0)
+            workspace=self.scenario.workspace, expiration_delay=dt.timedelta(0)
         )
 
         self.assertQuerySetEqual(
@@ -104,17 +105,17 @@ class ArtifactManagerTests(TestCase):
         """expired() return only the expired artifacts."""
         artifact_1 = Artifact.objects.create(
             workspace=self.scenario.workspace,
-            expiration_delay=timedelta(days=1),
+            expiration_delay=dt.timedelta(days=1),
         )
-        artifact_1.created_at = timezone.now() - timedelta(days=2)
+        artifact_1.created_at = timezone.now() - dt.timedelta(days=2)
         artifact_1.save()
 
         Artifact.objects.create(
             workspace=self.scenario.workspace,
-            expiration_delay=timedelta(days=1),
+            expiration_delay=dt.timedelta(days=1),
         )
         Artifact.objects.create(
-            workspace=self.scenario.workspace, expiration_delay=timedelta(0)
+            workspace=self.scenario.workspace, expiration_delay=dt.timedelta(0)
         )
 
         self.assertQuerySetEqual(
@@ -181,6 +182,100 @@ class ArtifactManagerTests(TestCase):
         ):
             Artifact.objects.in_current_workspace()
 
+    def test_visible_to_current_workspace(self) -> None:
+        """Test the visible_to_current_workspace() QuerySet filter."""
+        with override_permission(Workspace, "can_create_artifacts", AllowAll):
+            artifact = Artifact.objects.create(
+                workspace=self.scenario.workspace
+            )
+
+        with context.local():
+            context.set_scope(self.scenario.scope)
+            context.set_user(self.scenario.user)
+
+            with self.subTest(workspace="direct"), context.local():
+                self.scenario.workspace.set_current()
+                self.assertQuerySetEqual(
+                    Artifact.objects.visible_to_current_workspace(),
+                    [artifact],
+                )
+
+            workspace1 = self.playground.create_workspace(
+                name="other", public=True
+            )
+            with (
+                self.subTest(workspace="other", inherits=False),
+                context.local(),
+            ):
+                workspace1.set_current()
+                self.assertQuerySetEqual(
+                    Artifact.objects.visible_to_current_workspace(),
+                    [],
+                )
+            workspace1.set_inheritance([self.scenario.workspace])
+            with (
+                self.subTest(workspace="other", inherits=True),
+                context.local(),
+            ):
+                workspace1.set_current()
+                self.assertQuerySetEqual(
+                    Artifact.objects.visible_to_current_workspace(),
+                    [artifact],
+                )
+
+            grandchild_workspace = self.playground.create_workspace(
+                name="grandchild", public=True
+            )
+            grandchild_workspace.set_inheritance([workspace1])
+            with self.subTest(workspace="grandchild"), context.local():
+                grandchild_workspace.set_current()
+                self.assertQuerySetEqual(
+                    Artifact.objects.visible_to_current_workspace(),
+                    [artifact],
+                )
+
+            workspace2 = self.playground.create_workspace(
+                name="multiple-children", public=True
+            )
+            with override_permission(
+                Workspace, "can_create_artifacts", AllowAll
+            ):
+                workspace2_artifact = Artifact.objects.create(
+                    workspace=workspace2
+                )
+            for i in range(2):
+                child_workspace = self.playground.create_workspace(
+                    name=f"child-{i}", public=True
+                )
+                child_workspace.set_inheritance([workspace2])
+            with self.subTest(workspace="multiple-children"), context.local():
+                workspace2.set_current()
+                self.assertQuerySetEqual(
+                    Artifact.objects.visible_to_current_workspace(),
+                    [workspace2_artifact],
+                )
+
+        scope1 = Scope.objects.create(name="Scope1")
+        workspace2 = self.playground.create_workspace(
+            name="workspace2", scope=scope1, public=True
+        )
+
+        with context.local():
+            context.set_scope(scope1)
+            context.set_user(self.scenario.user)
+            workspace2.set_current()
+            with self.subTest(scope="other", inherits=False):
+                self.assertQuerySetEqual(
+                    Artifact.objects.in_current_scope(),
+                    [],
+                )
+            workspace2.set_inheritance([self.scenario.workspace])
+            with self.subTest(scope="other", inherits=True):
+                self.assertQuerySetEqual(
+                    Artifact.objects.visible_to_current_workspace(),
+                    [artifact],
+                )
+
 
 class ArtifactTests(TestCase):
     """Tests for the Artifact class."""
@@ -238,28 +333,28 @@ class ArtifactTests(TestCase):
         """Test Artifact.expired() method."""
         artifact = Artifact()
 
-        artifact.created_at = timezone.now() - timedelta(days=2)
-        artifact.expiration_delay = timedelta(days=1)
+        artifact.created_at = timezone.now() - dt.timedelta(days=2)
+        artifact.expiration_delay = dt.timedelta(days=1)
         self.assertTrue(artifact.expired(timezone.now()))
 
-        artifact.expiration_delay = timedelta(0)
+        artifact.expiration_delay = dt.timedelta(0)
         self.assertFalse(artifact.expired(timezone.now()))
 
         artifact.created_at = timezone.now()
-        artifact.expiration_delay = timedelta(days=1)
+        artifact.expiration_delay = dt.timedelta(days=1)
         self.assertFalse(artifact.expired(timezone.now()))
 
     @context.disable_permission_checks()
     def test_default_expiration(self) -> None:
         """Test workspace's default_expiration_delay."""
         self.assertEqual(
-            self.scenario.workspace.default_expiration_delay, timedelta(0)
+            self.scenario.workspace.default_expiration_delay, dt.timedelta(0)
         )
         artifact = Artifact.objects.create(workspace=self.scenario.workspace)
         self.assertIsNone(artifact.expiration_delay)
 
         expiring_ws = self.playground.create_workspace(name="test")
-        expiring_ws.default_expiration_delay = timedelta(days=7)
+        expiring_ws.default_expiration_delay = dt.timedelta(days=7)
         test_creation_time = timezone.now()
         artifact = Artifact.objects.create(workspace=expiring_ws)
         delta = expiring_ws.default_expiration_delay
@@ -279,6 +374,14 @@ class ArtifactTests(TestCase):
             f"Workspace: {artifact.workspace.id}",
         )
 
+    def test_ui(self) -> None:
+        from debusine.web.views.ui.artifacts import ArtifactUI
+
+        artifact, _ = self.playground.create_artifact()
+        request = RequestFactory().get("/")
+        ui = artifact.ui(request)
+        self.assertIsInstance(ui, ArtifactUI)
+
     @context.disable_permission_checks()
     def test_get_absolute_url(self) -> None:
         """Test for Artifact.get_absolute_url."""
@@ -431,6 +534,7 @@ class FileInArtifactTests(TestCase):
         workspace: Workspace | None = None,
         artifact: Artifact | None = None,
         path: str | None = None,
+        complete: bool = False,
     ) -> FileInArtifact:
         """Return FileInArtifact to be used in tests."""
         if fileobj is None:
@@ -443,7 +547,10 @@ class FileInArtifactTests(TestCase):
             path = "/usr/bin/test"
 
         return FileInArtifact.objects.create(
-            artifact=artifact, file=fileobj, path=path
+            artifact=artifact,
+            file=fileobj,
+            path=path,
+            complete=complete,
         )
 
     def test_artifact_path_unique_constraint(self) -> None:
@@ -477,6 +584,14 @@ class FileInArtifactTests(TestCase):
             f"File: {file_in_artifact.file.id}",
         )
 
+    def test_ui(self) -> None:
+        from debusine.web.views.ui.artifacts import FileInArtifactUI
+
+        file_in_artifact = self.create_file_in_artifact()
+        request = RequestFactory().get("/")
+        ui = file_in_artifact.ui(request)
+        self.assertIsInstance(ui, FileInArtifactUI)
+
     def test_get_absolute_url(self) -> None:
         """Test FileInArtifact.get_absolute_url."""
         file_in_artifact = self.create_file_in_artifact()
