@@ -13,13 +13,15 @@ import base64
 
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import ImproperlyConfigured, ValidationError
-from django.db import IntegrityError
+from django.db import IntegrityError, transaction
 
 from debusine.assets import (
     AWSProviderAccountConfiguration,
     AWSProviderAccountCredentials,
     AWSProviderAccountData,
     AssetCategory,
+    BasicAPTAuthenticationData,
+    BasicAPTCredentials,
     KeyPurpose,
     SigningKeyData,
 )
@@ -178,13 +180,40 @@ class AssetManagerTests(TestCase):
             Asset.objects.can_manage_permissions(user).first(), asset
         )
 
-    def test_get_by_slug(self) -> None:
+    def test_get_by_slug_signing_key(self) -> None:
         asset = self.playground.create_signing_key_asset()
         self.assertEqual(
             Asset.objects.get_by_slug(category=asset.category, slug=asset.slug),
             asset,
         )
 
+    def test_get_by_slug_apt_authentication_by_name(self) -> None:
+        asset = self.playground.create_apt_authentication_asset(name="test-1")
+        self.playground.create_apt_authentication_asset(name="test-2")
+        self.assertEqual(
+            Asset.objects.get_by_slug(
+                category=asset.category,
+                slug="test-1",
+                workspace=self.playground.get_default_workspace(),
+            ),
+            asset,
+        )
+
+    def test_get_by_slug_apt_authentication_by_workspace_and_name(self) -> None:
+        other_workspace = self.playground.create_workspace(name="other")
+        asset = self.playground.create_apt_authentication_asset(
+            workspace=other_workspace, name="test"
+        )
+        self.playground.create_apt_authentication_asset(name="test")
+        self.assertEqual(
+            Asset.objects.get_by_slug(
+                category=asset.category,
+                slug="other:test",
+                workspace=self.playground.get_default_workspace(),
+            ),
+            asset,
+        )
+
     def test_get_by_slug_unknown_category(self) -> None:
         with self.assertRaisesRegex(
             ValueError, r"No slug defined for category 'unknown'"
@@ -257,12 +286,57 @@ class AssetTests(TestCase):
             created_by=self.playground.get_default_user(),
         )
 
+    def create_apt_authentication_asset(self) -> Asset:
+        """Create an unsaved apt-authentication Asset."""
+        return Asset(
+            category=AssetCategory.APT_AUTHENTICATION,
+            workspace=self.playground.get_default_workspace(),
+            data=BasicAPTAuthenticationData(
+                name="test",
+                credentials=BasicAPTCredentials(
+                    username="test-username", password="test-password"
+                ),
+            ).model_dump(),
+            created_by=self.playground.get_default_user(),
+        )
+
     def test_signing_key_unique_constraint(self) -> None:
         """Test that signing-key assets have a unique constraint."""
         self.playground.create_signing_key_asset(fingerprint="ABC123")
         with self.assertRaises(IntegrityError):
             self.playground.create_signing_key_asset(fingerprint="ABC123")
 
+    def test_apt_authentication_name(self) -> None:
+        """apt-authentication assets must have valid names."""
+        valid_names = ("secret", "foo_bar", "foo.bar", "foo+bar", "tail_")
+        invalid_names = (
+            "+foo",
+            "-foo",
+            ".foo",
+            "_foo",
+            "foo:bar",
+            "/etc/passwd",
+            "../foo",
+        )
+
+        for name in valid_names:
+            with self.subTest(name=name):
+                self.playground.create_apt_authentication_asset(name=name)
+
+        for name in invalid_names:
+            with (
+                self.subTest(name=name),
+                transaction.atomic(),
+                self.assertRaises(IntegrityError),
+            ):
+                self.playground.create_apt_authentication_asset(name=name)
+
+    def test_apt_authentication_unique_constraint(self) -> None:
+        """Test that apt-authentication assets have a unique constraint."""
+        self.playground.create_apt_authentication_asset(name="test-apt")
+        with self.assertRaises(IntegrityError):
+            self.playground.create_apt_authentication_asset(name="test-apt")
+
     def test_clean(self) -> None:
         """Test that clean() validates the model."""
         asset = self.playground.create_signing_key_asset()
@@ -273,10 +347,15 @@ class AssetTests(TestCase):
         ):
             asset.clean()
 
-    def test_slug(self) -> None:
+    def test_slug_signing_key(self) -> None:
         asset = self.create_signing_key_asset()
         self.assertEqual(asset.slug, "openpgp:ABC123")
 
+    def test_slug_apt_authentication(self) -> None:
+        asset = self.create_apt_authentication_asset()
+        workspace = self.playground.get_default_workspace()
+        self.assertEqual(asset.slug, f"{workspace.name}:test")
+
     def test_slug_unknown_category(self) -> None:
         asset = self.create_signing_key_asset()
         asset.category = "unknown"
@@ -466,3 +545,28 @@ class AssetUsageTests(TestCase):
                 self.assertPermissionPredicate(
                     asset_usage.can_sign_repository_with, u, expected
                 )
+
+    def test_can_use_apt_authentication_with(self) -> None:
+        user = self.scenario.user
+        asset = self.playground.create_apt_authentication_asset()
+        asset_usage = self.playground.create_asset_usage(resource=asset)
+        for u, workspace_roles, asset_usage_roles, expected in (
+            (AnonymousUser(), [], [], False),
+            (user, [], [], False),
+            (user, [Workspace.Roles.OWNER], [], True),
+            (user, [], [AssetUsage.Roles.APT_AUTHENTICATOR], True),
+        ):
+            with (
+                self.subTest(
+                    user=repr(u),
+                    workspace_roles=repr(workspace_roles),
+                    asset_usage_roles=repr(asset_usage_roles),
+                ),
+                self.scenario.assign_role(
+                    self.scenario.workspace, *workspace_roles
+                ),
+                self.scenario.assign_role(asset_usage, *asset_usage_roles),
+            ):
+                self.assertPermissionPredicate(
+                    asset_usage.can_use_apt_authentication_with, u, expected
+                )
