@@ -146,24 +146,28 @@ class ScopeTests(TestCase):
     def test_unique(self) -> None:
         """Check that scope names and labels are unique."""
         Scope.objects.create(name="test", label="Test")
-        with transaction.atomic():
-            with self.assertRaisesRegex(
+        with (
+            transaction.atomic(),
+            self.assertRaisesRegex(
                 IntegrityError,
                 # Only match constraint name to support non-english locales
                 # "duplicate key value violates unique constraint"
                 r"(?s)duplicate key value violates unique constraint.+"
                 r"Key \(name\)=\(test\) already exists",
-            ):
-                Scope.objects.create(name="test", label="Other test")
-        with transaction.atomic():
-            with self.assertRaisesRegex(
+            ),
+        ):
+            Scope.objects.create(name="test", label="Other test")
+        with (
+            transaction.atomic(),
+            self.assertRaisesRegex(
                 IntegrityError,
                 # Only match constraint name to support non-english locales
                 # "duplicate key value violates unique constraint"
                 r"(?s)duplicate key value violates unique constraint.+"
                 r"Key \(label\)=\(Test\) already exists",
-            ):
-                Scope.objects.create(name="test1", label="Test")
+            ),
+        ):
+            Scope.objects.create(name="test1", label="Test")
         Scope.objects.create(name="test1", label="Test1")
 
     def test_fallback(self) -> None:
