@@ -23,8 +23,8 @@ from debusine.db.models import (
     ArtifactRelation,
     TaskDatabase,
     WorkRequest,
-    Workspace,
 )
+from debusine.db.playground import scenarios
 from debusine.tasks.models import LookupMultiple
 from debusine.tasks.server import ArtifactInfo, CollectionInfo
 from debusine.test.django import TestCase
@@ -34,25 +34,24 @@ from debusine.test.test_utils import create_system_tarball_data
 class TaskDatabaseTests(TestCase):
     """Test database interaction in worker tasks."""
 
-    workspace: ClassVar[Workspace]
+    scenario = scenarios.DefaultContext()
     work_request: ClassVar[WorkRequest]
 
     @classmethod
     def setUpTestData(cls) -> None:
         """Set up common data for tests."""
         super().setUpTestData()
-        cls.workspace = cls.playground.create_workspace(
-            name="test", public=True
-        )
         cls.work_request = cls.playground.create_worker_task(
-            task_name="noop", workspace=cls.workspace
+            task_name="noop", workspace=cls.scenario.workspace
         )
 
     @context.disable_permission_checks()
     def test_lookup_single_artifact(self) -> None:
         """Look up a single artifact."""
         collection = self.playground.create_collection(
-            "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
+            "debian",
+            CollectionCategory.ENVIRONMENTS,
+            workspace=self.scenario.workspace,
         )
         artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SYSTEM_TARBALL,
@@ -96,7 +95,9 @@ class TaskDatabaseTests(TestCase):
     def test_lookup_multiple_artifacts(self) -> None:
         """Look up multiple artifacts."""
         collection = self.playground.create_collection(
-            "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
+            "debian",
+            CollectionCategory.ENVIRONMENTS,
+            workspace=self.scenario.workspace,
         )
         artifacts: list[Artifact] = []
         for codename in "bookworm", "trixie":
@@ -220,7 +221,9 @@ class TaskDatabaseTests(TestCase):
     def test_lookup_single_collection(self) -> None:
         """Look up a single collection."""
         collection = self.playground.create_collection(
-            "debian", CollectionCategory.ENVIRONMENTS, workspace=self.workspace
+            "debian",
+            CollectionCategory.ENVIRONMENTS,
+            workspace=self.scenario.workspace,
         )
         task_db = TaskDatabase(self.work_request)
 
@@ -228,8 +231,8 @@ class TaskDatabaseTests(TestCase):
             task_db.lookup_single_collection("debian@debian:environments"),
             CollectionInfo(
                 id=collection.id,
-                scope_name=self.workspace.scope.name,
-                workspace_name=self.workspace.name,
+                scope_name=self.scenario.workspace.scope.name,
+                workspace_name=self.scenario.workspace.name,
                 category=collection.category,
                 name=collection.name,
                 data=collection.data,
@@ -242,8 +245,8 @@ class TaskDatabaseTests(TestCase):
             ),
             CollectionInfo(
                 id=collection.id,
-                scope_name=self.workspace.scope.name,
-                workspace_name=self.workspace.name,
+                scope_name=self.scenario.workspace.scope.name,
+                workspace_name=self.scenario.workspace.name,
                 category=collection.category,
                 name=collection.name,
                 data=collection.data,
@@ -258,8 +261,12 @@ class TaskDatabaseTests(TestCase):
 
     def test_lookup_singleton_collection(self) -> None:
         """Look up a singleton collection."""
+        workspace = self.playground.create_workspace(name="test", public=True)
+        self.work_request.workspace = workspace
+        self.work_request.save()
         collection = self.playground.create_singleton_collection(
-            CollectionCategory.PACKAGE_BUILD_LOGS, workspace=self.workspace
+            CollectionCategory.PACKAGE_BUILD_LOGS,
+            workspace=workspace,
         )
         task_db = TaskDatabase(self.work_request)
 
@@ -269,8 +276,8 @@ class TaskDatabaseTests(TestCase):
             ),
             CollectionInfo(
                 id=collection.id,
-                scope_name=self.workspace.scope.name,
-                workspace_name=self.workspace.name,
+                scope_name=workspace.scope.name,
+                workspace_name=workspace.name,
                 category=collection.category,
                 name=collection.name,
                 data=collection.data,
@@ -285,7 +292,9 @@ class TaskDatabaseTests(TestCase):
 
     def test_export_suite_signing_keys_none(self) -> None:
         collection = self.playground.create_collection(
-            "trixie", CollectionCategory.SUITE, workspace=self.workspace
+            "trixie",
+            CollectionCategory.SUITE,
+            workspace=self.scenario.workspace,
         )
         task_db = TaskDatabase(self.work_request)
 
@@ -302,14 +311,14 @@ class TaskDatabaseTests(TestCase):
                     purpose=KeyPurpose.OPENPGP_REPOSITORY,
                     fingerprint=key.fingerprint,
                     public_key=str(key),
-                    workspace=self.workspace,
+                    workspace=self.scenario.workspace,
                 ),
-                workspace=self.workspace,
+                workspace=self.scenario.workspace,
             )
         collection = self.playground.create_collection(
             "trixie",
             CollectionCategory.SUITE,
-            workspace=self.workspace,
+            workspace=self.scenario.workspace,
             data={"signing_keys": [key.fingerprint for key in keys]},
         )
         task_db = TaskDatabase(self.work_request)
