@@ -388,8 +388,10 @@ class WorkerManagerTests(TestCase):
         worker.save()
 
         idle_time = (
-            Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time
+            Worker.objects.with_idle_time()
+            .values_list("idle_time", flat=True)
+            .get(pk=worker.pk)
+        )
         self.assertGreaterEqual(idle_time, 86400)
         self.assertLessEqual(idle_time, 86400 + 3600)
 
@@ -403,8 +405,10 @@ class WorkerManagerTests(TestCase):
         worker.save()
 
         idle_time = (
-            Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time
+            Worker.objects.with_idle_time()
+            .values_list("idle_time", flat=True)
+            .get(pk=worker.pk)
+        )
         self.assertGreaterEqual(idle_time, 86400)
         self.assertLessEqual(idle_time, 86400 + 3600)
 
@@ -421,9 +425,30 @@ class WorkerManagerTests(TestCase):
         self.assertTrue(worker.is_busy())
 
         idle_time = (
-            Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time
-        self.assertGreaterEqual(idle_time, 0)
+            Worker.objects.with_idle_time()
+            .values_list("idle_time", flat=True)
+            .get(pk=worker.pk)
+        )
+        self.assertEqual(idle_time, 0)
+
+    def test_with_idle_time_pending_task(self) -> None:
+        """Compute idle_time with a pending task."""
+        worker = Worker.objects.create_with_fqdn(
+            'worker-a', token=Token.objects.create(enabled=True)
+        )
+        worker.registered_at = timezone.now() - dt.timedelta(days=2)
+        worker.instance_created_at = timezone.now() - dt.timedelta(days=1)
+        worker.save()
+        self.playground.create_worker_task(worker=worker, mark_pending=True)
+        worker.refresh_from_db()
+        self.assertTrue(worker.is_busy())
+
+        idle_time = (
+            Worker.objects.with_idle_time()
+            .values_list("idle_time", flat=True)
+            .get(pk=worker.pk)
+        )
+        self.assertEqual(idle_time, 0)
 
     def test_with_idle_time_completed_task(self) -> None:
         """Compute idle_time with a completed task."""
@@ -443,11 +468,37 @@ class WorkerManagerTests(TestCase):
             work_request.save()
 
         idle_time = (
-            Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time
+            Worker.objects.with_idle_time()
+            .values_list("idle_time", flat=True)
+            .get(pk=worker.pk)
+        )
         self.assertGreaterEqual(idle_time, 6 * 3600)
         self.assertLessEqual(idle_time, 7 * 3600)
 
+    def test_with_idle_time_old_completed_task(self) -> None:
+        """Compute idle_time with a completed task from a previous instance."""
+        worker = Worker.objects.create_with_fqdn(
+            'worker-a', token=Token.objects.create(enabled=True)
+        )
+        worker.registered_at = timezone.now() - dt.timedelta(days=7)
+        worker.instance_created_at = timezone.now() - dt.timedelta(seconds=5)
+        worker.save()
+        for hours_ago in (10, 6):
+            work_request = self.playground.create_worker_task(
+                worker=worker, result=WorkRequest.Results.SUCCESS
+            )
+            work_request.completed_at = timezone.now() - dt.timedelta(
+                hours=hours_ago
+            )
+            work_request.save()
+
+        idle_time = (
+            Worker.objects.with_idle_time()
+            .values_list("idle_time", flat=True)
+            .get(pk=worker.pk)
+        )
+        self.assertLessEqual(idle_time, 60)
+
     def test_mark_all_disconnected(self) -> None:
         """WorkerManager.mark_all_disconnected() affects non-Celery workers."""
         worker_connected_external = self.playground.create_worker(
