@@ -1551,13 +1551,13 @@ class WorkspaceTests(TestCase):
             )
         self.assertIsNotNone(test2.pk)
 
-        with override_permission(Scope, "can_create_workspace", DenyAll):
-            with self.assertRaisesRegex(
+        with (
+            override_permission(Scope, "can_create_workspace", DenyAll),
+            self.assertRaisesRegex(
                 PermissionDenied, r"None cannot create workspaces in debusine"
-            ):
-                Workspace.objects.create(
-                    name="test3", scope=self.scenario.scope
-                )
+            ),
+        ):
+            Workspace.objects.create(name="test3", scope=self.scenario.scope)
         self.assertQuerySetEqual(Workspace.objects.filter(name="test3"), [])
 
     def test_can_create_artifacts_anonymous(self) -> None:
@@ -1636,16 +1636,24 @@ class WorkspaceTests(TestCase):
 
     def test_with_expiration_time_no_expiry(self) -> None:
         """Test with_expiration_time with no expiry."""
-        ws = Workspace.objects.with_expiration_time().get(name="System")
-        self.assertIsNone(ws.expiration_time)
+        expiration_time = (
+            Workspace.objects.with_expiration_time()
+            .values_list("expiration_time", flat=True)
+            .get(name="System")
+        )
+        self.assertIsNone(expiration_time)
 
     def test_with_expiration_time_created_at(self) -> None:
         """Test with_expiration_time with only created_at."""
         self.a.expiration_delay = dt.timedelta(days=3)
         self.a.save()
-        ws = Workspace.objects.with_expiration_time().get(pk=self.a.pk)
+        expiration_time = (
+            Workspace.objects.with_expiration_time()
+            .values_list("expiration_time", flat=True)
+            .get(pk=self.a.pk)
+        )
         self.assertEqual(
-            ws.expiration_time, self.a.created_at + self.a.expiration_delay
+            expiration_time, self.a.created_at + self.a.expiration_delay
         )
 
     def test_with_expiration_time_work_request_completed(self) -> None:
@@ -1657,9 +1665,13 @@ class WorkspaceTests(TestCase):
         )
         wr.completed_at = self.a.created_at + dt.timedelta(days=42)
         wr.save()
-        ws = Workspace.objects.with_expiration_time().get(pk=self.a.pk)
+        expiration_time = (
+            Workspace.objects.with_expiration_time()
+            .values_list("expiration_time", flat=True)
+            .get(pk=self.a.pk)
+        )
         self.assertEqual(
-            ws.expiration_time,
+            expiration_time,
             self.a.created_at + self.a.expiration_delay + dt.timedelta(days=42),
         )
 
@@ -1686,7 +1698,7 @@ class WorkspaceTests(TestCase):
         expiration_time = self.a.created_at + self.a.expiration_delay
         a = Workspace.objects.with_expiration_time().get(pk=self.a.pk)
         self.assertEqual(
-            a.expiration_time,
+            a.expiration_time,  # type: ignore[attr-defined]
             expiration_time,
         )
         self.assertEqual(a.expire_at, expiration_time)
@@ -1918,15 +1930,15 @@ class WorkspaceDeleteTests(TestCase):
             ],
             ordered=False,
         )
-        self.assertEqual(operation.artifact_relations.count(), 25)
-        self.assertEqual(operation.artifacts.count(), 55)
+        self.assertEqual(operation.artifact_relations.count(), 38)
+        self.assertEqual(operation.artifacts.count(), 65)
         self.assertEqual(operation.assets.count(), 1)
         self.assertEqual(operation.asset_usages.count(), 1)
-        self.assertEqual(operation.collection_items.count(), 72)
+        self.assertEqual(operation.collection_items.count(), 78)
         self.assertEqual(operation.collections.count(), 17)
-        self.assertEqual(operation.file_in_artifacts.count(), 33)
+        self.assertEqual(operation.file_in_artifacts.count(), 39)
         self.assertEqual(operation.groups.count(), 1)
-        self.assertEqual(operation.work_requests.count(), 74)
+        self.assertEqual(operation.work_requests.count(), 75)
         self.assertEqual(operation.workspace_roles.count(), 1)
 
     def test_delete(self) -> None:
