@@ -194,22 +194,26 @@ class WorkerPool(models.Model):
             )
 
         # Trigger worker termination
-        self.provider_interface.terminate_worker(worker)
+        with transaction.atomic():
+            self.provider_interface.terminate_worker(worker)
 
-        # De-assign any pending tasks assigned to the worker
-        for pending in WorkRequest.objects.reassignable(worker=worker):
-            pending.de_assign_worker()
+        with transaction.atomic():
+            # De-assign any pending tasks assigned to the worker
+            for pending in WorkRequest.objects.reassignable(worker=worker):
+                pending.de_assign_worker()
 
-        # Retry any work requests that were previously running on the worker
-        for running in WorkRequest.objects.running(worker=worker):
-            running.mark_aborted()
-            try:
-                with context.disable_permission_checks():
-                    running.retry(reason=WorkRequestRetryReason.WORKER_FAILED)
-            except CannotRetry as e:
-                logger.debug(  # noqa: G200
-                    "Cannot retry previously-running work request: %s", e
-                )
+            # Retry any work requests that were previously running on the worker
+            for running in WorkRequest.objects.running(worker=worker):
+                running.mark_aborted()
+                try:
+                    with context.disable_permission_checks():
+                        running.retry(
+                            reason=WorkRequestRetryReason.WORKER_FAILED
+                        )
+                except CannotRetry as e:
+                    logger.debug(  # noqa: G200
+                        "Cannot retry previously-running work request: %s", e
+                    )
 
     def clean(self) -> None:
         """
