@@ -12,7 +12,7 @@
 import copy
 import datetime as dt
 import hashlib
-from typing import Any, Optional, TYPE_CHECKING, cast, overload
+from typing import Any, Optional, TYPE_CHECKING, TypedDict, cast, overload
 
 from django.db import IntegrityError, models, transaction
 from django.db.models import (
@@ -40,6 +40,7 @@ from debusine.tasks.models import WorkerType
 
 if TYPE_CHECKING:
     from django.http import HttpRequest
+    from django_stubs_ext import WithAnnotations
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.web.views.ui.workers import WorkerUI
@@ -47,6 +48,12 @@ else:
     TypedModelMeta = object
 
 
+class IdleTimeDict(TypedDict):
+    """Additional fields returned by WorkerQuerySet.with_idle_time."""
+
+    idle_time: float
+
+
 class WorkerQuerySet[A](QuerySet["Worker", A]):
     """Custom QuerySet for Worker."""
 
@@ -104,17 +111,24 @@ class WorkerQuerySet[A](QuerySet["Worker", A]):
 
         return workers
 
-    def with_idle_time(self) -> "WorkerQuerySet[Any]":
+    def with_idle_time(
+        self,
+    ) -> "WorkerQuerySet[WithAnnotations[Worker, IdleTimeDict]]":
         """
         Annotate workers with `idle_time`.
 
-        `Worker.idle_time` will be 0 for workers currently assigned a running
-        work request, or the number of seconds the worker has been spending
-        without a running work request assigned.
+        `Worker.idle_time` will be 0 for workers currently assigned a work
+        request, or the number of seconds the worker has been spending without
+        a work request assigned.
         """
         tasks = WorkRequest.objects.filter(worker=OuterRef("pk"))
 
-        running_tasks = tasks.filter(status=WorkRequest.Statuses.RUNNING)
+        active_tasks = tasks.filter(
+            status__in=(
+                WorkRequest.Statuses.RUNNING,
+                WorkRequest.Statuses.PENDING,
+            )
+        )
 
         last_tasks = tasks.filter(
             completed_at__gt=F("worker__instance_created_at")
@@ -132,7 +146,7 @@ class WorkerQuerySet[A](QuerySet["Worker", A]):
         return self.annotate(
             idle_time=Case(
                 When(
-                    Exists(running_tasks),
+                    Exists(active_tasks),
                     then=0,
                 ),
                 When(
