@@ -12,17 +12,8 @@
 import datetime as dt
 import enum
 import re
-from collections.abc import Generator, Sequence
-from typing import (
-    Any,
-    Generic,
-    Self,
-    TYPE_CHECKING,
-    TypeAlias,
-    TypeVar,
-    Union,
-    cast,
-)
+from collections.abc import Callable, Generator, Sequence
+from typing import Any, Self, TYPE_CHECKING, TypeAlias, TypedDict, Union, cast
 
 from django.conf import settings
 from django.contrib.auth.models import AnonymousUser
@@ -67,6 +58,7 @@ from debusine.utils.typing_utils import copy_signature_from
 
 if TYPE_CHECKING:
     from django.http import HttpRequest
+    from django_stubs_ext import WithAnnotations
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models import Collection, User
@@ -74,8 +66,6 @@ if TYPE_CHECKING:
 else:
     TypedModelMeta = object
 
-A = TypeVar("A")
-
 #: Workspace names reserved for use in toplevel URL path components
 RESERVED_WORKSPACE_NAMES = frozenset(
     # TODO: trim after !1274
@@ -185,10 +175,18 @@ class WorkspaceRoles(permissions.Roles, WorkspaceRoleBase, enum.ReprEnum):
 WorkspaceRoles.setup()
 
 
-class WorkspaceQuerySet(QuerySet["Workspace", A], Generic[A]):
+class ExpirationTimeDict(TypedDict):
+    """Additional fields returned by WorkspaceQuerySet.with_expiration_time."""
+
+    expiration_time: dt.datetime
+
+
+class WorkspaceQuerySet[A](QuerySet["Workspace", A]):
     """Custom QuerySet for Workspace."""
 
-    def with_expiration_time(self) -> "WorkspaceQuerySet[Any]":
+    def with_expiration_time(
+        self,
+    ) -> "WorkspaceQuerySet[WithAnnotations[Workspace, ExpirationTimeDict]]":
         """Annotate the queryset with expiration times."""
         return self.annotate(
             expiration_time=F("expiration_delay")
@@ -370,7 +368,7 @@ class WorkspaceManager(models.Manager["Workspace"]):
 class DeleteWorkspaces:
     """Delete all workspaces in a queryset."""
 
-    def __init__(self, workspaces: WorkspaceQuerySet["Workspace"]) -> None:
+    def __init__(self, workspaces: WorkspaceQuerySet["Any"]) -> None:
         """Store workspaces to delete and compute affected objects."""
         # Import here to prevent circular dependencies
         from debusine.db import models as dmodels
@@ -385,12 +383,17 @@ class DeleteWorkspaces:
         # simulate a fully populated database that is kept up to date as
         # new models get added, so for the moment we limit ourselves to
         # adding to this as the need arises.
-        self.workflow_templates = dmodels.WorkflowTemplate.objects.filter(
+        self.artifact_relations = dmodels.ArtifactRelation.objects.filter(
+            Q(artifact__workspace__in=workspaces)
+            | Q(target__workspace__in=workspaces)
+        )
+        self.artifacts = dmodels.Artifact.objects.filter(
             workspace__in=workspaces
         )
-        self.work_requests = dmodels.WorkRequest.objects.filter(
+        self.asset_usages = dmodels.AssetUsage.objects.filter(
             workspace__in=workspaces
         )
+        self.assets = dmodels.Asset.objects.filter(workspace__in=workspaces)
         self.collection_items = dmodels.CollectionItem.objects.filter(
             Q(parent_collection__workspace__in=workspaces)
             | Q(artifact__workspace__in=workspaces)
@@ -402,20 +405,19 @@ class DeleteWorkspaces:
         self.file_in_artifacts = dmodels.FileInArtifact.objects.filter(
             artifact__workspace__in=workspaces
         )
-        self.artifact_relations = dmodels.ArtifactRelation.objects.filter(
-            Q(artifact__workspace__in=workspaces)
-            | Q(target__workspace__in=workspaces)
+        # Group role tables use CASCADE on the foreign key to group, so we
+        # don't need to track where workspace-scoped groups are used and we can
+        # just delete them
+        self.groups = dmodels.Group.objects.filter(workspace__in=workspaces)
+        self.work_requests = dmodels.WorkRequest.objects.filter(
+            workspace__in=workspaces
         )
-        self.artifacts = dmodels.Artifact.objects.filter(
+        self.workflow_templates = dmodels.WorkflowTemplate.objects.filter(
             workspace__in=workspaces
         )
         self.workspace_roles = WorkspaceRole.objects.filter(
             resource__in=workspaces
         )
-        self.asset_usages = dmodels.AssetUsage.objects.filter(
-            workspace__in=workspaces
-        )
-        self.assets = dmodels.Asset.objects.filter(workspace__in=workspaces)
 
     def perform_deletions(self) -> None:
         """Delete the workspaces and all their resources."""
@@ -429,6 +431,7 @@ class DeleteWorkspaces:
         self.asset_usages.delete()
         self.assets.delete()
         self.workspace_roles.delete()
+        self.groups.delete()
         self.workspaces.delete()
 
 
@@ -445,8 +448,6 @@ class Workspace(models.Model):
 
     Roles: TypeAlias = WorkspaceRoles
 
-    objects = WorkspaceManager.from_queryset(WorkspaceQuerySet)()
-
     name = models.CharField(
         max_length=255, validators=[validate_workspace_name]
     )
@@ -480,6 +481,8 @@ class Workspace(models.Model):
         ),
     )
 
+    objects = WorkspaceManager.from_queryset(WorkspaceQuerySet)()
+
     class Meta(TypedModelMeta):
         base_manager_name = "objects"
         constraints = [
@@ -489,11 +492,21 @@ class Workspace(models.Model):
             ),
         ]
 
-    def ui(self, request: "HttpRequest") -> "WorkspaceUI":
-        """Return a UI helper for this instance."""
-        from debusine.web.views.ui.workspaces import WorkspaceUI
+    def __str__(self) -> str:
+        """Return basic information of Workspace."""
+        return f"{self.scope.name}/{self.name}"
 
-        return WorkspaceUI.get(request, self)
+    @copy_signature_from(models.Model.save)
+    def save(self, **kwargs: Any) -> None:
+        """Wrap save with permission checks."""
+        if self._state.adding:
+            # Create
+            enforce(self.scope.can_create_workspace)
+        else:
+            # Update
+            ...  # TODO: check for update permissions
+
+        return super().save(**kwargs)
 
     def get_absolute_url(self) -> str:
         """Return an absolute URL to this workspace."""
@@ -502,6 +515,12 @@ class Workspace(models.Model):
         with urlconf_scope(self.scope.name):
             return reverse("workspaces:detail", kwargs={"wname": self.name})
 
+    def ui(self, request: "HttpRequest") -> "WorkspaceUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.workspaces import WorkspaceUI
+
+        return WorkspaceUI.get(request, self)
+
     def get_absolute_url_configure(self) -> str:
         """Return an absolute URL to configure this workspace."""
         from debusine.server.scopes import urlconf_scope
@@ -518,18 +537,6 @@ class Workspace(models.Model):
                 "workspaces:update_inheritance", kwargs={"wname": self.name}
             )
 
-    @copy_signature_from(models.Model.save)
-    def save(self, **kwargs: Any) -> None:
-        """Wrap save with permission checks."""
-        if self._state.adding:
-            # Create
-            enforce(self.scope.can_create_workspace)
-        else:
-            # Update
-            ...  # TODO: check for update permissions
-
-        return super().save(**kwargs)
-
     def set_current(self) -> None:
         """
         Set this as the current workspace.
@@ -730,16 +737,16 @@ class Workspace(models.Model):
         # otherwise, compute it and set it
 
         if not hasattr(self, "expiration_time"):
-            query = Workspace.objects.with_expiration_time().filter(pk=self.pk)
-            # expiration_time is introduced by with_expiration_time but it's
-            # currently hard to explain to the type system
-            (expiration_time,) = query.values_list(
-                "expiration_time", flat=True
-            )  # type: ignore[misc]
+            expiration_time = (
+                Workspace.objects.with_expiration_time()
+                .values_list("expiration_time", flat=True)
+                .get(pk=self.pk)
+            )
             setattr(self, "expiration_time", expiration_time)
 
         return cast(
-            dt.datetime, self.expiration_time  # type: ignore[attr-defined]
+            dt.datetime,
+            self.expiration_time,  # type: ignore[attr-defined]
         )
 
     def file_needs_upload(self, fileobj: File) -> bool:
@@ -788,42 +795,30 @@ class Workspace(models.Model):
                 child=self, parent=workspace, order=idx
             )
 
-    def get_collection(
-        self,
-        *,
-        # TODO: allow user to be None to mean take it from context?
-        user: Union["User", "AnonymousUser"],
-        category: str,
-        name: str,
-    ) -> "Collection":
+    def make_inheritance_chain_cte(
+        self, visible: With
+    ) -> Callable[[With], "WorkspaceQuerySet[Workspace]"]:
         """
-        Lookup a collection by category and name.
+        Make a recursive CTE factory to follow a workspace's inheritance chain.
 
-        If the collection is not found in this workspace, it follows the
-        workspace inheritance chain using a depth-first search.
-
-        :param user: user to use for permission checking
-        :param category: collection category
-        :param name: collection name
-        :raises Collection.DoesNotExist: if the collection was not found
+        :param visible: A Common Table Expression with the results of
+          ``Workspace.objects.can_display(user)``, which will be used by
+          this CTE.
+        :return: a function that can be passed as the first argument to
+          ``With.recursive``.  The ``path_order`` field in the ``QuerySet``
+          it returns allows ordering the results as for a depth-first
+          search.
         """
-        from debusine.db.models import Collection
 
-        visible = With(Workspace.objects.can_display(user), name="visible")
-
-        def chain_cte(cte: With) -> QuerySet[Any]:
-            """
-            Follow a workspace's inheritance chain.
-
-            The ``path_order`` field allows ordering the results as for a
-            depth-first search.
-            """
+        def chain_cte(cte: With) -> WorkspaceQuerySet[Workspace]:
             path_order_field: ArrayField[int, int] = ArrayField(
                 IntegerField(), name="path_order"
             )
             path_field: ArrayField[int, int] = ArrayField(
                 BigIntegerField(), name="path"
             )
+            path_field_contains = path_field.get_lookup("contains")
+            assert path_field_contains is not None
             recursive = (
                 visible.join(Workspace, id=visible.col.id)
                 .filter(id=self.id)
@@ -833,8 +828,11 @@ class Workspace(models.Model):
                     path_order=RawSQL(
                         "ARRAY[]::integer[]", (), output_field=path_order_field
                     ),
-                    path=RawSQL(
-                        "ARRAY[db_workspace.id]", (), output_field=path_field
+                    path=Func(
+                        F("id"),
+                        function="ARRAY",
+                        template="%(function)s[%(expressions)s]",
+                        output_field=path_field,
                     ),
                 )
                 .union(
@@ -844,9 +842,14 @@ class Workspace(models.Model):
                     )
                     # Break inheritance loops.
                     .annotate(
-                        is_cycle=RawSQL(
-                            "chain.path @> ARRAY[db_workspacechain.parent_id]",
-                            (),
+                        is_cycle=path_field_contains(
+                            cte.col.path,
+                            Func(
+                                F("parent_id"),
+                                function="ARRAY",
+                                template="%(function)s[%(expressions)s]",
+                                output_field=path_field,
+                            ),
                         )
                     )
                     .filter(is_cycle=False)
@@ -869,10 +872,31 @@ class Workspace(models.Model):
                     all=True,
                 )
             )
-            assert isinstance(recursive, QuerySet)
+            assert isinstance(recursive, WorkspaceQuerySet)
             return recursive
 
-        chain = With.recursive(chain_cte, name="chain")
+        return chain_cte
+
+    def get_collection(
+        self, *, user: PermissionUser, category: str, name: str
+    ) -> "Collection":
+        """
+        Look up a collection by category and name.
+
+        If the collection is not found in this workspace, it follows the
+        workspace inheritance chain using a depth-first search.
+
+        :param user: user to use for permission checking
+        :param category: collection category
+        :param name: collection name
+        :raises Collection.DoesNotExist: if the collection was not found
+        """
+        from debusine.db.models import Collection
+
+        visible = With(Workspace.objects.can_display(user), name="visible")
+        chain = With.recursive(
+            self.make_inheritance_chain_cte(visible), name="chain"
+        )
         collection = (
             chain.join(
                 CTEQuerySet(Collection, using=Collection.objects._db),
@@ -946,10 +970,6 @@ class Workspace(models.Model):
                 user, category, ws_seen=ws_seen, coll_seen=coll_seen
             )
 
-    def __str__(self) -> str:
-        """Return basic information of Workspace."""
-        return f"{self.scope.name}/{self.name}"
-
 
 class WorkspaceChain(models.Model):
     """Workspace chaining model."""
