@@ -51,6 +51,8 @@ from debusine.assets import (
     AWSProviderAccountData,
     AssetCategory,
     BaseAssetDataModel,
+    BasicAPTAuthenticationData,
+    BasicAPTCredentials,
     CloudProviderAccountData,
     CloudProvidersType,
     DummyProviderAccountData,
@@ -60,11 +62,7 @@ from debusine.assets import (
     KeyPurpose,
     SigningKeyData,
 )
-from debusine.client.models import (
-    RuntimeParameter,
-    RuntimeParameters,
-    model_to_json_serializable_dict,
-)
+from debusine.client.models import RuntimeParameter, RuntimeParameters
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
@@ -92,6 +90,7 @@ from debusine.db.models import (
 from debusine.db.models.permissions import (
     PermissionUser,
     get_resource_scope,
+    get_resource_workspace,
     resolve_role,
 )
 from debusine.db.tests.utils import _calculate_hash_from_data
@@ -265,6 +264,7 @@ class Playground(ArtifactPlayground):
         name: str,
         users: list[User] | None = None,
         scope: Scope | None = None,
+        workspace: Workspace | None = None,
         ephemeral: bool = False,
     ) -> Group:
         """Create a group, setting membership to users."""
@@ -272,8 +272,8 @@ class Playground(ArtifactPlayground):
             scope = self.get_default_scope()
         if users is None:
             users = []
-        group, created = Group.objects.get_or_create(
-            scope=scope, name=name, ephemeral=ephemeral
+        group, _ = Group.objects.get_or_create(
+            scope=scope, workspace=workspace, name=name, ephemeral=ephemeral
         )
         group.users.set(users)
         return group
@@ -288,13 +288,19 @@ class Playground(ArtifactPlayground):
     ) -> Group:
         """Create a group with the given role on a resource."""
         if name is None:
-            name = f"{str(resource).replace('/', '-')}-{role}"
+            if isinstance(resource, Workspace):
+                name = role
+            else:
+                name = f"{str(resource).replace('/', '-')}-{role}"
         if users is None:
             users = []
-
         role = resolve_role(resource, role)
-        group_scope = get_resource_scope(resource)
-        group = self.create_group(name, users, scope=group_scope)
+        group = self.create_group(
+            name,
+            users,
+            workspace=get_resource_workspace(resource),
+            scope=get_resource_scope(resource),
+        )
         roles_model = getattr(resource.__class__, "objects").get_roles_model()
         roles_model.objects.filter(group=group).delete()
         group.assign_role(resource, role)
@@ -529,9 +535,7 @@ class Playground(ArtifactPlayground):
                 )
 
         if isinstance(task_data, BaseWorkflowData):
-            task_data = model_to_json_serializable_dict(
-                task_data, exclude_unset=True
-            )
+            task_data = task_data.model_dump(mode="json", exclude_unset=True)
 
         with contextlib.ExitStack() as stack:
             stack.enter_context(context.local())
@@ -828,8 +832,8 @@ class Playground(ArtifactPlayground):
         if "created_by" not in kwargs:
             defaults["created_by"] = self.get_default_user()
         if isinstance(task_data, BaseTaskData):
-            defaults["task_data"] = model_to_json_serializable_dict(
-                task_data, exclude_unset=True
+            defaults["task_data"] = task_data.model_dump(
+                mode="json", exclude_unset=True
             )
         else:
             defaults["task_data"] = task_data or {}
@@ -984,10 +988,10 @@ class Playground(ArtifactPlayground):
             enabled=enabled,
             architectures=architectures,
             tags=tags,
-            specifications=specifications.dict(),
+            specifications=specifications.model_dump(),
             instance_wide=instance_wide,
             ephemeral=ephemeral,
-            limits=limits.dict(),
+            limits=limits.model_dump(),
             registered_at=timezone.now(),
         )
         return pool
@@ -1094,7 +1098,7 @@ class Playground(ArtifactPlayground):
 
         match data:
             case ArtifactData():
-                raw_data = model_to_json_serializable_dict(data)
+                raw_data = data.model_dump(mode="json")
             case dict():
                 raw_data = data
             case None:
@@ -1497,7 +1501,7 @@ class Playground(ArtifactPlayground):
             paths={filename: contents},
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             workspace=workspace or self.get_default_workspace(),
-            data=data.dict(),
+            data=data.model_dump(),
             work_request=work_request,
             created_by=created_by,
             create_files=True,
@@ -1581,7 +1585,11 @@ class Playground(ArtifactPlayground):
         return Asset.objects.create(
             category=category,
             workspace=workspace,
-            data=data.dict() if isinstance(data, BaseAssetDataModel) else data,
+            data=(
+                data.model_dump()
+                if isinstance(data, BaseAssetDataModel)
+                else data
+            ),
             created_by=created_by,
             created_by_work_request=created_by_work_request,
         )
@@ -1674,6 +1682,32 @@ class Playground(ArtifactPlayground):
             created_by_work_request=created_by_work_request,
         )
 
+    def create_apt_authentication_asset(
+        self,
+        name: str | None = None,
+        username: str | None = None,
+        password: str | None = None,
+        workspace: Workspace | None = None,
+        created_by: User | None = None,
+        created_by_work_request: WorkRequest | None = None,
+    ) -> Asset:
+        """Create a debusine:signing-key asset."""
+        if workspace is None:
+            workspace = self.get_default_workspace()
+        return self.create_asset(
+            category=AssetCategory.APT_AUTHENTICATION,
+            data=BasicAPTAuthenticationData(
+                name=name or "test",
+                credentials=BasicAPTCredentials(
+                    username=username or "test-username",
+                    password=password or "test-password",
+                ),
+            ),
+            workspace=workspace,
+            created_by=created_by,
+            created_by_work_request=created_by_work_request,
+        )
+
     @context.disable_permission_checks()
     def create_asset_usage(
         self,
@@ -1715,7 +1749,7 @@ class Playground(ArtifactPlayground):
         vendor: str = "Debian",
         mirror: str = "https://deb.debian.org",
         components: list[str] | None = None,
-        pkglist: list[str] | None = None,
+        pkglist: dict[str, str] | None = None,
         with_init: bool = True,
         with_dev: bool = True,
         create_files: bool = False,
@@ -1760,7 +1794,7 @@ class Playground(ArtifactPlayground):
                 "filename": "test",
                 "vendor": vendor,
                 "mirror": mirror,
-                "pkglist": pkglist or [],
+                "pkglist": pkglist or {},
                 "codename": codename,
                 "components": components,
                 "architecture": architecture,
@@ -1915,6 +1949,11 @@ class Playground(ArtifactPlayground):
                 binary,
                 ArtifactRelation.Relations.RELATES_TO,
             )
+        self.create_artifact_relation(
+            buildlog_artifact,
+            upload_artifacts.upload,
+            ArtifactRelation.Relations.RELATES_TO,
+        )
 
         if workflow is not None:
             work_request.add_event_reaction(
