@@ -101,7 +101,6 @@ class Scenario:
 
     def set_current(self) -> None:
         """Set the current user and workspace from the scenario."""
-        pass
 
 
 class DefaultScopeUser(Scenario):
@@ -540,6 +539,13 @@ class UIRegressionTesting(DebianWorkspace):
                     "priority": "optional",
                 },
             )
+        for arch in binaries:
+            build_log = self.playground.create_build_log_artifact(
+                source=source_name, version=version, build_arch=arch
+            )
+            self.playground.create_artifact_relation(
+                artifact=build_log, target=binaries[arch]
+            )
         return source, binaries
 
     def _add_regression_analysis(self, **analysis: RegressionAnalysis) -> None:
@@ -676,7 +682,7 @@ class UIRegressionTesting(DebianWorkspace):
         parent: WorkRequest,
         result: WorkRequest.Results = WorkRequest.Results.SUCCESS,
     ) -> Artifact:
-        """Add a Piuparts QA result artifact to collection_qa_results_sid."""
+        """Add a blhc QA result artifact to collection_qa_results_sid."""
         work_request = parent.children.get(
             task_type=TaskTypes.WORKER,
             task_name="blhc",
@@ -693,6 +699,37 @@ class UIRegressionTesting(DebianWorkspace):
         self.simulate_work_request_result(artifact, work_request, result)
         return artifact
 
+    def simulate_sbuild(self) -> None:
+        """Simulate results of sbuild workflow."""
+        workflow_sbuild = self.workflow_debian_pipeline.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+        )
+        for sbuild in workflow_sbuild.children.filter(
+            task_type=TaskTypes.WORKER, task_name="sbuild"
+        ):
+            assert sbuild.workflow_data.step is not None
+            arch = sbuild.workflow_data.step.removeprefix("build-")
+            upload = self.playground.create_upload_artifacts(
+                src_name=self.new_source.data["name"],
+                version=self.new_source.data["version"],
+                binary=True,
+                source=False,
+                binaries=[(self.new_source.data["name"], arch)],
+                workspace=sbuild.workspace,
+                work_request=sbuild,
+            )
+            build_log = self.playground.create_build_log_artifact(
+                source=self.new_source.data["name"],
+                version=self.new_source.data["version"],
+                build_arch=arch,
+                workspace=sbuild.workspace,
+                work_request=sbuild,
+            )
+            self.playground.create_artifact_relation(
+                artifact=build_log, target=upload.upload
+            )
+            self.set_work_request_result(sbuild, WorkRequest.Results.SUCCESS)
+
     def simulate_autopkgtest(self) -> None:
         """Simulate results of autopkgtest workflow."""
         # Results from a previous run
@@ -1112,6 +1149,12 @@ class UIRegressionTesting(DebianWorkspace):
             self.simulate_rev_dep_autopkgtest()
             self.simulate_lintian()
             self.simulate_piuparts()
+
+            # Simulate sbuild completion so that the blhc workflow can be
+            # populated.
+            self.simulate_sbuild()
+            orchestrate_workflow(self.workflow_debian_pipeline)
+
             self.simulate_blhc()
 
 
