@@ -9,13 +9,12 @@
 
 """Tests for the database playground."""
 
-import base64
-
 import django.test
 from django.conf import settings
 from django.contrib.auth.hashers import check_password
 from django.utils import timezone
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -72,7 +71,7 @@ from debusine.test import TestCase
 from debusine.test.playground import Playground
 
 
-class PlaygroundTest(django.test.TestCase, TestCase):
+class PlaygroundTests(django.test.TestCase, TestCase):
     """Test playground functions."""
 
     def assert_artifact_relations(
@@ -174,10 +173,10 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(pool.architectures, ["amd64"])
         self.assertEqual(pool.tags, ["a"])
         self.assertEqual(pool.provider_account, provider_account)
-        self.assertEqual(pool.specifications, specifications.dict())
+        self.assertEqual(pool.specifications, specifications.model_dump())
         self.assertFalse(pool.instance_wide)
         self.assertTrue(pool.ephemeral)
-        self.assertEqual(pool.limits, limits.dict())
+        self.assertEqual(pool.limits, limits.model_dump())
 
     def test_create_worker_pool_defaults(self) -> None:
         playground = Playground()
@@ -188,12 +187,11 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(pool.tags, [])
         self.assertIsNotNone(pool.provider_account, Asset)
         self.assertEqual(
-            pool.specifications,
-            DummyWorkerPoolSpecification().dict(),
+            pool.specifications, DummyWorkerPoolSpecification().model_dump()
         )
         self.assertTrue(pool.instance_wide)
         self.assertFalse(pool.ephemeral)
-        self.assertEqual(pool.limits, WorkerPoolLimits().dict())
+        self.assertEqual(pool.limits, WorkerPoolLimits().model_dump())
 
     def test_create_worker_pool_aws(self) -> None:
         playground = Playground()
@@ -204,7 +202,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         )
         self.assertEqual(
             pool.specifications,
-            AWSEC2WorkerPoolSpecification(launch_templates=[]).dict(),
+            AWSEC2WorkerPoolSpecification(launch_templates=[]).model_dump(),
         )
 
     def test_create_worker_pool_hetzner(self) -> None:
@@ -219,7 +217,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             HetznerCloudWorkerPoolSpecification(
                 server_type="cx22",
                 image_name="debian-13",
-            ).dict(),
+            ).model_dump(),
         )
 
     def test_create_worker_pool_unknown_provider(self) -> None:
@@ -320,7 +318,9 @@ class PlaygroundTest(django.test.TestCase, TestCase):
     def test_create_upload_artifacts(self) -> None:
         """Test creating a set of upload artifacts."""
         playground = Playground()
-        upload_artifacts = playground.create_upload_artifacts(binaries=["bin1"])
+        upload_artifacts = playground.create_upload_artifacts(
+            binaries=[("bin1", "all")]
+        )
         upload = upload_artifacts.upload
         self.assertEqual(upload.category, ArtifactCategory.UPLOAD)
         self.assertEqual(upload.workspace, playground.get_default_workspace())
@@ -397,7 +397,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         """Test creating a set of upload binary artifacts."""
         playground = Playground()
         upload_artifacts = playground.create_upload_artifacts(
-            source=False, binaries=["foo"]
+            source=False, binaries=[("foo", "all")]
         )
         upload = upload_artifacts.upload
         self.assertEqual(upload.category, ArtifactCategory.UPLOAD)
@@ -547,7 +547,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(data_model.purpose, KeyPurpose.OPENPGP_UPLOAD)
         self.assertGreater(len(data_model.fingerprint), 8)
         self.assertTrue(
-            base64.b64decode(data_model.public_key).startswith(
+            data_model.public_key.startswith(
                 b"-----BEGIN PGP PUBLIC KEY BLOCK-----"
             )
         )
@@ -573,21 +573,21 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         assert isinstance(data_model, SigningKeyData)
         self.assertEqual(data_model.purpose, KeyPurpose.UEFI)
         self.assertEqual(data_model.fingerprint, "ABCDEF")
-        self.assertEqual(base64.b64decode(data_model.public_key), b"PUBLIC KEY")
+        self.assertEqual(data_model.public_key, b"PUBLIC KEY")
         self.assertEqual(data_model.description, "Custom Asset")
 
     def test_create_cloud_provider_account_asset_defaults(self) -> None:
         playground = Playground()
         asset = playground.create_cloud_provider_account_asset()
         self.assertEqual(
-            asset.data, DummyProviderAccountData(name="test").dict()
+            asset.data, DummyProviderAccountData(name="test").model_dump()
         )
 
     def test_create_cloud_provider_account_asset_custom_data(self) -> None:
         playground = Playground()
         data = DummyProviderAccountData(name="test", secret="custom-secret")
         asset = playground.create_cloud_provider_account_asset(data=data)
-        self.assertEqual(asset.data, data.dict())
+        self.assertEqual(asset.data, data.model_dump())
 
     def test_create_cloud_provider_account_asset_aws(self) -> None:
         playground = Playground()
@@ -605,7 +605,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                     access_key_id="access-key",
                     secret_access_key="secret-key",
                 ),
-            ).dict(),
+            ).model_dump(),
         )
 
     def test_create_asset_usage_defaults(self) -> None:
@@ -683,7 +683,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 "components": ["main"],
                 'filename': 'test',
                 'mirror': 'https://deb.debian.org',
-                'pkglist': [],
+                'pkglist': {},
                 'variant': None,
                 'vendor': 'Debian',
                 "with_dev": True,
@@ -730,7 +730,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 "components": ["main"],
                 'filename': 'test',
                 'mirror': 'https://deb.debian.org',
-                'pkglist': [],
+                'pkglist': {},
                 "variant": "apt",
                 'vendor': 'Debian',
                 "with_dev": True,
@@ -784,7 +784,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 "components": ["main"],
                 'filename': 'test',
                 'mirror': 'https://deb.debian.org',
-                'pkglist': [],
+                'pkglist': {},
                 'variant': None,
                 'vendor': 'Debian',
                 "with_dev": True,
@@ -881,7 +881,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             task_data | {"task_configuration": task_config_collection.id},
         )
         self.assertEqual(
-            SbuildDynamicData.parse_obj(wr.dynamic_task_data),
+            SbuildDynamicData.model_validate(wr.dynamic_task_data),
             SbuildDynamicData(
                 environment_id=environment.pk,
                 input_source_artifact_id=source.pk,
@@ -894,6 +894,15 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 configuration_context="bookworm",
             ),
         )
+        self.assertCountEqual(
+            wr.scheduler_tags_required,
+            [
+                wtags.TASK_PREFIX + "worker:sbuild:version:1",
+                wtags.WORKER_TYPE_EXTERNAL,
+                wtags.EXECUTOR_PREFIX + "unshare",
+                wtags.CAP_PREFIX + "sbuild",
+            ],
+        )
         self.assertIsNone(wr.parent)
         self.assertEqual(wr.event_reactions_json, {})
 
@@ -955,6 +964,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             [
                 (source, ArtifactRelation.Relations.RELATES_TO),
                 (binary, ArtifactRelation.Relations.RELATES_TO),
+                (upload, ArtifactRelation.Relations.RELATES_TO),
             ],
         )
         self.assert_artifact_relations(source, [])
@@ -1033,7 +1043,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             },
         )
         self.assertEqual(
-            SbuildDynamicData.parse_obj(wr.dynamic_task_data),
+            SbuildDynamicData.model_validate(wr.dynamic_task_data),
             SbuildDynamicData(
                 environment_id=environment.pk,
                 input_source_artifact_id=source.pk,
@@ -1046,6 +1056,15 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 configuration_context="bookworm",
             ),
         )
+        self.assertCountEqual(
+            wr.scheduler_tags_required,
+            [
+                wtags.TASK_PREFIX + "worker:sbuild:version:1",
+                wtags.WORKER_TYPE_EXTERNAL,
+                wtags.EXECUTOR_PREFIX + "unshare",
+                wtags.CAP_PREFIX + "sbuild",
+            ],
+        )
         self.assertIsNone(wr.parent)
         self.assertEqual(wr.event_reactions_json, {})
 
@@ -1101,6 +1120,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             [
                 (source, ArtifactRelation.Relations.RELATES_TO),
                 (binary, ArtifactRelation.Relations.RELATES_TO),
+                (upload, ArtifactRelation.Relations.RELATES_TO),
             ],
         )
         self.assert_artifact_relations(source, [])
