@@ -111,7 +111,7 @@ class DefaultScopeUserTestBase[DSU: scenarios.DefaultScopeUser](
         self.assertEqual(self.other_scenario.user.username, "other")
 
 
-class DefaultScopeUserTest(
+class DefaultScopeUserTests(
     DefaultScopeUserTestBase[scenarios.DefaultScopeUser]
 ):
     """Test DefaultScopeUser scenario."""
@@ -122,7 +122,7 @@ class DefaultScopeUserTest(
         self.assertIsNone(context.workspace)
 
 
-class DefaultScopeUserAPITest(
+class DefaultScopeUserAPITests(
     DefaultScopeUserTestBase[scenarios.DefaultScopeUserAPI]
 ):
     """Test DefaultScopeUserAPI scenario."""
@@ -167,13 +167,11 @@ class DefaultContextTestBase[DC: scenarios.DefaultContext](
         )
 
 
-class DefaultContextTest(DefaultContextTestBase[scenarios.DefaultContext]):
+class DefaultContextTests(DefaultContextTestBase[scenarios.DefaultContext]):
     """Test DefaultContext scenario."""
 
-    pass
 
-
-class DefaultContextAPITest(
+class DefaultContextAPITests(
     DefaultContextTestBase[scenarios.DefaultContextAPI]
 ):
     """Test DefaultContextAPI scenario."""
@@ -184,7 +182,7 @@ class DefaultContextAPITest(
         self.assertEqual(self.scenario.user_token.user, self.scenario.user)
 
 
-class UIPlaygroundTest(DefaultContextTestBase[scenarios.UIPlayground]):
+class UIPlaygroundTests(DefaultContextTestBase[scenarios.UIPlayground]):
     """Test UIPlayground scenario."""
 
     def test_members(self) -> None:
