@@ -174,7 +174,7 @@ class EmitMetricDirectlyTests(TestCase):
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
-@shared_task  # type: ignore[misc]
+@shared_task  # type: ignore[untyped-decorator]
 def _emit_metric(
     metric_type: Literal["counter", "gauge", "summary", "histogram"],
     name: str,
@@ -265,19 +265,17 @@ class EmitMetricFromCeleryTaskTests(TransactionTestCase):
                 perform_ping_check=False,
                 hostname=nodename("celery", gethostname()),
             ) as celery_worker,
+            responses.RequestsMock(),
+            self.assertLogs(
+                logger=logger,
+            ) as logs,
         ):
-            with (
-                responses.RequestsMock(),
-                self.assertLogs(
-                    logger=logger,
-                ) as logs,
-            ):
-                _emit_metric.delay(
-                    metric_type="counter",
-                    name="test_counter",
-                    labels={"foo": "bar"},
-                    value=1.0,
-                ).get()
+            _emit_metric.delay(
+                metric_type="counter",
+                name="test_counter",
+                labels={"foo": "bar"},
+                value=1.0,
+            ).get()
         signals.worker_shutdown.send(sender=celery_worker)
         self.assertRegex(
             logs.output[0],
