@@ -86,7 +86,8 @@ class DebianSuiteManager(CollectionManagerInterface):
         super().__init__(collection)
         self.may_rewind_versions = False
 
-    def get_archive(self) -> Collection | None:
+    @property
+    def parent_archive(self) -> Collection | None:
         """Get the parent archive, if there is one."""
         try:
             return Collection.objects.get(
@@ -97,12 +98,23 @@ class DebianSuiteManager(CollectionManagerInterface):
         except Collection.DoesNotExist:
             return None
 
+    @property
+    def may_reuse_versions(self) -> bool:
+        """True if versions in this suite may be reused."""
+        if self.collection.data.get("may_reuse_versions", False):
+            return True
+        if (archive := self.parent_archive) is None:
+            return False
+        if archive.data.get("may_reuse_versions", False):
+            return True
+        return False
+
     def get_signing_keys(self) -> QuerySet[Asset]:
         """Return signing keys for this suite."""
         if self.collection.data.get("signing_keys") is not None:
             fingerprints = self.collection.data["signing_keys"]
         else:
-            if (archive := self.get_archive()) is not None:
+            if (archive := self.parent_archive) is not None:
                 fingerprints = archive.data.get("signing_keys", [])
             else:
                 return Asset.objects.none()
@@ -340,7 +352,7 @@ class DebianSuiteManager(CollectionManagerInterface):
                     ArtifactCategory.BINARY_PACKAGE,
                 }
             )
-            and self.collection.data.get("may_reuse_versions", False)
+            and self.may_reuse_versions
         ):
             # This deletes both suite and archive constraints.
             CollectionItemMatchConstraint.objects.filter(
