@@ -18,7 +18,7 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, LookupResultType
 from debusine.db.models import CollectionItem, default_workspace
 from debusine.server.collections import (
     DebianPackageBuildLogsManager,
@@ -519,14 +519,11 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             ),
         )
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .filter(condition),
-            [items[0]],
+            self.collection.child_items.active().filter(condition), [items[0]]
         )
         self.assertCountEqual(
             lookup_multiple(
-                LookupMultiple.parse_obj(
+                LookupMultiple.model_validate(
                     {
                         "collection": self.collection_lookup,
                         "lookup__same_work_request": subordinate_lookup,
@@ -538,7 +535,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             ),
             [
                 LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT,
+                    result_type=LookupResultType.ARTIFACT,
                     parent_collection_lookup=self.collection_lookup,
                     collection_item=items[0],
                     artifact=build_logs[0],
@@ -577,7 +574,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             )
             for build_log in build_logs
         ]
-        subordinate_lookup = LookupMultiple.parse_obj(
+        subordinate_lookup = LookupMultiple.model_validate(
             [f"{upload.id}@artifacts" for upload in uploads]
         )
 
@@ -597,15 +594,13 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             ),
         )
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .filter(condition),
+            self.collection.child_items.active().filter(condition),
             items[:2],
             ordered=False,
         )
         self.assertCountEqual(
             lookup_multiple(
-                LookupMultiple.parse_obj(
+                LookupMultiple.model_validate(
                     {
                         "collection": self.collection_lookup,
                         "lookup__same_work_request": subordinate_lookup,
@@ -617,7 +612,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             ),
             [
                 LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT,
+                    result_type=LookupResultType.ARTIFACT,
                     parent_collection_lookup=self.collection_lookup,
                     collection_item=item,
                     artifact=build_log,
