@@ -146,18 +146,55 @@ class DebianSuiteManagerTests(TestCase):
         with self.assertRaisesRegex(ValueError, msg):
             DebianSuiteManager(collection)
 
-    def test_get_archive(self) -> None:
+    def test_parent_archive(self) -> None:
         self.assertEqual(
-            self.manager.get_archive(),
+            self.manager.parent_archive,
             self.workspace.collections.get(category=CollectionCategory.ARCHIVE),
         )
 
-    def test_get_archive_none(self) -> None:
+    def test_parent_archive_none(self) -> None:
         self.workspace.collections.filter(
             category=CollectionCategory.ARCHIVE
         ).delete()
 
-        self.assertIsNone(self.manager.get_archive())
+        self.assertIsNone(self.manager.parent_archive)
+
+    def test_may_reuse_versions_suite_reuses_versions(self) -> None:
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        self.collection.data["may_reuse_versions"] = True
+        self.collection.save()
+        archive.manager.add_collection(self.collection, user=self.user)
+
+        self.assertTrue(self.manager.may_reuse_versions)
+
+    def test_may_reuse_versions_no_parent_archive(self) -> None:
+        self.workspace.collections.filter(
+            category=CollectionCategory.ARCHIVE
+        ).delete()
+
+        self.assertFalse(self.manager.may_reuse_versions)
+
+    def test_may_reuse_versions_archive_reuses_versions(self) -> None:
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        archive.data["may_reuse_versions"] = True
+        archive.save()
+        archive.manager.add_collection(self.collection, user=self.user)
+
+        self.assertTrue(self.manager.may_reuse_versions)
+
+    def test_may_reuse_versions_neither_archive_nor_suite_reuses_versions(
+        self,
+    ) -> None:
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        archive.manager.add_collection(self.collection, user=self.user)
+
+        self.assertFalse(self.manager.may_reuse_versions)
 
     def test_get_signing_keys_from_suite(self) -> None:
         """Signing keys may be configured directly on the suite."""
