@@ -22,7 +22,7 @@ from debusine.artifacts.models import (
     CollectionCategory,
     DebusinePromise,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, LookupResultType
 from debusine.db.models import (
     Artifact,
     Collection,
@@ -146,15 +146,15 @@ class LookupMixin(TestCase):
         match obj:
             case Artifact():
                 return LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT, artifact=obj
+                    result_type=LookupResultType.ARTIFACT, artifact=obj
                 )
             case Collection():
                 return LookupResult(
-                    result_type=CollectionItem.Types.COLLECTION, collection=obj
+                    result_type=LookupResultType.COLLECTION, collection=obj
                 )
             case CollectionItem():
                 return LookupResult(
-                    result_type=CollectionItem.Types(obj.child_type),
+                    result_type=LookupResultType(obj.child_type),
                     parent_collection_lookup=(
                         parent_collection_lookup
                         or f"{obj.parent_collection.id}@collections"
@@ -1317,11 +1317,10 @@ class ArtifactIdsTests(LookupMixin, TestCase):
 
         items = [
             LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+                result_type=LookupResultType.ARTIFACT, artifact=artifact
             ),
             LookupResult(
-                result_type=CollectionItem.Types.COLLECTION,
-                collection=collection,
+                result_type=LookupResultType.COLLECTION, collection=collection
             ),
         ]
         self.assertEqual(artifact_ids(items), [artifact.id])
