@@ -24,7 +24,7 @@ from debusine.artifacts.models import (
     RuntimeStatistics,
     TaskTypes,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, LookupResultType
 from debusine.db.models import Collection, CollectionItem, WorkRequest
 from debusine.db.playground import scenarios
 from debusine.server.collections.base import ItemAdditionError
@@ -65,7 +65,7 @@ class TaskHistoryManagerTests(TestCase):
         collection: Collection | None = None,
     ) -> CollectionItem:
         """Create and return a historical task run in a collection."""
-        created_at = dt.datetime(2020, 1, 1) + dt.timedelta(
+        created_at = dt.datetime(2020, 1, 1, tzinfo=dt.UTC) + dt.timedelta(
             seconds=work_request_id
         )
         return CollectionItem.objects.create_from_bare_data(
@@ -648,7 +648,7 @@ class TaskHistoryManagerTests(TestCase):
             ),
             [
                 LookupResult(
-                    result_type=CollectionItem.Types.BARE,
+                    result_type=LookupResultType.BARE,
                     parent_collection_lookup=self.collection_lookup,
                     collection_item=items[0],
                 )
@@ -709,7 +709,7 @@ class TaskHistoryManagerTests(TestCase):
             ),
             [
                 LookupResult(
-                    result_type=CollectionItem.Types.BARE,
+                    result_type=LookupResultType.BARE,
                     parent_collection_lookup=self.collection_lookup,
                     collection_item=item,
                 )
@@ -762,7 +762,7 @@ class TaskHistoryManagerTests(TestCase):
             ),
             [
                 LookupResult(
-                    result_type=CollectionItem.Types.BARE,
+                    result_type=LookupResultType.BARE,
                     parent_collection_lookup=self.collection_lookup,
                     collection_item=item,
                 )
@@ -824,7 +824,7 @@ class TaskHistoryManagerTests(TestCase):
             ),
             [
                 LookupResult(
-                    result_type=CollectionItem.Types.BARE,
+                    result_type=LookupResultType.BARE,
                     parent_collection_lookup=self.collection_lookup,
                     collection_item=item,
                 )
