@@ -9,20 +9,13 @@
 
 """Models used for file backend configuration."""
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
+import pydantic
 
 
 class FileBackendConfiguration(pydantic.BaseModel):
     """Base class for file backend configuration."""
 
-    class Config:
-        """Set up stricter pydantic Config."""
-
-        validate_assignment = True
-        extra = "forbid"
+    model_config = pydantic.ConfigDict(validate_assignment=True, extra="forbid")
 
 
 class LocalFileBackendConfiguration(FileBackendConfiguration):
