@@ -11,18 +11,14 @@
 
 import datetime as dt
 import filecmp
-import os
 import tempfile
 from collections.abc import Callable, Generator
 from contextlib import contextmanager
-from importlib.metadata import version
 from pathlib import Path
 from typing import Self, TYPE_CHECKING
 from urllib.parse import urlsplit
 
 import boto3
-from botocore.config import Config
-from packaging.version import Version
 
 from debusine.assets.models import AWSProviderAccountData, CloudProvidersType
 from debusine.db.models import File, FileStore
@@ -64,16 +60,15 @@ class S3FileBackendEntry(FileBackendEntryInterface["S3FileBackend", str]):
     @contextmanager
     def get_temporary_local_path(self) -> Generator[Path]:
         """Yield a temporary local path for the file."""
-        # TODO: Once we can assume Python >= 3.12, it would be better to use
-        # delete_on_close=False here and call temp_file.close() instead of
-        # temp_file.flush().
-        with tempfile.NamedTemporaryFile(prefix="debusine-s3-") as temp_file:
+        with tempfile.NamedTemporaryFile(
+            prefix="debusine-s3-", delete_on_close=False
+        ) as temp_file:
             self.backend.client.download_fileobj(
                 self.backend.configuration.bucket_name,
                 self._identifier,
                 temp_file,
             )
-            temp_file.flush()
+            temp_file.close()
             yield Path(temp_file.name)
 
     def get_url(self, content_type: str | None = None) -> str:
@@ -97,20 +92,15 @@ class S3FileBackendEntry(FileBackendEntryInterface["S3FileBackend", str]):
 
     def add(self, local_path: Path) -> None:
         """Add the contents of `local_path` to the underlying storage."""
-        extra_args = {}
-        # Hetzner object storage doesn't currently support data integrity
-        # protection for uploads.  See:
-        #   https://status.hetzner.com/incident/b6382a74-0fd9-4789-b997-65249187fbc7
-        #   https://github.com/boto/boto3/issues/4392
-        if not self.backend._is_hetzner:
-            extra_args["ChecksumAlgorithm"] = "SHA256"
+        extra_args = {
+            "ChecksumAlgorithm": "SHA256",
+        }
         if self.backend.configuration.storage_class is not None:
             extra_args["StorageClass"] = (
                 self.backend.configuration.storage_class
             )
         self.backend.client.upload_file(
-            # TODO: Drop os.fspath() once we can rely on boto3 >= 1.26.32.
-            os.fspath(local_path),
+            str(local_path),
             self.backend.configuration.bucket_name,
             self._identifier,
             ExtraArgs=extra_args,
@@ -166,15 +156,6 @@ class S3FileBackend(FileBackendInterface[S3FileBackendConfiguration]):
 
         This exists mainly so that it can be patched by tests.
         """
-        config: Config | None = None
-        # Hetzner object storage doesn't currently support data integrity
-        # protection for uploads, so tell boto3 not to require it.  See:
-        #   https://status.hetzner.com/incident/b6382a74-0fd9-4789-b997-65249187fbc7
-        #   https://github.com/boto/boto3/issues/4392
-        if self._is_hetzner and Version(version("boto3")) >= Version(
-            "1.36.0"
-        ):  # pragma: no cover
-            config = Config(request_checksum_calculation="when_required")
         return boto3.client(
             "s3",
             region_name=provider_account_data.configuration.region_name,
@@ -183,7 +164,6 @@ class S3FileBackend(FileBackendInterface[S3FileBackendConfiguration]):
             aws_secret_access_key=(
                 provider_account_data.credentials.secret_access_key
             ),
-            config=config,
         )
 
     def get_entry(self, fileobj: File) -> S3FileBackendEntry:
@@ -192,7 +172,7 @@ class S3FileBackend(FileBackendInterface[S3FileBackendConfiguration]):
 
     def list_entries(
         self, mtime_filter: Callable[[dt.datetime], bool] | None = None
-    ) -> Generator[S3FileBackendEntry, None, None]:
+    ) -> Generator[S3FileBackendEntry]:
         """
         Yield entries in this backend.
 
