@@ -43,7 +43,10 @@ class FakeFileBackendEntry(FileBackendEntryInterface["FakeFileBackend", bytes]):
         """Yield the path to a possibly-temporary local copy of the file."""
         raise NotImplementedError()
 
-    def get_url(self, content_type: str | None = None) -> None:  # noqa: U100
+    def get_url(
+        self,
+        content_type: str | None = None,  # noqa: ARG002, U100
+    ) -> None:
         """Return URL of entry."""
         return None
 
@@ -51,7 +54,7 @@ class FakeFileBackendEntry(FileBackendEntryInterface["FakeFileBackend", bytes]):
         """Record the addition."""
         self.backend.add_file_calls.append(local_path.read_bytes())
 
-    def same_contents(self, local_path: Path) -> bool:  # noqa: U100
+    def same_contents(self, local_path: Path) -> bool:  # noqa: ARG002, U100
         """Check whether this entry has the same contents as `local_path`."""
         return True
 
@@ -86,7 +89,9 @@ class FakeFileBackend(FileBackendInterface[FileBackendConfiguration]):
 
     def get_entry(self, fileobj: File) -> FakeFileBackendEntry:
         """Return the entry in this backend corresponding to this file."""
-        return FakeFileBackendEntry(backend=self, identifier=fileobj.sha256)
+        return FakeFileBackendEntry(
+            backend=self, identifier=bytes(fileobj.sha256)
+        )
 
     def list_entries(
         self,
@@ -212,9 +217,9 @@ class FileBackendInterfaceTests(TestCase):
         temp_file_path.write_bytes(new_contents)
 
         expected_message = (
-            fr"^add_file file size mismatch\. Path: "
-            fr"{temp_file_path} Size in disk: {len(new_contents)} "
-            fr"fileobj\.size: {len(original_contents)}$"
+            rf"^add_file file size mismatch\. Path: "
+            rf"{temp_file_path} Size in disk: {len(new_contents)} "
+            rf"fileobj\.size: {len(original_contents)}$"
         )
 
         with self.assertRaisesRegex(ValueError, expected_message):
