@@ -125,9 +125,8 @@ class MemoryFileBackendTests(TestCase):
 
         self.backend.get_entry(file).remove()
 
-        with self.assertRaises(KeyError):
-            with self.backend.get_stream(file) as fd:
-                pass  # pragma: no cover
+        with self.assertRaises(KeyError), self.backend.get_stream(file) as fd:
+            pass  # pragma: no cover
 
     def test_entry_removes_twice(self) -> None:
         """
