@@ -462,8 +462,9 @@ class S3FileBackendTests(TestCase):
         with stubber:
             self.assertCountEqual(
                 backend.list_entries(
-                    mtime_filter=lambda mtime: mtime
-                    < now - dt.timedelta(days=6)
+                    mtime_filter=lambda mtime: (
+                        mtime < now - dt.timedelta(days=6)
+                    )
                 ),
                 expected_entries[:4],
             )
