@@ -31,6 +31,6 @@ class Command(DebusineBaseCommand):
 
     def handle(self, *args: Any, **options: Any) -> None:
         """List the work requests."""
-        work_requests = WorkRequest.objects.all().order_by('created_at')
+        work_requests = WorkRequest.objects.order_by("created_at")
         WorkRequests(options["yaml"]).print(work_requests, self.stdout)
         raise SystemExit(0)
