@@ -1062,7 +1062,7 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         self.assert_dry_run_does_not_delete()
 
 
-class DeleteExpiredEphemeralGroupsTest(TestCase):
+class DeleteExpiredEphemeralGroupsTests(TestCase):
     """Tests for the DeleteExpiredEphemeralGroups class."""
 
     scenario = scenarios.DefaultScopeUser()
@@ -1257,7 +1257,7 @@ class DeleteExpiredWorkspacesTests(TestCase):
         self.assert_dry_run_does_not_delete()
 
 
-class DeleteExpiredTokensTest(TestCase):
+class DeleteExpiredTokensTests(TestCase):
     """Tests for the DeleteExpiredTokens class."""
 
     def setUp(self) -> None:
