@@ -203,7 +203,7 @@ class FileStoreCommandTests(TestCase):
             CommandError,
             r"^Cloud provider account asset 'nonexistent' not found",
         ) as exc:
-            stdout, stderr, exit_code = call_command(
+            call_command(
                 "file_store",
                 "create",
                 "--provider-account",
@@ -564,7 +564,7 @@ class FileStoreCommandTests(TestCase):
 
         with self.assertRaisesRegex(
             CommandError,
-            fr"File store {name!r} still contains 3 files; drain it first",
+            rf"File store {name!r} still contains 3 files; drain it first",
         ) as exc:
             call_command("file_store", "delete", name)
 
@@ -600,8 +600,8 @@ class FileStoreCommandTests(TestCase):
 
         with self.assertRaisesRegex(
             CommandError,
-            fr"File store {name!r} is still used by 2 scopes; use "
-            fr"'debusine-admin scope remove_file_store' first",
+            rf"File store {name!r} is still used by 2 scopes; use "
+            rf"'debusine-admin scope remove_file_store' first",
         ) as exc:
             call_command("file_store", "delete", name)
 
