@@ -78,11 +78,9 @@ class InfoTests(TestCase):
                 "socket.getfqdn",
                 return_value=fqdn,
             ),
+            override_settings(ALLOWED_HOSTS=[fqdn, "localhost", "127.0.0.1"]),
         ):
-            with override_settings(
-                ALLOWED_HOSTS=[fqdn, "localhost", "127.0.0.1"]
-            ):
-                stdout, stderr, exit_code = call_command("info")
+            stdout, stderr, exit_code = call_command("info")
 
         self.assertIn(f" * DEBUSINE_FQDN = {fqdn!r}", stdout)
         self.assertIn(
@@ -105,16 +103,18 @@ class InfoTests(TestCase):
 
     def test_info_changed_debusine_fqdn(self) -> None:
         """Info output with overridden DEBUSINE_FQDN."""
-        with mock.patch(
-            "shutil.get_terminal_size",
-            return_value=os.terminal_size((65536, 25)),
-        ):
-            with override_settings(
+        with (
+            mock.patch(
+                "shutil.get_terminal_size",
+                return_value=os.terminal_size((65536, 25)),
+            ),
+            override_settings(
                 DEBUSINE_FQDN="test.example.org",
                 # ALLOWED_HOSTS is built from DEBUSINE_FQDN
                 ALLOWED_HOSTS=["foo", "testserver"],
-            ):
-                stdout, stderr, exit_code = call_command("info")
+            ),
+        ):
+            stdout, stderr, exit_code = call_command("info")
 
         fqdn = socket.getfqdn()
         self.assertIn(" * DEBUSINE_FQDN = 'test.example.org'", stdout)
