@@ -123,13 +123,13 @@ class VacuumStorageCommandTransactionTests(
         # The last_activity_at field is the default "now",
         # but it will be deleted because the Artifact was
         # creating and still incomplete more than one day ago
-        temp_file = tempfile.NamedTemporaryFile(
+        with tempfile.NamedTemporaryFile(
             prefix="missing-file-",
             dir=settings.DEBUSINE_UPLOAD_DIRECTORY,
             # Can be deleted by FileUpload.delete via its on_commit hook.
             delete=False,
-        )
-        temp_file.close()
+        ) as temp_file:
+            pass
         file_upload = FileUpload.objects.create(
             file_in_artifact=file_in_artifact, path=Path(temp_file.name).name
         )
