@@ -26,14 +26,14 @@ class ManagementUtilsTests(TestCase):
     def test_column_to_data(self) -> None:
         """Test Column.to_data."""
         col = management_utils.Column("foo", "Foo")
-        for val in (None, 1, "str", dt.datetime.today()):
+        for val in (None, 1, "str", dt.datetime.now(tz=dt.UTC)):
             with self.subTest(val=repr(val)):
                 self.assertIs(col.to_data(val), val)
 
     def test_column_to_rich(self) -> None:
         """Test Column.to_rich."""
         col = management_utils.Column("foo", "Foo")
-        d = dt.datetime.today()
+        d = dt.datetime.now(tz=dt.UTC)
         for val, expected in (
             (None, "-"),
             (1, "1"),
