@@ -60,7 +60,7 @@ from debusine.db.models import (
     WorkflowTemplate,
     Workspace,
 )
-from debusine.tasks import BaseTask
+from debusine.db.models.tasks import DBTask
 from debusine.tasks.executors import executor_backends
 from debusine.tasks.models import WorkerType
 
@@ -380,7 +380,7 @@ class DebusineCollector(Collector):
                 backends = executor_backends()
             gauge.fill_zeros(
                 (task_type,),
-                BaseTask.task_names(task_type),
+                DBTask.task_names(task_type),
                 self._scopes(),
                 WorkRequest.Statuses,
                 architectures,
@@ -568,9 +568,7 @@ class DebusineCollector(Collector):
                 (stat["task_name"], stat["workspace__scope__name"]),
                 stat["count"],
             )
-        gauge.fill_zeros(
-            BaseTask.task_names(TaskTypes.WORKFLOW), self._scopes()
-        )
+        gauge.fill_zeros(DBTask.task_names(TaskTypes.WORKFLOW), self._scopes())
         return gauge
 
     def count_workspaces(self) -> Metric:
