@@ -282,9 +282,7 @@ class ScopeWorkerAllocator:
             ),
         )
 
-    def allocate_workers(
-        self, pool: WorkerPool, amount: int  # noqa: U100
-    ) -> int:
+    def allocate_workers(self, pool: WorkerPool, amount: int) -> int:
         """
         Allocate workers on pool for the given scope.
 
@@ -359,10 +357,7 @@ class ScopeWorkerAllocator:
             # than the number of pending tasks.
             workers[arch] = max(
                 1,
-                min(
-                    int(math.ceil(pending_seconds / TARGET_LATENCY_SECONDS)),
-                    tasks,
-                ),
+                min(math.ceil(pending_seconds / TARGET_LATENCY_SECONDS), tasks),
             )
         return workers
 
@@ -470,7 +465,7 @@ def scrub_workers() -> None:
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
-@shared_task()  # type: ignore[misc]
+@shared_task()  # type: ignore[untyped-decorator]
 @transaction.atomic
 def provision() -> None:
     """Provision or terminate workers from pools to handle pending tasks."""
@@ -497,7 +492,7 @@ def provision() -> None:
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
-@shared_task()  # type: ignore[misc]
+@shared_task()  # type: ignore[untyped-decorator]
 @transaction.atomic
 def scrub() -> None:
     """
