@@ -35,6 +35,7 @@ from django.dispatch import receiver
 from django.utils import timezone
 from django_celery_results.models import TaskResult
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import TaskTypes
 from debusine.db.models import WorkRequest, Worker
 from debusine.db.models.work_requests import SkipWorkRequest
@@ -93,6 +94,13 @@ def _assign_work_request_to_worker(
     :return: The assigned work request, or None if it could not be assigned.
     :raises AssignToWorkerError: if the work request is broken.
     """
+    # Disable tag-based scheduling by default for now, but allow to enable it
+    # with a feature flag so we can test it, before offering it for mainstream
+    # adoption
+    FEATURE_TAG_BASED_SCHEDULING = getattr(
+        settings, "FEATURE_TAG_BASED_SCHEDULING", False
+    )
+
     try:
         task = work_request.get_task(worker=worker)
     except TaskConfigError as exc:
@@ -100,7 +108,9 @@ def _assign_work_request_to_worker(
             f"Failed to configure: {exc}", "configure-failed"
         ) from exc
 
-    if not task.can_run_on(worker.metadata()):
+    if not FEATURE_TAG_BASED_SCHEDULING and not task.can_run_on(
+        worker.metadata()
+    ):
         return None
 
     try:
@@ -212,6 +222,33 @@ def _assign_work_request_to_worker_or_error(
         return None
 
 
+def get_provided_worker_tags(worker: Worker) -> set[str]:
+    """Get the set of tags provided by a worker."""
+    from debusine.server.tags import make_worker_provided_tagset
+
+    tagset = make_worker_provided_tagset()
+    tagset.add(
+        wtags.PROVENANCE_WORKER, worker.dynamic_metadata.get("provided_tags")
+    )
+    tagset.add(
+        wtags.PROVENANCE_ADMIN, worker.static_metadata.get("provided_tags")
+    )
+    tagset.finalize()
+    return tagset.tags
+
+
+def get_required_task_tags(worker: Worker) -> set[str]:
+    """Get the set of task tags required by a worker."""
+    from debusine.server.tags import make_worker_required_tagset
+
+    tagset = make_worker_required_tagset()
+    tagset.add(
+        wtags.PROVENANCE_ADMIN, worker.static_metadata.get("required_tags")
+    )
+    tagset.finalize()
+    return tagset.tags
+
+
 @transaction.atomic
 def schedule_for_worker(worker: Worker) -> WorkRequest | None:
     """
@@ -223,9 +260,18 @@ def schedule_for_worker(worker: Worker) -> WorkRequest | None:
       be found.
     :rtype: WorkRequest | None.
     """
+    # Disable tag-based scheduling by default for now, but allow to enable it
+    # with a feature flag so we can test it, before offering it for mainstream
+    # adoption
+    FEATURE_TAG_BASED_SCHEDULING = getattr(
+        settings, "FEATURE_TAG_BASED_SCHEDULING", False
+    )
+
     if not _possible_to_schedule_for_worker(worker):
         return None
 
+    provided_worker_tags = get_provided_worker_tags(worker)
+    required_task_tags = get_required_task_tags(worker)
     worker_metadata = worker.metadata()
     tasks_allowlist = worker_metadata.get("tasks_allowlist", None)
     tasks_denylist = worker_metadata.get("tasks_denylist", [])
@@ -243,6 +289,12 @@ def schedule_for_worker(worker: Worker) -> WorkRequest | None:
     elif tasks_denylist:
         qs = qs.exclude(task_name__in=tasks_denylist)
 
+    if FEATURE_TAG_BASED_SCHEDULING:
+        qs = qs.filter(
+            scheduler_tags_required__contained_by=list(provided_worker_tags),
+            scheduler_tags_provided__contains=list(required_task_tags),
+        )
+
     for work_request in qs:
         assigned_work_request = _assign_work_request_to_worker_or_error(
             work_request, worker
@@ -515,7 +567,7 @@ def schedule() -> list[WorkRequest]:
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
-@shared_task()  # type: ignore[misc]
+@shared_task()  # type: ignore[untyped-decorator]
 def schedule_task() -> None:
     """Run the scheduler as a celery task."""
     schedule()
