@@ -31,6 +31,7 @@ from debusine.assets import AssetCategory, asset_data_model
 from debusine.client.models import (
     Link,
     LookupChildType,
+    LookupResultType,
     WorkspaceInheritanceChainElement,
 )
 from debusine.db.context import context
@@ -176,6 +177,12 @@ class WorkRequestSerializer(
     event_reactions = serializers.JSONField(
         source='event_reactions_json', required=False
     )
+    scheduler_tags_provided = serializers.ListField(
+        child=serializers.CharField(), read_only=True
+    )
+    scheduler_tags_required = serializers.ListField(
+        child=serializers.CharField(), read_only=True
+    )
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         """
@@ -261,6 +268,8 @@ class WorkRequestSerializer(
             'artifacts',
             'scope',
             'workspace',
+            'scheduler_tags_provided',
+            'scheduler_tags_required',
         ]
 
 
@@ -727,7 +736,7 @@ class LookupResponseSerializer(serializers.Serializer[LookupResult]):
 
     media_type = "application/json"
 
-    result_type = serializers.ChoiceField(choices=list(CollectionItem.Types))
+    result_type = serializers.ChoiceField(choices=list(LookupResultType))
     collection_item: "serializers.PrimaryKeyRelatedField[CollectionItem]" = (
         serializers.PrimaryKeyRelatedField(read_only=True, allow_null=True)
     )
