@@ -115,18 +115,20 @@ class SignonProviders(TestCase):
         self, response_token: str, time: int = 123450
     ) -> Generator[None]:
         """Mock the remote call in OAuth2Session.fetch_token."""
-        with mock.patch(
-            "requests_oauthlib.OAuth2Session.fetch_token",
-            return_value={
-                "id_token": response_token,
-                "access_token": "accesstoken",
-            },
-        ):
-            with mock.patch(
+        with (
+            mock.patch(
+                "requests_oauthlib.OAuth2Session.fetch_token",
+                return_value={
+                    "id_token": response_token,
+                    "access_token": "accesstoken",
+                },
+            ),
+            mock.patch(
                 "time.time",
                 return_value=time,
-            ):
-                yield
+            ),
+        ):
+            yield
 
     @contextlib.contextmanager
     def mock_load_tokens(
@@ -136,9 +138,11 @@ class SignonProviders(TestCase):
         time: int = 123450,
     ) -> Generator[None]:
         """Do the mocking needed to call load_tokens."""
-        with self.mock_load_keyset(provider):
-            with self.mock_fetch_token(response_token, time):
-                yield
+        with (
+            self.mock_load_keyset(provider),
+            self.mock_fetch_token(response_token, time),
+        ):
+            yield
 
     def _make_request(
         self,
@@ -355,12 +359,14 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider)
 
-        with self.assertRaisesRegex(
-            providers.OIDCValidationError,
-            r"expected state not found in session",
+        with (
+            self.assertRaisesRegex(
+                providers.OIDCValidationError,
+                r"expected state not found in session",
+            ),
+            self.mock_load_tokens(provider, response_token),
         ):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -374,12 +380,14 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider)
 
-        with self.assertRaisesRegex(
-            providers.OIDCValidationError,
-            "options not found in session",
+        with (
+            self.assertRaisesRegex(
+                providers.OIDCValidationError,
+                "options not found in session",
+            ),
+            self.mock_load_tokens(provider, response_token),
         ):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -393,12 +401,14 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider)
 
-        with self.assertRaisesRegex(
-            providers.OIDCValidationError,
-            r"state not found in remote response",
+        with (
+            self.assertRaisesRegex(
+                providers.OIDCValidationError,
+                r"state not found in remote response",
+            ),
+            self.mock_load_tokens(provider, response_token),
         ):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -412,11 +422,13 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider)
 
-        with self.assertRaisesRegex(
-            providers.OIDCValidationError, r"Request state mismatch"
+        with (
+            self.assertRaisesRegex(
+                providers.OIDCValidationError, r"Request state mismatch"
+            ),
+            self.mock_load_tokens(provider, response_token),
         ):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -430,9 +442,11 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider)
 
-        with self.assertRaises(jwt.JWTExpired):
-            with self.mock_load_tokens(provider, response_token, time=123556):
-                bound.load_tokens()
+        with (
+            self.assertRaises(jwt.JWTExpired),
+            self.mock_load_tokens(provider, response_token, time=123556),
+        ):
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -448,9 +462,11 @@ class SignonProviders(TestCase):
         wrong_key = jwk.JWK.generate(kty='oct', size=256)
         response_token = self._make_response_token(provider, key=wrong_key)
 
-        with self.assertRaises(jwt.JWTMissingKey):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+        with (
+            self.assertRaises(jwt.JWTMissingKey),
+            self.mock_load_tokens(provider, response_token),
+        ):
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -464,11 +480,13 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider, iss="WRONG")
 
-        with self.assertRaisesRegex(
-            providers.OIDCValidationError, r"Issuer mismatch"
+        with (
+            self.assertRaisesRegex(
+                providers.OIDCValidationError, r"Issuer mismatch"
+            ),
+            self.mock_load_tokens(provider, response_token),
         ):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -482,11 +500,13 @@ class SignonProviders(TestCase):
         bound = provider.bind(request)
         response_token = self._make_response_token(provider, aud="WRONG")
 
-        with self.assertRaisesRegex(
-            providers.OIDCValidationError, r"Audience mismatch"
+        with (
+            self.assertRaisesRegex(
+                providers.OIDCValidationError, r"Audience mismatch"
+            ),
+            self.mock_load_tokens(provider, response_token),
         ):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
@@ -501,9 +521,11 @@ class SignonProviders(TestCase):
         response_token = self._make_response_token(provider)
         response_token = response_token[:4] + '0' + response_token[5:]
 
-        with self.assertRaises(jws.InvalidJWSObject):
-            with self.mock_load_tokens(provider, response_token):
-                bound.load_tokens()
+        with (
+            self.assertRaises(jws.InvalidJWSObject),
+            self.mock_load_tokens(provider, response_token),
+        ):
+            bound.load_tokens()
 
         self.assertIsNone(bound.tokens)
         self.assertIsNone(bound.id_token_claims)
