@@ -138,7 +138,7 @@ class SalsaSignonViews(TestCase):
         identity = self._make_identity(user=self.scenario.user)
 
         # Simulate a session with a logged in user
-        url = reverse("logout")
+        url = reverse("login")
         request = self._make_post_request(url)
         assert isinstance(request, RequestSignonProtocol)
         request.signon.activate_identity(identity)
@@ -155,6 +155,37 @@ class SalsaSignonViews(TestCase):
         )
         self.assertEqual(len(external_urls), 0)
 
+    def test_login_without_provider_icon(self) -> None:
+        response = self.client.get(reverse("login"))
+
+        tree = self.assertResponseHTML(response)
+        login_item = self.assertHasElement(
+            tree, "//li[@class='list-group-item list-group-item-action']"
+        )
+        self.assertFalse(login_item.xpath("img"))
+
+    @override_settings(
+        SIGNON_PROVIDERS=[
+            providers.GitlabProvider(
+                name="salsa",
+                label="Salsa",
+                icon="signon/salsa.svg",
+                url="https://salsa.debian.org",
+                client_id="clientid",
+                client_secret="clientsecret",
+            ),
+        ],
+    )
+    def test_login_with_provider_icon(self) -> None:
+        response = self.client.get(reverse("login"))
+
+        tree = self.assertResponseHTML(response)
+        icon = self.assertHasElement(
+            tree, "//li[@class='list-group-item list-group-item-action']/img"
+        )
+        self.assertEqual(icon.get("alt"), "Salsa icon")
+        self.assertEqual(icon.get("src"), "/static/signon/salsa.svg")
+
     def test_logout(self) -> None:
         """Logout also deactivates signon identities."""
         identity = self._make_identity(user=self.scenario.user)
