@@ -39,7 +39,7 @@ class TarArtifact:
 
         tar_file = self._temp_directory / f"artifact-{artifact.id}.tar.gz"
 
-        self._tar = tarfile.open(tar_file.as_posix(), mode="w|gz")
+        self._tar = tarfile.open(tar_file.as_posix(), mode="w|gz")  # noqa: SIM115
         self._tar_closed = False
 
         self._tar_file = tar_file.open("rb")
