@@ -12,14 +12,17 @@ Server-side Debusine tasks.
 
 The tasks in this package run in a Celery worker.
 """
+
 from debusine.server.tasks.base import BaseServerTask  # isort: split
 
-# Sub-tasks need to be imported in order to be available to BaseTask
-# (e.g. for BaseTask.is_valid_task_name). They are registered via
-# BaseTask.__init_subclass__.
+# Sub-tasks need to be imported in order to be available to DBTask
+# (e.g. for DBTask.is_valid_task_name). They are registered via
+# DBTask.__init_subclass__.
 from debusine.server.tasks.aptmirror import APTMirror
-from debusine.server.tasks.base import ServerTaskPermissionDenied
-from debusine.server.tasks.cloud_provisioning import CloudProvisioning
+from debusine.server.tasks.base import (
+    ServerTaskPermissionDenied,
+    analyze_celery_worker_tasks,
+)
 from debusine.server.tasks.copy_collection_items import CopyCollectionItems
 from debusine.server.tasks.create_child_workspace import CreateChildWorkspace
 from debusine.server.tasks.generate_suite_indexes import GenerateSuiteIndexes
@@ -29,11 +32,11 @@ from debusine.server.tasks.package_upload import PackageUpload
 __all__ = [
     "APTMirror",
     "BaseServerTask",
-    "CloudProvisioning",
     "CopyCollectionItems",
     "CreateChildWorkspace",
     "GenerateSuiteIndexes",
     "PackageUpload",
     "ServerNoop",
     "ServerTaskPermissionDenied",
+    "analyze_celery_worker_tasks",
 ]
