@@ -17,7 +17,7 @@ from typing import Any, override
 from django.conf import settings
 
 from debusine.artifacts.models import BareDataCategory
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, LookupResultType
 from debusine.db.models import (
     Artifact,
     Collection,
@@ -27,14 +27,10 @@ from debusine.db.models import (
     WorkRequest,
     Workspace,
 )
-from debusine.server.collections.lookup import (
-    LookupResult,
-    lookup_multiple,
-    lookup_single,
-)
+from debusine.db.models.tasks import DefaultDynamicData
+from debusine.server.collections.lookup import LookupResult
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import CopyCollectionItemsData
-from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
@@ -123,17 +119,17 @@ class CopyCollectionItems(
     ) -> None:
         """Copy a single item into a target collection."""
         match source_item.result_type:
-            case CollectionItem.Types.BARE:
+            case LookupResultType.BARE:
                 assert source_item.collection_item is not None
                 source_workspace = (
                     source_item.collection_item.parent_collection.workspace
                 )
                 reference_data = {}
-            case CollectionItem.Types.ARTIFACT:
+            case LookupResultType.ARTIFACT:
                 assert source_item.artifact is not None
                 source_workspace = source_item.artifact.workspace
                 reference_data = source_item.artifact.data
-            case CollectionItem.Types.COLLECTION:  # pragma: no cover
+            case LookupResultType.COLLECTION:  # pragma: no cover
                 raise CannotCopy("Cannot copy entire collections")
             case _ as unreachable:
                 raise AssertionError(
@@ -171,7 +167,7 @@ class CopyCollectionItems(
             item_variables = expanded_variables
 
         match source_item.result_type:
-            case CollectionItem.Types.BARE:
+            case LookupResultType.BARE:
                 assert source_item.collection_item is not None
                 self.append_to_log_file(
                     "copy-collection-items.log",
@@ -192,7 +188,7 @@ class CopyCollectionItems(
                     name=item_name,
                     replace=replace,
                 )
-            case CollectionItem.Types.ARTIFACT:
+            case LookupResultType.ARTIFACT:
                 assert source_item.artifact is not None
                 self.append_to_log_file(
                     "copy-collection-items.log",
@@ -217,30 +213,19 @@ class CopyCollectionItems(
     @override
     def _execute(self) -> WorkRequest.Results:
         """Execute the task."""
-        assert self.work_request is not None
-        assert self.workspace is not None
-
         target_collection_lookups = {
             copies.target_collection for copies in self.data.copies
         }
         target_collections = {
-            lookup: lookup_single(
-                lookup=lookup,
-                workspace=self.work_request.workspace,
-                user=self.work_request.created_by,
-                workflow_root=self.work_request.workflow_root,
-                expect_type=LookupChildType.COLLECTION,
+            lookup: self.work_request.lookup_single(
+                lookup=lookup, expect_type=LookupChildType.COLLECTION
             ).collection
             for lookup in target_collection_lookups
         }
 
         for copies in self.data.copies:
-            source_items = lookup_multiple(
-                copies.source_items,
-                self.workspace,
-                user=self.work_request.created_by,
-                workflow_root=self.work_request.workflow_root,
-                expect_type=LookupChildType.ANY,
+            source_items = self.work_request.lookup_multiple(
+                copies.source_items, expect_type=LookupChildType.ANY
             )
             for source_item in source_items:
                 try:
