@@ -22,9 +22,9 @@ from debusine.artifacts.models import (
 )
 from debusine.db.context import context
 from debusine.db.models import Collection, Group, WorkflowTemplate, Workspace
+from debusine.db.models.tasks import DefaultDynamicData
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import CreateChildWorkspaceData
-from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
@@ -48,7 +48,6 @@ class CreateChildWorkspace(
     @override
     def _execute(self) -> WorkRequestResults:
         """Execute the task."""
-        assert self.work_request is not None
         child_workspace_name = self.get_child_workspace_name()
         base_workspace = self.work_request.workspace
 
