@@ -295,13 +295,9 @@ class GenerateSuiteIndexes(
 
     TASK_VERSION = 1
 
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize the task."""
-        super().__init__(task_data, dynamic_task_data)
+    @override
+    def post_init(self) -> None:
+        super().post_init()
         self.index_info: dict[PurePath, IndexInfo] = {}
         self.indexes: dict[PurePath, Artifact] = {}
 
@@ -316,12 +312,6 @@ class GenerateSuiteIndexes(
             ).id
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        # Our dynamic data includes a collection, but no artifacts.
-        return []
-
     @cached_property
     def suite(self) -> Collection:
         """The `debian:suite` collection this task is operating on."""
@@ -372,9 +362,6 @@ class GenerateSuiteIndexes(
         content_type: str | None = None,
         newer_item: CollectionItem | None,
     ) -> Artifact:
-        assert self.work_request is not None
-        assert self.workspace is not None
-
         artifact = Artifact.objects.create_from_local_artifact(
             RepositoryIndex.create(
                 file=path,
@@ -897,8 +884,6 @@ class GenerateSuiteIndexes(
 
     def supersede_older_items(self) -> None:
         """Remove old items superseded by this task."""
-        assert self.work_request is not None
-
         previous_generation = (
             self._find_indexes()
             .filter(
