@@ -12,9 +12,9 @@
 from typing import override
 
 from debusine.artifacts.models import WorkRequestResults
+from debusine.db.models.tasks import DefaultDynamicData
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import ServerNoopData
-from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
