@@ -22,7 +22,9 @@ from debusine.server.tasks.base import ServerTaskPermissionDenied
 from debusine.server.tasks.tests.helpers import SampleBaseServerTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, WorkerType
 from debusine.test.django import TestCase
-from debusine.worker.system_information import native_architecture
+from debusine.worker.system_information import (
+    system_metadata,
+)
 
 
 class SampleBaseServerTask1(
@@ -32,7 +34,7 @@ class SampleBaseServerTask1(
 
     @override
     def _execute(self) -> WorkRequest.Results:
-        """Unused abstract method from BaseTask."""
+        """Unused abstract method from DBTask."""
         raise NotImplementedError()
 
 
@@ -51,7 +53,7 @@ class SampleBaseServerTask2(
 
     @override
     def _execute(self) -> WorkRequest.Results:
-        """Unused abstract method from BaseTask."""
+        """Unused abstract method from DBTask."""
         raise NotImplementedError()
 
 
@@ -66,9 +68,17 @@ class BaseServerTaskTests(TestCase):
     def setUpTestData(cls) -> None:
         """Create the shared attributes."""
         super().setUpTestData()
-        cls.task = SampleBaseServerTask1({})
-        cls.task2 = SampleBaseServerTask2({"foo": "bar"})
-        cls.worker_metadata = {"system:worker_type": WorkerType.CELERY}
+        task = cls.playground.create_server_task(
+            "samplebaseservertask1", {}
+        ).get_task()
+        assert isinstance(task, SampleBaseServerTask1)
+        cls.task = task
+        task2 = cls.playground.create_server_task(
+            "samplebaseservertask2", {"foo": "bar"}
+        ).get_task()
+        assert isinstance(task2, SampleBaseServerTask2)
+        cls.task2 = task2
+        cls.worker_metadata = system_metadata(WorkerType.CELERY)
 
     def test_can_run_on_no_version(self) -> None:
         """Ensure can_run_on returns True if no version is specified."""
@@ -83,29 +93,17 @@ class BaseServerTaskTests(TestCase):
         metadata["server:samplebaseservertask1:version"] = 1
         self.assertEqual(self.task.can_run_on(metadata), False)
 
-    def test_set_work_request(self) -> None:
-        """set_work_request sets appropriate attributes."""
-        work_request = self.playground.create_worker_task()
-
-        self.task.set_work_request(work_request)
-
-        self.assertEqual(self.task.work_request, work_request)
-        self.assertEqual(self.task.workspace, work_request.workspace)
-        self.assertEqual(self.task.work_request_id, work_request.id)
-        self.assertEqual(self.task.workspace_name, work_request.workspace.name)
-        self.assertEqual(
-            self.task.worker_native_architecture, native_architecture()
-        )
-
     def test_execute_sets_up_context(self) -> None:
         """`execute` sets up a suitable context."""
-        work_request = self.playground.create_worker_task()
-        self.task.set_work_request(work_request)
 
         def fake_execute() -> WorkRequest.Results:
-            self.assertEqual(context.scope, work_request.workspace.scope)
-            self.assertEqual(context.user, work_request.created_by)
-            self.assertEqual(context.workspace, work_request.workspace)
+            self.assertEqual(
+                context.scope, self.task.work_request.workspace.scope
+            )
+            self.assertEqual(context.user, self.task.work_request.created_by)
+            self.assertEqual(
+                context.workspace, self.task.work_request.workspace
+            )
             return WorkRequest.Results.SUCCESS
 
         with mock.patch.object(
@@ -126,7 +124,6 @@ class BaseServerTaskTests(TestCase):
             file.write("log")
         assert self.task._debug_log_files_directory is not None
 
-        self.task.set_work_request(self.playground.create_worker_task())
         self.task._source_artifacts_ids = [
             artifact.id for artifact in source_artifacts or []
         ]
@@ -216,9 +213,7 @@ class BaseServerTaskTests(TestCase):
 
     def test_enforce(self) -> None:
         """Test enforcing permissions."""
-        work_request = self.playground.create_worker_task()
-        work_request.set_current()
-        self.task.set_work_request(work_request)
+        self.task.work_request.set_current()
 
         workspace = self.playground.get_default_workspace()
 
@@ -228,7 +223,8 @@ class BaseServerTaskTests(TestCase):
             ServerTaskPermissionDenied,
             re.escape(
                 format_permission_check_error(
-                    workspace.can_create_work_requests, work_request.created_by
+                    workspace.can_create_work_requests,
+                    self.task.work_request.created_by,
                 )
             ),
         ):
