@@ -31,19 +31,19 @@ from debusine.db.models import (
     WorkRequest,
     Workspace,
 )
-from debusine.server.collections.lookup import LookupResult, lookup_multiple
+from debusine.db.models.tasks import DBTask
+from debusine.server.collections.lookup import LookupResult
 from debusine.server.tasks import CopyCollectionItems
 from debusine.server.tasks.copy_collection_items import CannotCopy
 from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
 )
-from debusine.tasks import BaseTask
 from debusine.tasks.models import LookupMultiple, WorkerType
 from debusine.test.django import TestCase
 
 
-def cleanup_task(task: BaseTask[Any, Any]) -> None:
+def cleanup_task(task: DBTask[Any, Any]) -> None:
     """Delete debug log files directory if it exists."""
     if task._debug_log_files_directory:
         task._debug_log_files_directory.cleanup()
@@ -146,7 +146,8 @@ class CopyCollectionItemsTests(TestCase):
                 **lookup,
             }
         )
-        task = CopyCollectionItems(
+        copy_collection_items = self.playground.create_server_task(
+            task_name="copycollectionitems",
             task_data=CopyCollectionItemsData(
                 copies=[
                     CopyCollectionItemsCopies(
@@ -154,13 +155,14 @@ class CopyCollectionItemsTests(TestCase):
                         target_collection=self.target_build_logs.id,
                     )
                 ]
-            ).model_dump(mode="json", exclude_unset=True)
+            ),
         )
+        task = copy_collection_items.get_task()
         self.addCleanup(cleanup_task, task)
-        results = lookup_multiple(
+        assert isinstance(task, CopyCollectionItems)
+        results = task.work_request.lookup_multiple(
             source_items,
-            self.source_workspace,
-            user=self.user,
+            workspace=self.source_workspace,
             expect_type=LookupChildType.ANY,
         )
         return task, results
@@ -264,15 +266,15 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact(self) -> None:
         """Copy an artifact item."""
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
             self.target_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            users=[work_request.created_by],
+            users=[sbuild_work_request.created_by],
         )
-        artifact_item = self.create_build_log(work_request=work_request)
+        artifact_item = self.create_build_log(work_request=sbuild_work_request)
         artifact = artifact_item.artifact
         assert artifact is not None
         build_log_file = artifact.files.get()
@@ -284,11 +286,11 @@ class CopyCollectionItemsTests(TestCase):
             build_log_file
         ).get()
 
-        work_request.set_current()
+        task.work_request.set_current()
         task.copy_item(source_items[0], self.target_build_logs, user=self.user)
 
         copied_collection_item = self.target_build_logs.child_items.get(
-            name=f"debian_bookworm_amd64_hello_1.0-1_{work_request.id}",
+            name=f"debian_bookworm_amd64_hello_1.0-1_{sbuild_work_request.id}",
             child_type=CollectionItem.Types.ARTIFACT,
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             collection__isnull=True,
@@ -326,10 +328,10 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_needs_unembargo(self) -> None:
         """Copying an artifact from private to public needs unembargo=True."""
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
-        self.create_build_log(work_request=work_request)
+        self.create_build_log(work_request=sbuild_work_request)
         task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
@@ -348,15 +350,15 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_with_unembargo(self) -> None:
         """Copying an artifact from private to public can unembargo it."""
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace, assign_contributor_role=True
         )
         self.playground.create_group_role(
             self.target_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            users=[work_request.created_by],
+            users=[sbuild_work_request.created_by],
         )
-        artifact_item = self.create_build_log(work_request=work_request)
+        artifact_item = self.create_build_log(work_request=sbuild_work_request)
         artifact = artifact_item.artifact
         assert artifact is not None
         task, source_items = self.prepare_copy_item_test(
@@ -366,7 +368,7 @@ class CopyCollectionItemsTests(TestCase):
         self.source_workspace.public = False
         self.source_workspace.save()
 
-        work_request.set_current()
+        task.work_request.set_current()
         task.copy_item(
             source_items[0],
             self.target_build_logs,
@@ -375,7 +377,7 @@ class CopyCollectionItemsTests(TestCase):
         )
 
         copied_collection_item = self.target_build_logs.child_items.get(
-            name=f"debian_bookworm_amd64_hello_1.0-1_{work_request.id}",
+            name=f"debian_bookworm_amd64_hello_1.0-1_{sbuild_work_request.id}",
             child_type=CollectionItem.Types.ARTIFACT,
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             collection__isnull=True,
@@ -393,13 +395,13 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_variables(self) -> None:
         """Pass additional variables while copying an artifact."""
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
             self.target_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            users=[work_request.created_by],
+            users=[sbuild_work_request.created_by],
         )
         self.create_build_log(extra_data={"nested": {"foo": "bar"}})
         task, source_items = self.prepare_copy_item_test(
@@ -412,7 +414,7 @@ class CopyCollectionItemsTests(TestCase):
             workspace=self.target_workspace,
         )
 
-        work_request.set_current()
+        task.work_request.set_current()
         task.copy_item(
             source_items[0],
             target_collection,
@@ -452,13 +454,13 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_incomplete_file(self) -> None:
         """We cannot copy artifacts with incomplete files."""
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
             self.target_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            users=[work_request.created_by],
+            users=[sbuild_work_request.created_by],
         )
         artifact_item = self.create_build_log(skip_add_files_in_store=True)
         task, source_items = self.prepare_copy_item_test(
@@ -466,7 +468,7 @@ class CopyCollectionItemsTests(TestCase):
         )
         self.assertEqual(len(source_items), 1)
 
-        work_request.set_current()
+        task.work_request.set_current()
         with self.assertRaisesRegex(
             CannotCopy,
             f"Cannot copy incomplete 'hello_1.0-1_amd64.buildlog' from "
@@ -478,13 +480,13 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_not_in_any_file_store(self) -> None:
         """We cannot copy artifacts that are not in any file store."""
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
             self.target_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            users=[work_request.created_by],
+            users=[sbuild_work_request.created_by],
         )
         artifact_item = self.create_build_log()
         task, source_items = self.prepare_copy_item_test(
@@ -493,7 +495,7 @@ class CopyCollectionItemsTests(TestCase):
         self.assertEqual(len(source_items), 1)
         self.source_scope.file_stores.get().files.clear()
 
-        work_request.set_current()
+        task.work_request.set_current()
         with self.assertRaisesRegex(
             CannotCopy,
             f"Cannot copy 'hello_1.0-1_amd64.buildlog' from artifact ID "
@@ -516,17 +518,18 @@ class CopyCollectionItemsTests(TestCase):
             configuration={"base_directory": settings.DEBUSINE_STORE_DIRECTORY},
         )
         self.source_scope.file_stores.set([file_store])
-        work_request = self.playground.create_worker_task(
+        sbuild_work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
             self.target_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            users=[work_request.created_by],
+            users=[sbuild_work_request.created_by],
         )
         build_log_contents = b"a build log\n"
         artifact_item = self.create_build_log(
-            work_request=work_request, build_log_contents=build_log_contents
+            work_request=sbuild_work_request,
+            build_log_contents=build_log_contents,
         )
         artifact = artifact_item.artifact
         assert artifact is not None
@@ -539,11 +542,11 @@ class CopyCollectionItemsTests(TestCase):
             build_log_file
         ).get()
 
-        work_request.set_current()
+        task.work_request.set_current()
         task.copy_item(source_items[0], self.target_build_logs, user=self.user)
 
         copied_collection_item = self.target_build_logs.child_items.get(
-            name=f"debian_bookworm_amd64_hello_1.0-1_{work_request.id}",
+            name=f"debian_bookworm_amd64_hello_1.0-1_{sbuild_work_request.id}",
             child_type=CollectionItem.Types.ARTIFACT,
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             collection__isnull=True,
@@ -775,8 +778,9 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_get_label(self) -> None:
         """Test get_label."""
-        task = CopyCollectionItems(
-            {
+        copy_collection_items = self.playground.create_server_task(
+            task_name="copycollectionitems",
+            task_data={
                 "copies": [
                     {
                         "source_items": {"collection": 1},
@@ -787,8 +791,9 @@ class CopyCollectionItemsTests(TestCase):
                         "target_collection": "trixie@debian:suite",
                     },
                 ]
-            }
+            },
         )
+        task = copy_collection_items.get_task()
 
         self.assertEqual(
             task.get_label(),
