@@ -18,6 +18,7 @@ from typing import Any
 
 from django_pglocks import advisory_lock
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import CollectionCategory
 from debusine.db.locks import LockError, LockType
 from debusine.db.models import (
@@ -94,6 +95,20 @@ class GenerateSuiteIndexesTests(TestCase):
         assert isinstance(task, GenerateSuiteIndexes)
         return task
 
+    def test_compute_system_required_tags(self) -> None:
+        self.create_suite_collection("bookworm")
+        suite_collection = f"bookworm@{CollectionCategory.SUITE}"
+        task = self.create_task(
+            suite_collection=suite_collection, assign_new_worker=False
+        )
+        self.assertCountEqual(
+            task.compute_system_required_tags(),
+            [
+                wtags.TASK_PREFIX + "server:generatesuiteindexes:version:1",
+                wtags.WORKER_TYPE_CELERY,
+            ],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         collection = self.create_suite_collection("bookworm")
