@@ -12,26 +12,30 @@
 import logging
 import os
 from pathlib import Path
+from typing import Any, cast
 from unittest import mock
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     DebianUpload,
     WorkRequestResults,
 )
-from debusine.db.models import Artifact, File
+from debusine.db.models import Artifact, File, Workspace
 from debusine.server.tasks import PackageUpload
 from debusine.server.tasks.models import PackageUploadDynamicData
 from debusine.server.tasks.tests.ftp_server import FakeFTPServerThread
 from debusine.server.tasks.tests.sftp_server import FakeSFTPServerThread
 from debusine.tasks.server import ArtifactInfo
-from debusine.tasks.tests.helper_mixin import FakeTaskDatabase, TaskHelperMixin
+from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
 from debusine.test.django import TestCase
 
 
-class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
+class PackageUploadTests(TestCase):
     """Tests for :py:class:`PackageUpload`."""
 
+    task: PackageUpload
+
     SAMPLE_TASK_DATA = {
         "input": {"upload": "internal@collections/name:upload-source"},
         "target": "ftp://upload.example.org/queue/",
@@ -53,6 +57,34 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
             self.task._debug_log_files_directory.cleanup()
         super().tearDown()
 
+    def configure_task(
+        self,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+    ) -> None:
+        """
+        Run self.task.configure(task_data) with different inputs.
+
+        Copy self.SAMPLE_TASK_DATA or task_data and modify it, call
+        self.task.configure(modified_SAMPLE_TASK_DATA).
+
+        :param task_data: if provided use this as a base dictionary. If None,
+          use self.SAMPLE_TASK_DATA.
+        :param override: dictionary with the keys to modify (only root keys,
+          not recursive search) by its values. Modify or add them.
+        :param remove: list of keys to remove.
+        """
+        if task_data is None:  # pragma: no cover
+            task_data = self.SAMPLE_TASK_DATA.copy()
+        if override:
+            for key, value in override.items():
+                task_data[key] = value
+
+        work_request = self.playground.create_server_task(
+            task_name="packageupload", task_data=task_data
+        )
+        self.task = cast(PackageUpload, work_request.get_task())
+
     def create_source_upload(self) -> Artifact:
         """Create a minimal source upload."""
         filenames = [
@@ -73,6 +105,15 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         )
         return artifact
 
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [
+                wtags.WORKER_TYPE_CELERY,
+                wtags.TASK_PREFIX + "server:packageupload:version:1",
+            ],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         task_db = FakeTaskDatabase(
@@ -106,7 +147,6 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
 
     def test_fetch_upload_wrong_category(self) -> None:
         """fetch_upload checks the category of the input.upload artifact."""
-        self.task.set_work_request(self.playground.create_worker_task())
         artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST
         )
@@ -130,7 +170,6 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
     def test_fetch_upload_gathers_upload_paths(self) -> None:
         """fetch_upload gathers information from the input.upload artifact."""
         debian_upload_artifact = self.create_source_upload()
-        self.task.set_work_request(self.playground.create_worker_task())
         self.task.dynamic_data = PackageUploadDynamicData(
             input_upload_id=debian_upload_artifact.id
         )
@@ -155,8 +194,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
 
     def test_execute_no_upload_paths(self) -> None:
         """execute() returns FAILURE if fetch_upload returned no paths."""
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST
@@ -179,8 +220,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         self.configure_task(
             override={"target": f"ftp://{username}@{host}:{port}/queue/"}
         )
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         directory = self.create_temporary_directory()
         deb_path = directory / "foo_1.0-1_amd64.deb"
@@ -210,8 +253,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         self.configure_task(
             override={"target": f"ftp://{username}@{host}:{port}"}
         )
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         directory = self.create_temporary_directory()
         deb_path = directory / "foo_1.0-1_amd64.deb"
@@ -247,8 +292,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
                 "delayed_days": 3,
             }
         )
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         directory = self.create_temporary_directory()
         deb_path = directory / "foo_1.0-1_amd64.deb"
@@ -275,7 +322,6 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
 
     def test_execute_ftp_retries_timeouts(self) -> None:
         """execute() retries FTP upload timeouts."""
-        self.task.set_work_request(self.playground.create_worker_task())
         directory = self.create_temporary_directory()
         changes_path = directory / "foo_1.0-1_amd64.changes"
         self.write_changes_file(changes_path, [])
@@ -325,8 +371,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         self.configure_task(
             override={"target": f"sftp://user@{host}:{port}{queue_directory}"}
         )
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         directory = self.create_temporary_directory()
         deb_path = directory / "foo_1.0-1_amd64.deb"
@@ -369,8 +417,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
                 "delayed_days": 3,
             }
         )
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         directory = self.create_temporary_directory()
         deb_path = directory / "foo_1.0-1_amd64.deb"
@@ -399,7 +449,6 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         self.configure_task(
             override={"target": "sftp://upload.example.org/queue/"}
         )
-        self.task.set_work_request(self.playground.create_worker_task())
         directory = self.create_temporary_directory()
         changes_path = directory / "foo_1.0-1_amd64.changes"
         self.write_changes_file(changes_path, [])
@@ -443,8 +492,10 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         self.configure_task(
             override={"target": f"ftp://{username}@{host}:{port}/queue/"}
         )
-        self.task.set_work_request(
-            self.playground.create_worker_task(assign_contributor_role=True)
+        self.playground.create_group_role(
+            self.task.workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[self.task.work_request.created_by],
         )
         assert self.task.workspace is not None
         debian_upload_artifact = self.create_source_upload()
