@@ -9,9 +9,12 @@
 
 """Wait task to delay until a given time is reached."""
 
+from typing import override
+
+from debusine.artifacts.models import WorkRequestResults
+from debusine.db.models.tasks import DefaultDynamicData
 from debusine.server.tasks.wait import BaseWaitTask
 from debusine.server.tasks.wait.models import DelayData
-from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
@@ -22,10 +25,12 @@ class Delay(
 
     TASK_VERSION = 1
 
-    def _execute(self) -> bool:
+    @override
+    def _execute(self) -> WorkRequestResults:
         """Never called; the scheduler completes these tasks directly."""
         raise NotImplementedError()
 
+    @override
     def get_label(self) -> str:
         """Return the task label."""
         return f"delay until {self.data.delay_until}"
