@@ -38,13 +38,6 @@ class ExternalDebsign(
             ).id,
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return [self.dynamic_data.unsigned_id]
-
     @override
     def _execute(self) -> WorkRequestResults:
         """Do nothing, successfully."""
