@@ -9,7 +9,9 @@
 
 """Models used by debusine wait tasks."""
 
-from datetime import datetime
+import datetime as dt
+from enum import StrEnum
+from typing import override
 
 from debusine.tasks.models import (
     BaseDynamicTaskData,
@@ -21,7 +23,7 @@ from debusine.tasks.models import (
 class DelayData(BaseTaskData):
     """In-memory task data for the Delay task."""
 
-    delay_until: datetime
+    delay_until: dt.datetime
 
 
 class ExternalDebsignData(BaseTaskData):
@@ -34,3 +36,24 @@ class ExternalDebsignDynamicData(BaseDynamicTaskData):
     """Dynamic data for the ExternalDebsign task."""
 
     unsigned_id: int
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [self.unsigned_id]
+
+
+class DenyAction(StrEnum):
+    """The action to take when denying a Confirm task."""
+
+    FAIL = "fail"
+    ABORT = "abort"
+    ABORT_WORKFLOW = "abort-workflow"
+
+
+class ConfirmData(BaseTaskData):
+    """In-memory task data for the Confirm task."""
+
+    auto_confirm_if_no_failure: bool = False
+    deny_action: DenyAction = DenyAction.ABORT_WORKFLOW
+    confirm_label: str = "Continue the workflow"
+    deny_label: str = "Abort the workflow"
