@@ -9,12 +9,13 @@
 
 """Unit tests for the Confirm task."""
 
+from django.core.exceptions import ValidationError
+
 from debusine.db.models import WorkRequest
 from debusine.db.playground import scenarios
 from debusine.server.tasks.wait import Confirm
 from debusine.server.tasks.wait.models import ConfirmData, DenyAction
 from debusine.server.workflows.models import WorkRequestWorkflowData
-from debusine.tasks import TaskConfigError
 from debusine.tasks.models import Confirmation, OutputData
 from debusine.test.django import TestCase
 
@@ -26,11 +27,19 @@ class ConfirmTaskTests(TestCase):
 
     def test_no_additional_properties(self) -> None:
         error_msg = "Extra inputs are not permitted"
-        with self.assertRaisesRegex(TaskConfigError, error_msg):
-            Confirm({"input": {}})
+        with self.assertRaisesRegex(ValidationError, error_msg):
+            self.playground.create_wait_task(
+                task_name="confirm",
+                task_data={"input": {}},
+                workflow_data=WorkRequestWorkflowData(needs_input=True),
+            )
 
     def test_label(self) -> None:
-        task = Confirm({})
+        task = self.playground.create_wait_task(
+            task_name="confirm",
+            task_data={},
+            workflow_data=WorkRequestWorkflowData(needs_input=True),
+        ).get_task()
         self.assertEqual(task.get_label(), "wait for confirmation")
 
     def test_respond_confirm(self) -> None:
