@@ -764,18 +764,19 @@ class ScopeWorkerAllocatorTests(TestCase):
             (8, 2),  # For the 7th task, we need another worker
             (12, 2),  # Two workers can do 12 tasks in an hour
         ):
-            with self.subTest(queue_size=queue_size):
-                with mock.patch.object(
+            with (
+                self.subTest(queue_size=queue_size),
+                mock.patch.object(
                     allocator,
                     "pending_task_count",
                     return_value={"amd64": queue_size},
-                ):
-                    # Calculations are based on the effort required to run all
-                    # but the last pending task.
-                    self.assertEqual(
-                        allocator.workers_required(),
-                        {"amd64": workers_needed},
-                    )
+                ),
+            ):
+                # Calculations are based on the effort required to run all
+                # but the last pending task.
+                self.assertEqual(
+                    allocator.workers_required(), {"amd64": workers_needed}
+                )
 
     def test_workers_required_multi_archs(self) -> None:
         worker_allocator = WorkerAllocator()
