@@ -26,6 +26,7 @@ from django.test import override_settings
 from django.utils import timezone
 from django_celery_results.models import TaskResult
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -34,7 +35,6 @@ from debusine.artifacts.models import (
 from debusine.assets import KeyPurpose
 from debusine.db.models import (
     CollectionItem,
-    TaskDatabase,
     Token,
     WorkRequest,
     Worker,
@@ -45,6 +45,7 @@ from debusine.project.celery import TaskWithBetterArgumentRepresentation
 from debusine.server.scheduler import (
     _work_request_changed,
     _worker_changed,
+    get_provided_worker_tags,
     schedule,
     schedule_for_worker,
 )
@@ -63,7 +64,6 @@ from debusine.tasks.models import (
     EventReactions,
     LintianData,
     LintianInput,
-    LookupSingle,
     OutputData,
     OutputDataError,
     SbuildBuildComponent,
@@ -71,31 +71,21 @@ from debusine.tasks.models import (
     SbuildInput,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo
-from debusine.tasks.tests.helper_mixin import FakeTaskDatabaseBase
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
+from debusine.tasks.tests.helper_mixin import MockTaskDatabase
 from debusine.test.django import (
     PlaygroundTestCase,
     TestCase,
     TransactionTestCase,
+    preserve_db_task_registry,
 )
-from debusine.test.test_utils import (
-    create_system_tarball_data,
-    preserve_task_registry,
-)
+from debusine.test.test_utils import create_system_tarball_data
+from debusine.worker.system_information import native_architecture
 
 if TYPE_CHECKING:
     from debusine.test.playground import Playground
 
 
-class FakeTaskDatabase(FakeTaskDatabaseBase, TaskDatabase):
-    """Use TaskDatabase's configure."""
-
-    def __init__(self, *, work_request: WorkRequest, **kwargs: Any):
-        """Also store a WorkRequest reference."""
-        super().__init__(**kwargs)
-        self.work_request = work_request
-
-
 class SchedulerTestCase(PlaygroundTestCase):
     """Additional methods to help test scheduler related functions."""
 
@@ -110,11 +100,13 @@ class SchedulerTestCase(PlaygroundTestCase):
         ],
     )
 
-    single_lookups: dict[
-        tuple[LookupSingle, CollectionCategory | None], ArtifactInfo
-    ] = {
-        # source_artifact
-        (10, None): ArtifactInfo(
+    field_values = {
+        "environment": ArtifactInfo(
+            id=2,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        ),
+        "input_source_artifact": ArtifactInfo(
             id=10,
             category=ArtifactCategory.SOURCE_PACKAGE,
             data=DebianSourcePackage(
@@ -124,16 +116,9 @@ class SchedulerTestCase(PlaygroundTestCase):
                 dsc_fields={"Package": "hello", "Version": "1.0-1"},
             ),
         ),
-        # environment
-        (
-            "debian/match:codename=sid:architecture=amd64:format=tarball:"
-            "backend=unshare:variant=",
-            CollectionCategory.ENVIRONMENTS,
-        ): ArtifactInfo(
-            id=2,
-            category=ArtifactCategory.SYSTEM_TARBALL,
-            data=create_system_tarball_data(),
-        ),
+        "input_extra_binary_artifacts": MultipleArtifactInfo(),
+        "extra_repositories": None,
+        "debusine_fqdn": "debusine.example.net",
     }
 
     playground: "Playground"
@@ -142,11 +127,7 @@ class SchedulerTestCase(PlaygroundTestCase):
         """Patch :py:class:`debusine.server.scheduler.TaskDatabase`."""
         task_database_patcher = mock.patch(
             "debusine.db.models.task_database.TaskDatabase",
-            side_effect=lambda work_request: FakeTaskDatabase(
-                work_request=work_request,
-                single_lookups=self.single_lookups,
-                settings={"DEBUSINE_FQDN": "debusine.example.net"},
-            ),
+            side_effect=lambda _: MockTaskDatabase(**self.field_values),
         )
         task_database_patcher.start()
         self.addCleanup(task_database_patcher.stop)
@@ -168,6 +149,7 @@ class SchedulerTestCase(PlaygroundTestCase):
                 "system:architectures": ["amd64"],
                 "sbuild:version": Sbuild.TASK_VERSION,
                 "sbuild:available": True,
+                "signing:generatekey:version": 1,
                 "executor:unshare:available": True,
             }
         )
@@ -428,7 +410,7 @@ class SchedulerTests(SchedulerTestCase, TestCase):
         self.assertEqual(work_request.status, WorkRequest.Statuses.BLOCKED)
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_wait_other_task(self) -> None:
         """schedule() leaves non-Delay Wait tasks running."""
 
@@ -716,16 +698,19 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
         work_request.save()
         self.worker.dynamic_metadata["executor:unshare:available"] = True
 
-        with self.assertLogs(
-            "debusine.server.scheduler", level=logging.WARNING
-        ) as log:
+        with (
+            mock.patch(
+                "debusine.server.scheduler.WorkRequest.assign_worker",
+                side_effect=LookupError("expected failure"),
+            ),
+            self.assertLogs(
+                "debusine.server.scheduler", level=logging.WARNING
+            ) as log,
+        ):
             result = schedule_for_worker(self.worker)
         work_request.refresh_from_db()
 
-        expected_message = (
-            f"Failed to compute dynamic data: "
-            f"('bad-lookup', {CollectionCategory.ENVIRONMENTS!r})"
-        )
+        expected_message = "Failed to compute dynamic data: expected failure"
         self.assertEqual(
             log.output,
             [
@@ -772,8 +757,13 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
             ),
             mark_pending=True,
         )
-        self.single_lookups = {
-            (1, None): ArtifactInfo(
+        self.field_values = {
+            "environment": ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            "input_source_artifact": ArtifactInfo(
                 id=1,
                 category=ArtifactCategory.SOURCE_PACKAGE,
                 data=DebianSourcePackage(
@@ -783,15 +773,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
                     dsc_fields={"Package": "hello", "Version": "1.0-1"},
                 ),
             ),
-            (
-                "debian/match:codename=bookworm:architecture=i386:"
-                "format=tarball:backend=unshare:variant=",
-                CollectionCategory.ENVIRONMENTS,
-            ): ArtifactInfo(
-                id=2,
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=create_system_tarball_data(),
-            ),
+            "input_binary_artifacts": MultipleArtifactInfo(),
         }
         task_config_collection = (
             self.scenario.default_task_configuration_collection
@@ -886,8 +868,16 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
     def test_signing_work_request(self) -> None:
         """schedule_for_worker() assigns a signing work request."""
         signing_worker = self._create_worker(
-            enabled=True, worker_type=WorkerType.SIGNING
+            enabled=True,
+            worker_type=WorkerType.SIGNING,
         )
+        signing_worker.static_metadata = {
+            "provided_tags": [
+                wtags.WORKER_TYPE_SIGNING,
+                wtags.TASK_PREFIX + "signing:generatekey:version:1",
+            ]
+        }
+        signing_worker.save()
         with override_settings(DISABLE_AUTOMATIC_SCHEDULING=True):
             work_request = self.playground.create_signing_task(
                 task_name="generatekey",
@@ -938,8 +928,13 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
         worker = Worker.objects.get_or_create_celery()
         worker.concurrency = 3
         worker.dynamic_metadata = {
+            "system:architectures": [native_architecture()],
             "system:worker_type": WorkerType.CELERY,
             "server:servernoop:version": ServerNoop.TASK_VERSION,
+            "provided_tags": [
+                wtags.WORKER_TYPE_CELERY,
+                wtags.TASK_PREFIX + "server:servernoop:version:1",
+            ],
         }
 
         for _ in range(3):
@@ -953,6 +948,22 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
         )
         self.assertIsNone(schedule_for_worker(worker))
 
+    def test_get_provided_worker_tags(self) -> None:
+        worker = Worker.objects.get_or_create_celery()
+        for worker_tags, admin_tags, expected in (
+            ([], [], set()),
+            (None, None, set()),
+            (["a", "b"], [], {"a", "b"}),
+            ([], ["a", "b"], {"a", "b"}),
+            (["a"], ["b"], {"a", "b"}),
+        ):
+            with self.subTest(
+                worker_tags=repr(worker_tags), admin_tags=repr(admin_tags)
+            ):
+                worker.dynamic_metadata = {"provided_tags": worker_tags}
+                worker.static_metadata = {"provided_tags": admin_tags}
+                self.assertEqual(get_provided_worker_tags(worker), expected)
+
     def test_work_request_changed_has_kwargs(self) -> None:
         """
         Check method _work_request_changed has kwargs argument.
@@ -970,6 +981,83 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
         self.assertTrue(_method_has_kwargs(_worker_changed))
 
 
+class TagBasedScheduleForWorkerTests(ScheduleForWorkerTests):
+    """Version of ScheduleForWorkerTests with tag based scheduling on."""
+
+    @override
+    def setUp(self) -> None:
+        super().setUp()
+        self.enterContext(override_settings(FEATURE_TAG_BASED_SCHEDULING=True))
+
+    @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
+    def test_task_required_tags_enforced(self) -> None:  # pragma: no cover
+        work_request = self.create_sample_sbuild_work_request()
+        for required, provided, accepted in (
+            ([], [], True),
+            ([], ["a"], True),
+            (["a"], [], False),
+            (["a"], ["a"], True),
+            (["a"], ["b"], False),
+            (["a", "b", "c"], ["a", "b"], False),
+            (["a", "b", "c"], ["a", "b", "c"], True),
+            (["a", "b", "c"], ["a", "b", "c", "d"], True),
+        ):
+            with (
+                self.subTest(required=repr(required), provided=repr(provided)),
+                mock.patch(
+                    "debusine.server.scheduler.get_provided_worker_tags",
+                    return_value=set(provided),
+                ),
+                mock.patch(
+                    "debusine.server.scheduler"
+                    "._assign_work_request_to_worker_or_error"
+                ) as assign_work_request,
+            ):
+                work_request.scheduler_tags_required = required
+                work_request.save()
+                schedule_for_worker(self.worker), work_request
+                if accepted:
+                    assign_work_request.assert_called_with(
+                        work_request, self.worker
+                    )
+                else:
+                    assign_work_request.assert_not_called()
+
+    @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
+    def test_worker_required_tags_enforced(self) -> None:  # pragma: no cover
+        work_request = self.create_sample_sbuild_work_request()
+        for required, provided, accepted in (
+            ([], [], True),
+            ([], ["a"], True),
+            (["a"], [], False),
+            (["a"], ["a"], True),
+            (["a"], ["b"], False),
+            (["a", "b", "c"], ["a", "b"], False),
+            (["a", "b", "c"], ["a", "b", "c"], True),
+            (["a", "b", "c"], ["a", "b", "c", "d"], True),
+        ):
+            with (
+                self.subTest(required=repr(required), provided=repr(provided)),
+                mock.patch(
+                    "debusine.server.scheduler.get_required_task_tags",
+                    return_value=set(required),
+                ),
+                mock.patch(
+                    "debusine.server.scheduler"
+                    "._assign_work_request_to_worker_or_error"
+                ) as assign_work_request,
+            ):
+                work_request.scheduler_tags_provided = provided
+                work_request.save()
+                schedule_for_worker(self.worker), work_request
+                if accepted:
+                    assign_work_request.assert_called_with(
+                        work_request, self.worker
+                    )
+                else:
+                    assign_work_request.assert_not_called()
+
+
 class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     """Test schedule_for_worker()."""
 
@@ -1150,8 +1238,14 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
         """schedule_for_worker() dispatches server tasks to Celery."""
         worker = Worker.objects.get_or_create_celery()
         worker.dynamic_metadata = {
+            "system:architectures": [native_architecture()],
             "system:worker_type": WorkerType.CELERY,
             "server:servernoop:version": ServerNoop.TASK_VERSION,
+            "provided_tags": [
+                wtags.WORKER_TYPE_CELERY,
+                wtags.TASK_PREFIX
+                + f"server:servernoop:version:{ServerNoop.TASK_VERSION}",
+            ],
         }
         work_request = self.playground.create_server_task(
             task_name="servernoop", mark_pending=True
