@@ -117,6 +117,8 @@ class WorkRequestSerializerTests(TestCase):
                 'priority_adjustment',
                 'priority_base',
                 'result',
+                'scheduler_tags_provided',
+                'scheduler_tags_required',
                 'scope',
                 'started_at',
                 'status',
@@ -290,6 +292,26 @@ class WorkRequestSerializerTests(TestCase):
                 },
             )
 
+    def test_readonly_fields(self) -> None:
+        work_request_serialized = WorkRequestSerializer(self.work_request).data
+        for field, edited_value in (
+            ("scope", "newscope"),
+            ("scheduler_tags_provided", ["changed"]),
+            ("scheduler_tags_required", ["changed"]),
+        ):
+            with self.subTest(field=field), context.local():
+                context.set_scope(self.playground.get_default_scope())
+                data = work_request_serialized.copy()
+                data[field] = edited_value
+                serializer = WorkRequestSerializer(self.work_request, data=data)
+                self.assertTrue(serializer.is_valid())
+                self.assertEqual(serializer.errors, {})
+                new_instance = serializer.save()
+                assert (
+                    WorkRequestSerializer(new_instance).data
+                    == work_request_serialized
+                )
+
 
 class WorkerRegisterSerializerTests(TestCase):
     """Test for WorkerRegisterSerializer class."""
@@ -916,12 +938,14 @@ class WorkspaceChainItemFieldTests(TestCase):
         field = WorkspaceChainItemField()
         value: Any
         for value in (None, [], 3.14, False, True, "invalid", 42):
-            with self.subTest(value=value):
-                with self.assertRaisesRegex(
+            with (
+                self.subTest(value=value),
+                self.assertRaisesRegex(
                     ValidationError,
                     r"Workspace indicator should be a dict",
-                ):
-                    field.to_internal_value(value)
+                ),
+            ):
+                field.to_internal_value(value)
 
     def test_from_empty_dict(self) -> None:
         field = WorkspaceChainItemField()
