@@ -54,11 +54,10 @@ class TarArtifactTests(TestCase):
         result = io.BytesIO(b"".join(tar_artifact))
         result.seek(0)
 
-        tar_artifact_result = tarfile.open(fileobj=result, mode="r:gz")
-
-        self.assertEqual(
-            tar_artifact_result.getnames(), ["doc/README", "doc/README2"]
-        )
+        with tarfile.open(fileobj=result, mode="r:gz") as tar_artifact_result:
+            self.assertEqual(
+                tar_artifact_result.getnames(), ["doc/README", "doc/README2"]
+            )
 
         self.assertTrue(tar_artifact._tar_file.closed)
 
@@ -89,27 +88,26 @@ class TarArtifactTests(TestCase):
 
         result.seek(0)
 
-        tar_file = tarfile.open(fileobj=result, mode="r:gz")
+        with tarfile.open(fileobj=result, mode="r:gz") as tar_file:
+            # Assert file's contents are correct
+            for path in file_paths:
+                file = tar_file.extractfile(path)
+                assert file is not None
+                self.assertEqual(file.read(), files_contents[path])
 
-        # Assert file's contents are correct
-        for path in file_paths:
-            file = tar_file.extractfile(path)
-            assert file is not None
-            self.assertEqual(file.read(), files_contents[path])
+            # Assert file names are ordered
+            self.assertEqual(
+                tar_file.getnames(),
+                list(
+                    artifact.fileinartifact_set.values_list(
+                        "path", flat=True
+                    ).order_by("path")
+                ),
+            )
 
-        # Assert file names are ordered
-        self.assertEqual(
-            tar_file.getnames(),
-            list(
-                artifact.fileinartifact_set.values_list(
-                    "path", flat=True
-                ).order_by("path")
-            ),
-        )
-
-        # Assert mtime of each file is artifact.created_at
-        for member in tar_file.getmembers():
-            self.assertEqual(member.mtime, artifact.created_at.timestamp())
+            # Assert mtime of each file is artifact.created_at
+            for member in tar_file.getmembers():
+                self.assertEqual(member.mtime, artifact.created_at.timestamp())
 
         # Assert TarArtifact deleted the temporary directory when it finished
         # iterating
@@ -131,11 +129,10 @@ class TarArtifactTests(TestCase):
         result = io.BytesIO(b"".join(tar_artifact))
         result.seek(0)
 
-        tar_artifact_result = tarfile.open(fileobj=result, mode="r:gz")
-
-        self.assertCountEqual(
-            tar_artifact_result.getnames(), ["README2", "doc/README3"]
-        )
+        with tarfile.open(fileobj=result, mode="r:gz") as tar_artifact_result:
+            self.assertCountEqual(
+                tar_artifact_result.getnames(), ["README2", "doc/README3"]
+            )
 
         self.assertTrue(tar_artifact._tar_file.closed)
 
