@@ -13,6 +13,7 @@ import binascii
 import logging
 import tempfile
 from collections.abc import Mapping
+from contextlib import ExitStack
 from pathlib import Path
 from typing import Any, IO, Literal
 
@@ -123,12 +124,12 @@ class ArtifactView(GenericAPIViewBase[Artifact], BaseAPIView):
         if created or artifact.workspace.file_needs_upload(fileobj):
             if file_size == 0:
                 # Create the file upload and add the file in the store
-                temp_empty_file = tempfile.NamedTemporaryFile(
+                with tempfile.NamedTemporaryFile(
                     prefix="file-uploading-empty-",
                     dir=settings.DEBUSINE_UPLOAD_DIRECTORY,
                     delete=False,
-                )
-                temp_empty_file.close()
+                ) as temp_empty_file:
+                    pass
                 file_upload = FileUpload.objects.create(
                     file_in_artifact=file_in_artifact,
                     path=Path(temp_empty_file.name).name,
@@ -295,54 +296,59 @@ class FileUploadParser(BaseParser):
                 f"file size ({file_size})"
             )
 
-        uploads_file: IO[bytes]
-        try:
-            file_upload = FileUpload.objects.get(
-                file_in_artifact=file_in_artifact
-            )
-            temporary_file_path = file_upload.absolute_file_path()
-            self._raise_error_if_gap_in_file(range_start, temporary_file_path)
-            uploads_file = open(temporary_file_path, "r+b")
-        except FileUpload.DoesNotExist:
-            uploads_file = tempfile.NamedTemporaryFile(
-                prefix='file-uploading-',
-                dir=settings.DEBUSINE_UPLOAD_DIRECTORY,
-                delete=False,
-                mode="a+b",
-            )
-            file_upload = FileUpload.objects.create(
-                file_in_artifact=file_in_artifact,
-                path=Path(uploads_file.name).name,
-            )
-
-        uploads_file.seek(range_start)
-
-        bytes_received = 0
-        # DATA_UPLOAD_MAX_MEMORY_SIZE: arbitrary size that should make sense
-        # that can be dealt with
-        while chunk := stream.read(settings.DATA_UPLOAD_MAX_MEMORY_SIZE):
-            bytes_received += len(chunk)
-
-            if range_start + bytes_received > file_size:
-                raise ValueError(
-                    f"Cannot process range: attempted to write after end "
-                    f"of the file. Range start: {range_start} "
-                    f"Received: {bytes_received} File size: {file_size}"
+        with ExitStack() as stack:
+            uploads_file: IO[bytes]
+            try:
+                file_upload = FileUpload.objects.get(
+                    file_in_artifact=file_in_artifact
+                )
+                temporary_file_path = file_upload.absolute_file_path()
+                self._raise_error_if_gap_in_file(
+                    range_start, temporary_file_path
+                )
+                uploads_file = stack.enter_context(
+                    open(temporary_file_path, "r+b")
+                )
+            except FileUpload.DoesNotExist:
+                uploads_file = stack.enter_context(
+                    tempfile.NamedTemporaryFile(
+                        prefix='file-uploading-',
+                        dir=settings.DEBUSINE_UPLOAD_DIRECTORY,
+                        delete=False,
+                        mode="a+b",
+                    )
+                )
+                file_upload = FileUpload.objects.create(
+                    file_in_artifact=file_in_artifact,
+                    path=Path(uploads_file.name).name,
                 )
 
-            uploads_file.write(chunk)
+            uploads_file.seek(range_start)
 
-        if (
-            range_end is not None
-            and (expected_bytes := range_end - range_start + 1)
-            != bytes_received
-        ):
-            raise ValueError(
-                f"Expected {expected_bytes} bytes (based on range header: "
-                f"end-start+1). Received: {bytes_received} bytes"
-            )
+            bytes_received = 0
+            # DATA_UPLOAD_MAX_MEMORY_SIZE: arbitrary size that should make
+            # sense that can be dealt with
+            while chunk := stream.read(settings.DATA_UPLOAD_MAX_MEMORY_SIZE):
+                bytes_received += len(chunk)
 
-        uploads_file.close()
+                if range_start + bytes_received > file_size:
+                    raise ValueError(
+                        f"Cannot process range: attempted to write after end "
+                        f"of the file. Range start: {range_start} "
+                        f"Received: {bytes_received} File size: {file_size}"
+                    )
+
+                uploads_file.write(chunk)
+
+            if (
+                range_end is not None
+                and (expected_bytes := range_end - range_start + 1)
+                != bytes_received
+            ):
+                raise ValueError(
+                    f"Expected {expected_bytes} bytes (based on range header: "
+                    f"end-start+1). Received: {bytes_received} bytes"
+                )
 
         file_upload.last_activity_at = timezone.now()
         file_upload.save()
