@@ -536,7 +536,7 @@ class ArtifactViewTests(TestCase):
     def test_create_artifact_do_not_reuse_invisible_files(self) -> None:
         """Files not in this workspace must be reuploaded."""
         other_workspace = self.playground.create_workspace(name="other")
-        other_artifact, other_files_contents = self.playground.create_artifact(
+        _, other_files_contents = self.playground.create_artifact(
             paths=["foo"], workspace=other_workspace, create_files=True
         )
         artifact_serialized = {
@@ -574,7 +574,7 @@ class ArtifactViewTests(TestCase):
     @override_settings(ALLOWED_HOSTS=["*"])
     def test_create_artifact_reuse_visible_files(self) -> None:
         """Files in this workspace do not need to be reuploaded."""
-        artifact, files_contents = self.playground.create_artifact(
+        _, files_contents = self.playground.create_artifact(
             paths=["foo"], workspace=self.workspace, create_files=True
         )
         artifact_serialized = {
@@ -1048,6 +1048,35 @@ class ArtifactViewTests(TestCase):
             status_code=status.HTTP_404_NOT_FOUND,
         )
 
+    def test_workspace_artifact_multiple_child_workspaces(self) -> None:
+        """Unrelated child workspaces don't interfere."""
+        artifact, _ = self.playground.create_artifact()
+        for i in range(2):
+            child_workspace = self.playground.create_workspace(
+                name=f"Child-{i}", public=True
+            )
+            child_workspace.set_inheritance([self.scenario.workspace])
+
+        response = self.client.get(
+            reverse(
+                "api:workspace-artifact",
+                kwargs={
+                    "workspace": self.scenario.workspace.name,
+                    "artifact_id": artifact.id,
+                },
+            ),
+            headers={
+                "Token": self.scenario.user_token.key,
+                "X-Debusine-Scope": self.scenario.scope.name,
+            },
+        )
+
+        with urlconf_scope(artifact.workspace.scope.name):
+            expected = self.artifact_as_dict_response(artifact, "testserver")
+
+        data = self.assertAPIResponseOk(response)
+        self.assertEqual(data, expected)
+
 
 class UploadFileViewTests(TestCase):
     """Tests for UploadFileView class."""
@@ -1142,11 +1171,10 @@ class UploadFileViewTests(TestCase):
             data = data[range_start : range_end + 1]
             file_size = len(data)
             parameters.update(
-                **{
-                    "HTTP_CONTENT_LENGTH": range_end + 1 - range_start,
-                    "HTTP_CONTENT_RANGE": f"bytes {range_start}-{range_end}"
-                    f"/{file_size}",
-                }
+                HTTP_CONTENT_LENGTH=range_end + 1 - range_start,
+                HTTP_CONTENT_RANGE=(
+                    f"bytes {range_start}-{range_end}/{file_size}"
+                ),
             )
 
         if token is not None:
@@ -1202,7 +1230,13 @@ class UploadFileViewTests(TestCase):
     def test_put_file_user_token(self) -> None:
         """A request with a user token can PUT a file."""
         token = self.playground.create_user_token()
+        assert token.user
 
+        self.playground.create_group_role(
+            self.playground.get_default_workspace(),
+            Workspace.Roles.CONTRIBUTOR,
+            users=[token.user],
+        )
         response = self.put_file(token=token)
 
         self.assertEqual(response.status_code, status.HTTP_201_CREATED)
@@ -1660,6 +1694,27 @@ class UploadFileViewTests(TestCase):
             status_code=status.HTTP_404_NOT_FOUND,
         )
 
+    def test_put_private_workspace_viewer(self) -> None:
+        """Artifacts in private workspaces 404 to the unauthorized."""
+        private_workspace = self.playground.create_workspace(name="Private")
+        self.artifact.workspace = private_workspace
+        self.artifact.save()
+        token = self.playground.create_user_token()
+        assert token.user is not None
+
+        self.playground.create_group_role(
+            private_workspace, Workspace.Roles.VIEWER, users=[token.user]
+        )
+        response = self.put_file(
+            token=token, scope=private_workspace.scope.name
+        )
+
+        self.assertResponseProblem(
+            response,
+            f"{token.user} cannot create artifacts in {private_workspace}",
+            status_code=status.HTTP_403_FORBIDDEN,
+        )
+
     def test_put_private_workspace_authorized(self) -> None:
         """Artifacts in private workspaces succeed for the authorized."""
         private_workspace = self.playground.create_workspace(name="Private")
@@ -1668,7 +1723,7 @@ class UploadFileViewTests(TestCase):
         token = self.playground.create_user_token()
         assert token.user is not None
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, users=[token.user]
+            private_workspace, Workspace.Roles.CONTRIBUTOR, users=[token.user]
         )
         self.assertEqual(FileInStore.objects.count(), 0)
 
