@@ -8,6 +8,7 @@
 # contained in the LICENSE file.
 """Tests for the base view code."""
 
+import json
 from typing import Any, Union
 
 from django.http import HttpRequest, HttpResponse
@@ -15,12 +16,13 @@ from django.test import Client
 from django.urls import reverse
 from rest_framework import status
 from rest_framework.request import Request
+from rest_framework.test import APIRequestFactory
 from rest_framework.views import APIView
 
 from debusine.db.context import context
 from debusine.db.models import Scope, Token, Workspace
 from debusine.server.exceptions import DebusineAPIException
-from debusine.server.views.base import BaseAPIView
+from debusine.server.views.base import BaseAPIView, SupersededAPIView
 from debusine.server.views.rest import (
     IsGet,
     IsTokenAuthenticated,
@@ -29,6 +31,7 @@ from debusine.server.views.rest import (
 )
 from debusine.test.django import (
     AllowAll,
+    BaseDjangoTestCase,
     DenyAll,
     TestCase,
     TestResponseType,
@@ -339,3 +342,75 @@ class TestWhoami(TestCase):
             self.whoami(token),
             {'auth': False, 'user': "<anonymous>", 'worker_token': False},
         )
+
+
+class TestSupersededEndpoint(BaseDjangoTestCase):
+    """Test the superseded API endpoint."""
+
+    def setUp(self) -> None:
+        """Set up common objects."""
+        super().setUp()
+        self.factory = APIRequestFactory()
+
+    def request(self, method: str = "GET", **kwargs: str) -> HttpResponse:
+        """
+        Assert that a method request to view returns an error with detail.
+
+        We're not mounted at URL, so we skip django's test client and just put
+        the pieces together ourselves.
+        """
+        match method:
+            case "GET":
+                request = self.factory.get("/some-api")
+            case "POST":
+                request = self.factory.post("/some-api", {})
+            case "PATCH":
+                request = self.factory.patch("/some-api", {})
+            case "DELETE":
+                request = self.factory.delete("/some-api")
+            case "OPTIONS":
+                request = self.factory.options("/some-api")
+            case _:  # pragma: no cover
+                raise NotImplementedError()
+        request._debusine_token = None  # type: ignore[attr-defined]
+        response = SupersededAPIView.as_view()(request, **kwargs)
+        response.json = lambda: json.loads(response.content)  # type: ignore[attr-defined]
+        return response
+
+    def test_unconfigured(self) -> None:
+        detail = r"This endpoint is no longer supported\."
+        for method in ("GET", "POST", "PATCH", "DELETE"):
+            with self.subTest(method=method):
+                response = self.request(method=method)
+                self.assertResponseProblem(
+                    response,
+                    title="Superseded API Endpoint",
+                    detail_pattern=detail,
+                )
+
+    def test_options(self) -> None:
+        response = self.request(method="OPTIONS")
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+
+    def test_with_replacement(self) -> None:
+        response = self.request(new_endpoint="/new/")
+        self.assertResponseProblem(
+            response,
+            title="Superseded API Endpoint",
+            detail_pattern=(
+                r"This endpoint is no longer supported\. "
+                r"It has been replaced by /new/\."
+            ),
+        )
+
+    def test_with_replacement_and_version(self) -> None:
+        response = self.request(new_endpoint="/new/", client_version="1.0")
+        self.assertResponseProblem(
+            response,
+            title="Superseded API Endpoint",
+            detail_pattern=(
+                r"This endpoint is no longer supported\. "
+                r"It has been replaced by /new/, which is supported by "
+                r"debusine-client \(>= 1\.0\)\. Please upgrade your client\."
+            ),
+        )
