@@ -9,7 +9,6 @@
 
 """Tests for the workflow views."""
 
-import copy
 from typing import Any, ClassVar, assert_never
 
 from django.conf import settings
@@ -26,7 +25,6 @@ from debusine.client.models import (
     StrictBaseModel,
     WorkflowTemplateData,
     WorkflowTemplateDataNew,
-    model_to_json_serializable_dict,
 )
 from debusine.db.models import (
     Scope,
@@ -70,7 +68,7 @@ class WorkflowTemplateViewSetTests(TestCase):
             static_parameters=cls.sample.static_parameters,
             runtime_parameters=cls.sample.runtime_parameters,
         )
-        cls.sample_client_with_links = cls.sample_client.copy()
+        cls.sample_client_with_links = cls.sample_client.model_copy()
         cls.sample_client_with_links.links = cls.get_links(cls.sample)
 
     @classmethod
@@ -104,7 +102,7 @@ class WorkflowTemplateViewSetTests(TestCase):
             case dict():
                 return data
             case StrictBaseModel():
-                return model_to_json_serializable_dict(data)
+                return data.model_dump(mode="json")
             case None:
                 return {}
             case _ as unreachable:
@@ -149,7 +147,6 @@ class WorkflowTemplateViewSetTests(TestCase):
                 return {"token": self.scenario.user_token.key}
             case False:
                 return {}
-                pass
             case _ as unreachable:
                 assert_never(unreachable)
 
@@ -404,7 +401,7 @@ class WorkflowTemplateViewSetTests(TestCase):
             )
             self.assertResponseProblem(
                 response,
-                ("playground cannot configure workspace" " debusine/System"),
+                "playground cannot configure workspace debusine/System",
                 status_code=status.HTTP_403_FORBIDDEN,
             )
 
@@ -675,11 +672,11 @@ class WorkflowTemplateViewSetTests(TestCase):
     def test_patch_rename(self) -> None:
         response = self.patch(data={"name": "renamed"})
         new_data = self.assertAPIResponseOk(response)
-        changed = self.sample_client.dict()
+        changed = self.sample_client.model_dump()
         changed["name"] = "renamed"
         self.sample.refresh_from_db()
         changed["links"] = self.get_links(self.sample)
-        assert WorkflowTemplateData(**new_data) == changed
+        assert WorkflowTemplateData(**new_data).model_dump() == changed
         self.assertEqual(self.sample.name, "renamed")
         self.assertEqual(self.sample.task_name, "noop")
         self.assertEqual(self.sample.priority, 0)
@@ -801,7 +798,7 @@ class WorkflowTemplateViewSetTests(TestCase):
 
         response = self.patch(data={"name": "renamed"})
         new_data = self.assertAPIResponseOk(response)
-        changed = self.sample_client.copy()
+        changed = self.sample_client.model_copy()
         changed.name = "renamed"
         self.sample.refresh_from_db()
         changed.links = self.get_links(self.sample)
@@ -849,7 +846,7 @@ class WorkflowTemplateViewSetTests(TestCase):
     @override_permission(WorkflowTemplate, "can_configure", AllowAll)
     def test_debusine_workflow_template_update(self) -> None:
         debusine = self.get_debusine()
-        changed = copy.copy(self.sample_client)
+        changed = self.sample_client.model_copy()
         changed.name = "renamed"
         changed.priority = -2
         changed.static_parameters = {"changed": True}
@@ -858,7 +855,7 @@ class WorkflowTemplateViewSetTests(TestCase):
             self.scenario.workspace.name, changed
         )
         self.sample.refresh_from_db()
-        changed_with_links = changed.copy()
+        changed_with_links = changed.model_copy()
         changed_with_links.links = self.get_links(self.sample)
         self.assertEqual(result, changed_with_links)
         self.assertEqual(self.sample.name, "renamed")
@@ -1024,7 +1021,7 @@ class WorkflowViewTests(TestCase):
             response,
             "Cannot create workflow",
             detail_pattern=(
-                r"extra fields not permitted \(type=value_error\.extra\)"
+                r"Extra inputs are not permitted.*type=extra_forbidden"
             ),
         )
 
