@@ -14,6 +14,10 @@ from typing import Any, assert_never
 from django.urls import reverse
 from rest_framework import status
 
+from debusine.client.models import (
+    WorkspaceInheritanceChain,
+    WorkspaceInheritanceChainElement,
+)
 from debusine.db.models import Workspace
 from debusine.db.models.workspaces import WorkspaceChain
 from debusine.db.playground import scenarios
@@ -50,7 +54,6 @@ class WorkspaceInheritanceViewTests(TestCase):
                 headers = {"token": self.scenario.user_token.key}
             case False:
                 headers = {}
-                pass
             case _ as unreachable:
                 assert_never(unreachable)
 
@@ -83,7 +86,6 @@ class WorkspaceInheritanceViewTests(TestCase):
                 headers = {"token": self.scenario.user_token.key}
             case False:
                 headers = {}
-                pass
             case _ as unreachable:
                 assert_never(unreachable)
 
@@ -156,7 +158,10 @@ class WorkspaceInheritanceViewTests(TestCase):
     def test_get_empty_chain(self) -> None:
         response = self.get()
         data = self.assertAPIResponseOk(response)
-        self.assertEqual(data, {"chain": []})
+        self.assertEqual(
+            WorkspaceInheritanceChain(**data),
+            WorkspaceInheritanceChain(chain=[]),
+        )
 
     def test_get_chain(self) -> None:
         first = self.playground.create_workspace(name="first")
@@ -165,17 +170,19 @@ class WorkspaceInheritanceViewTests(TestCase):
         response = self.get()
         data = self.assertAPIResponseOk(response)
         self.assertEqual(
-            data,
-            {
-                "chain": [
-                    {"id": first.pk, "scope": "debusine", "workspace": "first"},
-                    {
-                        "id": second.pk,
-                        "scope": "debusine",
-                        "workspace": "second",
-                    },
+            WorkspaceInheritanceChain(**data),
+            WorkspaceInheritanceChain(
+                chain=[
+                    WorkspaceInheritanceChainElement(
+                        id=first.pk, scope="debusine", workspace="first"
+                    ),
+                    WorkspaceInheritanceChainElement(
+                        id=second.pk,
+                        scope="debusine",
+                        workspace="second",
+                    ),
                 ]
-            },
+            ),
         )
 
         # Try again after swapping order, to see if order is preserved
@@ -187,17 +194,19 @@ class WorkspaceInheritanceViewTests(TestCase):
         response = self.get()
         data = self.assertAPIResponseOk(response)
         self.assertEqual(
-            data,
-            {
-                "chain": [
-                    {
-                        "id": second.pk,
-                        "scope": "debusine",
-                        "workspace": "second",
-                    },
-                    {"id": first.pk, "scope": "debusine", "workspace": "first"},
+            WorkspaceInheritanceChain(**data),
+            WorkspaceInheritanceChain(
+                chain=[
+                    WorkspaceInheritanceChainElement(
+                        id=second.pk,
+                        scope="debusine",
+                        workspace="second",
+                    ),
+                    WorkspaceInheritanceChainElement(
+                        id=first.pk, scope="debusine", workspace="first"
+                    ),
                 ]
-            },
+            ),
         )
 
     @override_permission(Workspace, "can_configure", AllowAll)
@@ -214,17 +223,19 @@ class WorkspaceInheritanceViewTests(TestCase):
         )
         data = self.assertAPIResponseOk(response)
         self.assertEqual(
-            data,
-            {
-                "chain": [
-                    {"id": first.pk, "scope": "debusine", "workspace": "first"},
-                    {
-                        "id": second.pk,
-                        "scope": "debusine",
-                        "workspace": "second",
-                    },
+            WorkspaceInheritanceChain(**data),
+            WorkspaceInheritanceChain(
+                chain=[
+                    WorkspaceInheritanceChainElement(
+                        id=first.pk, scope="debusine", workspace="first"
+                    ),
+                    WorkspaceInheritanceChainElement(
+                        id=second.pk,
+                        scope="debusine",
+                        workspace="second",
+                    ),
                 ]
-            },
+            ),
         )
 
     @override_permission(Workspace, "can_display", AllowAll)
