@@ -37,6 +37,7 @@ from debusine.db.models import (
     TaskDatabase,
     WorkRequest,
 )
+from debusine.db.models.tasks import DBTask
 from debusine.db.models.work_requests import (
     CannotAbort,
     CannotRetry,
@@ -68,7 +69,7 @@ from debusine.server.views.rest import (
     IsTokenUserAuthenticated,
     IsWorkerAuthenticated,
 )
-from debusine.tasks import BaseTask, TaskConfigError
+from debusine.tasks import TaskConfigError
 from debusine.tasks.models import ActionTypes
 
 logger = logging.getLogger(__name__)
@@ -240,12 +241,12 @@ class WorkRequestView(
             task_types.append(TaskTypes.SERVER)
 
         for task_type in task_types:
-            if BaseTask.is_valid_task_name(task_type, task_name):
+            if DBTask.is_valid_task_name(task_type, task_name):
                 return task_type
 
         valid_task_names = ', '.join(
             itertools.chain(
-                *(BaseTask.task_names(task_type) for task_type in task_types),
+                *(DBTask.task_names(task_type) for task_type in task_types),
             ),
         )
         logger.debug(
@@ -560,9 +561,7 @@ class WorkRequestExternalDebsignView(
                 detail="Signed upload does not change the .changes file"
             )
         if not all(
-            path.endswith(".changes")
-            or path.endswith(".dsc")
-            or path.endswith(".buildinfo")
+            path.endswith((".changes", ".dsc", ".buildinfo"))
             for path in changed_paths
         ):
             raise ValidationError(
