@@ -311,7 +311,7 @@ class AWSEC2WorkerPool(
             )
         else:
             # mypy_boto3 types this as a very long set of Literal options
-            data["InstanceType"] = launch_template.InstanceType  # type: ignore
+            data["InstanceType"] = launch_template.InstanceType  # type: ignore[typeddict-item]
 
         if launch_template.KeyName:
             data["KeyName"] = launch_template.KeyName
