@@ -76,6 +76,8 @@ class HetznerCloudWorkerPool(
         self._terminate_existing_instances(worker.worker_pool_data)
         worker.worker_pool_data = {}
         worker.instance_created_at = None
+        worker.token = None
+        worker.activation_token = None
         worker.save()
 
     @override
@@ -110,7 +112,7 @@ class HetznerCloudWorkerPool(
                 worker.name,
                 instance_id,
             )
-            self.terminate_worker(worker)
+            self.worker_pool.terminate_worker(worker)
 
     def _make_client(
         self, provider_account_data: HetznerProviderAccountData
