@@ -10,6 +10,7 @@
 """Autopkgtest workflow."""
 
 import datetime as dt
+from collections.abc import Callable
 from functools import cached_property
 from typing import Any, TypeAlias, override
 
@@ -20,14 +21,10 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     DebianAutopkgtest,
     DebianAutopkgtestResultStatus,
-    TaskTypes,
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, WorkRequest
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-)
+from debusine.server.collections.lookup import artifact_ids
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.base import WorkflowValidationError
 from debusine.server.workflows.models import (
@@ -64,16 +61,13 @@ class AutopkgtestWorkflow(
 
     @override
     def post_init(self) -> None:
-        """Workflow initialization."""
+        super().post_init()
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
     @cached_property
     def architectures(self) -> set[str]:
         """Concrete architectures to run tests on."""
-        assert self.work_request is not None
-        assert self.workspace is not None
-
         try:
             architectures = workflow_utils.get_lookup_architectures(
                 self, self.data.binary_artifacts
@@ -138,8 +132,6 @@ class AutopkgtestWorkflow(
 
     def _populate_single(self, architecture: str) -> None:
         """Create an autopkgtest work request for a single architecture."""
-        assert self.work_request is not None
-
         if (
             self.data.update_qa_results
             and self._has_current_reference_qa_result(architecture)
@@ -250,7 +242,7 @@ class AutopkgtestWorkflow(
                 task_name="workflow",
                 workflow_data=WorkRequestWorkflowData(
                     allow_dependency_failures=True,
-                    step="regression-analysis",
+                    step=f"regression-analysis-{architecture}",
                     display_name=f"Regression analysis for {architecture}",
                     visible=False,
                 ),
@@ -269,26 +261,8 @@ class AutopkgtestWorkflow(
     @override
     def populate(self) -> None:
         """Create autopkgtest work requests for all architectures."""
-        assert self.work_request is not None
-
-        children = self.work_request.children.filter(
-            task_type=TaskTypes.WORKER, task_name="autopkgtest"
-        )
-        existing_architectures = {
-            child.task_data["build_architecture"] for child in children
-        }
-
-        # We don't expect there to be a scenario where there are existing
-        # work requests that we no longer need.  Leave an assertion so that
-        # if this happens we can work out what to do in that corner case.
-        if old_architectures := existing_architectures - self.architectures:
-            raise AssertionError(
-                f"Unexpected work requests found: {old_architectures}"
-            )
-
-        if new_architectures := self.architectures - existing_architectures:
-            for architecture in sorted(new_architectures):
-                self._populate_single(architecture)
+        for architecture in sorted(self.architectures):
+            self._populate_single(architecture)
 
     def _extract_artifact_details(
         self, artifact: Artifact | None
@@ -390,6 +364,14 @@ class AutopkgtestWorkflow(
         self.work_request.save()
         return True
 
+    @override
+    def get_callback(self, step: str) -> Callable[[], bool]:
+        if not step.startswith("regression-analysis"):
+            raise NotImplementedError(
+                f"Unhandled workflow callback step: {step}"
+            )
+        return self.callback_regression_analysis
+
     @override
     def build_dynamic_data(
         self,
@@ -407,18 +389,12 @@ class AutopkgtestWorkflow(
         source_artifact = workflow_utils.source_package(self)
         source_data = SourcePackage.create_data(source_artifact.data)
 
-        binary_artifacts_results = lookup_multiple(
+        binary_artifacts_results = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
-        context_artifacts_results = lookup_multiple(
+        context_artifacts_results = self.work_request.lookup_multiple(
             self.data.context_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
