@@ -17,10 +17,7 @@ import logging
 from abc import ABCMeta, abstractmethod
 from collections.abc import Callable
 from functools import partial
-from typing import Any, Literal, TypedDict, cast, override
-
-from django.contrib.auth.models import AnonymousUser
-from django.db.models import Q
+from typing import Any, Literal, cast, override
 
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -33,7 +30,6 @@ from debusine.db.models import (
     Collection,
     CollectionItem,
     TaskDatabase,
-    User,
     WorkRequest,
     Workspace,
 )
@@ -41,7 +37,6 @@ from debusine.db.models.work_requests import (
     InternalTaskError,
     StatusChangeError,
 )
-from debusine.server.collections.lookup import lookup_multiple, lookup_single
 from debusine.server.tasks.base import BaseServerTask
 from debusine.server.workflows.models import (
     BaseWorkflowData,
@@ -91,26 +86,6 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
 
     TASK_TYPE = TaskTypes.WORKFLOW
 
-    work_request: WorkRequest
-    workspace: Workspace
-
-    @override
-    def __init__(
-        self,
-        *,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize Workflow."""
-        super().__init__(
-            task_data=task_data, dynamic_task_data=dynamic_task_data
-        )
-        self.post_init()
-
-    def post_init(self) -> None:
-        """Specific post-init code."""
-        pass
-
     @classmethod
     def from_name(cls, name: str) -> type['Workflow[Any, Any]']:
         """Instantiate a workflow by name."""
@@ -185,6 +160,27 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         :py:class:`WorkRequest` structure as calling it once.
         """
 
+    def get_callback(self, step: str) -> Callable[[], bool]:
+        """
+        Get the callback implementation for a given workflow step name.
+
+        By default, subclasses may provide ``callback_{step.replace('-',
+        '_')}`` methods to implement workflow callbacks with the
+        corresponding ``step`` set in their workflow data.  Alternatively,
+        they may override this method to provide a more complex mapping from
+        work request to callback implementation.
+        """
+        callback_method = getattr(
+            self, f"callback_{step.replace('-', '_')}", None
+        )
+        if callback_method is None:
+            raise NotImplementedError(
+                f"Unhandled workflow callback step: {step}"
+            )
+        assert callable(callback_method)
+        # https://github.com/python/mypy/issues/20748
+        return callback_method  # type: ignore[no-any-return]
+
     def callback(self, work_request: "WorkRequest") -> bool:
         """
         Perform an orchestration step.
@@ -202,7 +198,7 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
 
         Subclasses may provide ``callback_{step.replace('-', '_')}`` methods
         to implement workflow callbacks with the corresponding ``step`` set
-        in their workflow data.
+        in their workflow data, or override ``get_callback``.
 
         :return: True if the callback succeeded, otherwise False.
         """
@@ -211,14 +207,7 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
             raise NotImplementedError(
                 "Workflow callback called without setting `step`"
             )
-        callback_method = getattr(
-            self, f"callback_{step.replace('-', '_')}", None
-        )
-        if callback_method is None:
-            raise NotImplementedError(
-                f"Unhandled workflow callback step: {step}"
-            )
-        succeeded = callback_method()
+        succeeded = self.get_callback(step)()
         assert isinstance(succeeded, bool)
         return succeeded
 
@@ -288,11 +277,8 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         assert work_request.parent is not None
 
         try:
-            lookup_single(
+            work_request.lookup_single(
                 f"internal@collections/name:{name}",
-                work_request.workspace,
-                user=work_request.created_by or AnonymousUser(),
-                workflow_root=work_request.workflow_root,
                 expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
             )
         except KeyError:
@@ -335,26 +321,16 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         :param lookup: resolve the lookup and iterate over the results
           (for PROMISES only)
         """
-
-        class LookupKwargs(TypedDict):
-            """Dictionary containing keyword arguments for lookup functions."""
-
-            workspace: Workspace
-            user: User | AnonymousUser
-            workflow_root: WorkRequest | None
-            expect_type: LookupChildType
-
-        lookup_kwargs: LookupKwargs = {
-            "workspace": work_request.workspace,
-            "user": work_request.created_by or AnonymousUser(),
-            "workflow_root": work_request.workflow_root,
-            "expect_type": LookupChildType.ARTIFACT_OR_PROMISE,
-        }
-
         if isinstance(lookup, LookupMultiple):
-            results = lookup_multiple(lookup, **lookup_kwargs)
+            results = work_request.lookup_multiple(
+                lookup, expect_type=LookupChildType.ARTIFACT_OR_PROMISE
+            )
         else:
-            results = [lookup_single(lookup, **lookup_kwargs)]
+            results = [
+                work_request.lookup_single(
+                    lookup, expect_type=LookupChildType.ARTIFACT_OR_PROMISE
+                )
+            ]
 
         for result in results:
             collection_item = result.collection_item
@@ -380,6 +356,10 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         """
         Create a child WORKER work request if one does not already exist.
 
+        This assumes that ``workflow_data.step`` is unique among work
+        request children with this task type and name, and that it is stable
+        even if the workflow is orchestrated multiple times.
+
         :param task_name: the task name for the child work request.
         :param task_data: the task data for the child work request.
         :param workflow_data: the workflow data for the child work request.
@@ -390,9 +370,9 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         """
         try:
             return self.work_request.children.get(
-                task_name=task_name,
-                workflow_data_json=workflow_data.model_dump(exclude_unset=True),
                 task_type=TaskTypes.WORKER,
+                task_name=task_name,
+                workflow_data_json__step=workflow_data.step,
             )
         except WorkRequest.DoesNotExist:
             return self.work_request.create_child_worker(
@@ -407,33 +387,25 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         *,
         task_name: str,
         task_data: BaseTaskData | None = None,
-        task_data_filter: Q | None = None,
         workflow_data: WorkRequestWorkflowData,
     ) -> WorkRequest:
         """
-        Create a child work request if one does not already exist.
+        Create a child SERVER work request if one does not already exist.
+
+        This assumes that ``workflow_data.step`` is unique among work
+        request children with this task type and name, and that it is stable
+        even if the workflow is orchestrated multiple times.
 
         :param task_name: the task name for the child work request.
         :param task_data: the task data for the child work request.
-        :param task_data_filter: if given, use these conditions to test
-          whether an existing work request has matching task data; by
-          default, look for work requests that exactly match the given
-          ``task_data``.
         :param workflow_data: the workflow data for the child work request.
         :return: new or existing :py:class:`WorkRequest`.
         """
-        if task_data_filter is None:
-            task_data_filter = Q(
-                task_data=(task_data or BaseTaskData()).model_dump(
-                    mode="json", exclude_unset=True
-                )
-            )
         try:
             return self.work_request.children.get(
-                task_data_filter,
-                task_name=task_name,
-                workflow_data_json=workflow_data.model_dump(exclude_unset=True),
                 task_type=TaskTypes.SERVER,
+                task_name=task_name,
+                workflow_data_json__step=workflow_data.step,
             )
         except WorkRequest.DoesNotExist:
             return self.work_request.create_child_server(
@@ -450,7 +422,11 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         relative_priority: int = 0,
     ) -> WorkRequest:
         """
-        Create a child work request if one does not already exist.
+        Create a child INTERNAL work request if one does not already exist.
+
+        This assumes that ``workflow_data.step`` is unique among work
+        request children with this task type and name, and that it is stable
+        even if the workflow is orchestrated multiple times.
 
         :param task_name: the task name for the child work request.
         :param task_data: the task data for the child work request.
@@ -462,9 +438,9 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         """
         try:
             return self.work_request.children.get(
-                task_name=task_name,
-                workflow_data_json=workflow_data.model_dump(exclude_unset=True),
                 task_type=TaskTypes.INTERNAL,
+                task_name=task_name,
+                workflow_data_json__step=workflow_data.step,
             )
         except WorkRequest.DoesNotExist:
             return self.work_request.create_child_internal(
@@ -478,17 +454,18 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         *,
         task_name: str,
         task_data: BaseTaskData | None = None,
-        task_data_filter: Q | None = None,
         workflow_data: WorkRequestWorkflowData,
         relative_priority: int = 0,
     ) -> WorkRequest:
         """
-        Create a child work request if one does not already exist.
+        Create a child WORKFLOW work request if one does not already exist.
+
+        This assumes that ``workflow_data.step`` is unique among work
+        request children with this task type and name, and that it is stable
+        even if the workflow is orchestrated multiple times.
 
         :param task_name: the task name for the child work request.
         :param task_data: the task data for the child work request.
-        :param task_data_filter: if given, use these conditions to test
-          whether an existing work request has matching task data.
         :param workflow_data: the workflow data for the child work request.
         :param relative_priority: if creating a new work request, set the
           base priority of the child to the effective priority of this
@@ -497,10 +474,9 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         """
         try:
             return self.work_request.children.get(
-                task_data_filter or Q(),
-                task_name=task_name,
-                workflow_data_json=workflow_data.model_dump(exclude_unset=True),
                 task_type=TaskTypes.WORKFLOW,
+                task_name=task_name,
+                workflow_data_json__step=workflow_data.step,
             )
         except WorkRequest.DoesNotExist:
             return self.work_request.create_child_workflow(
@@ -518,7 +494,11 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         workflow_data: WorkRequestWorkflowData,
     ) -> WorkRequest:
         """
-        Create a child work request if one does not already exist.
+        Create a child SIGNING work request if one does not already exist.
+
+        This assumes that ``workflow_data.step`` is unique among work
+        request children with this task type and name, and that it is stable
+        even if the workflow is orchestrated multiple times.
 
         :param task_name: the task name for the child work request.
         :param task_data: the task data for the child work request.
@@ -530,9 +510,9 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         """
         try:
             return self.work_request.children.get(
-                task_name=task_name,
-                workflow_data_json=workflow_data.model_dump(exclude_unset=True),
                 task_type=TaskTypes.SIGNING,
+                task_name=task_name,
+                workflow_data_json__step=workflow_data.step,
             )
         except WorkRequest.DoesNotExist:
             return self.work_request.create_child_signing(
@@ -549,7 +529,11 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         workflow_data: WorkRequestWorkflowData,
     ) -> WorkRequest:
         """
-        Create a child work request if one does not already exist.
+        Create a child WAIT work request if one does not already exist.
+
+        This assumes that ``workflow_data.step`` is unique among work
+        request children with this task type and name, and that it is stable
+        even if the workflow is orchestrated multiple times.
 
         :param task_name: the task name for the child work request.
         :param task_data: the task data for the child work request.
@@ -558,9 +542,9 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         """
         try:
             return self.work_request.children.get(
-                task_name=task_name,
-                workflow_data_json=workflow_data.model_dump(exclude_unset=True),
                 task_type=TaskTypes.WAIT,
+                task_name=task_name,
+                workflow_data_json__step=workflow_data.step,
             )
         except WorkRequest.DoesNotExist:
             return self.work_request.create_child_wait(
@@ -576,11 +560,9 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
         workspace: Workspace | None = None,
     ) -> Collection:
         """Look up a singleton collection related to this workflow."""
-        return lookup_single(
+        return self.work_request.lookup_single(
             f"_@{category}",
             workspace=workspace or self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.COLLECTION,
         ).collection
 
@@ -604,7 +586,7 @@ class Workflow[WD: BaseWorkflowData, DTD: BaseDynamicTaskData](
 
     @override
     def _execute(self) -> WorkRequest.Results:
-        """Unused abstract method from BaseTask: populate() is used instead."""
+        """Unused abstract method from DBTask: populate() is used instead."""
         raise NotImplementedError()
 
 
