@@ -13,21 +13,46 @@ import logging
 
 from celery import shared_task
 from django.db import transaction
+from django.utils import timezone
+from prometheus_client import Histogram
 
 from debusine.artifacts.models import TaskTypes
 from debusine.db.context import context
+from debusine.db.metrics import emit_metric
 from debusine.db.models import WorkRequest
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.tasks.models import OutputData, OutputDataError
 
 logger = logging.getLogger(__name__)
 
+workflow_orchestrator_duration = Histogram(
+    "debusine_workflow_orchestrator_duration",
+    "Histogram of how long workflow orchestrators took to run.",
+    labelnames=("task_name",),
+    # Orchestrators can take some time.  Try to provide a reasonable range
+    # of buckets.
+    buckets=(
+        1.0,
+        5.0,
+        15.0,
+        60.0,  # 1 minute
+        300.0,  # 5 minutes
+        900.0,  # 15 minutes
+        3600.0,  # 1 hour
+        10800.0,  # 3 hours
+        21600.0,  # 6 hours
+        43200.0,  # 12 hours
+        86400.0,  # 1 day
+        float("inf"),
+    ),
+)
+
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
-@shared_task  # type: ignore[misc]
+@shared_task  # type: ignore[untyped-decorator]
 @transaction.atomic
-def run_workflow_task(work_request_id: int) -> bool:
+def run_workflow_task(work_request_id: int) -> WorkRequest.Results:
     """Run a workflow callback or workflow in Celery."""
     try:
         work_request = (
@@ -59,7 +84,7 @@ def run_workflow_task(work_request_id: int) -> bool:
                 errors=[OutputDataError(message=message, code="wrong-status")]
             ),
         )
-        return False
+        return WorkRequest.Results.ERROR
 
     if (
         work_request.task_type == TaskTypes.WORKFLOW
@@ -83,9 +108,23 @@ def run_workflow_task(work_request_id: int) -> bool:
                 ]
             ),
         )
-        return False
+        return WorkRequest.Results.ERROR
 
     context.reset()
     work_request.set_current()
+    start_time = timezone.now()
 
-    return orchestrate_workflow(work_request)
+    result = (
+        WorkRequest.Results.SUCCESS
+        if orchestrate_workflow(work_request)
+        else WorkRequest.Results.ERROR
+    )
+
+    emit_metric(
+        metric_type="histogram",
+        name="debusine_workflow_orchestrator_duration",
+        labels={"task_name": work_request.task_name},
+        value=(timezone.now() - start_time).total_seconds(),
+    )
+
+    return result
