@@ -28,8 +28,6 @@ from debusine.db.models import Artifact, ArtifactRelation, WorkRequest
 from debusine.server.collections.lookup import (
     LookupResult,
     artifact_ids,
-    lookup_multiple,
-    lookup_single,
     reconstruct_lookup,
 )
 from debusine.server.workflows import Workflow, workflow_utils
@@ -85,11 +83,8 @@ class DebDiffWorkflow(
         new_source_package_name = get_source_package_name(new_source_data)
         version_new = new_source_data.version
 
-        debian_suite = lookup_single(
-            self.data.original,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
-            expect_type=LookupChildType.COLLECTION,
+        debian_suite = self.work_request.lookup_single(
+            self.data.original, expect_type=LookupChildType.COLLECTION
         ).collection
 
         original_source_package = debian_suite.manager.lookup(
@@ -132,11 +127,8 @@ class DebDiffWorkflow(
         else:
             parameter_summary = f"{debian_suite.name}: missing → {version_new}"
 
-        binary_artifacts_results = lookup_multiple(
+        binary_artifacts_results = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
@@ -189,11 +181,8 @@ class DebDiffWorkflow(
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
 
         # Populate DebDiff tasks for the source package
-        debian_suite = lookup_single(
-            self.data.original,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
-            expect_type=LookupChildType.COLLECTION,
+        debian_suite = self.work_request.lookup_single(
+            self.data.original, expect_type=LookupChildType.COLLECTION
         ).collection
 
         original_source = debian_suite.manager.lookup(
@@ -219,14 +208,26 @@ class DebDiffWorkflow(
                 new_source_data.name, original_source_version
             )
         )
+        archs_old: set[Architecture] = set()
+        for archs_lookup in original_binaries_arch_lookup.values():
+            archs_old.update(archs_lookup.keys())
+
+        concrete_archs_old = archs_old - {Architecture("all")}
+
         new_binaries_arch_lookup = self._get_new_packages_by_name_arch()
 
-        archs = self._get_architectures(new_binaries_arch_lookup)
-        concrete_archs = {a for a in archs if a != Architecture("all")}
+        archs_new = self._get_architectures(new_binaries_arch_lookup)
+        concrete_archs_new = archs_new - {Architecture("all")}
 
-        if concrete_archs:
-            archs_to_compare = concrete_archs
+        if concrete_archs_new:
+            # New binaries provide concrete archs: compare per concrete arch
+            archs_to_compare = concrete_archs_new
+        elif concrete_archs_old and Architecture("all") not in archs_old:
+            # Transition any → all: new provides only "all" (because no
+            # concrete_archs_new), old provides only concretes
+            archs_to_compare = concrete_archs_old
         else:
+            # At least one side provides "all" (and no concrete archs on new)
             archs_to_compare = {Architecture("all")}
 
         for arch in sorted(archs_to_compare):
@@ -276,9 +277,10 @@ class DebDiffWorkflow(
         if original_relevants and new_relevants:
             build_arch = (
                 self.data.arch_all_build_architecture
-                if architecture == "all"
+                if architecture == Architecture("all")
                 else architecture
             )
+
             self._populate_binary_artifact_debdiff(
                 originals=LookupMultiple.model_validate(
                     sorted(original_relevants)
@@ -308,10 +310,8 @@ class DebDiffWorkflow(
 
         lookup = LookupMultiple.model_validate(kwargs)
 
-        return lookup_multiple(
+        return self.work_request.lookup_multiple(
             lookup,
-            self.workspace,
-            user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
             default_category=CollectionCategory.SUITE,
         )
@@ -430,11 +430,8 @@ class DebDiffWorkflow(
         self,
     ) -> PackageArchLookupMap:
         """Return mapping of new binary packages by name and arch."""
-        new_binary_results = lookup_multiple(
+        new_binary_results = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
         packages_by_name_arch: defaultdict[
