@@ -12,15 +12,11 @@
 from functools import partial
 from typing import assert_never, override
 
-from django.contrib.auth.models import AnonymousUser
-from django.db.models import Q
-
 from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import ArtifactCategory, DebianSourcePackage
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import WorkRequest
-from debusine.server.collections.lookup import lookup_single
 from debusine.server.tasks.wait.models import ConfirmData
 from debusine.server.workflows import (
     Workflow,
@@ -65,12 +61,8 @@ class DebianPipelineWorkflow(
         """Thorough validation of input data."""
         if self.data.qa_suite is not None:
             try:
-                lookup_single(
-                    self.data.qa_suite,
-                    self.workspace,
-                    user=self.work_request.created_by,
-                    workflow_root=self.work_request.workflow_root,
-                    expect_type=LookupChildType.COLLECTION,
+                self.work_request.lookup_single(
+                    self.data.qa_suite, expect_type=LookupChildType.COLLECTION
                 )
             except LookupError as e:
                 raise WorkflowValidationError(str(e)) from e
@@ -112,12 +104,8 @@ class DebianPipelineWorkflow(
 
     def suite_has_active_indexes(self, suite_lookup: LookupSingle) -> bool:
         """Return True if this suite has an active ``InRelease`` file."""
-        suite = lookup_single(
-            suite_lookup,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
-            expect_type=LookupChildType.COLLECTION,
+        suite = self.work_request.lookup_single(
+            suite_lookup, expect_type=LookupChildType.COLLECTION
         ).collection
         return (
             suite.child_items.active()
@@ -195,15 +183,6 @@ class DebianPipelineWorkflow(
             }
         )
 
-        # sbuild workflow promises the buildlog-{arch} artifacts
-        package_build_logs = LookupMultiple.model_validate(
-            {
-                "collection": "internal@collections",
-                "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                "name__startswith": "buildlog-",
-            }
-        )
-
         to_upload = [
             (
                 self.data.source_artifact,
@@ -230,7 +209,6 @@ class DebianPipelineWorkflow(
             populate_qa = partial(
                 self._populate_qa,
                 source_artifact=self.data.source_artifact,
-                package_build_logs=package_build_logs,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 architectures=effective_architectures,
@@ -258,11 +236,8 @@ class DebianPipelineWorkflow(
                     f"{self.data.qa_suite}/source:{source_data.name}"
                 )
                 try:
-                    reference_source = lookup_single(
+                    reference_source = self.work_request.lookup_single(
                         reference_source_lookup,
-                        self.workspace,
-                        user=self.work_request.created_by,
-                        workflow_root=self.work_request.workflow_root,
                         expect_type=LookupChildType.ARTIFACT,
                     ).artifact
                 except KeyError:
@@ -466,7 +441,6 @@ class DebianPipelineWorkflow(
         reference_prefix: str = "",
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
-        package_build_logs: LookupMultiple,
         reference_source_artifact: LookupSingle | None = None,
         vendor: str,
         codename: str,
@@ -497,7 +471,6 @@ class DebianPipelineWorkflow(
             reference_prefix=reference_prefix,
             source_artifact=source_artifact,
             binary_artifacts=binary_artifacts,
-            package_build_logs=package_build_logs,
             reference_source_artifact=reference_source_artifact,
             vendor=vendor,
             codename=codename,
@@ -529,10 +502,7 @@ class DebianPipelineWorkflow(
                 display_name="QA", step="qa"
             )
         wr = self.work_request_ensure_child_workflow(
-            task_name="qa",
-            task_data=data,
-            task_data_filter=Q(task_data__update_qa_results=update_qa_results),
-            workflow_data=workflow_data,
+            task_name="qa", task_data=data, workflow_data=workflow_data
         )
         # The qa workflow's children will have dependencies on the work
         # requests creating binary_artifacts, but the qa workflow itself
@@ -581,11 +551,8 @@ class DebianPipelineWorkflow(
                 signing_artifact = (
                     f"internal@collections/name:signing-template-{arch}-{name}"
                 )
-                lookup_single(
+                self.work_request.lookup_single(
                     signing_artifact,
-                    self.workspace,
-                    user=self.work_request.created_by or AnonymousUser(),
-                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
                 )
 
