@@ -9,6 +9,7 @@
 
 """lintian workflow."""
 
+from collections.abc import Callable
 from typing import Any, override
 
 from debian.debian_support import Version
@@ -28,10 +29,7 @@ from debusine.db.models import (
     ArtifactRelation,
     WorkRequest,
 )
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-)
+from debusine.server.collections.lookup import artifact_ids
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.models import (
     LintianWorkflowData,
@@ -299,7 +297,7 @@ class LintianWorkflow(
                 task_name="workflow",
                 workflow_data=WorkRequestWorkflowData(
                     allow_dependency_failures=True,
-                    step="regression-analysis",
+                    step=f"regression-analysis-{architecture}",
                     display_name=f"Regression analysis for {architecture}",
                     visible=False,
                 ),
@@ -331,11 +329,8 @@ class LintianWorkflow(
         source_artifact = workflow_utils.source_package(self)
         source_data = SourcePackage.create_data(source_artifact.data)
 
-        binary_artifacts_results = lookup_multiple(
+        binary_artifacts_results = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
         return LintianWorkflowDynamicData(
@@ -493,3 +488,11 @@ class LintianWorkflow(
         self.work_request.output_data = output_data
         self.work_request.save()
         return True
+
+    @override
+    def get_callback(self, step: str) -> Callable[[], bool]:
+        if not step.startswith("regression-analysis"):
+            raise NotImplementedError(
+                f"Unhandled workflow callback step: {step}"
+            )
+        return self.callback_regression_analysis
