@@ -16,11 +16,7 @@ from debusine.artifacts.models import ArtifactCategory
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import WorkRequest
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-    reconstruct_lookup,
-)
+from debusine.server.collections.lookup import artifact_ids, reconstruct_lookup
 from debusine.server.workflows import Workflow, workflow_utils
 from debusine.server.workflows.models import (
     MakeSignedSourceWorkflowData,
@@ -66,11 +62,8 @@ class MakeSignedSourceWorkflow(
         :binary_artifacts_ids: ids of the binary_artifacts
         :signing_template_artifacts_ids: ids of the signing_template_artifacts
         """
-        binaries = lookup_multiple(
+        binaries = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            workflow_root=self.work_request.workflow_root,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
@@ -85,11 +78,8 @@ class MakeSignedSourceWorkflow(
             )
         )
 
-        signing_templates = lookup_multiple(
+        signing_templates = self.work_request.lookup_multiple(
             self.data.signing_template_artifacts,
-            workflow_root=self.work_request.workflow_root,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
@@ -146,11 +136,8 @@ class MakeSignedSourceWorkflow(
         workflow_root = self.work_request.workflow_root
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
 
-        signing_templates = lookup_multiple(
+        signing_templates = self.work_request.lookup_multiple(
             self.data.signing_template_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
