@@ -165,13 +165,9 @@ class UpdateEnvironmentsWorkflowTarget(BaseWorkflowData):
         default_factory=dict
     )
     variants: str | None | list[str | None] = pydantic.Field(
-        # TODO: Why is mypy confused about what type default_factory should be?
-        default_factory=list  # type: ignore[arg-type]
-    )
-    backends: str | list[str] = pydantic.Field(
-        # TODO: Why is mypy confused about what type default_factory should be?
-        default_factory=list  # type: ignore[arg-type]
+        default_factory=list
     )
+    backends: str | list[str] = pydantic.Field(default_factory=list)
     architectures: Annotated[
         list[str],
         pydantic.Field(min_length=1),
@@ -423,7 +419,8 @@ class BlhcWorkflowData(RegressionTrackingWorkflowData):
     """`blhc` workflow data."""
 
     source_artifact: LookupSingle | None = None
-    package_build_logs: LookupMultiple
+    binary_artifacts: LookupMultiple | None = None
+    package_build_logs: LookupMultiple | None = None
     extra_flags: list[BlhcFlags] = pydantic.Field(default_factory=list)
 
     vendor: str
@@ -441,11 +438,22 @@ class BlhcWorkflowData(RegressionTrackingWorkflowData):
             )
         return self
 
+    @pydantic.model_validator(mode="after")
+    def check_package_build_logs(self) -> Self:
+        """We need either ``binary_artifacts`` or ``package_build_logs``."""
+        if (self.binary_artifacts is None) == (self.package_build_logs is None):
+            raise ValueError(
+                "Exactly one of binary_artifacts"
+                " or package_build_logs must be set"
+            )
+        return self
+
 
 class BlhcWorkflowDynamicData(BaseDynamicTaskData):
     """Dynamic data for Blhc workflow."""
 
     source_artifact_id: int | None = None
+    binary_artifacts_ids: list[int] | None = None
     package_build_logs_ids: list[int] | None = None
 
 
@@ -462,6 +470,7 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
 
     source_artifact: LookupSingle
     binary_artifacts: LookupMultiple
+    # Obsolete, but may be present in old rows.
     package_build_logs: LookupMultiple | None = None
 
     # debian_pipeline sets this if enable_regression_tracking is set, but it
@@ -528,15 +537,6 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
             )
         return self
 
-    @pydantic.model_validator(mode="after")
-    def enable_blhc_consistency(self) -> Self:
-        """Check consistency of enable_blhc options."""
-        if self.enable_blhc and self.package_build_logs is None:
-            raise ValueError(
-                '"package_build_logs" is required if "enable_blhc" is set'
-            )
-        return self
-
 
 class QAWorkflowDynamicData(BaseDynamicTaskData):
     """Dynamic data for the QA workflow."""
@@ -544,6 +544,7 @@ class QAWorkflowDynamicData(BaseDynamicTaskData):
     source_artifact_id: int | None = None
     binary_artifacts_ids: list[int] | None = None
     reference_source_artifact_id: int | None = None
+    # Obsolete, but may be present in old rows.
     package_build_logs_ids: list[int] | None = None
 
 
