@@ -10,9 +10,9 @@
 
 from typing import override
 
+from debusine.db.models.tasks import DefaultDynamicData
 from debusine.server.workflows.base import Workflow
 from debusine.server.workflows.models import BaseWorkflowData
-from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
