@@ -10,7 +10,7 @@
 """Package publishing workflow."""
 
 import logging
-from typing import Any
+from typing import Any, override
 
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -28,11 +28,7 @@ from debusine.db.models import (
     WorkflowTemplate,
     Workspace,
 )
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-    lookup_single,
-)
+from debusine.server.collections.lookup import artifact_ids
 from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
@@ -65,11 +61,8 @@ class PackagePublishWorkflow(
     @property
     def target_suite(self) -> Collection:
         """Look up the target suite."""
-        return lookup_single(
+        return self.work_request.lookup_single(
             lookup=self.data.target_suite,
-            workspace=self.work_request.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.COLLECTION,
         ).collection
 
@@ -96,12 +89,8 @@ class PackagePublishWorkflow(
         if self.data.source_artifact is None:
             return
 
-        original_source_artifact = lookup_single(
-            self.data.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
-            expect_type=LookupChildType.ARTIFACT,
+        original_source_artifact = self.work_request.lookup_single(
+            self.data.source_artifact, expect_type=LookupChildType.ARTIFACT
         ).artifact
         source_artifact = workflow_utils.locate_debian_source_package(
             "source_artifact", original_source_artifact
@@ -157,8 +146,8 @@ class PackagePublishWorkflow(
 
         copies.append(
             CopyCollectionItemsCopies(
-                source_items=LookupMultiple.parse_obj(
-                    [f"{source_artifact.id}@artifacts"]
+                source_items=LookupMultiple(
+                    (f"{source_artifact.id}@artifacts",)
                 ),
                 # Identify the target collection by ID to avoid strange
                 # behaviour if the lookup result changes between now and
@@ -232,8 +221,8 @@ class PackagePublishWorkflow(
 
         copies.append(
             CopyCollectionItemsCopies(
-                source_items=LookupMultiple.parse_obj(
-                    [f"{binary_artifact.id}@artifacts"]
+                source_items=LookupMultiple(
+                    (f"{binary_artifact.id}@artifacts",)
                 ),
                 # Identify the target collection by ID to avoid strange
                 # behaviour if the lookup result changes between now and
@@ -254,7 +243,7 @@ class PackagePublishWorkflow(
         target_workspace: Workspace,
     ) -> None:
         """Add copy items for build logs."""
-        if self.data.binary_artifacts.__root__:
+        if self.data.binary_artifacts:
             try:
                 source_build_logs_collection = self.lookup_singleton_collection(
                     CollectionCategory.PACKAGE_BUILD_LOGS
@@ -275,7 +264,7 @@ class PackagePublishWorkflow(
             ):
                 copies.append(
                     CopyCollectionItemsCopies(
-                        source_items=LookupMultiple.parse_obj(
+                        source_items=LookupMultiple.model_validate(
                             {
                                 "collection": source_build_logs_collection.id,
                                 "lookup__same_work_request": (
@@ -295,7 +284,7 @@ class PackagePublishWorkflow(
         target_workspace: Workspace,
     ) -> None:
         """Add copy items for task history."""
-        if self.data.binary_artifacts.__root__:
+        if self.data.binary_artifacts:
             try:
                 source_task_history_collection = (
                     self.lookup_singleton_collection(
@@ -321,7 +310,7 @@ class PackagePublishWorkflow(
             ):
                 copies.append(
                     CopyCollectionItemsCopies(
-                        source_items=LookupMultiple.parse_obj(
+                        source_items=LookupMultiple.model_validate(
                             {
                                 "collection": source_task_history_collection.id,
                                 "lookup__same_workflow": (
@@ -335,6 +324,7 @@ class PackagePublishWorkflow(
                     )
                 )
 
+    @override
     def populate(self) -> None:
         """Create work requests."""
         assert self.dynamic_data is not None
@@ -350,11 +340,8 @@ class PackagePublishWorkflow(
             "binary_artifacts",
             [
                 result.artifact
-                for result in lookup_multiple(
+                for result in self.work_request.lookup_multiple(
                     self.data.binary_artifacts,
-                    self.workspace,
-                    user=self.work_request.created_by,
-                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 )
             ],
@@ -406,12 +393,13 @@ class PackagePublishWorkflow(
         )
         WorkRequest.objects.create_workflow(
             template=template,
-            data=UpdateSuitesData(only_suites=[self.target_suite.name]).dict(
-                exclude_unset=True
-            ),
+            data=UpdateSuitesData(
+                only_suites=[self.target_suite.name]
+            ).model_dump(exclude_unset=True),
         ).mark_pending()
         return True
 
+    @override
     def build_dynamic_data(
         self,
         task_database: TaskDatabaseInterface,  # noqa: U100
@@ -427,11 +415,8 @@ class PackagePublishWorkflow(
         :binary_artifacts_ids: ids of the binary artifacts
         :target_suite_id: collection id of the target_suite
         """
-        binaries = lookup_multiple(
+        binaries = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            workflow_root=self.work_request.workflow_root,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
@@ -446,12 +431,8 @@ class PackagePublishWorkflow(
             )
         )
 
-        target_suite_id = lookup_single(
-            self.data.target_suite,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
-            expect_type=LookupChildType.COLLECTION,
+        target_suite_id = self.work_request.lookup_single(
+            self.data.target_suite, expect_type=LookupChildType.COLLECTION
         ).collection.id
 
         try:
@@ -468,6 +449,7 @@ class PackagePublishWorkflow(
             target_suite_id=target_suite_id,
         )
 
+    @override
     def get_input_artifacts(self) -> list[InputArtifact]:
         """Return input artifacts."""
         artifacts: list[InputArtifact] = []
