@@ -15,12 +15,7 @@ from typing import override
 from debusine.artifacts.models import ArtifactCategory
 from debusine.client.models import LookupChildType
 from debusine.db.models import WorkRequest
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-    lookup_single,
-    reconstruct_lookup,
-)
+from debusine.server.collections.lookup import artifact_ids, reconstruct_lookup
 from debusine.server.tasks.models import PackageUploadData, PackageUploadInput
 from debusine.server.tasks.wait.models import ExternalDebsignData
 from debusine.server.workflows import (
@@ -58,7 +53,7 @@ class PackageUploadWorkflow(
 
     @override
     def post_init(self) -> None:
-        """Initialize."""
+        super().post_init()
         self._artifacts_pending_upload: list[tuple[LookupSingle, bool]] = []
 
     @cached_property
@@ -68,11 +63,8 @@ class PackageUploadWorkflow(
             return None
 
         return workflow_utils.lookup_result_artifact_category(
-            lookup_single(
+            self.work_request.lookup_single(
                 self.data.source_artifact,
-                self.workspace,
-                user=self.work_request.created_by,
-                workflow_root=self.work_request.workflow_root,
                 expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
             )
         )
@@ -121,11 +113,8 @@ class PackageUploadWorkflow(
                 artifacts_pending_upload.append((source_artifact, False))
 
         # Upload binary_artifacts
-        results = lookup_multiple(
+        results = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
@@ -157,11 +146,8 @@ class PackageUploadWorkflow(
         :source_artifact_id: id of source_artifact or None
         :binary_artifacts_ids: ids of the binary_artifacts
         """
-        binaries = lookup_multiple(
+        binaries = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            workflow_root=self.work_request.workflow_root,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
         source_package_names = workflow_utils.get_source_package_names(
@@ -175,11 +161,8 @@ class PackageUploadWorkflow(
 
         source_artifact_id = None
         if self.data.source_artifact is not None:
-            source_artifact = lookup_single(
+            source_artifact = self.work_request.lookup_single(
                 self.data.source_artifact,
-                workflow_root=self.work_request.workflow_root,
-                workspace=self.workspace,
-                user=self.work_request.created_by,
                 expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
             )
 
