@@ -9,6 +9,7 @@
 
 """Piuparts workflow."""
 
+from collections.abc import Callable
 from typing import Any, override
 
 from django.utils import timezone
@@ -21,7 +22,7 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, ArtifactRelation, WorkRequest
-from debusine.server.collections.lookup import artifact_ids, lookup_multiple
+from debusine.server.collections.lookup import artifact_ids
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.models import (
     PiupartsWorkflowData,
@@ -63,7 +64,7 @@ class PiupartsWorkflow(
 
     @override
     def post_init(self) -> None:
-        """Initialize."""
+        super().post_init()
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
@@ -93,9 +94,17 @@ class PiupartsWorkflow(
 
     def _populate_single(self, architecture: str) -> None:
         """Create a Piuparts task for a single architecture."""
+        # The concrete architecture, or {arch_all_build_architecture} if only
+        # Architecture: all binary packages are being checked by this task.
+        build_architecture = (
+            self.data.arch_all_build_architecture
+            if architecture == "all"
+            else architecture
+        )
+
         if (
             self.data.update_qa_results
-            and self._has_current_reference_qa_result(architecture)
+            and self._has_current_reference_qa_result(build_architecture)
         ):
             return
 
@@ -111,14 +120,6 @@ class PiupartsWorkflow(
             )
         )
 
-        # The concrete architecture, or {arch_all_build_architecture} if only
-        # Architecture: all binary packages are being checked by this task
-        build_architecture = (
-            self.data.arch_all_build_architecture
-            if architecture == "all"
-            else architecture
-        )
-
         extra_repositories = workflow_utils.configure_for_overlay_suite(
             self,
             extra_repositories=self.data.extra_repositories,
@@ -150,8 +151,8 @@ class PiupartsWorkflow(
                 extra_repositories=extra_repositories,
             ),
             workflow_data=WorkRequestWorkflowData(
-                display_name=f"Piuparts {architecture}",
-                step=f"piuparts-{architecture}",
+                display_name=f"Piuparts {build_architecture}",
+                step=f"piuparts-{build_architecture}",
                 **workflow_data_kwargs,
             ),
         )
@@ -161,7 +162,7 @@ class PiupartsWorkflow(
         # dependencies. Binary promises must include an architecture field
         # in their data.
         self.requires_artifact(wr, arch_subset_binary_artifacts)
-        promise_name = f"{self.data.prefix}piuparts-{architecture}"
+        promise_name = f"{self.data.prefix}piuparts-{build_architecture}"
         self.provides_artifact(wr, ArtifactCategory.PIUPARTS, promise_name)
 
         if self.data.update_qa_results:
@@ -172,7 +173,7 @@ class PiupartsWorkflow(
 
             # Back off if another workflow gets there first.
             self.skip_if_qa_result_changed(
-                wr, package=source_data.name, architecture=architecture
+                wr, package=source_data.name, architecture=build_architecture
             )
 
             # Record results in the reference collection.
@@ -181,7 +182,7 @@ class PiupartsWorkflow(
                 variables={
                     "package": source_data.name,
                     "version": source_data.version,
-                    "architecture": architecture,
+                    "architecture": build_architecture,
                     "timestamp": int(
                         (self.qa_suite_changed or timezone.now()).timestamp()
                     ),
@@ -200,8 +201,10 @@ class PiupartsWorkflow(
                 task_name="workflow",
                 workflow_data=WorkRequestWorkflowData(
                     allow_dependency_failures=True,
-                    step="regression-analysis",
-                    display_name=f"Regression analysis for {architecture}",
+                    step=f"regression-analysis-{build_architecture}",
+                    display_name=(
+                        f"Regression analysis for {build_architecture}"
+                    ),
                     visible=False,
                 ),
             )
@@ -209,7 +212,7 @@ class PiupartsWorkflow(
                 self.requires_artifact(
                     regression_analysis,
                     f"internal@collections/name:"
-                    f"{self.data.reference_prefix}piuparts-{architecture}",
+                    f"{self.data.reference_prefix}piuparts-{build_architecture}",
                 )
             except KeyError:
                 pass
@@ -314,6 +317,14 @@ class PiupartsWorkflow(
         self.work_request.save()
         return True
 
+    @override
+    def get_callback(self, step: str) -> Callable[[], bool]:
+        if not step.startswith("regression-analysis"):
+            raise NotImplementedError(
+                f"Unhandled workflow callback step: {step}"
+            )
+        return self.callback_regression_analysis
+
     @override
     def build_dynamic_data(
         self,
@@ -328,11 +339,8 @@ class PiupartsWorkflow(
         :source_artifact_id: id of the ``source_artifact``
         :binary_artifacts_ids: ids of the ``binary_artifacts``
         """
-        binaries = lookup_multiple(
+        binaries = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            workflow_root=self.work_request.workflow_root,
-            workspace=self.workspace,
-            user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
         source_package_names = " ".join(
