@@ -17,10 +17,7 @@ from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import TaskTypes
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, WorkRequest
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-)
+from debusine.server.collections.lookup import artifact_ids
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
@@ -72,7 +69,6 @@ class QAWorkflow(
         :source_artifact_id: id of the source artifact
         :binary_artifacts_ids: ids of the binary artifacts or None
         :reference_source_artifact_id: id of the reference artifact or None
-        :package_build_logs_ids: ids of the package build logs or None
         """
         source_artifact = workflow_utils.source_package(self)
         source_data = SourcePackage.create_data(source_artifact.data)
@@ -84,26 +80,11 @@ class QAWorkflow(
         else:
             reference_source_artifact = None
 
-        binary_artifacts_results = lookup_multiple(
+        binary_artifacts_results = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
-        if self.data.package_build_logs is not None:
-            build_logs_results = lookup_multiple(
-                self.data.package_build_logs,
-                self.workspace,
-                user=self.work_request.created_by,
-                workflow_root=self.work_request.workflow_root,
-                expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
-            )
-            build_logs_artifacts_ids = artifact_ids(build_logs_results)
-        else:
-            build_logs_artifacts_ids = None
-
         return QAWorkflowDynamicData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
@@ -114,7 +95,6 @@ class QAWorkflow(
                 if reference_source_artifact is not None
                 else None
             ),
-            package_build_logs_ids=build_logs_artifacts_ids,
         )
 
     @override
@@ -154,19 +134,6 @@ class QAWorkflow(
                 )
             )
 
-        if self.data.package_build_logs is not None:
-            artifacts.append(
-                InputArtifactMultiple(
-                    lookup=self.data.package_build_logs,
-                    label="package_build_logs",
-                    artifact_ids=(
-                        None
-                        if self.dynamic_data is None
-                        else self.dynamic_data.package_build_logs_ids
-                    ),
-                )
-            )
-
         return artifacts
 
     @override
@@ -304,14 +271,11 @@ class QAWorkflow(
             )
 
         if self.data.enable_blhc:
-            # Checked by
-            # QAWorkflowData.enable_blhc_consistency.
-            assert self.data.package_build_logs
             self._populate_blhc(
                 prefix=self.data.prefix,
                 reference_prefix=self.data.reference_prefix,
                 source_artifact=self.data.source_artifact,
-                package_build_logs=self.data.package_build_logs,
+                binary_artifacts=filtered_binary_artifacts,
                 qa_suite=self.data.qa_suite,
                 reference_qa_results=self.data.reference_qa_results,
                 enable_regression_tracking=self.data.enable_regression_tracking,
@@ -569,7 +533,7 @@ class QAWorkflow(
         prefix: str,
         reference_prefix: str,
         source_artifact: LookupSingle,
-        package_build_logs: LookupMultiple,
+        binary_artifacts: LookupMultiple,
         qa_suite: LookupSingle | None,
         reference_qa_results: LookupSingle | None,
         enable_regression_tracking: bool,
@@ -585,7 +549,7 @@ class QAWorkflow(
                 prefix=prefix,
                 reference_prefix=reference_prefix,
                 source_artifact=source_artifact,
-                package_build_logs=package_build_logs,
+                binary_artifacts=binary_artifacts,
                 qa_suite=qa_suite,
                 reference_qa_results=reference_qa_results,
                 enable_regression_tracking=enable_regression_tracking,
@@ -600,6 +564,11 @@ class QAWorkflow(
                 allow_failure=allow_failure,
             ),
         )
+
+        # The blhc workflow needs the binary artifacts to exist with
+        # relations to package build logs in order to populate itself.
+        self.requires_artifact(wr, binary_artifacts)
+
         self.orchestrate_child(wr)
 
     def _find_regression_analysis_sources(self) -> QuerySet[WorkRequest]:
@@ -630,7 +599,7 @@ class QAWorkflow(
         for callback in WorkRequest.objects.filter(
             task_type=TaskTypes.INTERNAL,
             task_name="workflow",
-            workflow_data_json__step="regression-analysis",
+            workflow_data_json__step__startswith="regression-analysis",
             parent__in=self._find_regression_analysis_sources(),
         ):
             # Each individual regression analysis callback is a descendant
