@@ -39,11 +39,7 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import Collection, CollectionItem
-from debusine.server.collections.lookup import (
-    artifact_ids,
-    lookup_multiple,
-    lookup_single,
-)
+from debusine.server.collections.lookup import artifact_ids
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
@@ -172,7 +168,7 @@ class ReverseDependenciesAutopkgtestWorkflow(
 
     @override
     def post_init(self) -> None:
-        """Initialize."""
+        super().post_init()
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
@@ -188,11 +184,8 @@ class ReverseDependenciesAutopkgtestWorkflow(
     def get_binary_names(self) -> set[str]:
         """Return names of binary artifacts whose rdeps we want to test."""
         binary_names: set[str] = set()
-        for result in lookup_multiple(
+        for result in self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         ):
             if result.artifact is not None:
@@ -223,12 +216,8 @@ class ReverseDependenciesAutopkgtestWorkflow(
         :return: a list of tuples, each of which is a source package and its
           list of associated binary packages.
         """
-        source_artifact = lookup_single(
-            self.data.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
-            expect_type=LookupChildType.ARTIFACT,
+        source_artifact = self.work_request.lookup_single(
+            self.data.source_artifact, expect_type=LookupChildType.ARTIFACT
         ).artifact
         source_artifact = workflow_utils.locate_debian_source_package(
             "source_artifact", source_artifact
@@ -395,8 +384,6 @@ class ReverseDependenciesAutopkgtestWorkflow(
         self, source: _SourcePackage, binaries: Iterable[_BinaryPackage]
     ) -> None:
         """Create an autopkgtest sub-workflow for a single source package."""
-        assert self.work_request is not None
-
         context_lookup = LookupMultiple.model_validate(
             _lookup_multiple_to_list(self.data.binary_artifacts)
             + _lookup_multiple_to_list(self.data.context_artifacts)
@@ -465,26 +452,16 @@ class ReverseDependenciesAutopkgtestWorkflow(
         source_artifact = workflow_utils.source_package(self)
         source_data = SourcePackage.create_data(source_artifact.data)
 
-        binary_artifacts_result = lookup_multiple(
+        binary_artifacts_result = self.work_request.lookup_multiple(
             self.data.binary_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
-        context_artifacts_result = lookup_multiple(
+        context_artifacts_result = self.work_request.lookup_multiple(
             self.data.context_artifacts,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
-        qa_suite_id = lookup_single(
-            self.data.qa_suite,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
-            expect_type=LookupChildType.COLLECTION,
+        qa_suite_id = self.work_request.lookup_single(
+            self.data.qa_suite, expect_type=LookupChildType.COLLECTION
         ).collection.id
 
         return ReverseDependenciesAutopkgtestWorkflowDynamicData(
