@@ -10,9 +10,11 @@
 
 import re
 from functools import cached_property
+from typing import override
 
 from debian.debian_support import DpkgArchTable
 
+from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -22,20 +24,20 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, TaskDatabase
-from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.base import Workflow, WorkflowValidationError
 from debusine.server.workflows.models import (
     SbuildWorkflowData,
+    SbuildWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
-from debusine.tasks import get_environment
 from debusine.tasks.models import (
     ActionRetryWithDelays,
     ActionUpdateCollectionWithArtifacts,
     ActionUpdateCollectionWithData,
     BackendType,
-    BaseDynamicTaskData,
+    InputArtifact,
+    InputArtifactSingle,
     SbuildBuildComponent,
     SbuildBuildDepResolver,
     SbuildData,
@@ -43,30 +45,39 @@ from debusine.tasks.models import (
 from debusine.tasks.server import TaskDatabaseInterface
 
 
-class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
+class SbuildWorkflow(Workflow[SbuildWorkflowData, SbuildWorkflowDynamicData]):
     """Build a source package for all architectures of a target distribution."""
 
     TASK_NAME = "sbuild"
 
+    @override
     def post_init(self) -> None:
-        """Initialize."""
+        super().post_init()
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
+    @override
     def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> BaseDynamicTaskData:
+        self,
+        task_database: TaskDatabaseInterface,  # noqa: U100
+    ) -> SbuildWorkflowDynamicData:
         r"""
         Compute dynamic data for this workflow.
 
         :subject: package name of ``input.source_artifact``
         :parameter_summary: package name of
           ``input.source_artifact``\_"version"
+        :source_artifact_id: id or None of the input.source_artifact.
         """
-        source_data = workflow_utils.source_package_data(self)
-        return BaseDynamicTaskData(
+        source_artifact = workflow_utils.source_package(
+            self, configuration_key="input.source_artifact"
+        )
+        source_data = SourcePackage.create_data(source_artifact.data)
+
+        return SbuildWorkflowDynamicData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
+            source_artifact_id=source_artifact.id,
         )
 
     @cached_property
@@ -74,9 +85,9 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         """Binary names that may be built."""
         return re.split(
             r"\s*,\s*",
-            workflow_utils.source_package_data(self).dsc_fields.get(
-                "Binary", ""
-            ),
+            workflow_utils.source_package_data(
+                self, configuration_key="input.source_artifact"
+            ).dsc_fields.get("Binary", ""),
         )
 
     @cached_property
@@ -85,7 +96,9 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         workflow_arches = set(self.data.architectures)
 
         package_arches = set(
-            workflow_utils.source_package_data(self)
+            workflow_utils.source_package_data(
+                self, configuration_key="input.source_artifact"
+            )
             .dsc_fields.get("Architecture", "")
             .split()
         )
@@ -158,23 +171,24 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         build_architecture = self._get_build_architecture(architecture)
 
         # set in BaseServerTask.__init__() along with work_request
-        assert self.workspace is not None
-        assert self.work_request is not None
         try:
-            env_id = get_environment(
-                TaskDatabase(self.work_request),
-                lookup,
-                architecture=build_architecture,
-                # TODO: This duplicates logic from
-                # BaseTaskWithExecutor.backend.
-                backend=(
-                    BackendType.UNSHARE
-                    if self.data.backend == BackendType.AUTO
-                    else self.data.backend
-                ),
-                default_category=CollectionCategory.ENVIRONMENTS,
-                try_variant="sbuild",
-            ).id
+            env_id = (
+                TaskDatabase(self.work_request)
+                .get_environment(
+                    lookup,
+                    architecture=build_architecture,
+                    # TODO: This duplicates logic from
+                    # BaseTaskWithExecutor.backend.
+                    backend=(
+                        BackendType.UNSHARE
+                        if self.data.backend == BackendType.AUTO
+                        else self.data.backend
+                    ),
+                    default_category=CollectionCategory.ENVIRONMENTS,
+                    try_variant="sbuild",
+                )
+                .id
+            )
         except KeyError:
             raise WorkflowValidationError(
                 "environment not found for"
@@ -183,6 +197,7 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
             )
         return Artifact.objects.get(id=env_id)
 
+    @override
     def validate_input(self) -> None:
         """Thorough validation of input data."""
         # Validate target_distribution
@@ -195,45 +210,25 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
         # self.architectures
         self.architectures
 
-        vendor, codename = self.data.target_distribution.split(":", 1)
+        _vendor, _codename = self.data.target_distribution.split(":", 1)
 
         # Make sure we have environments for all architectures we need
         for arch in self.architectures:
             # Attempt to get the environment to validate its lookup
             self._get_environment(arch)
 
+    @override
     def populate(self) -> None:
         """Create sbuild WorkRequests for all architectures."""
-        assert self.work_request is not None
-        children = self.work_request.children.all()
-        existing_arches: set[str] = set()
-        for child in children:
-            if child.task_data["build_components"] == [
-                SbuildBuildComponent.ALL
-            ]:
-                existing_arches.add("all")
-            else:
-                existing_arches.add(child.task_data["build_architecture"])
-
-        # Idempotence
-        # It is unlikely that there are workrequests already created that we do
-        # not need anymore, and I cannot think of a scenario when that may
-        # happen. Still, I'm leaving an assertion to catch it, so that if it
-        # happens it can be detected and studied to figure out what corner case
-        # has been hit and how to handle it
-        assert not existing_arches - self.architectures
-
-        if arches := self.architectures - existing_arches:
-            for architecture in sorted(arches):
-                self._populate_single(
-                    architecture, environment=self._get_environment_lookup()
-                )
+        for architecture in sorted(self.architectures):
+            self._populate_single(
+                architecture, environment=self._get_environment_lookup()
+            )
 
     def _populate_single(self, architecture: str, *, environment: str) -> None:
         """Create a single sbuild WorkRequest."""
-        assert self.work_request is not None
         build_architecture = self._get_build_architecture(architecture)
-        input_ = self.data.input.copy()
+        input_ = self.data.input.model_copy()
 
         input_.source_artifact = (
             workflow_utils.locate_debian_source_package_lookup(
@@ -241,12 +236,8 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
             )
         )
 
-        source_artifact = lookup_single(
-            input_.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.workflow_root,
-            expect_type=LookupChildType.ARTIFACT,
+        source_artifact = self.work_request.lookup_single(
+            input_.source_artifact, expect_type=LookupChildType.ARTIFACT
         ).artifact
         assert source_artifact.category == ArtifactCategory.SOURCE_PACKAGE
         debian_source_package = DebianSourcePackage(**source_artifact.data)
@@ -332,15 +323,16 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
             build_logs_collection_id = None
 
         if build_logs_collection_id is not None:
+            source_data = workflow_utils.source_package_data(
+                self, configuration_key="input.source_artifact"
+            )
             variables = {
                 "work_request_id": wr.id,
                 "vendor": vendor,
                 "codename": codename,
                 "architecture": build_architecture,
-                "srcpkg_name": workflow_utils.source_package_data(self).name,
-                "srcpkg_version": workflow_utils.source_package_data(
-                    self
-                ).version,
+                "srcpkg_name": source_data.name,
+                "srcpkg_version": source_data.version,
             }
             wr.add_event_reaction(
                 "on_creation",
@@ -383,3 +375,18 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                     "data__deb_fields__Package": binary_package_name
                 },
             )
+
+    @override
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return workflow input artifacts (the source package)."""
+        return [
+            InputArtifactSingle(
+                lookup=self.data.input.source_artifact,
+                label="input.source_artifact",
+                artifact_id=(
+                    None
+                    if self.dynamic_data is None
+                    else self.dynamic_data.source_artifact_id
+                ),
+            )
+        ]
