@@ -16,6 +16,7 @@ from typing import Any, ClassVar, NewType
 from django.test import override_settings
 from django.utils import timezone
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -24,7 +25,7 @@ from debusine.artifacts.models import (
     DebianSourcePackage,
     TaskTypes,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, LookupResultType
 from debusine.db.models import (
     Artifact,
     Collection,
@@ -34,7 +35,7 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.db.models.work_requests import SkipWorkRequest
-from debusine.server.collections.lookup import LookupResult, lookup_single
+from debusine.server.collections.lookup import LookupResult
 from debusine.server.scheduler import schedule
 from debusine.server.workflows import BlhcWorkflow, WorkflowValidationError
 from debusine.server.workflows.base import orchestrate_workflow
@@ -60,13 +61,12 @@ from debusine.tasks.models import (
     InputArtifactMultiple,
     InputArtifactSingle,
     LookupMultiple,
-    LookupSingle,
     OutputData,
     RegressionAnalysisStatus,
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 Architecture = NewType("Architecture", str)
 
@@ -181,30 +181,37 @@ class BlhcWorkflowTests(
     def create_blhc_workflow(
         self,
         *,
-        package_build_logs: list[LookupSingle] | None = None,
+        binary_artifacts: LookupMultiple | None = None,
+        package_build_logs: LookupMultiple | None = None,
         extra_task_data: dict[str, Any] | None = None,
         include_source_artifact: bool = True,
     ) -> BlhcWorkflow:
         """
         Create and schedule a Blhc workflow.
 
-        :param package_build_logs: build logs for the workflow. If None
-          use all the expected promises by setUpTestData()
-        :param extra_task_data: any extra task data to pass to the workflow
-        :param include_source_artifact: True to include self.source_artifact
+        :param binary_artifacts: binary artifacts for the workflow.
+        :param package_build_logs: build logs for the workflow. If both
+          ``binary_artifacts`` and ``package_build_logs`` are None, use all
+          the build log artifacts created by ``setUpTestData()``.
+        :param extra_task_data: any extra task data to pass to the workflow.
+        :param include_source_artifact: True to include self.source_artifact.
         """
-        if package_build_logs is None:
-            package_build_logs = []
-            for package_build_log in self.package_build_log_artifacts.values():
-                package_build_logs.append(package_build_log.id)
+        if binary_artifacts is None and package_build_logs is None:
+            package_build_logs = LookupMultiple(
+                tuple(
+                    package_build_log.id
+                    for package_build_log in (
+                        self.package_build_log_artifacts.values()
+                    )
+                )
+            )
 
         task_data = BlhcWorkflowData(
             source_artifact=(
                 self.source_artifact.id if include_source_artifact else None
             ),
-            package_build_logs=LookupMultiple.model_validate(
-                package_build_logs
-            ),
+            binary_artifacts=binary_artifacts,
+            package_build_logs=package_build_logs,
             vendor="debian",
             codename="bookworm",
         )
@@ -251,13 +258,13 @@ class BlhcWorkflowTests(
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_validate_input(self) -> None:
         """validate_input passes a valid case."""
         # validate_input() is called and does not raise
         self.create_blhc_workflow()
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_validate_input_bad_qa_suite(self) -> None:
         """validate_input raises errors in looking up a suite."""
         w = self.create_blhc_workflow(
@@ -270,7 +277,7 @@ class BlhcWorkflowTests(
         ):
             w.validate_input()
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_validate_input_bad_reference_qa_results(self) -> None:
         """validate_input raises errors in looking up reference QA results."""
         w = self.create_blhc_workflow(
@@ -312,8 +319,7 @@ class BlhcWorkflowTests(
         self.assertFalse(
             self.get_workflow(wr)._has_current_reference_qa_result(
                 LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT,
-                    artifact=build_log,
+                    result_type=LookupResultType.ARTIFACT, artifact=build_log
                 ),
                 "amd64",
             )
@@ -346,8 +352,7 @@ class BlhcWorkflowTests(
         self.assertFalse(
             self.get_workflow(wr)._has_current_reference_qa_result(
                 LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT,
-                    artifact=build_log,
+                    result_type=LookupResultType.ARTIFACT, artifact=build_log
                 ),
                 "amd64",
             )
@@ -380,7 +385,7 @@ class BlhcWorkflowTests(
         with self.assertRaisesRegex(ValueError, r"^Unexpected result"):
             self.get_workflow(wr)._has_current_reference_qa_result(
                 LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT,
+                    result_type=LookupResultType.ARTIFACT,
                     artifact=self.playground.create_artifact()[0],
                 ),
                 "amd64",
@@ -413,8 +418,7 @@ class BlhcWorkflowTests(
         self.assertTrue(
             self.get_workflow(wr)._has_current_reference_qa_result(
                 LookupResult(
-                    result_type=CollectionItem.Types.ARTIFACT,
-                    artifact=build_log,
+                    result_type=LookupResultType.ARTIFACT, artifact=build_log
                 ),
                 "amd64",
             )
@@ -461,14 +465,14 @@ class BlhcWorkflowTests(
         self.assertTrue(
             self.get_workflow(wr)._has_current_reference_qa_result(
                 LookupResult(
-                    result_type=CollectionItem.Types.BARE,
+                    result_type=LookupResultType.BARE,
                     collection_item=build_log_promise,
                 ),
                 "amd64",
             )
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         blhc_workflow = self.create_blhc_workflow(
             extra_task_data={
@@ -509,14 +513,22 @@ class BlhcWorkflowTests(
             ],
         )
 
-    def test_populate_with_upload(self) -> None:
-        upload_artifacts = self.playground.create_upload_artifacts()
+    def test_populate_with_uploads(self) -> None:
+        source_upload = self.playground.create_upload_artifacts(
+            binary=False
+        ).upload
+        binary_upload = self.playground.create_upload_artifacts(
+            source=False, binaries=[("hello", "amd64")]
+        ).upload
         log = self.package_build_log_artifacts[Architecture("amd64")]
+        self.playground.create_artifact_relation(
+            artifact=log, target=binary_upload
+        )
         wr = self.playground.create_workflow(
             task_name="blhc",
             task_data=BlhcWorkflowData(
-                source_artifact=upload_artifacts.upload.id,
-                package_build_logs=LookupMultiple((log.id,)),
+                source_artifact=source_upload.id,
+                binary_artifacts=LookupMultiple((binary_upload.id,)),
                 vendor="debian",
                 codename="sid",
             ),
@@ -653,10 +665,8 @@ class BlhcWorkflowTests(
             child, result=WorkRequest.Results.SUCCESS
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"sid@{CollectionCategory.QA_RESULTS}/latest:blhc_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).artifact,
             result,
@@ -712,17 +722,15 @@ class BlhcWorkflowTests(
             ),
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"sid@{CollectionCategory.QA_RESULTS}/latest:blhc_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).collection_item,
             racing_qa_result,
         )
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_callback_regression_analysis(self) -> None:
         self.playground.create_collection("sid", CollectionCategory.SUITE)
         self.playground.create_collection("sid", CollectionCategory.QA_RESULTS)
@@ -933,8 +941,8 @@ class BlhcWorkflowTests(
                 new_blhc_workflow, expected_callback_statuses
             )
 
-    @preserve_task_registry()
-    def test_compute_dynamic_data(self) -> None:
+    @preserve_db_task_registry()
+    def test_compute_system_required_tags(self) -> None:
         package_build_log_1 = self.playground.create_build_log_artifact(
             source="hello"
         )
@@ -943,10 +951,77 @@ class BlhcWorkflowTests(
         )
 
         workflow = self.create_blhc_workflow(
-            package_build_logs=[
-                f"{package_build_log_1.id}@artifacts",
-                f"{package_build_log_2.id}@artifacts",
-            ]
+            package_build_logs=LookupMultiple(
+                (
+                    f"{package_build_log_1.id}@artifacts",
+                    f"{package_build_log_2.id}@artifacts",
+                )
+            )
+        )
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
+    @preserve_db_task_registry()
+    def test_compute_dynamic_data_with_binary_artifacts(self) -> None:
+        binary_uploads = {
+            architecture: self.playground.create_upload_artifacts(
+                source=False, binaries=[("hello", architecture)]
+            ).upload
+            for architecture in ("amd64", "s390x")
+        }
+        package_build_logs = {
+            architecture: self.playground.create_build_log_artifact(
+                build_arch=architecture
+            )
+            for architecture in binary_uploads
+        }
+        for architecture in binary_uploads:
+            self.playground.create_artifact_relation(
+                artifact=package_build_logs[architecture],
+                target=binary_uploads[architecture],
+            )
+
+        workflow = self.create_blhc_workflow(
+            binary_artifacts=LookupMultiple(
+                tuple(
+                    f"{artifact.id}@artifacts"
+                    for artifact in binary_uploads.values()
+                )
+            )
+        )
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(workflow.work_request)),
+            BlhcWorkflowDynamicData(
+                subject="hello",
+                source_artifact_id=self.source_artifact.id,
+                binary_artifacts_ids=[
+                    artifact.id for artifact in binary_uploads.values()
+                ],
+                package_build_logs_ids=sorted(
+                    [artifact.id for artifact in package_build_logs.values()]
+                ),
+            ),
+        )
+
+    @preserve_db_task_registry()
+    def test_compute_dynamic_data_with_package_build_logs(self) -> None:
+        package_build_log_1 = self.playground.create_build_log_artifact(
+            source="hello"
+        )
+        package_build_log_2 = self.playground.create_build_log_artifact(
+            source="hello-traditional"
+        )
+
+        workflow = self.create_blhc_workflow(
+            package_build_logs=LookupMultiple(
+                (
+                    f"{package_build_log_1.id}@artifacts",
+                    f"{package_build_log_2.id}@artifacts",
+                )
+            )
         )
 
         self.assertEqual(
@@ -961,8 +1036,46 @@ class BlhcWorkflowTests(
             ),
         )
 
-    @preserve_task_registry()
-    def test_get_input_artifacts(self) -> None:
+    @preserve_db_task_registry()
+    def test_get_input_artifacts_with_binary_artifacts(self) -> None:
+        binary_uploads = {
+            architecture: self.playground.create_upload_artifacts(
+                source=False, binaries=[("hello", architecture)]
+            ).upload
+            for architecture in ("amd64", "s390x")
+        }
+
+        workflow = self.create_blhc_workflow(
+            binary_artifacts=LookupMultiple(
+                tuple(
+                    f"{artifact.id}@artifacts"
+                    for artifact in binary_uploads.values()
+                )
+            )
+        )
+
+        assert workflow.data.source_artifact is not None
+        assert workflow.data.binary_artifacts is not None
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=self.source_artifact.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[
+                        artifact.id for artifact in binary_uploads.values()
+                    ],
+                ),
+            ],
+        )
+
+    @preserve_db_task_registry()
+    def test_get_input_artifacts_with_package_build_logs(self) -> None:
         package_build_log_1 = self.playground.create_build_log_artifact(
             source="hello"
         )
@@ -971,13 +1084,16 @@ class BlhcWorkflowTests(
         )
 
         workflow = self.create_blhc_workflow(
-            package_build_logs=[
-                f"{package_build_log_1.id}@artifacts",
-                f"{package_build_log_2.id}@artifacts",
-            ],
+            package_build_logs=LookupMultiple(
+                (
+                    f"{package_build_log_1.id}@artifacts",
+                    f"{package_build_log_2.id}@artifacts",
+                )
+            )
         )
 
         assert workflow.data.source_artifact is not None
+        assert workflow.data.package_build_logs is not None
         self.assertEqual(
             workflow.get_input_artifacts(),
             [
@@ -997,7 +1113,7 @@ class BlhcWorkflowTests(
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_get_input_artifacts_without_source_artifact(self) -> None:
         package_build_log_1 = self.playground.create_build_log_artifact(
             source="hello"
@@ -1007,13 +1123,16 @@ class BlhcWorkflowTests(
         )
 
         workflow = self.create_blhc_workflow(
-            package_build_logs=[
-                f"{package_build_log_1.id}@artifacts",
-                f"{package_build_log_2.id}@artifacts",
-            ],
+            package_build_logs=LookupMultiple(
+                (
+                    f"{package_build_log_1.id}@artifacts",
+                    f"{package_build_log_2.id}@artifacts",
+                )
+            ),
             include_source_artifact=False,
         )
 
+        assert workflow.data.package_build_logs is not None
         self.assertEqual(
             workflow.get_input_artifacts(),
             [
