@@ -13,6 +13,7 @@ from collections.abc import Iterable
 from typing import Any, ClassVar
 from unittest.mock import call, patch
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -53,7 +54,7 @@ from debusine.tasks.models import (
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
@@ -252,7 +253,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
                 else self.new_source_artifact.id
             ),
             "binary_artifacts": binary_artifacts,
-            "original": original if original else str(self.debian_collection),
+            "original": original or str(self.debian_collection),
             "vendor": "debian",
             "codename": "bookworm",
         }
@@ -295,7 +296,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
     def new_binary_artifact_lookup(cls, arch: str) -> str:
         return f"{cls.new_binary_artifacts[Architecture(arch)].id}@artifacts"
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         # We'll deliberately add a binary package newer than the one we're
         # interested in, so allow this for test purposes
@@ -388,7 +389,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             )
 
         debdiff_work_requests = work_request.children.all()
-        bookworm = Collection.objects.get(name="bookworm")
+        bookworm = self.debian_collection
         bookworm_lookup = f"{bookworm.name}@{bookworm.category}"
 
         # Check work request to compare the source packages
@@ -489,7 +490,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             requires_artifact.call_args_list,
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_binary_artifacts_upload(self) -> None:
         upload_artifacts = self.playground.create_upload_artifacts(
             binaries=[("hello", "amd64"), ("bye", "amd64")],
@@ -546,7 +547,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_get_new_packages_by_name_arch_invalid_category(self) -> None:
         # Valid artifact categories are tested as part of other tests, such as
         # test_populate_binary_artifacts_upload(). The invalid category
@@ -563,7 +564,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
         ):
             workflow._get_new_packages_by_name_arch()
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_new_binary_artifacts_only_all_arch(self) -> None:
         work_request = self.create_debdiff_workflow(
             add_new_binary_artifacts_archs=["all"],
@@ -584,7 +585,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             workflow_data_json__step="debdiff-binaries-all"
         )
 
-        bookworm = Collection.objects.get(name="bookworm")
+        bookworm = self.debian_collection
         bookworm_lookup = f"{bookworm.name}@{bookworm.category}"
 
         self.assertEqual(
@@ -606,7 +607,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_origin_binary_artifacts_no_all_arch(self) -> None:
         work_request = self.create_debdiff_workflow(
             add_origin_binary_artifacts_archs=["amd64", "arm64"]
@@ -626,7 +627,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             workflow_data_json__step="debdiff-binaries-amd64"
         )
 
-        bookworm = Collection.objects.get(name="bookworm")
+        bookworm = self.debian_collection
         bookworm_lookup = f"{bookworm.name}@{bookworm.category}"
 
         self.assertEqual(
@@ -651,7 +652,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_new_package(self) -> None:
         """Run populate with a package not found in original collection."""
         work_request = self.create_debdiff_workflow(
@@ -664,7 +665,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
         # DebDiff task created.
         self.assertEqual(work_request.children.count(), 0)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_package_without_source_in_original(self) -> None:
         # Add binary packages that have the same name but are older
         # (to exercise returning only the newest)
@@ -706,7 +707,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             ][0]:
                 self.assertIn("1.0.0", binary_artifact)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_source_package_upload_artifact(self) -> None:
         upload_artifact = self.playground.create_upload_artifacts(
             src_name="hello",
@@ -729,7 +730,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             new_source_artifact, f"{upload_artifact.source.id}@artifacts"
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_one_debdiff_per_concrete_arch_includes_all_binaries(
         self,
     ) -> None:
@@ -750,7 +751,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             {f"debdiff-binaries-{arch}" for arch in expected_archs},
         )
 
-        bookworm = Collection.objects.get(name="bookworm")
+        bookworm = self.debian_collection
         bookworm_lookup = f"{bookworm.name}@{bookworm.category}"
 
         for arch in expected_archs:
@@ -777,7 +778,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
                 },
             )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_binary_for_all_does_not_include_arch_specific_originals(
         self,
     ) -> None:
@@ -799,7 +800,82 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             originals, [f"{debian_collection_lookup}/name:hello_1.0.0_all"]
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
+    def test_populate_origin_architecture_from_any_to_all(self) -> None:
+        """Binary debdiff tasks created if binaries change from any to all."""
+        work_request = self.create_debdiff_workflow(
+            add_origin_binary_artifacts_archs=["amd64", "arm64"],
+            add_new_binary_artifacts_archs=["all"],
+        )
+
+        debdiff_work_requests = work_request.children.all()
+
+        binary_wrs = debdiff_work_requests.filter(
+            workflow_data_json__step__startswith="debdiff-binaries"
+        )
+
+        expected_archs = {"amd64", "arm64"}
+
+        steps = binary_wrs.values_list("workflow_data_json__step", flat=True)
+
+        # Correct archs
+        self.assertCountEqual(
+            steps,
+            {f"debdiff-binaries-{arch}" for arch in expected_archs},
+        )
+
+        bookworm = self.debian_collection
+        bookworm_lookup = f"{bookworm.name}@{bookworm.category}"
+
+        # Correct task_data per architecture
+        amd64_work_request = debdiff_work_requests.get(
+            workflow_data_json__step="debdiff-binaries-amd64"
+        )
+
+        self.assertEqual(
+            amd64_work_request.task_data,
+            {
+                "environment": "debian/match:codename=bookworm",
+                "build_architecture": "amd64",
+                "extra_flags": [],
+                "input": {
+                    "binary_artifacts": [
+                        [f"{bookworm_lookup}/name:hello_1.0.0_amd64"],
+                        [self.new_binary_artifact_lookup("all")],
+                    ]
+                },
+            },
+        )
+
+        arm64_work_request = debdiff_work_requests.get(
+            workflow_data_json__step="debdiff-binaries-arm64"
+        )
+
+        self.assertEqual(
+            arm64_work_request.task_data,
+            {
+                "environment": "debian/match:codename=bookworm",
+                "build_architecture": "arm64",
+                "extra_flags": [],
+                "input": {
+                    "binary_artifacts": [
+                        [f"{bookworm_lookup}/name:hello_1.0.0_arm64"],
+                        [self.new_binary_artifact_lookup("all")],
+                    ]
+                },
+            },
+        )
+
+    @preserve_db_task_registry()
+    def test_compute_system_required_tags(self) -> None:
+        work_request = self.create_debdiff_workflow()
+        workflow = self.get_workflow(work_request)
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
+    @preserve_db_task_registry()
     def test_compute_dynamic_data_from_source(self) -> None:
         work_request = self.create_debdiff_workflow()
         workflow = self.get_workflow(work_request)
@@ -818,7 +894,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             ),
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_get_input_artifacts(self) -> None:
         work_request = self.create_debdiff_workflow()
         workflow = self.get_workflow(work_request)
@@ -851,7 +927,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_compute_dynamic_data_from_binary(self) -> None:
         # Add a binary package with older version: it will be discarded
         old_package = self.playground.create_minimal_binary_package_artifact(
@@ -891,7 +967,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             ),
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_compute_dynamic_data_package_not_in_origin_collection(
         self,
     ) -> None:
@@ -1001,12 +1077,12 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             ),
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_validate_input(self) -> None:
         # validate_input() is called and does not raise
         self.create_debdiff_workflow()
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_validate_input_invalid_original(self) -> None:
         work_request = self.create_debdiff_workflow(
             original="non-existing", expect_success=False
