@@ -11,6 +11,7 @@
 
 from typing import Any, ClassVar
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -124,6 +125,24 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
         ):
             workflow.validate_input()
 
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        wr = self.playground.create_workflow(
+            task_name="debian_pipeline",
+            task_data=DebianPipelineWorkflowData(
+                source_artifact=source_artifact.id,
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data includes a parameter summary."""
         source_artifact = self.playground.create_source_artifact(
@@ -539,16 +558,6 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                         },
                     }
                 ],
-                "package_build_logs": [
-                    {
-                        "collection": "internal@collections",
-                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                        "name_matcher": {
-                            "kind": CollectionItemMatcherKind.STARTSWITH,
-                            "value": "buildlog-",
-                        },
-                    }
-                ],
                 "codename": "bookworm",
                 "extra_repositories": [
                     {
@@ -604,16 +613,6 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                         },
                     }
                 ],
-                "package_build_logs": [
-                    {
-                        "collection": "internal@collections",
-                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                        "name_matcher": {
-                            "kind": CollectionItemMatcherKind.STARTSWITH,
-                            "value": "buildlog-",
-                        },
-                    }
-                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -664,16 +663,6 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                         },
                     }
                 ],
-                "package_build_logs": [
-                    {
-                        "collection": "internal@collections",
-                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                        "name_matcher": {
-                            "kind": CollectionItemMatcherKind.STARTSWITH,
-                            "value": "buildlog-",
-                        },
-                    }
-                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": True,
@@ -724,16 +713,6 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                         },
                     }
                 ],
-                "package_build_logs": [
-                    {
-                        "collection": "internal@collections",
-                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                        "name_matcher": {
-                            "kind": CollectionItemMatcherKind.STARTSWITH,
-                            "value": "buildlog-",
-                        },
-                    }
-                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -792,16 +771,6 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                         },
                     }
                 ],
-                "package_build_logs": [
-                    {
-                        "collection": "internal@collections",
-                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                        "name_matcher": {
-                            "kind": CollectionItemMatcherKind.STARTSWITH,
-                            "value": "buildlog-",
-                        },
-                    }
-                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -863,16 +832,6 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                         },
                     }
                 ],
-                "package_build_logs": [
-                    {
-                        "collection": "internal@collections",
-                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
-                        "name_matcher": {
-                            "kind": CollectionItemMatcherKind.STARTSWITH,
-                            "value": "buildlog-",
-                        },
-                    }
-                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -1108,7 +1067,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
             normal_autopkgtest.children.get(
                 task_type=TaskTypes.INTERNAL,
                 task_name="workflow",
-                workflow_data_json__step="regression-analysis",
+                workflow_data_json__step="regression-analysis-amd64",
             )
         )
         extra_autopkgtest = extra_qa.children.get(
@@ -2123,7 +2082,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
 
         # Completing the builds causes the publishing workflow to be
         # orchestrated.
-        binaries = self.simulate_sbuild_workflow_completion(sbuild_workflow)
+        binaries, _ = self.simulate_sbuild_workflow_completion(sbuild_workflow)
         package_publish.refresh_from_db()
         self.assertEqual(package_publish.status, WorkRequest.Statuses.PENDING)
         self.schedule_and_run_workflow(workflow)
@@ -2260,7 +2219,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
 
         # Completing the builds causes the publishing workflow to be
         # orchestrated.
-        binaries = self.simulate_sbuild_workflow_completion(
+        binaries, _ = self.simulate_sbuild_workflow_completion(
             initial_sbuild_workflow
         )
         self.playground.create_signing_input_artifact(
@@ -2429,7 +2388,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
 
         # Completing the build causes the signed publishing workflow to be
         # orchestrated.
-        signed_binaries = self.simulate_sbuild_workflow_completion(
+        signed_binaries, _ = self.simulate_sbuild_workflow_completion(
             sbuild_signed_amd64, section="admin", priority="extra"
         )
         package_publish_signed_amd64.refresh_from_db()
