@@ -15,6 +15,7 @@ from typing import Any, ClassVar
 from django.test import override_settings
 from django.utils import timezone
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -32,7 +33,6 @@ from debusine.db.models import (
     WorkRequest,
 )
 from debusine.db.models.work_requests import SkipWorkRequest
-from debusine.server.collections.lookup import lookup_single
 from debusine.server.scheduler import schedule
 from debusine.server.workflows import LintianWorkflow, WorkflowValidationError
 from debusine.server.workflows.base import orchestrate_workflow
@@ -65,7 +65,7 @@ from debusine.tasks.models import (
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class LintianWorkflowTests(
@@ -497,7 +497,7 @@ class LintianWorkflowTests(
             )
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
         architectures = ["amd64", "i386", "all"]
@@ -585,7 +585,7 @@ class LintianWorkflowTests(
                 ordered=False,
             )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_source_only(self) -> None:
         """Test populate with no binary artifacts."""
         root = self.orchestrate(
@@ -624,7 +624,7 @@ class LintianWorkflowTests(
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_without_architectures(self) -> None:
         """Test populate with only arch-indep packages."""
         binary_artifact = (
@@ -669,7 +669,7 @@ class LintianWorkflowTests(
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_without_architectures_arch_all_build_architecture(
         self,
     ) -> None:
@@ -717,7 +717,7 @@ class LintianWorkflowTests(
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_mixed_without_arch_all_build_architecture(self) -> None:
         """Mixed all/any binaries but without the usual arch-all arch."""
         binary_artifacts = {
@@ -775,7 +775,7 @@ class LintianWorkflowTests(
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_arch_dep_without_arch_all_build_architecture(
         self,
     ) -> None:
@@ -837,7 +837,7 @@ class LintianWorkflowTests(
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_with_no_architecture_overlap(self) -> None:
         """If architectures and binary artifacts are disjoint, do nothing."""
         root = self.orchestrate(
@@ -861,7 +861,7 @@ class LintianWorkflowTests(
 
         self.assertFalse(workflow.children.filter(task_name="lintian").exists())
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_no_binary_any_analysis(self) -> None:
         """Disabling binary-any analysis skips no-op work requests."""
         binary_artifacts = {
@@ -927,7 +927,7 @@ class LintianWorkflowTests(
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_upload(self) -> None:
         """The workflow accepts debian:upload source artifacts."""
         architectures = ["amd64", "i386", "all"]
@@ -1060,7 +1060,7 @@ class LintianWorkflowTests(
                     },
                 )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_has_current_reference_qa_result(self) -> None:
         """The workflow does nothing with a current reference QA result."""
         trixie_suite = self.playground.create_collection(
@@ -1109,7 +1109,7 @@ class LintianWorkflowTests(
 
         self.assertQuerySetEqual(workflow.children.all(), [])
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_no_previous_reference_qa_result(self) -> None:
         """The workflow produces a reference QA result if there is none."""
         trixie_suite = self.playground.create_collection(
@@ -1275,11 +1275,9 @@ class LintianWorkflowTests(
 
         for architecture, result in results.items():
             self.assertEqual(
-                lookup_single(
+                lintian.lookup_single(
                     f"trixie@{CollectionCategory.QA_RESULTS}/"
                     f"latest:lintian_hello_{architecture}",
-                    lintian.workspace,
-                    user=lintian.created_by,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact,
                 result,
@@ -1346,17 +1344,15 @@ class LintianWorkflowTests(
             ),
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"trixie@{CollectionCategory.QA_RESULTS}/"
                 f"latest:lintian_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).collection_item,
             racing_qa_result,
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_orchestrate_idempotent(self) -> None:
         """Calling orchestrate twice does not create new work requests."""
         binary = self.playground.create_minimal_binary_package_artifact(
@@ -1381,6 +1377,29 @@ class LintianWorkflowTests(
 
         self.assertQuerySetEqual(wr.children.all(), children, ordered=False)
 
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="lintian",
+            task_data=LintianWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple((binary_artifact.id,)),
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         source_artifact = self.playground.create_source_artifact(
             name="hello", version="1.0-1"
@@ -1469,7 +1488,7 @@ class LintianWorkflowTests(
         )
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_callback_regression_analysis(self) -> None:
         self.playground.create_collection("sid", CollectionCategory.SUITE)
         self.playground.create_collection("sid", CollectionCategory.QA_RESULTS)
