@@ -12,6 +12,7 @@
 from collections.abc import Sequence
 from typing import ClassVar
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     DebusineSigningInput,
@@ -21,7 +22,6 @@ from debusine.artifacts.models import (
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
-from debusine.server.collections.lookup import lookup_multiple, lookup_single
 from debusine.server.workflows import MakeSignedSourceWorkflow
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
@@ -43,7 +43,7 @@ from debusine.tasks.models import (
     LookupMultiple,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
@@ -156,6 +156,39 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
 
         return root
 
+    def test_compute_system_required_tags(self) -> None:
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+        upload_artifact = self.playground.create_upload_artifacts(
+            src_name="linux-base"
+        ).upload
+        signing_artifact = self.create_signing_template_artifact("amd64")
+
+        wr = self.playground.create_workflow(
+            task_name="make_signed_source",
+            task_data=MakeSignedSourceWorkflowData(
+                binary_artifacts=LookupMultiple(
+                    (binary_artifact.id, upload_artifact.id)
+                ),
+                vendor="debian",
+                codename="trixie",
+                signing_template_artifacts=LookupMultiple(
+                    (signing_artifact.id,)
+                ),
+                architectures=["amd64"],
+                purpose=KeyPurpose("uefi"),
+                key="ABC123",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         binary_artifact = (
             self.playground.create_minimal_binary_package_artifact(
@@ -237,7 +270,7 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
         # Expected architectures as per intersection of architectures
@@ -500,12 +533,8 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
             self.assertEqual(
                 [
                     result.artifact
-                    for result in lookup_multiple(
-                        unsigned,
-                        sign.workspace,
-                        user=sign.created_by,
-                        workflow_root=root,
-                        expect_type=LookupChildType.ARTIFACT,
+                    for result in sign.lookup_multiple(
+                        unsigned, expect_type=LookupChildType.ARTIFACT
                     )
                 ],
                 [signing_input],
@@ -526,13 +555,10 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
             self.assertEqual(
                 [
                     result.artifact
-                    for result in lookup_multiple(
+                    for result in assemble_signed_source.lookup_multiple(
                         LookupMultiple.model_validate(
                             assemble_signed_source.task_data["signed"]
                         ),
-                        assemble_signed_source.workspace,
-                        user=assemble_signed_source.created_by,
-                        workflow_root=root,
                         expect_type=LookupChildType.ARTIFACT,
                     )
                 ],
@@ -552,11 +578,8 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 assemble_signed_source, result=WorkRequest.Results.SUCCESS
             )
             self.assertEqual(
-                lookup_single(
+                sbuild_workflow.lookup_single(
                     sbuild_workflow.task_data["input"]["source_artifact"],
-                    sbuild_workflow.workspace,
-                    user=sbuild_workflow.created_by,
-                    workflow_root=root,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact,
                 upload_artifacts.upload,
@@ -577,7 +600,7 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 sbuild_workflow.status, WorkRequest.Statuses.RUNNING
             )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_with_artifacts(self) -> None:
         """
         Test populate: only children for specified architectures are generated.
