@@ -208,12 +208,15 @@ class BlhcWorkflowDataTests(
     """Tests for :py:class:`BlhcWorkflowData`."""
 
     def make_data(
-        self, source_artifact: LookupSingle | None = 1, **kwargs: Any
+        self,
+        source_artifact: LookupSingle | None = 1,
+        package_build_logs: LookupMultiple | None = LookupMultiple((2,)),
+        **kwargs: Any,
     ) -> BlhcWorkflowData:
         """Make a `BlhcWorkflowData`."""
         return BlhcWorkflowData(
             source_artifact=source_artifact,
-            package_build_logs=LookupMultiple((2,)),
+            package_build_logs=package_build_logs,
             vendor="debian",
             codename="sid",
             **kwargs,
@@ -242,6 +245,23 @@ class BlhcWorkflowDataTests(
                 enable_regression_tracking=True,
             )
 
+    def test_package_build_logs(self) -> None:
+        self.make_data(
+            binary_artifacts=LookupMultiple((1,)), package_build_logs=None
+        )
+        self.make_data(package_build_logs=LookupMultiple((2,)))
+
+        error_msg = (
+            "Exactly one of binary_artifacts or package_build_logs must be set"
+        )
+        with self.assertRaisesRegex(ValueError, error_msg):
+            self.make_data(package_build_logs=None)
+        with self.assertRaisesRegex(ValueError, error_msg):
+            self.make_data(
+                binary_artifacts=LookupMultiple((1,)),
+                package_build_logs=LookupMultiple((2,)),
+            )
+
 
 class QAWorkflowDataTests(RegressionTrackingWorkflowDataTests[QAWorkflowData]):
     """Tests for :py:class:`QAWorkflowData`."""
@@ -252,7 +272,6 @@ class QAWorkflowDataTests(RegressionTrackingWorkflowDataTests[QAWorkflowData]):
             {
                 "source_artifact": 1,
                 "binary_artifacts": LookupMultiple((2,)),
-                "package_build_logs": LookupMultiple((3,)),
                 "vendor": "debian",
                 "codename": "sid",
                 **kwargs,
@@ -287,16 +306,6 @@ class QAWorkflowDataTests(RegressionTrackingWorkflowDataTests[QAWorkflowData]):
         ):
             self.make_data(enable_debdiff=True)
 
-    def test_enable_blhc_consistency(self) -> None:
-        """Test enable_blhc_consistency validation."""
-        self.make_data()
-
-        with self.assertRaisesRegex(
-            ValueError,
-            '"package_build_logs" is required if "enable_blhc" is set',
-        ):
-            self.make_data(package_build_logs=None, enable_blhc=True)
-
     def test_default_fail_on(self) -> None:
         """Test default for ``fail_on``."""
         self.assertEqual(self.make_data().fail_on, QAFailOn.FAILURE)
