@@ -14,6 +14,7 @@ from collections.abc import Sequence
 from dataclasses import dataclass
 from typing import Any, ClassVar
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import CollectionCategory, TaskTypes
 from debusine.db.models import (
     Artifact,
@@ -44,7 +45,7 @@ from debusine.tasks.models import (
     LookupMultiple,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 @dataclass
@@ -264,7 +265,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         self.assertEqual(update_suites.status, WorkRequest.Statuses.RUNNING)
         self.assertTrue(update_suites.children.exists())
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_source_and_binary(self) -> None:
         """Test population with both source and binary artifacts."""
         source_artifact = self.playground.create_source_artifact(
@@ -297,7 +298,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         sbuild_workflow = root.children.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        binaries = self.simulate_sbuild_workflow_completion(sbuild_workflow)
+        binaries, _ = self.simulate_sbuild_workflow_completion(sbuild_workflow)
 
         package_publish.refresh_from_db()
         self.assertEqual(package_publish.status, WorkRequest.Statuses.PENDING)
@@ -316,7 +317,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         self.assert_copies(copy_collection_items, target_suite, expected)
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_source_only(self) -> None:
         """Test population with only a source artifact."""
         source_artifact = self.playground.create_source_artifact(
@@ -361,7 +362,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_source_upload_only(self) -> None:
         """Test population with only a source upload artifact."""
         upload_artifacts = self.playground.create_upload_artifacts(
@@ -396,7 +397,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_missing_binary_fields(self) -> None:
         """Population warns about missing binary component/section/priority."""
         source_artifact = self.playground.create_source_artifact(
@@ -453,7 +454,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_non_main_component(self) -> None:
         """Population handles sections with non-main components."""
         upload_artifacts = self.playground.create_upload_artifacts(
@@ -488,7 +489,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_honours_suite_variables(self) -> None:
         """Population honours ``suite_variables``."""
         source_artifact = self.playground.create_source_artifact(
@@ -541,7 +542,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_no_update_indexes(self) -> None:
         """The workflow can be told not to update indexes."""
         upload_artifacts = self.playground.create_upload_artifacts(
@@ -583,7 +584,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
             ).exists()
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_package_build_logs(self) -> None:
         """Test population with binary artifacts and copying build logs."""
         source_workspace = self.playground.create_workspace(name="source")
@@ -643,7 +644,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         sbuild_workflow = root.children.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        binaries = self.simulate_sbuild_workflow_completion(sbuild_workflow)
+        binaries, _ = self.simulate_sbuild_workflow_completion(sbuild_workflow)
 
         package_publish.refresh_from_db()
         self.assertEqual(package_publish.status, WorkRequest.Statuses.PENDING)
@@ -692,7 +693,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_package_task_history(self) -> None:
         """Test population with binary artifacts and copying build logs."""
         source_workspace = self.playground.create_workspace(name="source")
@@ -752,7 +753,7 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         sbuild_workflow = root.children.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        binaries = self.simulate_sbuild_workflow_completion(sbuild_workflow)
+        binaries, _ = self.simulate_sbuild_workflow_completion(sbuild_workflow)
 
         package_publish.refresh_from_db()
         self.assertEqual(package_publish.status, WorkRequest.Statuses.PENDING)
@@ -798,6 +799,27 @@ class PackagePublishWorkflowTests(WorkflowTestBase[PackagePublishWorkflow]):
         )
         self.assert_updates_indexes(package_publish, target_suite)
 
+    def test_compute_system_required_tags(self) -> None:
+        upload_artifact = (
+            self.playground.create_upload_artifacts(src_name="hello")
+        ).upload
+        target_suite = self.create_suite_collection("bookworm")
+
+        workflow = self.create_package_publish_workflow(
+            task_data=PackagePublishWorkflowData(
+                binary_artifacts=LookupMultiple((upload_artifact.id,)),
+                target_suite=(
+                    f"{target_suite.name}@{CollectionCategory.SUITE}"
+                ),
+                source_artifact=self.source_artifact.id,
+            )
+        )
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data_binary_package(self) -> None:
         upload_artifact = (
             self.playground.create_upload_artifacts(src_name="hello")
