@@ -13,13 +13,7 @@ from collections.abc import Sequence
 from typing import Any
 from unittest.mock import call, patch
 
-from debusine.server.workflows.tests.helpers import WorkflowTestBase
-
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -28,16 +22,16 @@ from debusine.artifacts.models import (
     DebusinePromise,
     TaskTypes,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, RuntimeParameter
 from debusine.db.models import (
     Artifact,
+    Collection,
     CollectionItem,
     TaskDatabase,
     WorkRequest,
     WorkflowTemplate,
     default_workspace,
 )
-from debusine.server.tasks.models import PackageUploadTarget
 from debusine.server.workflows import (
     PackageUploadWorkflow,
     WorkflowValidationError,
@@ -45,11 +39,14 @@ from debusine.server.workflows import (
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     PackageUploadWorkflowData,
+    PackageUploadWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
+from debusine.server.workflows.tests.helpers import WorkflowTestBase
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
-    BaseDynamicTaskData,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LookupMultiple,
     SbuildData,
     SbuildInput,
@@ -76,22 +73,9 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         )
         return self.get_workflow(wr)
 
-    def create_source_package(self, *, name: str = "hello") -> Artifact:
-        """Create a minimal `debian:source-package` artifact."""
-        artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.SOURCE_PACKAGE,
-            data={
-                "name": name,
-                "version": "1.0",
-                "type": "dpkg",
-                "dsc_fields": {},
-            },
-        )
-        return artifact
-
     def create_source_binary_artifacts(
         self, architectures: Sequence[str] | None = None
-    ) -> tuple[Artifact, list[Artifact]]:
+    ) -> tuple[Artifact, list[Artifact], Collection]:
         """Return one source and a list of binary packages (one per arch)."""
         if architectures is None:
             architectures = ("amd64",)
@@ -99,24 +83,26 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
             "test", CollectionCategory.TEST
         )
 
-        source = self.create_source_package()
-        binaries = [
-            self.playground.create_minimal_binary_packages_artifact(
-                "hello", "1.0-1", "1.0-1", architecture
-            )
-            for architecture in architectures
-        ]
-        for binary in binaries:
-            CollectionItem.objects.create_from_artifact(
-                binary,
-                parent_collection=collection,
-                name=f"binary-{binary.data['architecture']}",
-                created_by_user=self.playground.get_default_user(),
-                data={},
-                created_by_workflow=None,
+        source = self.playground.create_minimal_source_package_artifact()
+        uploads = []
+        for architecture in architectures:
+            upload = self.playground.create_upload_artifacts(
+                source=False,
+                binary=True,
+                binaries=[("hello", architecture)],
             )
+            uploads.append(upload.upload)
+            for binary in upload.binaries:
+                CollectionItem.objects.create_from_artifact(
+                    binary,
+                    parent_collection=collection,
+                    name=f"binary-{binary.data['deb_fields']['Architecture']}",
+                    created_by_user=self.playground.get_default_user(),
+                    data={},
+                    created_by_workflow=None,
+                )
 
-        return source, binaries
+        return source, uploads, collection
 
     def test_validate_input_source_binary_artifacts_not_set(self) -> None:
         """Invalid: source_artifact and binaries_artifacts not set."""
@@ -134,7 +120,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         self,
     ) -> None:
         """Raise WorkflowValidationError: vendor and codename required."""
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
 
         w = self.create_package_upload_workflow(
             extra_task_data={"source_artifact": source.id},
@@ -150,7 +136,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_create_orchestrator(self) -> None:
         """A PackageUploadWorkflow can be instantiated."""
-        source_artifact, [binary_artifact] = (
+        source_artifact, [binary_artifact], _ = (
             self.create_source_binary_artifacts(["arm64"])
         )
         target_distribution = "debian:bookworm"
@@ -167,8 +153,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertEqual(w.data.source_artifact, source_artifact.id)
         self.assertEqual(
-            w.data.binary_artifacts,
-            LookupMultiple.parse_obj([binary_artifact.id]),
+            w.data.binary_artifacts, LookupMultiple((binary_artifact.id,))
         )
 
         self.assertEqual(w.data.target_distribution, target_distribution)
@@ -179,11 +164,12 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
             name="package_upload",
             workspace=default_workspace(),
             task_name="package_upload",
+            runtime_parameters=RuntimeParameter.ANY,
         )
 
         wr = self.playground.create_workflow(
             template,
-            task_data={**{"target": "sftp://upload.example.org/queue"}, **data},
+            task_data={"target": "sftp://upload.example.org/queue", **data},
         )
         self.assertTrue(orchestrate_workflow(wr))
 
@@ -197,7 +183,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         -makesourcepackageupload
         -package_upload
         """
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
 
         with patch.object(
             PackageUploadWorkflow,
@@ -230,8 +216,8 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         self.assertEqual(
             make_source.task_data,
             {
+                "build_architecture": "amd64",
                 "environment": "debian/match:codename=bullseye",
-                "host_architecture": "amd64",
                 "input": {"source_artifact": source.id},
                 "since_version": "1.0",
                 "target_distribution": "debian:bookworm",
@@ -325,7 +311,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_populate_package_uploads_binaries_only(self) -> None:
         """The workflow create children package uploads: binaries only."""
-        _, [binary] = self.create_source_binary_artifacts()
+        _, [binary], _ = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -348,13 +334,13 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         assert parent.internal_collection is not None
 
         architectures = ("amd64", "i386")
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
         for architecture in architectures:
             sbuild = parent.create_child_worker(
                 task_name="sbuild",
                 task_data=SbuildData(
                     input=SbuildInput(source_artifact=source.id),
-                    host_architecture=architecture,
+                    build_architecture=architecture,
                     environment="debian/match:codename=bookworm",
                 ),
                 workflow_data=WorkRequestWorkflowData(
@@ -469,7 +455,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_upload_debsign(self) -> None:
         """Upload signed artifacts."""
-        source, [binary] = self.create_source_binary_artifacts()
+        source, [binary], _ = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -556,7 +542,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_upload_externaldebsign(self) -> None:
         """Upload external signed artifacts."""
-        source, [binary] = self.create_source_binary_artifacts()
+        source, [binary], _ = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -630,7 +616,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_upload_split_signing(self) -> None:
         """Upload artifacts: developer signs source, Debusine signs binary."""
-        source, [binary] = self.create_source_binary_artifacts()
+        source, [binary], _ = self.create_source_binary_artifacts()
 
         wr = self.orchestrate(
             data={
@@ -763,7 +749,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_upload_delayed(self) -> None:
         """Upload to a delayed queue."""
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
 
         wr = self.orchestrate(
             data={
@@ -790,10 +776,12 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_orchestrate_idempotent(self) -> None:
         """Calling orchestrate twice does not create new work requests."""
-        source, binaries = self.create_source_binary_artifacts(
+        self.playground.create_collection(
+            "test", CollectionCategory.TASK_CONFIGURATION
+        )
+        source, _, binaries_collection = self.create_source_binary_artifacts(
             architectures=("amd64", "i386")
         )
-        binaries_collection = binaries[0].parent_collections.earliest("id")
 
         # Force a predictable default ordering.
         with patch(
@@ -802,6 +790,9 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         ):
             wr = self.orchestrate(
                 data={
+                    "task_configuration": (
+                        f"test@{CollectionCategory.TASK_CONFIGURATION}"
+                    ),
                     "source_artifact": source.id,
                     "binary_artifacts": {
                         "collection": binaries_collection.id,
@@ -824,17 +815,37 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertQuerySetEqual(wr.children.all(), children, ordered=False)
 
-    def test_compute_dynamic_data_source_artifact(self) -> None:
-        source_artifact = self.create_source_package(name="hello")
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = (
+            self.playground.create_minimal_source_package_artifact()
+        )
 
         wr = self.playground.create_workflow(
             task_name="package_upload",
             task_data=PackageUploadWorkflowData(
                 source_artifact=source_artifact.id,
-                target=pydantic.parse_obj_as(
-                    PackageUploadTarget,
-                    "sftp://example.org",
-                ),
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
+    def test_compute_dynamic_data_source_artifact(self) -> None:
+        source_artifact = (
+            self.playground.create_minimal_source_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                source_artifact=source_artifact.id,
+                target="sftp://example.org",
                 vendor="debian",
                 codename="trixie",
             ),
@@ -843,8 +854,51 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello → sftp://example.org"
+            PackageUploadWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello → sftp://example.org",
+                source_artifact_id=source_artifact.id,
+                binary_artifacts_ids=[],
+            ),
+        )
+
+    def test_compute_dynamic_data_source_is_promise(self) -> None:
+        parent = self.playground.create_workflow()
+        assert parent.internal_collection is not None
+
+        signing = parent.create_child_worker(
+            task_name="noop",
+        )
+        self.playground.create_bare_data_item(
+            parent.internal_collection,
+            "source",
+            category=BareDataCategory.PROMISE,
+            data=DebusinePromise(
+                promise_work_request_id=signing.id,
+                promise_workflow_id=parent.id,
+                promise_category=ArtifactCategory.SOURCE_PACKAGE,
+                **{"source_package_name": "hello"},  # noqa: PIE804
+            ),
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                source_artifact="internal@collections/name:source",
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+            parent=parent,
+        )
+
+        workflow = self.get_workflow(wr)
+        self.assertEqual(
+            workflow.build_dynamic_data(TaskDatabase(wr)),
+            PackageUploadWorkflowDynamicData(
+                parameter_summary="hello → sftp://example.org",
+                subject="hello",
+                binary_artifacts_ids=[],
             ),
         )
 
@@ -858,12 +912,11 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         wr = self.playground.create_workflow(
             task_name="package_upload",
             task_data=PackageUploadWorkflowData(
-                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
-                source_artifact=None,
-                target=pydantic.parse_obj_as(
-                    PackageUploadTarget,
-                    "sftp://example.org",
+                binary_artifacts=LookupMultiple.model_validate(
+                    [binary_artifact.id]
                 ),
+                source_artifact=None,
+                target="sftp://example.org",
                 vendor="debian",
                 codename="trixie",
             ),
@@ -872,7 +925,84 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello → sftp://example.org"
+            PackageUploadWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello → sftp://example.org",
+                source_artifact_id=None,
+                binary_artifacts_ids=[binary_artifact.id],
             ),
         )
+
+    def test_get_input_artifacts_source_artifact(self) -> None:
+        source_artifact = (
+            self.playground.create_minimal_source_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                source_artifact=source_artifact.id,
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        workflow.dynamic_data = workflow.build_dynamic_data(
+            TaskDatabase(workflow.work_request)
+        )
+
+        assert workflow.data.source_artifact is not None
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=source_artifact.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[],
+                ),
+            ],
+        )
+
+    def test_get_input_artifacts_binary_artifacts(self) -> None:
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                binary_artifacts=LookupMultiple.model_validate(
+                    [binary_artifact.id]
+                ),
+                source_artifact=None,
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        workflow.dynamic_data = workflow.build_dynamic_data(
+            TaskDatabase(workflow.work_request)
+        )
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[binary_artifact.id],
+                )
+            ],
+        )
