@@ -15,6 +15,7 @@ from typing import Any, ClassVar
 from django.test import override_settings
 from django.utils import timezone
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -32,7 +33,6 @@ from debusine.db.models import (
     WorkRequest,
 )
 from debusine.db.models.work_requests import SkipWorkRequest
-from debusine.server.collections.lookup import lookup_single
 from debusine.server.scheduler import schedule
 from debusine.server.workflows import WorkflowValidationError
 from debusine.server.workflows.base import orchestrate_workflow
@@ -64,7 +64,7 @@ from debusine.tasks.models import (
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class PiupartsWorkflowTests(
@@ -388,7 +388,7 @@ class PiupartsWorkflowTests(
             self.get_workflow(wr)._has_current_reference_qa_result("amd64")
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
         # Expected architectures as per intersection of architectures of
@@ -778,11 +778,9 @@ class PiupartsWorkflowTests(
             child, result=WorkRequest.Results.SUCCESS
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"sid@{CollectionCategory.QA_RESULTS}/"
                 f"latest:piuparts_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).artifact,
             result,
@@ -851,18 +849,139 @@ class PiupartsWorkflowTests(
             ),
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"sid@{CollectionCategory.QA_RESULTS}/"
                 f"latest:piuparts_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).collection_item,
             racing_qa_result,
         )
 
+    def test_populate_reference_qa_result_uses_build_architecture(self) -> None:
+        sid_suite = self.playground.create_collection(
+            "sid", CollectionCategory.SUITE
+        )
+        sid_qa_results = self.playground.create_collection(
+            "sid", CollectionCategory.QA_RESULTS
+        )
+        # Add a spurious QA result for "all" to make sure it isn't considered.
+        self.add_qa_result(
+            sid_qa_results,
+            "hello",
+            "1.0-1",
+            "all",
+            binary_packages={"hello": "1.0-1"},
+            distribution="debian:sid",
+        )
+
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello",
+                srcpkg_version="1.0-1",
+                architecture="all",
+            )
+        )
+        source_item = sid_suite.manager.add_artifact(
+            source_artifact,
+            user=self.playground.get_default_user(),
+            variables={"component": "main", "section": "devel"},
+        )
+        wr = self.playground.create_workflow(
+            task_name="piuparts",
+            task_data=PiupartsWorkflowData(
+                prefix="reference-qa-result|",
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple((binary_artifact.id,)),
+                vendor="debian",
+                codename="sid",
+                arch_all_build_architecture="amd64",
+                qa_suite=f"sid@{CollectionCategory.SUITE}",
+                reference_qa_results=f"sid@{CollectionCategory.QA_RESULTS}",
+                update_qa_results=True,
+            ),
+        )
+
+        self.get_workflow(wr).populate()
+
+        [child] = wr.children.all()
+        self.assertEqual(child.status, WorkRequest.Statuses.BLOCKED)
+        self.assertEqual(child.task_type, TaskTypes.WORKER)
+        self.assertEqual(child.task_name, "piuparts")
+        self.assertEqual(
+            child.task_data,
+            {
+                "backend": BackendType.UNSHARE,
+                "base_tgz": "debian/match:codename=sid",
+                "build_architecture": "amd64",
+                "environment": "debian/match:codename=sid",
+                "extra_repositories": None,
+                "input": {
+                    "binary_artifacts": [f"{binary_artifact.id}@artifacts"]
+                },
+            },
+        )
+        qa_result_action = ActionUpdateCollectionWithArtifacts(
+            artifact_filters={"category": ArtifactCategory.PIUPARTS},
+            collection=f"sid@{CollectionCategory.QA_RESULTS}",
+            variables={
+                "package": "hello",
+                "version": "1.0-1",
+                "architecture": "amd64",
+                "timestamp": int(source_item.created_at.timestamp()),
+                "work_request_id": child.id,
+            },
+        )
+        self.assert_work_request_event_reactions(
+            child,
+            on_assignment=[
+                ActionSkipIfLookupResultChanged(
+                    lookup=(
+                        f"sid@{CollectionCategory.QA_RESULTS}/"
+                        f"latest:piuparts_hello_amd64"
+                    ),
+                    collection_item_id=None,
+                    promise_name=None,
+                )
+            ],
+            on_failure=[qa_result_action],
+            on_success=[
+                ActionUpdateCollectionWithArtifacts(
+                    artifact_filters={"category": ArtifactCategory.PIUPARTS},
+                    collection="internal@collections",
+                    name_template="reference-qa-result|piuparts-amd64",
+                ),
+                qa_result_action,
+            ],
+        )
+
+        # Completing the work request stores the QA result.
+        result, _ = self.playground.create_artifact(
+            category=ArtifactCategory.PIUPARTS,
+            data=DebianPiuparts(
+                piuparts_version="1.6.0",
+                binary_packages={"hello": "1.0-1"},
+                architecture="amd64",
+                distribution="debian:sid",
+            ),
+            work_request=child,
+        )
+        self.playground.advance_work_request(
+            child, result=WorkRequest.Results.SUCCESS
+        )
+        self.assertEqual(
+            child.lookup_single(
+                f"sid@{CollectionCategory.QA_RESULTS}/"
+                f"latest:piuparts_hello_amd64",
+                expect_type=LookupChildType.ARTIFACT,
+            ).artifact,
+            result,
+        )
+
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_callback_regression_analysis(self) -> None:
         self.playground.create_collection("sid", CollectionCategory.SUITE)
         self.playground.create_collection("sid", CollectionCategory.QA_RESULTS)
@@ -1106,6 +1225,176 @@ class PiupartsWorkflowTests(
                 new_piuparts_workflow, expected_callback_statuses
             )
 
+    @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
+    @preserve_db_task_registry()
+    def test_callback_regression_analysis_build_architecture(self) -> None:
+        self.playground.create_collection("sid", CollectionCategory.SUITE)
+        self.playground.create_collection("sid", CollectionCategory.QA_RESULTS)
+
+        reference_source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        reference_binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello", srcpkg_version="1.0-1", architecture="all"
+            )
+        )
+        new_source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-2"
+        )
+        new_binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello", srcpkg_version="1.0-2", architecture="all"
+            )
+        )
+
+        root = self.playground.create_workflow(mark_running=True)
+        reference = self.orchestrate(
+            task_data=PiupartsWorkflowData(
+                prefix="reference-qa-result|",
+                qa_suite=f"sid@{CollectionCategory.SUITE}",
+                reference_qa_results=f"sid@{CollectionCategory.QA_RESULTS}",
+                update_qa_results=True,
+                source_artifact=reference_source_artifact.id,
+                binary_artifacts=LookupMultiple(
+                    (reference_binary_artifact.id,)
+                ),
+                vendor="debian",
+                codename="sid",
+                arch_all_build_architecture="amd64",
+            ),
+            source_artifact=reference_source_artifact,
+            architectures=["all"],
+            parent=root,
+            pipeline_task_name="referencepipeline",
+        )
+        new = self.orchestrate(
+            task_data=PiupartsWorkflowData(
+                reference_prefix="reference-qa-result|",
+                qa_suite=f"sid@{CollectionCategory.SUITE}",
+                reference_qa_results=f"sid@{CollectionCategory.QA_RESULTS}",
+                enable_regression_tracking=True,
+                source_artifact=new_source_artifact.id,
+                binary_artifacts=LookupMultiple(
+                    ("internal@collections/name:build-all",)
+                ),
+                vendor="debian",
+                codename="sid",
+                arch_all_build_architecture="amd64",
+            ),
+            source_artifact=new_source_artifact,
+            architectures=["all"],
+            parent=root,
+        )
+
+        # Unblock the piuparts tasks.
+        for sbuild in reference.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+        ).children.filter(task_type=TaskTypes.WORKER, task_name="sbuild"):
+            self.playground.advance_work_request(
+                sbuild, result=WorkRequest.Results.SUCCESS
+            )
+        for sbuild in new.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+        ).children.filter(task_type=TaskTypes.WORKER, task_name="sbuild"):
+            self.playground.advance_work_request(
+                sbuild, result=WorkRequest.Results.SUCCESS
+            )
+
+        reference_piuparts_workflow = reference.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="piuparts"
+        )
+        reference_task = reference_piuparts_workflow.children.get(
+            task_type=TaskTypes.WORKER, task_name="piuparts"
+        )
+        reference_result, _ = self.playground.create_artifact(
+            category=ArtifactCategory.PIUPARTS,
+            data=DebianPiuparts(
+                piuparts_version="1.6.0",
+                binary_packages={"hello": "1.0-1"},
+                architecture="amd64",
+                distribution="debian:sid",
+            ),
+            workspace=reference.workspace,
+            work_request=reference_task,
+        )
+        self.playground.create_artifact_relation(
+            reference_result,
+            reference_binary_artifact,
+            ArtifactRelation.Relations.RELATES_TO,
+        )
+        new_piuparts_workflow = new.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="piuparts"
+        )
+        new_task = new_piuparts_workflow.children.get(
+            task_type=TaskTypes.WORKER, task_name="piuparts"
+        )
+        new_result, _ = self.playground.create_artifact(
+            category=ArtifactCategory.PIUPARTS,
+            data=DebianPiuparts(
+                piuparts_version="1.6.0",
+                binary_packages={"hello": "1.0-1"},
+                architecture="amd64",
+                distribution="debian:sid",
+            ),
+            workspace=new.workspace,
+            work_request=new_task,
+        )
+        self.playground.create_artifact_relation(
+            new_result,
+            new_binary_artifact,
+            ArtifactRelation.Relations.RELATES_TO,
+        )
+
+        self.assertIsNone(new_piuparts_workflow.output_data)
+        self.assertEqual(schedule(), [])
+        self.playground.advance_work_request(
+            reference_task, result=WorkRequest.Results.SUCCESS
+        )
+        self.assertEqual(schedule(), [])
+        self.playground.advance_work_request(
+            new_task, result=WorkRequest.Results.SUCCESS
+        )
+
+        self.schedule_and_run_workflow_callback(new_piuparts_workflow)
+
+        new_piuparts_workflow.refresh_from_db()
+        self.assert_regression_analysis(
+            new_piuparts_workflow,
+            "1.0-1",
+            {"amd64": reference_result},
+            "1.0-2",
+            {"amd64": new_result},
+            [("amd64", RegressionAnalysisStatus.STABLE, None)],
+        )
+
+        self.assert_callback_statuses(
+            new_piuparts_workflow, {"amd64": WorkRequest.Statuses.COMPLETED}
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = self.playground.create_source_artifact(name="hello")
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+        wr = self.playground.create_workflow(
+            task_name="piuparts",
+            task_data=PiupartsWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple((binary_artifact.id,)),
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data_binary_package(self) -> None:
         source_artifact = self.playground.create_source_artifact(name="hello")
         binary_artifact = (
@@ -1226,7 +1515,7 @@ class PiupartsWorkflowTests(
     def test_compute_dynamic_data_binary_promises(self) -> None:
         source_artifact = self.playground.create_source_artifact()
 
-        with preserve_task_registry():
+        with preserve_db_task_registry():
 
             class ExamplePipeline(
                 SampleWorkflow[BaseWorkflowData, BaseDynamicTaskData]
