@@ -14,6 +14,7 @@ from typing import Any, ClassVar
 
 from django.utils import timezone
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -57,7 +58,7 @@ from debusine.tasks.models import (
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
@@ -75,9 +76,14 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             name="hello"
         )
         cls.binary_artifact = (
-            cls.playground.create_minimal_binary_package_artifact()
+            cls.playground.create_minimal_binary_package_artifact(
+                architecture="amd64"
+            )
         )
         cls.package_build_log = cls.playground.create_build_log_artifact()
+        cls.playground.create_artifact_relation(
+            artifact=cls.package_build_log, target=cls.binary_artifact
+        )
 
     def create_qa_workflow(
         self,
@@ -89,7 +95,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         task_data = {
             "source_artifact": self.source_artifact.pk,
             "binary_artifacts": [self.binary_artifact.pk],
-            "package_build_logs": [self.package_build_log.pk],
             "vendor": "debian",
             "codename": "bookworm",
         }
@@ -173,12 +178,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
 
                 data = {
                     "source_artifact": source_artifact_id,
-                    "binary_artifacts": [
-                        self.binary_artifact.id,
-                    ],
-                    "package_build_logs": [
-                        self.package_build_log.id,
-                    ],
+                    "binary_artifacts": [self.binary_artifact.id],
                     "enable_blhc": False,
                     "vendor": "debian",
                     "codename": "bookworm",
@@ -220,7 +220,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         ):
             workflow.validate_input()
 
-    def test_compute_dynamic_data(self) -> None:
+    def test_compute_system_required_tags(self) -> None:
         wr = self.playground.create_workflow(
             task_name="qa",
             task_data=QAWorkflowData(
@@ -228,8 +228,28 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 binary_artifacts=LookupMultiple(
                     ("internal@collections/name:build-amd64",)
                 ),
-                package_build_logs=LookupMultiple(
-                    ("internal@collections/name:buildlog-amd64",)
+                vendor="debian",
+                codename="sid",
+            ),
+            internal_collection_artifacts=[
+                (self.binary_artifact, {"name": "build-amd64"}),
+                (self.package_build_log, {"name": "buildlog-amd64"}),
+            ],
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
+    def test_compute_dynamic_data(self) -> None:
+        wr = self.playground.create_workflow(
+            task_name="qa",
+            task_data=QAWorkflowData(
+                source_artifact=self.source_artifact.id,
+                binary_artifacts=LookupMultiple(
+                    ("internal@collections/name:build-amd64",)
                 ),
                 vendor="debian",
                 codename="sid",
@@ -248,7 +268,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 parameter_summary="hello_1.0-1",
                 source_artifact_id=self.source_artifact.id,
                 binary_artifacts_ids=[self.binary_artifact.id],
-                package_build_logs_ids=[self.package_build_log.id],
             ),
         )
 
@@ -267,9 +286,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 binary_artifacts=LookupMultiple(
                     ("internal@collections/name:build-amd64",)
                 ),
-                package_build_logs=LookupMultiple(
-                    ("internal@collections/name:buildlog-amd64",)
-                ),
                 reference_source_artifact=reference_source_artifact.id,
                 vendor="debian",
                 codename="sid",
@@ -289,7 +305,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 source_artifact_id=new_source_artifact.id,
                 reference_source_artifact_id=reference_source_artifact.id,
                 binary_artifacts_ids=[self.binary_artifact.id],
-                package_build_logs_ids=[self.package_build_log.id],
             ),
         )
 
@@ -310,9 +325,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                     binary_artifacts=LookupMultiple(
                         ("internal@collections/name:build-amd64",)
                     ),
-                    package_build_logs=LookupMultiple(
-                        ("internal@collections/name:buildlog-amd64",)
-                    ),
                     vendor="debian",
                     codename="sid",
                 ),
@@ -382,9 +394,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                     ("internal@collections/name:build-amd64",)
                 ),
                 reference_source_artifact=reference_source_artifact.id,
-                package_build_logs=LookupMultiple(
-                    ("internal@collections/name:buildlog-amd64",)
-                ),
                 enable_blhc=False,
                 vendor="debian",
                 codename="sid",
@@ -395,13 +404,11 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             ],
         )
         workflow = self.get_workflow(wr)
-        assert workflow.data.package_build_logs is not None
 
         workflow.dynamic_data = QAWorkflowDynamicData(
             source_artifact_id=upload_artifacts.source.id,
             binary_artifacts_ids=[self.binary_artifact.id],
             reference_source_artifact_id=reference_source_artifact.id,
-            package_build_logs_ids=[self.package_build_log.id],
         )
 
         assert workflow.data.reference_source_artifact is not None
@@ -423,11 +430,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                     label="reference_source_artifact",
                     artifact_id=reference_source_artifact.id,
                 ),
-                InputArtifactMultiple(
-                    lookup=workflow.data.package_build_logs,
-                    label="package_build_logs",
-                    artifact_ids=[self.package_build_log.id],
-                ),
             ],
         )
 
@@ -455,7 +457,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             workflow_utils.source_package(workflow), upload_artifacts.source
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
         architectures = ["amd64"]
@@ -660,7 +662,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_use_available_architectures(self) -> None:
         """
         Test populate uses available architectures.
@@ -690,7 +692,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             autopkgtest.task_data["architectures"], ["all", "amd64", "i386"]
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_disable_autopkgtest_lintian_piuparts(self) -> None:
         """Populate does not create autopkgtest, lintian nor piuparts."""
         source_artifact = self.playground.create_source_artifact(name="hello")
@@ -750,7 +752,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             task_data=QAWorkflowData(
                 source_artifact=f"{artifacts.upload.id}@artifacts",
                 binary_artifacts=LookupMultiple((artifacts.upload.id,)),
-                package_build_logs=LookupMultiple((self.package_build_log.id,)),
                 architectures=architectures,
                 vendor="debian",
                 codename="sid",
@@ -774,6 +775,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             },
             "blhc": {
                 "source_artifact": expected_src_artifact,
+                "binary_artifacts": expected_bin_artifacts,
             },
             "debdiff": {
                 "source_artifact": expected_src_artifact,
@@ -801,7 +803,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                         child.task_data.get(k), v, f"task_data[{k}]"
                     )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_enable_reverse_dependencies_autopkgtest(self) -> None:
         """Populate can enable reverse_dependencies_autopkgtest."""
         architectures = ["amd64"]
@@ -926,7 +928,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             ],
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_update_qa_results(self) -> None:
         """Populate can update QA results."""
         architectures = ["amd64"]
@@ -988,12 +990,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 # in qa_suite, but that doesn't really matter for testing
                 # purposes.
                 "binary_artifacts": ["internal@collections/name:build-amd64"],
-                # This would normally be the corresponding build log
-                # artifacts in a reference collection, but that doesn't
-                # really matter for testing purposes.
-                "package_build_logs": [
-                    "internal@collections/name:buildlog-amd64"
-                ],
                 "architectures": ["amd64"],
                 "qa_suite": f"sid@{CollectionCategory.SUITE}",
                 "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
@@ -1218,16 +1214,14 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         )
 
         assert workflow.parent is not None
-        self.assertEqual(blhc.status, WorkRequest.Statuses.RUNNING)
+        self.assertEqual(blhc.status, WorkRequest.Statuses.BLOCKED)
         self.assertEqual(
             blhc.task_data,
             {
                 "arch_all_build_architecture": "amd64",
+                "binary_artifacts": ["internal@collections/name:build-amd64"],
                 "codename": "bookworm",
                 "enable_regression_tracking": False,
-                "package_build_logs": [
-                    "internal@collections/name:buildlog-amd64"
-                ],
                 "prefix": "reference-qa-result|",
                 "qa_suite": f"sid@{CollectionCategory.SUITE}",
                 "reference_prefix": "",
@@ -1245,29 +1239,14 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 "step": "blhc",
             },
         )
-        self.assertQuerySetEqual(blhc.dependencies.all(), [])
-
-        # BlhcWorkflow.populate() was called and created its tasks.
         self.assertQuerySetEqual(
-            blhc.children.order_by("id").values_list(
-                "task_type",
-                "task_name",
-                "status",
-                "dependencies__task_type",
-                "dependencies__task_name",
-            ),
-            [
-                (
-                    TaskTypes.WORKER,
-                    "blhc",
-                    WorkRequest.Statuses.BLOCKED,
-                    TaskTypes.WORKER,
-                    "sbuild",
-                )
-            ],
+            blhc.dependencies.values_list("task_type", "task_name"),
+            [(TaskTypes.WORKER, "sbuild")],
         )
+        # No children yet; see test_populate_blhc.
+        self.assertFalse(blhc.children.exists())
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_enable_regression_tracking(self) -> None:
         """Populate can run in a mode that tracks regressions."""
         architectures = ["amd64"]
@@ -1328,12 +1307,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 # in qa_suite, but that doesn't really matter for testing
                 # purposes.
                 "binary_artifacts": ["internal@collections/name:build-amd64"],
-                # This would normally be the corresponding build log
-                # artifacts in a reference collection, but that doesn't
-                # really matter for testing purposes.
-                "package_build_logs": [
-                    "internal@collections/name:buildlog-amd64"
-                ],
                 "architectures": ["amd64"],
                 "qa_suite": f"sid@{CollectionCategory.SUITE}",
                 "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
@@ -1454,7 +1427,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         blhc = workflow.children.get(
             task_name="blhc", task_type=TaskTypes.WORKFLOW
         )
-        self.assertEqual(blhc.status, WorkRequest.Statuses.RUNNING)
+        self.assertEqual(blhc.status, WorkRequest.Statuses.BLOCKED)
         self.assertDictContainsAll(
             blhc.task_data,
             {
@@ -1472,7 +1445,25 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 "step": "blhc",
             },
         )
-        self.assertQuerySetEqual(blhc.dependencies.all(), [])
+        self.assertQuerySetEqual(
+            blhc.dependencies.values_list("task_type", "task_name"),
+            [(TaskTypes.WORKER, "sbuild")],
+        )
+        # No children yet.
+        self.assertFalse(blhc.children.exists())
+
+        # Complete sbuild to unblock the blhc workflow.
+        assert workflow.parent is not None
+        self.simulate_sbuild_workflow_completion(
+            workflow.parent.children.get(
+                task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+            )
+        )
+        blhc.refresh_from_db()
+        self.assertEqual(blhc.status, WorkRequest.Statuses.PENDING)
+        self.schedule_and_run_workflow(workflow.parent)
+        blhc.refresh_from_db()
+        self.assertEqual(blhc.status, WorkRequest.Statuses.RUNNING)
         self.assertTrue(blhc.children.exists())
 
         regression_analysis = workflow.children.get(
@@ -1484,7 +1475,9 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             regression_analysis.status, WorkRequest.Statuses.BLOCKED
         )
         regression_analysis_callbacks = [
-            wr.children.get(workflow_data_json__step="regression-analysis")
+            wr.children.get(
+                workflow_data_json__step="regression-analysis-amd64"
+            )
             for wr in (
                 autopkgtest,
                 sub_autopkgtest,
@@ -1499,7 +1492,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             ordered=False,
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_architectures_allowed(self) -> None:
         """Populate uses architectures and architectures_allowed."""
         source_artifact = self.playground.create_source_artifact(name="hello")
@@ -1557,7 +1550,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_architectures_deny(self) -> None:
         """Populate uses architectures and architectures_denylist."""
         source_artifact = self.playground.create_source_artifact(name="hello")
@@ -1616,7 +1609,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_subworkflow_configuration(self) -> None:
         """Populate configures sub-workflows according to specified data."""
         self.playground.create_debian_environment()
@@ -1660,7 +1653,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             piuparts.task_data["environment"], "debian/match:codename=trixie"
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_debdiff(self) -> None:
         """Populate creates a debdiff workflow."""
         self.playground.create_collection(
@@ -1754,7 +1747,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
 
         # Create build artifacts and complete sbuild to unblock the debdiff
         # workflow.
-        binaries = self.simulate_sbuild_workflow_completion(sbuild_workflow)
+        binaries, _ = self.simulate_sbuild_workflow_completion(sbuild_workflow)
 
         debdiff_workflow.refresh_from_db()
         self.assertEqual(debdiff_workflow.status, WorkRequest.Statuses.PENDING)
@@ -1804,7 +1797,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_blhc(self) -> None:
         """Populate enables blhc."""
         architectures = ["amd64"]
@@ -1818,9 +1811,6 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             extra_data={
                 "source_artifact": source_artifact.id,
                 "binary_artifacts": ["internal@collections/name:build-amd64"],
-                "package_build_logs": [
-                    "internal@collections/name:buildlog-amd64"
-                ],
                 "architectures": ["amd64"],
                 "qa_suite": f"sid@{CollectionCategory.SUITE}",
                 "enable_blhc": True,
@@ -1829,10 +1819,35 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             sbuild_workflow_source_artifact=source_artifact,
         )
 
+        assert workflow.parent is not None
+        sbuild_workflow = workflow.parent.children.get(
+            task_name="sbuild", task_type=TaskTypes.WORKFLOW
+        )
+        self.playground.advance_work_request(sbuild_workflow, mark_running=True)
+
         blhc_workflow = workflow.children.get(
             task_name="blhc", task_type=TaskTypes.WORKFLOW
         )
 
+        # No children and blocked: it needs the binary artifacts first.
+        self.assertFalse(blhc_workflow.children.exists())
+        self.assertEqual(blhc_workflow.status, WorkRequest.Statuses.BLOCKED)
+        self.assertQuerySetEqual(
+            blhc_workflow.dependencies.values_list("task_type", "task_name"),
+            [(TaskTypes.WORKER, "sbuild")],
+        )
+
+        # Create build artifacts and complete sbuild to unblock the blhc
+        # workflow.
+        _, build_logs = self.simulate_sbuild_workflow_completion(
+            sbuild_workflow
+        )
+
+        blhc_workflow.refresh_from_db()
+        self.assertEqual(blhc_workflow.status, WorkRequest.Statuses.PENDING)
+        self.schedule_and_run_workflow(workflow.parent)
+
+        blhc_workflow.refresh_from_db()
         self.assertEqual(blhc_workflow.children.count(), 1)
         self.assertEqual(blhc_workflow.status, WorkRequest.Statuses.RUNNING)
 
@@ -1852,9 +1867,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 "build_architecture": "amd64",
                 "environment": "debian/match:codename=bookworm",
                 "extra_flags": [],
-                "input": {
-                    "artifact": "internal@collections/name:buildlog-amd64"
-                },
+                "input": {"artifact": f"{build_logs['amd64'].id}@artifacts"},
             },
         )
 
@@ -1983,7 +1996,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         )
         return workflow, reference_autopkgtest_artifact, new_source_artifact
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_callback_regression_analysis_success(self) -> None:
         """The collation callback succeeds if there were no regressions."""
         workflow, reference_autopkgtest_artifact, new_source_artifact = (
@@ -2070,7 +2083,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             },
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_callback_regression_analysis_failure(self) -> None:
         """The collation callback fails if there were regressions."""
         workflow, reference_autopkgtest_artifact, new_source_artifact = (
