@@ -12,6 +12,7 @@
 from collections.abc import Iterable, Mapping, Sequence
 from typing import Any, ClassVar
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.local_artifact import Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -23,7 +24,6 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, Collection, TaskDatabase, WorkRequest
-from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows import (
     ReverseDependenciesAutopkgtestWorkflow,
     WorkflowValidationError,
@@ -59,7 +59,7 @@ from debusine.tasks.models import (
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class ReverseDependenciesAutopkgtestWorkflowTests(
@@ -346,12 +346,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         """Assert that reverse-dependencies of a workflow are correct."""
         expected = []
         for source_lookup, binary_lookups in lookups.items():
-            source = lookup_single(
+            source = workflow.work_request.lookup_single(
                 source_lookup,
-                workflow.workspace,
-                user=workflow.work_request.created_by,
                 default_category=CollectionCategory.SUITE,
-                workflow_root=workflow.work_request.workflow_root,
                 expect_type=LookupChildType.ARTIFACT,
             ).artifact
             source_data = source.create_data()
@@ -363,12 +360,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
             )
             expected_binaries = []
             for binary_lookup in binary_lookups:
-                binary = lookup_single(
+                binary = workflow.work_request.lookup_single(
                     binary_lookup,
-                    workflow.workspace,
-                    user=workflow.work_request.created_by,
                     default_category=CollectionCategory.SUITE,
-                    workflow_root=workflow.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact
                 binary_data = binary.create_data()
@@ -679,7 +673,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
             task_name="reverse_dependencies_autopkgtest",
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         """The workflow populates child work requests."""
         depends = self.create_source_package(
@@ -846,7 +840,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         children = list(WorkRequest.objects.filter(parent=rdep_autopkgtest))
         self.assertEqual(len(children), 2)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_passes_regression_tracking_parameters(self) -> None:
         """The workflow passes on parameters related to regression tracking."""
         self.playground.create_collection(
@@ -1118,6 +1112,34 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         new_children = list(WorkRequest.objects.filter(parent=new))
         self.assertEqual(len(new_children), 1)
 
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        context_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        wr = self.playground.create_workflow(
+            task_name="reverse_dependencies_autopkgtest",
+            task_data=ReverseDependenciesAutopkgtestWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple((binary_artifact.id,)),
+                context_artifacts=LookupMultiple((context_artifact.id,)),
+                vendor="debian",
+                codename="trixie",
+                qa_suite="sid@debian:suite",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         source_artifact = self.playground.create_source_artifact(
             name="hello", version="1.0-1"
