@@ -11,6 +11,7 @@
 
 from typing import Any
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -139,6 +140,26 @@ class SbuildWorkflowTests(
         assert w.data.extra_repositories is not None
         self.assertEqual(w.data.extra_repositories[0].suite, "bookworm")
 
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1", architectures={"all"}
+        )
+        self.playground.create_debian_environment(codename="bookworm")
+        wr = self.playground.create_workflow(
+            task_name="sbuild",
+            task_data=SbuildWorkflowData(
+                input=SbuildInput(source_artifact=source_artifact.id),
+                target_distribution="debian:bookworm",
+                architectures=["all"],
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data includes a parameter summary."""
         source_artifact = self.playground.create_source_artifact(
