@@ -11,6 +11,7 @@
 
 from typing import Any
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -53,73 +54,108 @@ class UpdateEnvironmentsWorkflowTests(
             runtime_parameters=RuntimeParameter.ANY,
         )
 
-    def test_create_orchestrator(self) -> None:
-        """Instantiate an UpdateEnvironmentsWorkflow."""
-        wr = self.playground.create_workflow(
-            task_data={
-                "vendor": "debian",
-                "targets": [
-                    {
-                        "codenames": ["bookworm"],
-                        "architectures": ["amd64"],
-                        "mmdebstrap_template": {},
-                    }
-                ],
-            },
-            workspace=default_workspace(),
-            task_name="update_environments",
-        )
-        w = self.get_workflow(wr)
-        self.assertEqual(w.data.vendor, "debian")
-        self.assertEqual(len(w.data.targets), 1)
-        self.assertEqual(w.data.targets[0].codenames, ["bookworm"])
-        self.assertEqual(w.data.targets[0].architectures, ["amd64"])
-        self.assertEqual(w.data.targets[0].mmdebstrap_template, {})
+    def add_target(
+        self,
+        codename: str = "bookworm",
+        codenames: list[str] | None = None,
+        aliases: list[str] | None = None,
+        architectures: list[str] | None = None,
+        backends: list[str] | None = None,
+        variants: list[str | None] | None = None,
+        mmdebstrap_template: dict[str, Any] | None = None,
+        simplesystemimagebuild_template: dict[str, Any] | None = None,
+    ) -> None:
+        if not hasattr(self, "targets"):
+            self.targets = []
+        # Compute default values for parameters
+        if codenames is None:
+            codenames = [codename]
+        if architectures is None:
+            architectures = ["amd64"]
+        if aliases is None:
+            aliases = [f"{codenames[0]}-alias"]
+        if (
+            mmdebstrap_template is None
+            and simplesystemimagebuild_template is None
+        ):
+            # Ensure we build something, defaults to tarball with mmdebstrap
+            mmdebstrap_template = {}
+        if mmdebstrap_template is not None:
+            mmdebstrap_template.setdefault(
+                "bootstrap_repositories",
+                [{"mirror": "https://deb.debian.org/debian"}],
+            )
+        if simplesystemimagebuild_template is not None:
+            simplesystemimagebuild_template.setdefault(
+                "bootstrap_repositories",
+                [{"mirror": "https://deb.debian.org/debian"}],
+            )
+            simplesystemimagebuild_template.setdefault(
+                "disk_image",
+                {
+                    "format": "qcow2",
+                    "partitions": [{"size": 2, "filesystem": "ext4"}],
+                },
+            )
 
-    def test_create_work_requests(self) -> None:
-        """Create specified work requests."""
+        # Generate the target entry
+        target: dict[str, Any] = {
+            "codenames": codenames,
+            "codename_aliases": {codenames[0]: aliases},
+            "architectures": architectures,
+            "mmdebstrap_template": mmdebstrap_template,
+            "simplesystemimagebuild_template": simplesystemimagebuild_template,
+        }
+        if backends is not None:
+            target["backends"] = backends
+        if variants is not None:
+            target["variants"] = variants
+
+        self.targets.append(target)
+
+    def orchestrate_workflow(self) -> WorkRequest:
         template = self.create_update_environments_template(
-            static_parameters={
-                "vendor": "debian",
-                "targets": [
-                    {
-                        "codenames": "bookworm",
-                        "architectures": ["amd64"],
-                        "mmdebstrap_template": {
-                            "bootstrap_repositories": [
-                                {"mirror": "https://deb.debian.org/debian"}
-                            ]
-                        },
-                    },
-                    {
-                        "codenames": ["trixie"],
-                        "codename_aliases": {"trixie": ["sid"]},
-                        "variants": [None, "autopkgtest", "sbuild"],
-                        "backends": [
-                            BackendType.UNSHARE,
-                            BackendType.INCUS_LXC,
-                        ],
-                        "architectures": ["amd64", "arm64"],
-                        "simplesystemimagebuild_template": {
-                            "bootstrap_options": {"variant": "minbase"},
-                            "bootstrap_repositories": [
-                                {"mirror": "https://deb.debian.org/debian"}
-                            ],
-                            "disk_image": {
-                                "format": "qcow2",
-                                "partitions": [
-                                    {"size": 2, "filesystem": "ext4"}
-                                ],
-                            },
-                        },
-                    },
-                ],
-            }
+            static_parameters={"vendor": "debian", "targets": self.targets}
         )
         wr = self.playground.create_workflow(template, task_data={})
         self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
         self.assertTrue(orchestrate_workflow(wr))
 
+        return wr
+
+    def test_create_orchestrator(self) -> None:
+        """Instantiate an UpdateEnvironmentsWorkflow."""
+        self.add_target(mmdebstrap_template={})
+        wr = self.orchestrate_workflow()
+        w = self.get_workflow(wr)
+
+        self.assertEqual(w.data.vendor, "debian")
+        self.assertEqual(len(w.data.targets), 1)
+        self.assertEqual(w.data.targets[0].codenames, ["bookworm"])
+        self.assertEqual(w.data.targets[0].architectures, ["amd64"])
+        assert w.data.targets[0].mmdebstrap_template is not None
+        self.assertIsNotNone(
+            w.data.targets[0].mmdebstrap_template.get("bootstrap_repositories")
+        )
+
+    def test_create_work_requests(self) -> None:
+        """Create specified work requests."""
+        self.add_target(mmdebstrap_template={}, aliases=[])
+        self.add_target(
+            codename="trixie",
+            aliases=["sid"],
+            architectures=["amd64", "arm64"],
+            variants=[None, "autopkgtest", "sbuild"],
+            backends=[
+                BackendType.UNSHARE,
+                BackendType.INCUS_LXC,
+            ],
+            simplesystemimagebuild_template={
+                "bootstrap_options": {"variant": "minbase"}
+            },
+        )
+        wr = self.orchestrate_workflow()
+
         children = list(wr.children.order_by("id"))
         self.assertEqual(len(children), 3)
 
@@ -150,9 +186,11 @@ class UpdateEnvironmentsWorkflowTests(
         self.assertEqual(
             children[0].workflow_data_json,
             {
-                "display_name": "Build tarball for bookworm/amd64",
-                "step": "mmdebstrap-bookworm-amd64",
-                "group": "bookworm",
+                "display_name": (
+                    "Build tarball for bookworm / amd64 / no-backend "
+                    "/ no-variant"
+                ),
+                "step": "mmdebstrap/bookworm/amd64/no-backend/no-variant",
             },
         )
         self.assert_work_request_event_reactions(
@@ -214,9 +252,14 @@ class UpdateEnvironmentsWorkflowTests(
         self.assertEqual(
             children[1].workflow_data_json,
             {
-                "display_name": "Build image for trixie/amd64",
-                "step": "simplesystemimagebuild-trixie-amd64",
-                "group": "trixie [None,autopkgtest,sbuild]",
+                "display_name": (
+                    "Build image for trixie / amd64 / unshare|"
+                    "incus-lxc / no-variant|autopkgtest|sbuild"
+                ),
+                "step": (
+                    "simplesystemimagebuild/trixie/amd64/"
+                    "unshare|incus-lxc/no-variant|autopkgtest|sbuild"
+                ),
             },
         )
         self.assert_work_request_event_reactions(
@@ -240,9 +283,14 @@ class UpdateEnvironmentsWorkflowTests(
         self.assertEqual(
             children[2].workflow_data_json,
             {
-                "display_name": "Build image for trixie/arm64",
-                "step": "simplesystemimagebuild-trixie-arm64",
-                "group": "trixie [None,autopkgtest,sbuild]",
+                "display_name": (
+                    "Build image for trixie / arm64 / "
+                    "unshare|incus-lxc / no-variant|autopkgtest|sbuild"
+                ),
+                "step": (
+                    "simplesystemimagebuild/trixie/arm64/"
+                    "unshare|incus-lxc/no-variant|autopkgtest|sbuild"
+                ),
             },
         )
         self.assert_work_request_event_reactions(
@@ -256,24 +304,9 @@ class UpdateEnvironmentsWorkflowTests(
 
     def test_event_reaction(self) -> None:
         """The event reaction created by the workflow can be handled."""
-        template = self.create_update_environments_template(
-            static_parameters={
-                "vendor": "debian",
-                "targets": [
-                    {
-                        "codenames": "bookworm",
-                        "architectures": ["amd64"],
-                        "mmdebstrap_template": {
-                            "bootstrap_repositories": [
-                                {"mirror": "https://deb.debian.org/debian"}
-                            ]
-                        },
-                    }
-                ],
-            }
-        )
-        wr = self.playground.create_workflow(template, task_data={})
-        self.assertTrue(orchestrate_workflow(wr))
+        self.add_target()
+        wr = self.orchestrate_workflow()
+
         child = WorkRequest.objects.get(parent=wr)
         child.status = WorkRequest.Statuses.PENDING
         collection = self.playground.create_collection(
@@ -295,57 +328,61 @@ class UpdateEnvironmentsWorkflowTests(
         assert item is not None
         self.assertEqual(item.artifact, tarball)
 
-    def test_compute_dynamic_data_one_target(self) -> None:
-        wr = self.playground.create_workflow(
-            task_data={
-                "vendor": "debian",
-                "targets": [
-                    {
-                        "codenames": ["bookworm"],
-                        "architectures": ["amd64"],
-                        "mmdebstrap_template": {},
-                    }
-                ],
-            },
-            workspace=default_workspace(),
-            task_name="update_environments",
+    def test_compute_system_required_tags(self) -> None:
+        self.add_target()
+        wr = self.orchestrate_workflow()
+        workflow = self.get_workflow(wr)
+
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
         )
+
+    def test_compute_dynamic_data(self) -> None:
+        self.add_target(architectures=["amd64", "i386"])
+        self.add_target(architectures=["arm64"])
+        wr = self.orchestrate_workflow()
         w = self.get_workflow(wr)
 
         self.assertEqual(
             w.compute_dynamic_data(TaskDatabase(wr)),
             BaseDynamicTaskData(
                 subject="debian",
-                parameter_summary="debian (1 target)",
+                parameter_summary="debian (3 environments)",
             ),
         )
 
-    def test_compute_dynamic_data_two_targets(self) -> None:
-        wr = self.playground.create_workflow(
-            task_data={
-                "vendor": "debian",
-                "targets": [
-                    {
-                        "codenames": ["bookworm"],
-                        "architectures": ["amd64"],
-                        "mmdebstrap_template": {},
-                    },
-                    {
-                        "codenames": ["bookworm"],
-                        "architectures": ["arm64"],
-                        "mmdebstrap_template": {},
-                    },
-                ],
-            },
-            workspace=default_workspace(),
-            task_name="update_environments",
+    def test_combination_logic_single_target(self) -> None:
+        self.add_target(
+            codenames=["bullseye", "bookworm", "trixie", "sid"],
+            architectures=["amd64", "arm64", "armhf"],
+            mmdebstrap_template={},
+            simplesystemimagebuild_template={},
         )
-        w = self.get_workflow(wr)
+        wr = self.orchestrate_workflow()
+        # 4 codenames, 3 architectures, 2 tasks (tarball+image)
+        self.assertEqual(wr.children.count(), 4 * 3 * 2)
 
-        self.assertEqual(
-            w.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="debian",
-                parameter_summary="debian (2 targets)",
-            ),
+    def test_combination_logic_multiple_targets(self) -> None:
+        # Add two variations just on the backend
+        self.add_target(backends=[BackendType.UNSHARE])
+        self.add_target(backends=[BackendType.INCUS_LXC])
+        # Add three variations just on the variant
+        self.add_target(variants=[None, "lintian"])
+        self.add_target(variants=["sbuild"])
+        self.add_target(variants=["variant1", "variant2"])
+        wr = self.orchestrate_workflow()
+        # We should have 5 work requests
+        self.assertEqual(wr.children.count(), 5)
+
+    def test_combination_logic_duplicate_between_targets(self) -> None:
+        # Each target represents 4 combinations
+        self.add_target(
+            codenames=["bookworm", "trixie"], architectures=["amd64", "i386"]
         )
+        self.add_target(
+            codenames=["trixie", "sid"], architectures=["i386", "s390x"]
+        )
+        wr = self.orchestrate_workflow()
+        # But 1 of them (trixie/i386) is common so 7 unique environments
+        self.assertEqual(wr.children.count(), 7)
