@@ -13,13 +13,9 @@ import logging
 
 from django.utils import timezone
 
-from debusine.artifacts.models import (
-    ArtifactCategory,
-    CollectionCategory,
-    TaskTypes,
-)
+from debusine.artifacts.models import CollectionCategory, TaskTypes
 from debusine.assets import KeyPurpose
-from debusine.assets.models import AssetCategory, SigningKeyData
+from debusine.assets.models import SigningKeyData
 from debusine.db.models import (
     Collection,
     CollectionItem,
@@ -53,9 +49,7 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
     def create_release_item(self, suite: Collection) -> CollectionItem:
         """Create a minimal debian:repository-index item in the suite."""
         return CollectionItem.objects.create_from_artifact(
-            self.playground.create_artifact(
-                category=ArtifactCategory.REPOSITORY_INDEX, data={}
-            )[0],
+            self.playground.create_repository_index("Release"),
             parent_collection=suite,
             name="index:Release",
             data={"path": "Release"},
@@ -207,16 +201,13 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
             for name in ("one", "two")
         ]
         for suite in suites:
-            archive.manager.add_collection(
-                suite, user=self.playground.get_default_user()
-            )
             self.create_binary_package_item(suite)
         workflow = self.playground.create_workflow(
             task_name="update_suites", workspace=workspace
         )
 
         workflow.set_current()
-        orchestrate_workflow(workflow)
+        self.assertTrue(orchestrate_workflow(workflow))
 
         self.assertQuerySetEqual(
             workflow.children.values_list(
@@ -281,19 +272,16 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
         generate_key = workflow.children.get(
             task_type=TaskTypes.SIGNING, task_name="generatekey"
         )
-        signing_key = self.playground.create_asset(
-            AssetCategory.SIGNING_KEY,
-            data=SigningKeyData(
-                purpose=KeyPurpose.OPENPGP_REPOSITORY,
-                fingerprint="ABC123",
-                public_key="Fake public key",
-                description=f"Generate signing key for {workspace}",
-            ),
+        signing_key = self.playground.create_signing_key_asset(
+            purpose=KeyPurpose.OPENPGP_REPOSITORY,
+            fingerprint="ABC123",
+            public_key="Fake public key",
+            description=f"Generate signing key for {workspace}",
             workspace=workspace,
             created_by_work_request=generate_key,
         )
-        self.assertTrue(
-            generate_key.mark_completed(WorkRequest.Results.SUCCESS)
+        self.playground.advance_work_request(
+            generate_key, result=WorkRequest.Results.SUCCESS
         )
 
         generated_key = workflow.children.get(
@@ -301,7 +289,7 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
         )
         self.assertEqual(generated_key.status, WorkRequest.Statuses.PENDING)
 
-        orchestrate_workflow(workflow)
+        self.assertTrue(orchestrate_workflow(workflow))
 
         generated_key.refresh_from_db()
         self.assertEqual(generated_key.status, WorkRequest.Statuses.COMPLETED)
@@ -346,34 +334,56 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
             logger="debusine.server.workflows.update_suites",
             level=logging.INFO,
         ):
-            orchestrate_workflow(workflow)
+            self.assertTrue(orchestrate_workflow(workflow))
 
         self.assertFalse(workflow.children.exists())
 
+    def test_populate_not_exported(self) -> None:
+        """``populate`` skips suites that are marked as not to be exported."""
+        workspace = self.scenario.workspace
+        self.playground.add_user(
+            self.scenario.workspace_owners, self.scenario.user
+        )
+        self.playground.create_singleton_collection(CollectionCategory.ARCHIVE)
+        suites = [
+            self.playground.create_collection(
+                name, CollectionCategory.SUITE, data=data
+            )
+            for name, data in (("one", {}), ("two", {"exported": False}))
+        ]
+        for suite in suites:
+            self.create_binary_package_item(suite)
+        workflow = self.playground.create_workflow(
+            task_name="update_suites", workspace=workspace
+        )
+
+        workflow.set_current()
+        orchestrate_workflow(workflow)
+
+        self.assertQuerySetEqual(
+            workflow.children.filter(
+                task_type=TaskTypes.WORKFLOW, task_name="update_suite"
+            ).values_list("task_data__suite_collection", flat=True),
+            [suites[0].id],
+        )
+
     def test_populate_already_exists(self) -> None:
         """``populate`` skips work requests that already exist."""
         workspace = self.scenario.workspace
         self.playground.add_user(
             self.scenario.workspace_owners, self.scenario.user
         )
-        archive = self.playground.create_singleton_collection(
-            CollectionCategory.ARCHIVE
-        )
+        self.playground.create_singleton_collection(CollectionCategory.ARCHIVE)
         suites = [
             self.playground.create_collection(name, CollectionCategory.SUITE)
             for name in ("one", "two")
         ]
-        for suite in suites:
-            archive.manager.add_collection(
-                suite, user=self.playground.get_default_user()
-            )
         self.create_binary_package_item(suites[0])
         workflow = self.playground.create_workflow(
-            task_name="update_suites", workspace=workspace
+            task_name="update_suites", workspace=workspace, mark_running=True
         )
 
-        workflow.mark_running()
-        orchestrate_workflow(workflow)
+        self.assertTrue(orchestrate_workflow(workflow))
 
         expected_children = [
             (
@@ -428,7 +438,7 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
 
         self.create_binary_package_item(suites[1])
 
-        orchestrate_workflow(workflow)
+        self.assertTrue(orchestrate_workflow(workflow))
 
         expected_children.append(
             (
@@ -470,16 +480,12 @@ class UpdateSuitesWorkflowTests(WorkflowTestBase[UpdateSuitesWorkflow]):
             for name in ("one", "two")
         ]
         for suite in suites:
-            archive.manager.add_collection(
-                suite, user=self.playground.get_default_user()
-            )
             self.create_binary_package_item(suite)
         workflow = self.playground.create_workflow(
-            task_name="update_suites", workspace=workspace
+            task_name="update_suites", workspace=workspace, mark_running=True
         )
 
-        workflow.mark_running()
-        orchestrate_workflow(workflow)
+        self.assertTrue(orchestrate_workflow(workflow))
 
         self.assertQuerySetEqual(
             workflow.children.values_list(
