@@ -19,6 +19,7 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import CollectionItem, default_workspace
+from debusine.db.models.tasks import DBTask
 from debusine.db.playground import scenarios
 from debusine.server.tasks.wait.models import ExternalDebsignData
 from debusine.server.workflows import NoopWorkflow, Workflow
@@ -28,7 +29,6 @@ from debusine.server.workflows.models import (
 )
 from debusine.server.workflows.tests.helpers import SampleWorkflow
 from debusine.signing.tasks.models import DebsignData
-from debusine.tasks import BaseTask
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     BaseDynamicTaskData,
@@ -36,8 +36,7 @@ from debusine.tasks.models import (
     InputArtifactSingle,
     LookupMultiple,
 )
-from debusine.test.django import TestCase
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import TestCase, preserve_db_task_registry
 
 
 class WorkflowTests(TestCase):
@@ -45,7 +44,7 @@ class WorkflowTests(TestCase):
 
     scenario = scenarios.DefaultContext()
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_create(self) -> None:
         """Test instantiating a Workflow."""
 
@@ -74,11 +73,8 @@ class WorkflowTests(TestCase):
             ),
         )
         self.assertEqual(wf.work_request, wr)
-        self.assertEqual(wf.workspace, default_workspace())
-        self.assertEqual(wf.work_request_id, wr.id)
-        self.assertEqual(wf.workspace_name, wr.workspace.name)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_registration(self) -> None:
         """Test class subclass registry."""
 
@@ -114,15 +110,13 @@ class WorkflowTests(TestCase):
         self.assertEqual(ExampleWorkflowName.name, "examplename")
 
         # Subclasses that are not ABC get registered
-        self.assertIn(
-            "exampleworkflow", BaseTask._sub_tasks[TaskTypes.WORKFLOW]
-        )
+        self.assertIn("exampleworkflow", DBTask._sub_tasks[TaskTypes.WORKFLOW])
         self.assertNotIn(
-            "exampleworkflowabc", BaseTask._sub_tasks[TaskTypes.WORKFLOW]
+            "exampleworkflowabc", DBTask._sub_tasks[TaskTypes.WORKFLOW]
         )
-        self.assertIn("examplename", BaseTask._sub_tasks[TaskTypes.WORKFLOW])
+        self.assertIn("examplename", DBTask._sub_tasks[TaskTypes.WORKFLOW])
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_provides_artifact(self) -> None:
         """Test provides_artifact() update event_reactions.on_creation."""
 
@@ -206,7 +200,7 @@ class WorkflowTests(TestCase):
         self.assertEqual(item.artifact, artifact)
         self.assert_work_request_event_reactions(child, on_success=on_success)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_provides_artifact_bad_name(self) -> None:
         r"""Test requires_artifact() ValueError: name contains a slash."""
 
@@ -234,7 +228,7 @@ class WorkflowTests(TestCase):
                 child, ArtifactCategory.TEST, "prefix/testing"
             )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_provides_artifact_bad_data_key(self) -> None:
         r"""Test requires_artifact() ValueError: key starts with promise\_."""
 
@@ -262,7 +256,7 @@ class WorkflowTests(TestCase):
                 child, ArtifactCategory.TEST, "testing", data={key: "value"}
             )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_requires_artifact_lookup_single(self) -> None:
         """Test requires_artifact() call work_request.add_dependency()."""
 
@@ -316,7 +310,7 @@ class WorkflowTests(TestCase):
         )
         self.assertEqual(child_requires.dependencies.count(), 1)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_requires_artifact_lookup_multiple(self) -> None:
         """Test requires_artifact() call work_request.add_dependency()."""
 
@@ -384,11 +378,11 @@ class WorkflowTests(TestCase):
     def test_lookup(self) -> None:
         """Test lookup of Workflow orchestrators."""
         self.assertEqual(
-            BaseTask.class_from_name(TaskTypes.WORKFLOW, "noop"), NoopWorkflow
+            DBTask.class_from_name(TaskTypes.WORKFLOW, "noop"), NoopWorkflow
         )
         self.assertEqual(Workflow.from_name("noop"), NoopWorkflow)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_work_request_ensure_child(self) -> None:
         """Test work_request_ensure_child create or return work request."""
 
@@ -471,7 +465,7 @@ class WorkflowTests(TestCase):
                 # Returned the same as had been created
                 self.assertEqual(wr_created, wr_returned)
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_get_input_artifacts_ids(self) -> None:
         class ExampleWorkflow(
             SampleWorkflow[BaseWorkflowData, BaseDynamicTaskData]
