@@ -18,7 +18,7 @@ from debusine.artifacts.models import (
     DebianSourcePackage,
     DebusinePromise,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, LookupResultType
 from debusine.db.models import ArtifactRelation, CollectionItem, WorkRequest
 from debusine.server.collections.lookup import LookupResult
 from debusine.server.workflows import (
@@ -125,7 +125,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -149,7 +149,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         self.assertEqual(
@@ -166,7 +166,7 @@ class WorkflowUtilsTests(TestCase):
             collection, "test", category=BareDataCategory.TEST
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         with self.assertRaisesRegex(
@@ -184,7 +184,7 @@ class WorkflowUtilsTests(TestCase):
             "test", CollectionCategory.TEST
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.COLLECTION, collection=collection
+            result_type=LookupResultType.COLLECTION, collection=collection
         )
 
         with self.assertRaisesRegex(
@@ -215,7 +215,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         self.assertEqual(
@@ -238,7 +238,7 @@ class WorkflowUtilsTests(TestCase):
             ),
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         with self.assertRaisesRegex(
@@ -255,7 +255,7 @@ class WorkflowUtilsTests(TestCase):
             architecture="i386",
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -280,7 +280,7 @@ class WorkflowUtilsTests(TestCase):
             created_by_user=self.playground.get_default_user(),
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT,
+            result_type=LookupResultType.ARTIFACT,
             collection_item=item,
             artifact=artifact,
         )
@@ -302,7 +302,7 @@ class WorkflowUtilsTests(TestCase):
         )
 
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -334,7 +334,7 @@ class WorkflowUtilsTests(TestCase):
             created_by_user=self.playground.get_default_user(),
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT,
+            result_type=LookupResultType.ARTIFACT,
             collection_item=item,
             artifact=artifact,
         )
@@ -358,7 +358,7 @@ class WorkflowUtilsTests(TestCase):
         )
 
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -384,7 +384,7 @@ class WorkflowUtilsTests(TestCase):
         )
 
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -405,7 +405,7 @@ class WorkflowUtilsTests(TestCase):
         )
 
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -436,7 +436,7 @@ class WorkflowUtilsTests(TestCase):
             created_by_user=self.playground.get_default_user(),
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT,
+            result_type=LookupResultType.ARTIFACT,
             collection_item=item,
             artifact=artifact,
         )
@@ -458,7 +458,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         with self.assertRaisesRegex(
@@ -472,7 +472,7 @@ class WorkflowUtilsTests(TestCase):
             "test", CollectionCategory.TEST
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.COLLECTION, collection=collection
+            result_type=LookupResultType.COLLECTION, collection=collection
         )
 
         with self.assertRaisesRegex(
@@ -500,7 +500,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         with self.assertRaisesRegex(
@@ -511,7 +511,7 @@ class WorkflowUtilsTests(TestCase):
 
     def test_lookup_result_architectures_unexpected_result(self) -> None:
         """lookup_result_architectures raise error: unexpected result."""
-        result = LookupResult(result_type=CollectionItem.Types.BARE)
+        result = LookupResult(result_type=LookupResultType.BARE)
 
         with self.assertRaisesRegex(ValueError, "^Unexpected result: .*"):
             workflow_utils.lookup_result_architectures(result)
@@ -533,7 +533,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         self.assertEqual(
@@ -556,7 +556,7 @@ class WorkflowUtilsTests(TestCase):
             ),
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         with self.assertRaisesRegex(
@@ -583,7 +583,7 @@ class WorkflowUtilsTests(TestCase):
         )
 
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         self.assertEqual(
@@ -615,7 +615,7 @@ class WorkflowUtilsTests(TestCase):
             created_by_user=self.playground.get_default_user(),
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT,
+            result_type=LookupResultType.ARTIFACT,
             collection_item=item,
             artifact=artifact,
         )
@@ -637,7 +637,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.ARTIFACT, artifact=artifact
+            result_type=LookupResultType.ARTIFACT, artifact=artifact
         )
 
         with self.assertRaisesRegex(
@@ -652,7 +652,7 @@ class WorkflowUtilsTests(TestCase):
             "test", CollectionCategory.TEST
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.COLLECTION, collection=collection
+            result_type=LookupResultType.COLLECTION, collection=collection
         )
 
         with self.assertRaisesRegex(
@@ -682,7 +682,7 @@ class WorkflowUtilsTests(TestCase):
             },
         )
         result = LookupResult(
-            result_type=CollectionItem.Types.BARE, collection_item=promise
+            result_type=LookupResultType.BARE, collection_item=promise
         )
 
         with self.assertRaisesRegex(
@@ -693,7 +693,7 @@ class WorkflowUtilsTests(TestCase):
 
     def test_lookup_result_binary_package_name_unexpected_result(self) -> None:
         """lookup_result_binary_package_name raise error: unexpected result."""
-        result = LookupResult(result_type=CollectionItem.Types.BARE)
+        result = LookupResult(result_type=LookupResultType.BARE)
 
         with self.assertRaisesRegex(ValueError, "^Unexpected result: .*"):
             workflow_utils.lookup_result_binary_package_name(result)
@@ -1063,8 +1063,7 @@ class WorkflowUtilsTests(TestCase):
 
         lookup_result = [
             LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=source_artifact,
+                result_type=LookupResultType.ARTIFACT, artifact=source_artifact
             )
         ]
 
@@ -1084,7 +1083,7 @@ class WorkflowUtilsTests(TestCase):
     ) -> None:
         lookup_result = [
             LookupResult(
-                result_type=CollectionItem.Types.BARE,
+                result_type=LookupResultType.BARE,
                 collection_item=CollectionItem(
                     data={
                         "promise_category": ArtifactCategory.TEST,
@@ -1127,23 +1126,22 @@ class WorkflowUtilsTests(TestCase):
 
         lookup_result = [
             LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
+                result_type=LookupResultType.ARTIFACT,
                 artifact=binary_artifact_1,
             ),
             LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
+                result_type=LookupResultType.ARTIFACT,
                 artifact=binary_artifact_2,
             ),
             LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
+                result_type=LookupResultType.ARTIFACT,
                 artifact=binaries_artifact,
             ),
             LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=upload_artifact,
+                result_type=LookupResultType.ARTIFACT, artifact=upload_artifact
             ),
             LookupResult(
-                result_type=CollectionItem.Types.BARE,
+                result_type=LookupResultType.BARE,
                 collection_item=CollectionItem(
                     # This isn't valid, without a source_package_name, but it
                     # gives coverage another path
@@ -1153,7 +1151,7 @@ class WorkflowUtilsTests(TestCase):
                 ),
             ),
             LookupResult(
-                result_type=CollectionItem.Types.BARE,
+                result_type=LookupResultType.BARE,
                 collection_item=CollectionItem(
                     data={
                         "promise_category": ArtifactCategory.BINARY_PACKAGE,
