@@ -32,20 +32,10 @@ from debusine.artifacts.models import (
     get_architectures,
     get_source_package_name,
 )
-from debusine.client.models import LookupChildType
-from debusine.db.models import (
-    Artifact,
-    ArtifactRelation,
-    CollectionItem,
-    TaskDatabase,
-)
-from debusine.server.collections.lookup import (
-    LookupResult,
-    lookup_multiple,
-    lookup_single,
-    reconstruct_lookup,
-)
-from debusine.tasks import BaseTask, TaskConfigError, get_environment
+from debusine.client.models import LookupChildType, LookupResultType
+from debusine.db.models import Artifact, ArtifactRelation, TaskDatabase
+from debusine.server.collections.lookup import LookupResult, reconstruct_lookup
+from debusine.tasks import TaskConfigError, ensure_artifact_categories
 from debusine.tasks.models import (
     BackendType,
     ExtraExternalRepository,
@@ -70,11 +60,8 @@ def source_package(
     If the source artifact is a :artifact:`debian:upload`, returns its
     :artifact:`debian:source-package`.
     """
-    artifact = lookup_single(
+    artifact = workflow.work_request.lookup_single(
         attrgetter(configuration_key)(workflow.data),
-        workflow.workspace,
-        user=workflow.work_request.created_by,
-        workflow_root=workflow.work_request.workflow_root,
         expect_type=LookupChildType.ARTIFACT,
     ).artifact
     return locate_debian_source_package(configuration_key, artifact)
@@ -99,12 +86,12 @@ def lookup_result_artifact_category(result: LookupResult) -> str:
     The result may be either an artifact or a promise.
     """
     if (
-        result.result_type == CollectionItem.Types.ARTIFACT
+        result.result_type == LookupResultType.ARTIFACT
         and result.artifact is not None
     ):
         return result.artifact.category
     elif (
-        result.result_type == CollectionItem.Types.BARE
+        result.result_type == LookupResultType.BARE
         and result.collection_item is not None
         and result.collection_item.category == BareDataCategory.PROMISE
     ):
@@ -173,12 +160,8 @@ def filter_artifact_lookup_by_arch(
     architectures: set[str],
 ) -> LookupMultiple:
     """Filter an artifact lookup by architecture."""
-    results = lookup_multiple(
-        lookup,
-        workflow.workspace,
-        user=workflow.work_request.created_by,
-        workflow_root=workflow.work_request.workflow_root,
-        expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+    results = workflow.work_request.lookup_multiple(
+        lookup, expect_type=LookupChildType.ARTIFACT_OR_PROMISE
     )
     relevant: list[str] = []
     for result in results:
@@ -218,12 +201,8 @@ def get_lookup_architectures(
     The architectures are extracted from each lookup result using
     :py:func:`lookup_result_architectures`.
     """
-    results = lookup_multiple(
-        lookup,
-        workflow.workspace,
-        user=workflow.work_request.created_by,
-        workflow_root=workflow.work_request.workflow_root,
-        expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+    results = workflow.work_request.lookup_multiple(
+        lookup, expect_type=LookupChildType.ARTIFACT_OR_PROMISE
     )
 
     architectures: set[str] = set()
@@ -260,7 +239,7 @@ def locate_debian_source_package(
       :artifact:`debian:upload`, return the related
       :artifact:`debian:source-package`.
     """
-    BaseTask.ensure_artifact_categories(
+    ensure_artifact_categories(
         configuration_key=configuration_key,
         category=artifact.category,
         expected=[ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD],
@@ -304,19 +283,15 @@ def locate_debian_source_package_lookup(
     return it.  If it returns a :artifact:`debian:upload`, find the related
     :artifact:`debian:source-package` and return a lookup to it.
     """
-    artifact = lookup_single(
-        lookup,
-        workflow.workspace,
-        user=workflow.work_request.created_by,
-        workflow_root=workflow.work_request.workflow_root,
-        expect_type=LookupChildType.ARTIFACT,
+    artifact = workflow.work_request.lookup_single(
+        lookup, expect_type=LookupChildType.ARTIFACT
     ).artifact
     if artifact.category == ArtifactCategory.UPLOAD:
         source_package = locate_debian_source_package(
             configuration_key, artifact
         )
         return f"{source_package.id}@artifacts"
-    BaseTask.ensure_artifact_categories(
+    ensure_artifact_categories(
         configuration_key=configuration_key,
         category=artifact.category,
         expected=[ArtifactCategory.SOURCE_PACKAGE],
@@ -341,7 +316,7 @@ def locate_debian_binary_packages(
     """
     binary_packages: list[Artifact] = []
     for i, artifact in enumerate(artifacts):
-        BaseTask.ensure_artifact_categories(
+        ensure_artifact_categories(
             configuration_key=f"{configuration_key}[{i}]",
             category=artifact.category,
             expected=[ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD],
@@ -383,7 +358,7 @@ def get_source_package_names(
       representing artifacts to be processed. Each entry is expected to be
       either an artifact or a promise.
     :param configuration_key: A string used by
-      :py:meth:`BaseTask.ensure_artifact_categories` for the exception
+      :py:meth:`ensure_artifact_categories` for the exception
       message.
     :param artifact_expected_categories: valid :py:class:`ArtifactCategory`
       that artifacts must belong to.
@@ -394,11 +369,11 @@ def get_source_package_names(
     for result in results:
         # lookup_multiple expect_type: only artifacts or promises
         match result.result_type:
-            case CollectionItem.Types.ARTIFACT:
+            case LookupResultType.ARTIFACT:
                 assert result.artifact is not None
                 category = result.artifact.category
 
-                BaseTask.ensure_artifact_categories(
+                ensure_artifact_categories(
                     configuration_key=configuration_key,
                     category=category,
                     expected=artifact_expected_categories,
@@ -419,10 +394,10 @@ def get_source_package_names(
             case _:
                 # Makes coverage happy
                 # It's a promise.
-                assert result.result_type == CollectionItem.Types.BARE
+                assert result.result_type == LookupResultType.BARE
                 assert result.collection_item is not None
 
-                BaseTask.ensure_artifact_categories(
+                ensure_artifact_categories(
                     configuration_key=configuration_key,
                     category=result.collection_item.data["promise_category"],
                     expected=artifact_expected_categories,
@@ -444,12 +419,10 @@ def get_available_architectures(
     """Get architectures available for use with this vendor/codename."""
     architectures = set()
 
-    for result in lookup_multiple(
+    for result in workflow.work_request.lookup_multiple(
         LookupMultiple.model_validate(
             {"collection": vendor, "data__codename": codename}
         ),
-        workflow.workspace,
-        user=workflow.work_request.created_by,
         default_category=CollectionCategory.ENVIRONMENTS,
         expect_type=LookupChildType.ARTIFACT,
     ):
@@ -489,8 +462,7 @@ def configure_for_overlay_suite(
         extra_repositories = []
 
     task_database = TaskDatabase(workflow.work_request)
-    environment_artifact = get_environment(
-        task_database,
+    environment_artifact = task_database.get_environment(
         environment,
         architecture=architecture,
         backend=backend,
