@@ -25,8 +25,11 @@ from debusine.signing.signing_utils import SensitiveTemporaryDirectory
 from debusine.utils import Base64BytesAsStr
 
 if TYPE_CHECKING:
+    from _typeshed import ReadableBuffer
     from nacl.public import PrivateKey, PublicKey
 
+    ReadableBuffer  # fake usage for vulture
+
 
 class ProtectedKeyError(Exception):
     """An error handling a protected key."""
@@ -42,7 +45,9 @@ class ProtectedKeyStorage(StrEnum):
 class BaseAvailableKey(ABC):
     """A protected key that has temporarily been made available."""
 
-    def __init__(self, *, temp_dir: Path, certificate: bytes) -> None:
+    def __init__(
+        self, *, temp_dir: Path, certificate: "ReadableBuffer"
+    ) -> None:
         """Construct a :py:class:`BaseAvailableKey`."""
         self.temp_dir = temp_dir
         self._certificate_path.write_bytes(certificate)
@@ -64,7 +69,9 @@ class BaseProtectedKey(pydantic.BaseModel, ABC):
 
     @contextmanager
     @abstractmethod
-    def available(self, public_key: bytes) -> Generator[BaseAvailableKey]:
+    def available(
+        self, public_key: "ReadableBuffer"
+    ) -> Generator[BaseAvailableKey]:
         """
         Make a protected key temporarily available.
 
@@ -78,7 +85,11 @@ class AvailableKeyFileSystem(BaseAvailableKey):
     """An available private key on the file system."""
 
     def __init__(
-        self, *, temp_dir: Path, certificate: bytes, key: bytes
+        self,
+        *,
+        temp_dir: Path,
+        certificate: "ReadableBuffer",
+        key: "ReadableBuffer",
     ) -> None:
         """Construct an :py:class:`AvailableKeyFileSystem`."""
         super().__init__(temp_dir=temp_dir, certificate=certificate)
@@ -103,7 +114,7 @@ class AvailableKeyPKCS11(BaseAvailableKey):
     """An available private key on a PKCS#11 token."""
 
     def __init__(
-        self, *, temp_dir: Path, certificate: bytes, pkcs11_uri: str
+        self, *, temp_dir: Path, certificate: "ReadableBuffer", pkcs11_uri: str
     ) -> None:
         """Construct an :py:class:`AvailableKeyPKCS11`."""
         super().__init__(temp_dir=temp_dir, certificate=certificate)
@@ -130,7 +141,9 @@ class ProtectedKeyNaCl(BaseProtectedKey):
     encrypted: Base64BytesAsStr
 
     @classmethod
-    def create(cls, *, public_key: bytes, encrypted: bytes) -> Self:
+    def create(
+        cls, *, public_key: "ReadableBuffer", encrypted: "ReadableBuffer"
+    ) -> Self:
         """Create a new :py:class:`ProtectedKeyNaCl`."""
         return cls(
             storage=ProtectedKeyStorage.NACL,
@@ -173,7 +186,9 @@ class ProtectedKeyNaCl(BaseProtectedKey):
         )
 
     @contextmanager
-    def available(self, public_key: bytes) -> Generator[BaseAvailableKey]:
+    def available(
+        self, public_key: "ReadableBuffer"
+    ) -> Generator[BaseAvailableKey]:
         """Make this protected key temporarily available."""
         with SensitiveTemporaryDirectory(
             "debusine-available-nacl-"
@@ -211,7 +226,9 @@ class ProtectedKeyPKCS11Static(BaseProtectedKey):
         return value
 
     @contextmanager
-    def available(self, public_key: bytes) -> Generator[BaseAvailableKey]:
+    def available(
+        self, public_key: "ReadableBuffer"
+    ) -> Generator[BaseAvailableKey]:
         """Make this protected key temporarily available."""
         with TemporaryDirectory(
             "debusine-available-pkcs11-static-"
@@ -234,7 +251,9 @@ class ProtectedKey(
     """A protected private key."""
 
     @contextmanager
-    def available(self, public_key: bytes) -> Generator[BaseAvailableKey]:
+    def available(
+        self, public_key: "ReadableBuffer"
+    ) -> Generator[BaseAvailableKey]:
         """
         Make a protected key temporarily available.
 
