@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,14 +12,19 @@
 import os
 import subprocess
 from pathlib import Path
-from typing import BinaryIO
+from typing import BinaryIO, TYPE_CHECKING
 
 from debusine.signing.models import ProtectedKey
 
+if TYPE_CHECKING:
+    from _typeshed import ReadableBuffer
+
+    ReadableBuffer  # fake usage for vulture
+
 
 def sbsign(
     private_key: ProtectedKey,
-    public_key: bytes,
+    public_key: "ReadableBuffer",
     data_path: Path,
     signature_path: Path,
     detached: bool = False,
