@@ -28,6 +28,8 @@ class GenerateKey(
 ):
     """Task that generates a new key."""
 
+    TASK_VERSION = 1
+
     _key: Key | None = None
 
     @override
@@ -72,8 +74,3 @@ class GenerateKey(
             workspace=self.workspace_name,
             work_request=self.work_request_id,
         )
-
-    @override
-    def get_label(self) -> str:
-        """Return the task label."""
-        return f"generate {self.data.purpose} key"
