@@ -9,7 +9,7 @@
 
 """Models used by debusine signing tasks."""
 
-from typing import Annotated
+from typing import Annotated, override
 
 import pydantic
 
@@ -62,6 +62,10 @@ class SignDynamicData(BaseDynamicTaskData):
     unsigned_ids: list[int]
     unsigned_binary_package_names: list[str | None] = []
 
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return self.unsigned_ids
+
 
 class DebsignData(BaseTaskData):
     """In-memory task data for the Debsign task."""
@@ -75,6 +79,10 @@ class DebsignDynamicData(BaseDynamicTaskData):
 
     unsigned_id: int
 
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [self.unsigned_id]
+
 
 class SignRepositoryIndexData(BaseTaskData):
     """In-memory task data for the SignRepositoryIndex task."""
@@ -100,3 +108,7 @@ class SignRepositoryIndexDynamicData(BaseDynamicTaskData):
 
     signing_keys: list[str]
     unsigned_id: int
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [self.unsigned_id]
