@@ -32,8 +32,3 @@ class SigningNoop(
     def run(self, execute_directory: Path) -> WorkRequestResults:  # noqa: U100
         """Do nothing."""
         return self.data.result
-
-    @override
-    def get_label(self) -> str:
-        """Return the task label."""
-        return "noop"
