@@ -21,6 +21,7 @@ from django.test import TestCase as DjangoTestCase
 from django.test import TransactionTestCase, override_settings
 from nacl.public import PrivateKey
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import (
     LocalArtifact,
     SourcePackage,
@@ -47,7 +48,7 @@ from debusine.signing.tasks.models import DebsignDynamicData
 from debusine.signing.tasks.tests.test_sign import SignTestMixin
 from debusine.tasks.models import WorkerType
 from debusine.tasks.server import ArtifactInfo
-from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
+from debusine.tasks.tests.helper_mixin import MockTaskDatabase
 from debusine.test import TestCase
 from debusine.test.test_utils import create_remote_artifact
 
@@ -55,6 +56,25 @@ from debusine.test.test_utils import create_remote_artifact
 class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
     """Unit tests for the :py:class:`Debsign` task."""
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.mock_is_command_available({"debsign": True})
+        task = Debsign(task_data={"unsigned": 2, "key": "ABC123"})
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "signing:debsign:version:1",
+                wtags.CAP_PREFIX + "debsign",
+            ],
+        )
+
+    def test_get_provided_worker_tags_debsign_not_available(self) -> None:
+        self.mock_is_command_available({"debsign": False})
+        task = Debsign(task_data={"unsigned": 2, "key": "ABC123"})
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [wtags.TASK_PREFIX + "signing:debsign:version:1"],
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"debsign": True})
@@ -108,28 +128,32 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             )
         )
 
+    def test_compute_system_required_tags(self) -> None:
+        task = Debsign(task_data={"unsigned": 2, "key": "ABC123"})
+        self.assertCountEqual(
+            task.compute_system_required_tags(), [wtags.CAP_PREFIX + "debsign"]
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (2, None): ArtifactInfo(
-                    id=2,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "source",
-                            "Files": [{"name": "hello_1.0.dsc"}],
-                        },
-                    ),
+        task_db = MockTaskDatabase(
+            unsigned=ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.UPLOAD,
+                data=DebianUpload(
+                    type="dpkg",
+                    changes_fields={
+                        "Architecture": "source",
+                        "Files": [{"name": "hello_1.0.dsc"}],
+                    },
                 ),
-            }
+            )
         )
 
         task = Debsign(task_data={"unsigned": 2, "key": "ABC123"})
+        task_db.resolve_inputs(task)
         self.assertEqual(
-            task.compute_dynamic_data(task_db),
-            DebsignDynamicData(unsigned_id=2),
+            task.build_dynamic_data(), DebsignDynamicData(unsigned_id=2)
         )
 
     def test_get_input_artifacts_ids(self) -> None:
@@ -565,8 +589,3 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             workspace="System",
             work_request=1,
         )
-
-    def test_label(self) -> None:
-        """Test get_label."""
-        task = Debsign(task_data={"unsigned": 2, "key": "ABC123"})
-        self.assertEqual(task.get_label(), "sign upload")
