@@ -19,8 +19,9 @@ from cryptography.x509 import load_pem_x509_certificate
 from django.test import TransactionTestCase, override_settings
 from nacl.public import PrivateKey, SealedBox
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import WorkRequestDebugLogs
-from debusine.artifacts.models import EmptyArtifactData
+from debusine.artifacts.models import EmptyArtifactData, WorkRequestResults
 from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.client.debusine import Debusine
 from debusine.signing.db.models import Key
@@ -38,6 +39,18 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
         task.configure_server_access(debusine_mock)
         return debusine_mock
 
+    def test_get_provided_worker_tags(self) -> None:
+        task = GenerateKey(
+            task_data={"purpose": "uefi", "description": "A UEFI key"},
+            dynamic_task_data={},
+        )
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "signing:generatekey:version:1",
+            ],
+        )
+
     def test_uefi(self) -> None:
         """Integration test: generate and store a real UEFI key."""
         service_private_key = PrivateKey.generate()
@@ -55,7 +68,7 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
         with override_settings(
             DEBUSINE_SIGNING_PRIVATE_KEYS=[service_private_key]
         ):
-            self.assertTrue(task.execute())
+            self.assertEqual(task.execute(), WorkRequestResults.SUCCESS)
 
         # The private key was encrypted using the configured service-wide
         # private key, and the public key has the correct subject.
@@ -108,7 +121,7 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
         with override_settings(
             DEBUSINE_SIGNING_PRIVATE_KEYS=[service_private_key]
         ):
-            self.assertTrue(task.execute())
+            self.assertEqual(task.execute(), WorkRequestResults.SUCCESS)
 
         # The private key was encrypted using the configured service-wide
         # private key, and the public key has the correct UID.
@@ -163,7 +176,7 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
         with override_settings(
             DEBUSINE_SIGNING_PRIVATE_KEYS=[service_private_key]
         ):
-            self.assertTrue(task.execute())
+            self.assertEqual(task.execute(), WorkRequestResults.SUCCESS)
 
         # The private key was encrypted using the configured service-wide
         # private key, and the public key has the correct UID.
@@ -216,7 +229,7 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
             "generate",
             side_effect=Exception("Key generation failed"),
         ):
-            self.assertFalse(task.execute())
+            self.assertEqual(task.execute(), WorkRequestResults.FAILURE)
 
         # No key was generated.
         self.assertQuerySetEqual(Key.objects.all(), [])
@@ -249,11 +262,3 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
             workspace="System",
             work_request=1,
         )
-
-    def test_label(self) -> None:
-        """Test get_label."""
-        task = GenerateKey(
-            task_data={"purpose": "uefi", "description": "A UEFI key"},
-            dynamic_task_data={},
-        )
-        self.assertEqual(task.get_label(), "generate uefi key")
