@@ -23,6 +23,7 @@ from django.test import TransactionTestCase, override_settings
 from django.utils import timezone
 from nacl.public import PrivateKey
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import (
     LocalArtifact,
     SigningInputArtifact,
@@ -51,9 +52,9 @@ from debusine.signing.gnupg import gpg_ephemeral_context
 from debusine.signing.models import SigningMode
 from debusine.signing.tasks import BaseSigningTask, Sign
 from debusine.signing.tasks.models import SignDynamicData
-from debusine.tasks.models import LookupMultiple, WorkerType
-from debusine.tasks.server import ArtifactInfo
-from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
+from debusine.tasks.models import WorkerType
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
+from debusine.tasks.tests.helper_mixin import MockTaskDatabase
 from debusine.test import TestCase
 from debusine.test.test_utils import (
     create_artifact_response,
@@ -153,6 +154,37 @@ class SignTestMixin:
 class SignTests(SignTestMixin, TestCase, DjangoTestCase):
     """Unit tests for the :py:class:`Sign` task."""
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.mock_is_command_available({"sbsign": True})
+        task = Sign(
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
+        )
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "signing:sign:version:1",
+                wtags.CAP_PREFIX + "sbsign",
+            ],
+        )
+
+    def test_get_provided_worker_tags_sbsign_not_available(self) -> None:
+        self.mock_is_command_available({"sbsign": False})
+        task = Sign(
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
+        )
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [wtags.TASK_PREFIX + "signing:sign:version:1"],
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"sbsign": True})
@@ -254,19 +286,30 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             )
         )
 
+    def test_compute_system_required_tags(self) -> None:
+        task = Sign(
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
+        )
+        self.assertCountEqual(
+            task.compute_system_required_tags(), [wtags.CAP_PREFIX + "sbsign"]
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        task_db = FakeTaskDatabase(
-            multiple_lookups={
-                # unsigned
-                (LookupMultiple((2,)), None): [
+        task_db = MockTaskDatabase(
+            unsigned=MultipleArtifactInfo(
+                (
                     ArtifactInfo(
                         id=2,
                         category=ArtifactCategory.SIGNING_INPUT,
                         data=DebusineSigningInput(),
-                    )
-                ]
-            },
+                    ),
+                )
+            )
         )
 
         task = Sign(
@@ -276,8 +319,9 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
                 "key": "ABC123",
             }
         )
+        task_db.resolve_inputs(task)
         self.assertEqual(
-            task.compute_dynamic_data(task_db),
+            task.build_dynamic_data(),
             SignDynamicData(
                 unsigned_ids=[2], unsigned_binary_package_names=[None]
             ),
@@ -285,17 +329,16 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
 
     def test_compute_dynamic_data_copies_binary_package_name(self) -> None:
         """Dynamic data copies `binary_package_name` from `unsigned`."""
-        task_db = FakeTaskDatabase(
-            multiple_lookups={
-                # unsigned
-                (LookupMultiple((2,)), None): [
+        task_db = MockTaskDatabase(
+            unsigned=MultipleArtifactInfo(
+                (
                     ArtifactInfo(
                         id=2,
                         category=ArtifactCategory.SIGNING_INPUT,
                         data=DebusineSigningInput(binary_package_name="hello"),
-                    )
-                ]
-            },
+                    ),
+                )
+            )
         )
 
         task = Sign(
@@ -305,8 +348,9 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
                 "key": "ABC123",
             }
         )
+        task_db.resolve_inputs(task)
         self.assertEqual(
-            task.compute_dynamic_data(task_db),
+            task.build_dynamic_data(),
             SignDynamicData(
                 unsigned_ids=[2], unsigned_binary_package_names=["hello"]
             ),
@@ -1180,14 +1224,3 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             self.assertEqual(
                 result.signatures[0].hash_algo, gpg.constants.MD_SHA512
             )
-
-    def test_label(self) -> None:
-        """Test get_label."""
-        task = Sign(
-            task_data={
-                "purpose": KeyPurpose.UEFI,
-                "unsigned": [2],
-                "key": "ABC123",
-            }
-        )
-        self.assertEqual(task.get_label(), "sign data for uefi")
