@@ -21,6 +21,7 @@ from django.test import TestCase as DjangoTestCase
 from django.test import TransactionTestCase, override_settings
 from nacl.public import PrivateKey
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import (
     LocalArtifact,
     RepositoryIndex,
@@ -44,9 +45,8 @@ from debusine.signing.models import SigningMode
 from debusine.signing.tasks import SignRepositoryIndex
 from debusine.signing.tasks.models import SignRepositoryIndexDynamicData
 from debusine.signing.tasks.tests.test_sign import SignTestMixin
-from debusine.tasks import TaskConfigError
 from debusine.tasks.server import ArtifactInfo, CollectionInfo
-from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
+from debusine.tasks.tests.helper_mixin import MockTaskDatabase
 from debusine.test import TestCase
 from debusine.test.test_utils import create_remote_artifact
 
@@ -54,25 +54,51 @@ from debusine.test.test_utils import create_remote_artifact
 class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
     """Unit tests for the :py:class:`SignRepositoryIndex` task."""
 
+    def test_get_provided_worker_tags(self) -> None:
+        task = SignRepositoryIndex(
+            task_data={
+                "suite_collection": f"sid@{CollectionCategory.SUITE}",
+                "unsigned": 2,
+                "mode": "clear",
+                "signed_name": "InRelease",
+            }
+        )
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "signing:signrepositoryindex:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        task = SignRepositoryIndex(
+            task_data={
+                "suite_collection": f"sid@{CollectionCategory.SUITE}",
+                "unsigned": 2,
+                "mode": "clear",
+                "signed_name": "InRelease",
+            }
+        )
+        self.assertCountEqual(task.compute_system_required_tags(), [])
+
     def test_compute_dynamic_data(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (2, None): ArtifactInfo(
-                    id=2,
-                    category=ArtifactCategory.REPOSITORY_INDEX,
-                    data=EmptyArtifactData(),
-                ),
-            },
-            single_collection_lookups={
-                (f"sid@{CollectionCategory.SUITE}", None): CollectionInfo(
+        task_db = MockTaskDatabase(
+            unsigned=ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.REPOSITORY_INDEX,
+                data=EmptyArtifactData(),
+            ),
+            suite_archive=[
+                CollectionInfo(
                     id=1,
                     scope_name="scope",
                     workspace_name="workspace",
                     category=CollectionCategory.SUITE,
                     name="sid",
                     data={"signing_keys": ["ABC123"]},
-                )
-            },
+                ),
+                None,
+            ],
         )
 
         task = SignRepositoryIndex(
@@ -83,61 +109,31 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
                 "signed_name": "InRelease",
             }
         )
+        task_db.resolve_inputs(task)
         self.assertEqual(
-            task.compute_dynamic_data(task_db),
+            task.build_dynamic_data(),
             SignRepositoryIndexDynamicData(
                 signing_keys=["ABC123"], unsigned_id=2
             ),
         )
 
-    def test_compute_dynamic_data_wrong_suite_collection_category(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (2, None): ArtifactInfo(
+    def test_compute_dynamic_data_signing_keys_from_archive(self) -> None:
+        task_db = MockTaskDatabase(
+            unsigned=ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.REPOSITORY_INDEX,
+                data=EmptyArtifactData(),
+            ),
+            suite_archive=[
+                CollectionInfo(
                     id=2,
-                    category=ArtifactCategory.REPOSITORY_INDEX,
-                    data=EmptyArtifactData(),
-                ),
-            },
-            single_collection_lookups={
-                ("sid", None): CollectionInfo(
-                    id=1,
                     scope_name="scope",
                     workspace_name="workspace",
-                    category=CollectionCategory.TEST,
+                    category=CollectionCategory.SUITE,
                     name="sid",
                     data={},
-                )
-            },
-        )
-
-        task = SignRepositoryIndex(
-            task_data={
-                "suite_collection": "sid",
-                "unsigned": 2,
-                "mode": "clear",
-                "signed_name": "InRelease",
-            }
-        )
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            rf"^suite_collection: unexpected collection category: "
-            rf"'{CollectionCategory.TEST}'\. "
-            rf"Expected: '{CollectionCategory.SUITE}'$",
-        ):
-            task.compute_dynamic_data(task_db)
-
-    def test_compute_dynamic_data_signing_keys_from_archive(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (2, None): ArtifactInfo(
-                    id=2,
-                    category=ArtifactCategory.REPOSITORY_INDEX,
-                    data=EmptyArtifactData(),
                 ),
-            },
-            single_collection_lookups={
-                (f"_@{CollectionCategory.ARCHIVE}", None): CollectionInfo(
+                CollectionInfo(
                     id=1,
                     scope_name="scope",
                     workspace_name="workspace",
@@ -145,15 +141,7 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
                     name="_",
                     data={"signing_keys": ["ABC123"]},
                 ),
-                (f"sid@{CollectionCategory.SUITE}", None): CollectionInfo(
-                    id=2,
-                    scope_name="scope",
-                    workspace_name="workspace",
-                    category=CollectionCategory.SUITE,
-                    name="sid",
-                    data={},
-                ),
-            },
+            ],
         )
 
         task = SignRepositoryIndex(
@@ -164,24 +152,23 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
                 "signed_name": "InRelease",
             }
         )
+        task_db.resolve_inputs(task)
         self.assertEqual(
-            task.compute_dynamic_data(task_db),
+            task.build_dynamic_data(),
             SignRepositoryIndexDynamicData(
                 signing_keys=["ABC123"], unsigned_id=2
             ),
         )
 
     def test_compute_dynamic_data_no_archive_or_signing_keys(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                (2, None): ArtifactInfo(
-                    id=2,
-                    category=ArtifactCategory.REPOSITORY_INDEX,
-                    data=EmptyArtifactData(),
-                ),
-            },
-            single_collection_lookups={
-                (f"sid@{CollectionCategory.SUITE}", None): CollectionInfo(
+        task_db = MockTaskDatabase(
+            unsigned=ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.REPOSITORY_INDEX,
+                data=EmptyArtifactData(),
+            ),
+            suite_archive=[
+                CollectionInfo(
                     id=1,
                     scope_name="scope",
                     workspace_name="workspace",
@@ -189,7 +176,8 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
                     name="sid",
                     data={},
                 ),
-            },
+                None,
+            ],
         )
 
         task = SignRepositoryIndex(
@@ -200,8 +188,9 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
                 "signed_name": "InRelease",
             }
         )
+        task_db.resolve_inputs(task)
         self.assertEqual(
-            task.compute_dynamic_data(task_db),
+            task.build_dynamic_data(),
             SignRepositoryIndexDynamicData(signing_keys=[], unsigned_id=2),
         )
 
@@ -908,17 +897,3 @@ class SignRepositoryIndexIntegrationTests(
             workspace="System",
             work_request=1,
         )
-
-    def test_label(self) -> None:
-        task = SignRepositoryIndex(
-            task_data={
-                "suite_collection": f"sid@{CollectionCategory.SUITE}",
-                "unsigned": 2,
-                "mode": "clear",
-                "signed_name": "InRelease",
-            }
-        )
-        self.assertEqual(
-            task.get_label(),
-            f"sign InRelease for sid@{CollectionCategory.SUITE}",
-        )
