@@ -23,10 +23,14 @@ from debusine.tasks._task import (  # isort: split
     BaseExternalTask,
     BaseTask,
     BaseTaskWithExecutor,
+    DefaultDynamicData,
     ExtraRepositoryMixin,
     RunCommandTask,
     TaskConfigError,
-    get_environment,
+    analyze_external_worker_tasks,
+    ensure_artifact_categories,
+    ensure_collection_category,
+    get_provided_external_worker_tags,
 )
 
 # Sub-tasks need to be imported in order to be available to BaseTask
@@ -57,6 +61,7 @@ __all__ = [
     "BaseTaskWithExecutor",
     "Blhc",
     "DebDiff",
+    "DefaultDynamicData",
     "ExtraRepositoryMixin",
     "ExtractForSigning",
     "Lintian",
@@ -69,5 +74,8 @@ __all__ = [
     "Sbuild",
     "SimpleSystemImageBuild",
     "TaskConfigError",
-    "get_environment",
+    "analyze_external_worker_tasks",
+    "ensure_artifact_categories",
+    "ensure_collection_category",
+    "get_provided_external_worker_tags",
 ]
