@@ -35,7 +35,6 @@ from abc import ABCMeta, abstractmethod
 from collections import defaultdict
 from collections.abc import Collection, Generator
 from contextlib import contextmanager
-from functools import cache
 from pathlib import Path
 from typing import (
     Any,
@@ -43,12 +42,9 @@ from typing import (
     BinaryIO,
     ClassVar,
     IO,
-    Literal,
-    Self,
     TYPE_CHECKING,
     TextIO,
     Union,
-    assert_never,
     overload,
     override,
 )
@@ -56,8 +52,6 @@ from typing import (
 if TYPE_CHECKING:
     from _typeshed import OpenBinaryModeWriting, OpenTextModeWriting
 
-    from debusine.db.models import WorkRequest
-
 from debian.debian_support import Version
 
 from debusine.artifacts import WorkRequestDebugLogs
@@ -70,13 +64,15 @@ from debusine.artifacts.models import (
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks.executors import (
-    ExecutorImageCategory,
     ExecutorInterface,
     InstanceInterface,
     executor_class,
 )
+from debusine.tasks.inputs import (
+    ExtraRepositoriesInput,
+    TaskInput,
+)
 from debusine.tasks.models import (
-    ActionRecordInTaskHistory,
     BackendType,
     BaseDynamicTaskData,
     BaseDynamicTaskDataWithExecutor,
@@ -84,18 +80,8 @@ from debusine.tasks.models import (
     BaseTaskData,
     BaseTaskDataWithExecutor,
     BaseTaskDataWithExtraRepositories,
-    EventReaction,
-    ExtraDebusineRepository,
-    ExtraExternalRepository,
-    ExtraRepository,
-    LookupSingle,
     WorkerType,
-    build_key_value_lookup_segment,
-    build_lookup_string_segments,
-    parse_key_value_lookup_segment,
-    parse_lookup_string_segments,
 )
-from debusine.tasks.server import ArtifactInfo, TaskDatabaseInterface
 from debusine.utils import extract_generic_type_arguments
 
 
@@ -142,19 +128,101 @@ class TaskConfigError(Exception):
             return str(self.args[0])
 
 
+def ensure_artifact_categories(
+    *,
+    configuration_key: str,
+    category: str,
+    expected: Collection[ArtifactCategory],
+) -> None:
+    """
+    Validate that the artifact's category is one of the expected categories.
+
+    :param configuration_key: Optional key to identify the source of the
+      artifact. Provides additional context in error messages.
+    :param category: The category to validate.
+    :param expected: A collection of valid artifact categories.
+    """
+    valid_categories = [cat.value for cat in sorted(expected)]
+    if category not in valid_categories:
+        raise TaskConfigError(
+            f"{configuration_key}: unexpected artifact "
+            f"category: '{category}'. "
+            f"Valid categories: {valid_categories}"
+        )
+
+
+def ensure_collection_category(
+    *, configuration_key: str, category: str, expected: CollectionCategory
+) -> None:
+    """
+    Validate that the collection's category is as expected.
+
+    :param configuration_key: Optional key to identify the source of the
+      artifact. Provides additional context in error messages.
+    :param category: The category to validate.
+    :param expected: The expected collection category.
+    """
+    if category != expected:
+        raise TaskConfigError(
+            f"{configuration_key}: unexpected collection category: "
+            f"'{category}'. Expected: '{expected}'"
+        )
+
+
+def analyze_external_worker_tasks() -> dict[str, Any]:
+    """
+    Return dictionary with metadata for each task in BaseTask._sub_tasks.
+
+    Subclasses of BaseTask get registered in BaseTask._sub_tasks. Return
+    a dictionary with the metadata of each of the subtasks.
+
+    This method is executed in the worker when submitting the dynamic
+    metadata.
+    """
+    metadata = {}
+
+    for registry in BaseTask._sub_tasks.values():
+        for task_class in registry.values():
+            metadata.update(task_class.analyze_worker())
+
+    return metadata
+
+
+def get_provided_external_worker_tags() -> set[str]:
+    """
+    Return the set of worker tags provided by all available tasks.
+
+    This method is called on the worker to collect information about the
+    worker.
+    """
+    tags: set[str] = set()
+
+    for registry in BaseTask._sub_tasks.values():
+        for task_class in registry.values():
+            tags |= task_class.get_provided_worker_tags()
+
+    return tags
+
+
 class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
     """
-    Base class for tasks.
+    Base class for external tasks.
 
-    A BaseTask object serves two purpose: encapsulating the logic of what
-    needs to be done to execute the task (cf :py:meth:`configure`
-    and :py:meth:`execute` that are run on a worker), and supporting the
-    scheduler by determining if a task is suitable for a given worker. That is
-    done in a two-step process, collating metadata from each worker (with the
-    :py:meth:`analyze_worker` method that is run on a worker) and then,
-    based on this metadata, see if a task is suitable (with
+    A BaseTask object serves two purposes: encapsulating the logic of what needs
+    to be done to execute the task on an external worker (cf
+    :py:meth:`configure` and :py:meth:`execute`, that are run on the worker),
+    and supporting the server's task-specific logic, such as scheduling and UI
+    presentation.
+
+    Scheduling support is done in a two-step process: collating metadata from
+    each worker (with the :py:meth:`analyze_worker` method that is run on a
+    worker) and then, based on this metadata, see if a task is suitable (with
     :py:meth:`can_run_on` that is executed on the scheduler).
 
+    If tag-based scheduling is in use, that is replaced by
+    :py:meth`get_provided_worker_tags`, which runs on the worker and reports
+    provided worker tags that need to match what tasks require.
+
     Most concrete task implementations should inherit from
     :py:class:`RunCommandTask` instead.
     """
@@ -182,6 +250,9 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
         defaultdict(dict)
     )
 
+    #: Inputs defined in this task class
+    inputs: dict[str, TaskInput[Any]] = {}
+
     def __init_subclass__(cls, **kwargs: Any) -> None:
         """
         Register the subclass into BaseTask._sub_tasks.
@@ -270,22 +341,10 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
             None | (tempfile.TemporaryDirectory[str])
         ) = None
 
-    def set_work_request(self, work_request: "WorkRequest") -> None:
-        """Set this task's work_request_id and workspace_name."""
-        self.work_request_id = work_request.id
-        self.workspace_name = work_request.workspace.name
+        self.post_init()
 
-    def instantiate_with_new_data(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> Self:
-        """Create a new task like this one, with different data."""
-        new_task = self.__class__(task_data, dynamic_task_data)
-        new_task.work_request_id = self.work_request_id
-        new_task.workspace_name = self.workspace_name
-        new_task.worker_native_architecture = self.worker_native_architecture
-        return new_task
+    def post_init(self) -> None:
+        """Specific post-init code."""
 
     def append_to_log_file(self, filename: str, lines: list[str]) -> None:
         """
@@ -297,47 +356,6 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
         with self.open_debug_log_file(filename) as file:
             file.writelines([line + "\n" for line in lines])
 
-    @staticmethod
-    def ensure_artifact_categories(
-        *,
-        configuration_key: str,
-        category: str,
-        expected: Collection[ArtifactCategory],
-    ) -> None:
-        """
-        Validate that the artifact's category is one of the expected categories.
-
-        :param configuration_key: Optional key to identify the source of the
-          artifact. Provides additional context in error messages.
-        :param category: The category to validate.
-        :param expected: A collection of valid artifact categories.
-        """
-        valid_categories = [cat.value for cat in sorted(expected)]
-        if category not in valid_categories:
-            raise TaskConfigError(
-                f"{configuration_key}: unexpected artifact "
-                f"category: '{category}'. "
-                f"Valid categories: {valid_categories}"
-            )
-
-    @staticmethod
-    def ensure_collection_category(
-        *, configuration_key: str, category: str, expected: CollectionCategory
-    ) -> None:
-        """
-        Validate that the collection's category is as expected.
-
-        :param configuration_key: Optional key to identify the source of the
-          artifact. Provides additional context in error messages.
-        :param category: The category to validate.
-        :param expected: The expected collection category.
-        """
-        if category != expected:
-            raise TaskConfigError(
-                f"{configuration_key}: unexpected collection category: "
-                f"'{category}'. Expected: '{expected}'"
-            )
-
     @overload
     def open_debug_log_file(
         self, filename: str, *, mode: "OpenTextModeWriting" = "a"
@@ -409,23 +427,20 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
         }
 
     @classmethod
-    def analyze_worker_all_tasks(cls) -> dict[str, Any]:
+    def get_provided_worker_tags(cls) -> set[str]:
         """
-        Return dictionary with metadata for each task in BaseTask._sub_tasks.
+        Return the set of worker tags provided by this task class.
 
-        Subclasses of BaseTask get registered in BaseTask._sub_tasks. Return
-        a dictionary with the metadata of each of the subtasks.
-
-        This method is executed in the worker when submitting the dynamic
-        metadata.
+        This method is called on the worker to collect information about the
+        worker.
         """
-        metadata = {}
+        # Prevent circular import
+        import debusine.worker.tags as wtags
 
-        for registry in cls._sub_tasks.values():
-            for task_class in registry.values():
-                metadata.update(task_class.analyze_worker())
-
-        return metadata
+        return {
+            wtags.TASK_PREFIX
+            + f"{cls.TASK_TYPE.lower()}:{cls.name}:version:{cls.TASK_VERSION}"
+        }
 
     def build_architecture(self) -> str | None:
         """
@@ -494,23 +509,21 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
 
         return True
 
+    def compute_system_required_tags(self) -> set[str]:
+        """Compute the system set of task-required tags."""
+        return set()
+
     @abstractmethod
-    def build_dynamic_data(self, task_database: TaskDatabaseInterface) -> DTD:
+    def build_dynamic_data(self) -> DTD:
         """
         Build a dynamic task data structure for this task.
 
-        :param task_database: TaskDatabaseInterface to use for lookups
+        This method can only be called after all the task fields have been
+        resolved.
+
         :returns: the newly created dynamic task data
         """
 
-    def compute_dynamic_data(self, task_database: TaskDatabaseInterface) -> DTD:
-        """
-        Compute dynamic data for this task.
-
-        This may involve resolving artifact lookups.
-        """
-        return self.build_dynamic_data(task_database)
-
     def _configure(
         self,
         task_data: dict[str, Any],
@@ -615,28 +628,6 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
 
         return cls
 
-    @staticmethod
-    def is_valid_task_name(task_type: TaskTypes, task_name: str) -> bool:
-        """Return True if task_name is registered (its class is imported)."""
-        if (registry := BaseTask._sub_tasks.get(task_type)) is None:
-            return False
-        return task_name.lower() in registry
-
-    @staticmethod
-    def task_names(task_type: TaskTypes) -> list[str]:
-        """Return list of sub-task names."""
-        return sorted(BaseTask._sub_tasks[task_type])
-
-    @staticmethod
-    def is_worker_task(task_name: str) -> bool:
-        """Check if task_name is a task that can run on external workers."""
-        return task_name.lower() in BaseTask._sub_tasks[TaskTypes.WORKER]
-
-    @staticmethod
-    def worker_task_names() -> list[str]:
-        """Return list of sub-task names not of type TaskTypes.SERVER."""
-        return sorted(BaseTask._sub_tasks[TaskTypes.WORKER].keys())
-
     @abstractmethod
     def _upload_work_request_debug_logs(self) -> None:
         """
@@ -649,7 +640,6 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
         WorkRequestDebugLogs to source_artifact_id.
         """
 
-    @abstractmethod
     def get_input_artifacts_ids(self) -> list[int]:
         """
         Return the list of input artifact IDs used by this task.
@@ -661,40 +651,9 @@ class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
         `_source_artifacts_ids` cannot be used for this purpose because it is
         only set during task execution.
         """
-
-    def get_label(self) -> str:
-        """Return a short human-readable label for the task."""
-        from debusine.db.models import WorkRequest
-
-        return WorkRequest.objects.get(id=self.work_request_id).get_label()
-
-    def get_subject(self) -> str | None:
-        """Return the subject if known or None."""
-        if self.dynamic_data is not None:
-            return self.dynamic_data.subject
-
-        return None
-
-    def get_event_reactions(
-        self,
-        event_name: Literal[
-            "on_creation",
-            "on_unblock",
-            "on_assignment",
-            "on_success",
-            "on_failure",
-        ],
-    ) -> list[EventReaction]:
-        """
-        Return event reactions for this task.
-
-        This allows tasks to provide actions that are processed by the
-        server at various points in the lifecycle of the work request.
-        """
-        event_reactions: list[EventReaction] = []
-        if event_name in {"on_success", "on_failure"}:
-            event_reactions.append(ActionRecordInTaskHistory())
-        return event_reactions
+        if self.dynamic_data is None:
+            return []
+        return self.dynamic_data.get_input_artifacts_ids()
 
 
 class BaseExternalTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
@@ -728,13 +687,8 @@ class BaseExternalTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
 
     TASK_TYPE = TaskTypes.WORKER
 
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize the task."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         self.debusine: Debusine | None = None
 
@@ -978,111 +932,10 @@ class BaseExternalTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
         self._debug_log_files_directory = None
 
 
-def get_environment(
-    task_database: TaskDatabaseInterface,
-    lookup: LookupSingle,
-    *,
-    architecture: str | None = None,
-    backend: BackendType | None = None,
-    default_category: CollectionCategory | None = None,
-    image_category: ExecutorImageCategory | None = None,
-    try_variant: str | None = None,
-) -> ArtifactInfo:
-    """
-    Get an environment.
-
-    This automatically fills in some additional constraints if needed.
-
-    :param task_database: the :py:class:`TaskDatabaseInterface` used to
-      perform the lookup
-    :param lookup: the base lookup provided by the task data
-    :param architecture: the task's build architecture, if available
-    :param backend: the task's backend, or None if the environment lookup
-      does not need to be constrained to a particular backend
-    :param default_category: the default category to use for the first
-      segment of the lookup
-    :param image_category: try to use an environment with this image
-      category; defaults to the image category needed by the executor for
-      `self.backend`
-    :param try_variant: None if the environment lookup does not need to be
-      constrained to a particular variant; otherwise, and if `lookup` does
-      not already specify a variant, then try looking up an environment with
-      this variant first, and fall back to looking for an environment with
-      no variant
-    :return: the ArtifactInfo of a suitable environment artifact
-    """
-    lookups: list[LookupSingle] = []
-
-    if (
-        isinstance(lookup, str)
-        and len(segments := parse_lookup_string_segments(lookup)) == 2
-        and (parsed := parse_key_value_lookup_segment(segments[1]))[0]
-        == "match"
-    ):
-        # Supplement the environment lookup with the task architecture,
-        # if required.
-        lookup_type, filters = parsed
-        assert lookup_type == "match"
-
-        # TODO: If filters already have values for architecture/format, then
-        # check their consistency.
-
-        if architecture is not None:
-            filters.setdefault("architecture", architecture)
-
-        if image_category is None and backend is not None:
-            image_category = executor_class(backend).image_category
-        match image_category:
-            case ExecutorImageCategory.TARBALL:
-                filters.setdefault("format", "tarball")
-            case ExecutorImageCategory.IMAGE:
-                filters.setdefault("format", "image")
-            case _ as unreachable:
-                raise AssertionError(
-                    f"Unexpected image category: {unreachable}"
-                )
-
-        if backend is not None:
-            filters.setdefault("backend", backend)
-
-        if try_variant is not None and "variant" not in filters:
-            for variant in (try_variant, ""):
-                lookups.append(
-                    build_lookup_string_segments(
-                        segments[0],
-                        build_key_value_lookup_segment(
-                            lookup_type, {**filters, "variant": variant}
-                        ),
-                    )
-                )
-        else:
-            lookups.append(
-                build_lookup_string_segments(
-                    segments[0],
-                    build_key_value_lookup_segment(lookup_type, filters),
-                )
-            )
-    else:
-        lookups.append(lookup)
-
-    for try_lookup in lookups[:-1]:
-        try:
-            return task_database.lookup_single_artifact(
-                try_lookup, default_category=default_category
-            )
-        except KeyError:
-            pass
-
-    return task_database.lookup_single_artifact(
-        lookups[-1], default_category=default_category
-    )
-
-
-# TODO: This should be an ABC.
 class BaseTaskWithExecutor[
     TDE: BaseTaskDataWithExecutor,
     DTDE: BaseDynamicTaskDataWithExecutor,
-](BaseExternalTask[TDE, DTDE]):
+](BaseExternalTask[TDE, DTDE], metaclass=ABCMeta):
     r"""
     Base for tasks with executor capabilities.
 
@@ -1094,46 +947,6 @@ class BaseTaskWithExecutor[
 
     DEFAULT_BACKEND = BackendType.UNSHARE
 
-    def get_environment(
-        self,
-        task_database: TaskDatabaseInterface,
-        lookup: LookupSingle,
-        default_category: CollectionCategory | None = None,
-        image_category: ExecutorImageCategory | None = None,
-        set_backend: bool = True,
-        try_variant: bool = True,
-    ) -> ArtifactInfo:
-        """
-        Get an environment for an executor-capable task.
-
-        This automatically fills in some additional constraints from the task
-        data if needed.
-
-        :param task_database: the :py:class:`TaskDatabaseInterface` used to
-          perform the lookup
-        :param lookup: the base lookup provided by the task data
-        :param default_category: the default category to use for the first
-          segment of the lookup
-        :param image_category: try to use an environment with this image
-          category; defaults to the image category needed by the executor
-          for `self.backend`
-        :param set_backend: if True (default), try to use an environment
-          matching `self.backend`
-        :param try_variant: if True (default), try to use an environment
-          whose variant is `self.name`, but fall back to looking up an
-          environment without a variant if the first lookup fails
-        :return: the ArtifactInfo of a suitable environment artifact
-        """
-        return get_environment(
-            task_database=task_database,
-            lookup=lookup,
-            architecture=self.build_architecture(),
-            backend=self.backend if set_backend else None,
-            default_category=default_category,
-            image_category=image_category,
-            try_variant=self.name if try_variant else None,
-        )
-
     def _prepare_executor(self) -> None:
         """
         Prepare the executor.
@@ -1178,6 +991,15 @@ class BaseTaskWithExecutor[
             backend = self.DEFAULT_BACKEND
         return backend
 
+    @override
+    def compute_system_required_tags(self) -> set[str]:
+        # Prevent circular import
+        import debusine.worker.tags as wtags
+
+        tags = super().compute_system_required_tags()
+        tags.add(wtags.EXECUTOR_PREFIX + self.backend)
+        return tags
+
     def get_package_version(self, package_name: str) -> Version:
         """
         Get the installed version of a package in the executor.
@@ -1429,7 +1251,7 @@ class RunCommandTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
 
                 self._write_utf8(cmd_log, self.CMD_LOG_SEPARATOR)
                 if capture_stdout_filename:
-                    self._write_utf8(cmd_log, capture_stdout.read_text())
+                    cmd_log.write(capture_stdout.read_bytes() + b"\n")
                     self._write_utf8(cmd_log, self.CMD_LOG_SEPARATOR)
 
                 out_file.close()
@@ -1517,7 +1339,6 @@ class RunCommandTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
                 self.logger.debug(
                     "Task PID %s not finished after SIGTERM", p.pid
                 )
-                pass
 
             # debusine sends a SIGKILL if:
             # - SIGTERM was sent to cmd AND cmd was running 5 seconds later:
@@ -1597,6 +1418,9 @@ class ExtraRepositoryMixin[
 
     data: TDER
 
+    #: Extra repositories configured in task data
+    extra_repositories = ExtraRepositoriesInput()
+
     # Deb822 style sources:
     extra_repository_sources: list[Path]
     # One-line sources:
@@ -1604,69 +1428,12 @@ class ExtraRepositoryMixin[
     # GPG keys (for apt < 2.3.10):
     extra_repository_keys: list[Path]
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        """Initialize class variables."""
-        super().__init__(*args, **kwargs)
+    def post_init(self) -> None:
+        super().post_init()
         self.extra_repository_sources = []
         self.extra_repository_lists = []
         self.extra_repository_keys = []
 
-    @cache
-    def _get_archive_fqdn(self, task_database: TaskDatabaseInterface) -> str:
-        """Return the primary Debusine archive FQDN."""
-        return task_database.get_server_setting(
-            "DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"
-        )
-
-    def _resolve_extra_repository(
-        self,
-        task_database: TaskDatabaseInterface,
-        extra_repository: ExtraRepository,
-    ) -> ExtraExternalRepository:
-        """Resolve a single extra repository into the external form."""
-        match extra_repository:
-            case ExtraDebusineRepository():
-                suite = task_database.lookup_single_collection(
-                    extra_repository.suite
-                )
-                components = extra_repository.components
-                if components is None:
-                    if "components" not in suite.data:
-                        raise TaskConfigError(
-                            f"'components' not set for {extra_repository.suite}"
-                        )
-                    components = suite.data["components"]
-                return ExtraExternalRepository(
-                    # Ideally we'd use HTTPS, but that might require
-                    # certificate setup within environments, and we know that
-                    # our own repositories are signed, so HTTP should be good
-                    # enough to use from workers for now.
-                    url=(
-                        f"http://{self._get_archive_fqdn(task_database)}"
-                        f"/{suite.scope_name}/{suite.workspace_name}"
-                    ),
-                    suite=suite.name,
-                    components=components,
-                    signing_key=task_database.export_suite_signing_keys(
-                        suite.id
-                    ),
-                )
-            case ExtraExternalRepository():
-                return extra_repository
-            case _ as unreachable:
-                assert_never(unreachable)
-
-    def resolve_extra_repositories(
-        self, task_database: TaskDatabaseInterface
-    ) -> list[ExtraExternalRepository] | None:
-        """Resolve Debusine repositories so that the worker can handle them."""
-        if self.data.extra_repositories is None:
-            return None
-        return [
-            self._resolve_extra_repository(task_database, extra_repository)
-            for extra_repository in self.data.extra_repositories
-        ]
-
     def supports_inline_signed_by(self, codename: str) -> bool:
         """Determine if codename supports inline signatures."""
         # apt < 2.3.10 has no support for keys inline in Signed-By
@@ -1718,14 +1485,6 @@ class DefaultDynamicData[TD: BaseTaskData](
     """Base class for tasks that do not add to dynamic task data."""
 
     @override
-    def build_dynamic_data(
-        self,
-        task_database: TaskDatabaseInterface,  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    def build_dynamic_data(self) -> BaseDynamicTaskData:
         """Return default dynamic data."""
         return BaseDynamicTaskData()
-
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of source artifact IDs used by this task."""
-        return []
