@@ -30,7 +30,6 @@ from debusine.artifacts import (
 )
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     WorkRequestResults,
     get_architectures,
@@ -40,11 +39,11 @@ from debusine.artifacts.models import (
 from debusine.assets import KeyPurpose
 from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
+from debusine.tasks.inputs import EnvironmentInput, MultiInput, SingleInput
 from debusine.tasks.models import (
     AssembleSignedSourceData,
     AssembleSignedSourceDynamicData,
 )
-from debusine.tasks.server import TaskDatabaseInterface
 
 # https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-f-source
 _re_package_name = re.compile(r"^[a-z0-9][a-z0-9+.-]+$")
@@ -64,14 +63,13 @@ class AssembleSignedSource(
 
     TASK_VERSION = 1
 
+    template = SingleInput(categories=[ArtifactCategory.BINARY_PACKAGE])
+    environment = EnvironmentInput()
+    signed = MultiInput()
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize the task."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         # Set by fetch_input.
         self._template_deb_name: str | None = None
@@ -100,22 +98,14 @@ class AssembleSignedSource(
         return bool(worker_metadata.get(executor_available_key, False))
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> AssembleSignedSourceDynamicData:
+    def build_dynamic_data(self) -> AssembleSignedSourceDynamicData:
         """
         Resolve artifact lookups for this task.
 
         :subject: binary package name out of ``template``
         :parameter_summary: binary package name and version (architecture)
         """
-        template = task_database.lookup_single_artifact(self.data.template)
-
-        self.ensure_artifact_categories(
-            configuration_key="template",
-            category=template.category,
-            expected=(ArtifactCategory.BINARY_PACKAGE,),
-        )
+        template = self.template
         assert isinstance(template.data, DebianBinaryPackage)
         package_name = get_binary_package_name(template.data)
         package_version = get_binary_package_version(template.data)
@@ -124,31 +114,14 @@ class AssembleSignedSource(
         package_architectures = architectures.pop()
 
         return AssembleSignedSourceDynamicData(
-            environment_id=self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ).id,
+            environment_id=self.environment.id,
             template_id=template.id,
-            signed_ids=task_database.lookup_multiple_artifacts(
-                self.data.signed
-            ).get_ids(),
+            signed_ids=self.signed.get_ids(),
             subject=package_name,
             parameter_summary=f"{package_name}_{package_version} "
             f"({package_architectures})",
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return [
-            self.dynamic_data.environment_id,
-            self.dynamic_data.template_id,
-            *self.dynamic_data.signed_ids,
-        ]
-
     def _check_category(
         self, artifact: ArtifactResponse, expected_category: ArtifactCategory
     ) -> bool:
