@@ -17,11 +17,11 @@ from typing import Any, cast, override
 
 import pydantic
 
+import debusine.worker.tags as wtags
 from debusine import utils
 from debusine.artifacts.local_artifact import AutopkgtestArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianAutopkgtest,
     DebianAutopkgtestResult,
     DebianAutopkgtestResultStatus,
@@ -39,12 +39,12 @@ from debusine.tasks import (
     ExtraRepositoryMixin,
     RunCommandTask,
 )
+from debusine.tasks.inputs import EnvironmentInput, MultiInput, SingleInput
 from debusine.tasks.models import (
     AutopkgtestData,
     AutopkgtestDynamicData,
     BackendType,
 )
-from debusine.tasks.server import TaskDatabaseInterface
 
 log = logging.getLogger(__name__)
 
@@ -63,14 +63,28 @@ class Autopkgtest(
     ARTIFACT_DIR = "artifact-dir"
     SUMMARY_FILE = "artifact-dir/summary"
 
+    input_source_artifact = SingleInput(
+        field="input.source_artifact",
+        categories=[ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD],
+    )
+    input_binary_artifacts = MultiInput(
+        field="input.binary_artifacts",
+        categories=[ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD],
+    )
+    input_context_artifacts = MultiInput(
+        field="input.context_artifacts",
+        categories=[ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD],
+    )
+    environment = EnvironmentInput(
+        categories=[
+            ArtifactCategory.SYSTEM_TARBALL,
+            ArtifactCategory.SYSTEM_IMAGE,
+        ]
+    )
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize object."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         self._source_package_name: str | None = None
         self._source_package_version: str | None = None
@@ -91,6 +105,20 @@ class Autopkgtest(
 
         return metadata
 
+    @override
+    @classmethod
+    def get_provided_worker_tags(cls) -> set[str]:
+        tags = super().get_provided_worker_tags()
+        if utils.is_command_available("autopkgtest"):
+            tags.add(wtags.CAP_PREFIX + "autopkgtest")
+        return tags
+
+    @override
+    def compute_system_required_tags(self) -> set[str]:
+        tags = super().compute_system_required_tags()
+        tags.add(wtags.CAP_PREFIX + "autopkgtest")
+        return tags
+
     @override
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
         """Check if the specified worker can run the task."""
@@ -105,9 +133,7 @@ class Autopkgtest(
         )
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> AutopkgtestDynamicData:
+    def build_dynamic_data(self) -> AutopkgtestDynamicData:
         """
         Resolve artifact lookups for this task.
 
@@ -116,70 +142,23 @@ class Autopkgtest(
         :runtime_context: ``build_architecture:executor_backend``
         :configuration_context: codename of ``environment``
         """
-        input_source_artifact = task_database.lookup_single_artifact(
-            self.data.input.source_artifact
-        )
-        input_binary_artifacts = task_database.lookup_multiple_artifacts(
-            self.data.input.binary_artifacts
-        )
-        input_context_artifacts = task_database.lookup_multiple_artifacts(
-            self.data.input.context_artifacts
-        )
-
-        self.ensure_artifact_categories(
-            configuration_key="input.source_artifact",
-            category=input_source_artifact.category,
-            expected=(ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD),
-        )
+        input_source_artifact = self.input_source_artifact
         assert isinstance(
             input_source_artifact.data, (DebianSourcePackage, DebianUpload)
         )
+        assert isinstance(
+            self.environment.data, (DebianSystemTarball, DebianSystemImage)
+        )
+
         package_name = get_source_package_name(input_source_artifact.data)
         package_version = get_source_package_version(input_source_artifact.data)
 
-        for i, binary_artifact in enumerate(input_binary_artifacts):
-            self.ensure_artifact_categories(
-                configuration_key=f"input.binary_artifacts[{i}]",
-                category=binary_artifact.category,
-                expected=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.UPLOAD,
-                ),
-            )
-
-        for i, context_artifact in enumerate(input_context_artifacts):
-            self.ensure_artifact_categories(
-                configuration_key=f"input.context_artifacts[{i}]",
-                category=context_artifact.category,
-                expected=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.UPLOAD,
-                ),
-            )
-
-        environment = self.get_environment(
-            task_database,
-            self.data.environment,
-            default_category=CollectionCategory.ENVIRONMENTS,
-        )
-        self.ensure_artifact_categories(
-            configuration_key="environment",
-            category=environment.category,
-            expected=(
-                ArtifactCategory.SYSTEM_TARBALL,
-                ArtifactCategory.SYSTEM_IMAGE,
-            ),
-        )
-        assert isinstance(
-            environment.data, (DebianSystemTarball, DebianSystemImage)
-        )
-
         return AutopkgtestDynamicData(
-            environment_id=environment.id,
-            extra_repositories=self.resolve_extra_repositories(task_database),
+            environment_id=self.environment.id,
+            extra_repositories=self.extra_repositories,
             input_source_artifact_id=input_source_artifact.id,
-            input_binary_artifacts_ids=input_binary_artifacts.get_ids(),
-            input_context_artifacts_ids=input_context_artifacts.get_ids(),
+            input_binary_artifacts_ids=self.input_binary_artifacts.get_ids(),
+            input_context_artifacts_ids=self.input_context_artifacts.get_ids(),
             subject=package_name,
             # TODO: It would probably be better to get the architecture of
             # the input binary artifacts and use that in the parameter
@@ -187,21 +166,9 @@ class Autopkgtest(
             parameter_summary=f"{package_name}_{package_version} "
             f"({self.data.build_architecture})",
             runtime_context=f"{self.build_architecture()}:{self.backend}",
-            configuration_context=environment.data.codename,
+            configuration_context=self.environment.data.codename,
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return [
-            self.dynamic_data.environment_id,
-            self.dynamic_data.input_source_artifact_id,
-            *self.dynamic_data.input_binary_artifacts_ids,
-            *self.dynamic_data.input_context_artifacts_ids,
-        ]
-
     @override
     def check_directory_for_consistency_errors(
         self, build_directory: Path
