@@ -10,7 +10,7 @@
 """Task to use blhc in debusine."""
 
 from pathlib import Path
-from typing import Any, override
+from typing import override
 
 from debian.debian_support import Version
 
@@ -18,7 +18,6 @@ from debusine import utils
 from debusine.artifacts import BlhcArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianPackageBuildLog,
     WorkRequestResults,
     get_architectures,
@@ -27,8 +26,8 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
+from debusine.tasks.inputs import EnvironmentInput, SingleInput
 from debusine.tasks.models import BlhcData, BlhcDynamicData
-from debusine.tasks.server import TaskDatabaseInterface
 
 
 class Blhc(
@@ -41,48 +40,34 @@ class Blhc(
 
     CAPTURE_OUTPUT_FILENAME = "blhc.txt"
 
+    input_artifact = SingleInput(
+        field="input.artifact", categories=[ArtifactCategory.PACKAGE_BUILD_LOG]
+    )
+    environment = EnvironmentInput()
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize object."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         # list of build logs to be checked
         self._blhc_target: Path | None = None
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> BlhcDynamicData:
+    def build_dynamic_data(self) -> BlhcDynamicData:
         """
         Resolve artifact lookups for this task.
 
         :subject: source package name out of the analyzed build log
         :parameter_summary: source package name and version (arch)
         """
-        input_artifact = task_database.lookup_single_artifact(
-            self.data.input.artifact
-        )
-
-        self.ensure_artifact_categories(
-            configuration_key="input.artifact",
-            category=input_artifact.category,
-            expected=(ArtifactCategory.PACKAGE_BUILD_LOG,),
-        )
+        input_artifact = self.input_artifact
         assert isinstance(input_artifact.data, DebianPackageBuildLog)
         package_name = get_source_package_name(input_artifact.data)
         package_version = get_source_package_version(input_artifact.data)
         architectures = " ".join(sorted(get_architectures(input_artifact.data)))
 
         return BlhcDynamicData(
-            environment_id=self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ).id,
+            environment_id=self.environment.id,
             input_artifact_id=input_artifact.id,
             # We don't need the dose-distcheck analysis, and it's
             # potentially complex.
@@ -94,16 +79,6 @@ class Blhc(
             f"({architectures})",
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return [
-            self.dynamic_data.input_artifact_id,
-            self.dynamic_data.environment_id,
-        ]
-
     def _blhc_supports_debian_option(self) -> bool:
         """
         Check whether the environment's blhc supports ``--debian``.
