@@ -13,13 +13,12 @@ from dataclasses import dataclass
 from itertools import chain
 from pathlib import Path
 from tempfile import mkdtemp
-from typing import Any, override
+from typing import override
 
 from debusine import utils
 from debusine.artifacts import DebDiffArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebianSourcePackage,
     DebianUpload,
@@ -30,8 +29,12 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask, TaskConfigError
+from debusine.tasks.inputs import (
+    EnvironmentInput,
+    MultiInputList,
+    SingleInputList,
+)
 from debusine.tasks.models import DebDiffData, DebDiffDynamicData
-from debusine.tasks.server import TaskDatabaseInterface
 
 
 @dataclass(frozen=True, kw_only=True)
@@ -67,14 +70,19 @@ class DebDiff(
 
     CAPTURE_OUTPUT_FILENAME = "debdiff.txt"
 
+    environment = EnvironmentInput()
+    source_artifacts = SingleInputList(
+        field="input.source_artifacts",
+        categories=[ArtifactCategory.SOURCE_PACKAGE],
+    )
+    binary_artifacts = MultiInputList(
+        field="input.binary_artifacts",
+        categories=[ArtifactCategory.UPLOAD, ArtifactCategory.BINARY_PACKAGE],
+    )
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize object."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         # list of directories that fetch_input() downloaded the artifacts
         self._original_dirs: list[Path] = []
@@ -100,32 +108,17 @@ class DebDiff(
                 f"(got {original_count} and {new_count})"
             )
 
-    def _summarize_source_artifacts(
-        self, task_database: TaskDatabaseInterface
-    ) -> SourceArtifactsSummary | None:
-        if not self.data.input.source_artifacts:
+    def _summarize_source_artifacts(self) -> SourceArtifactsSummary | None:
+        source_artifacts = self.source_artifacts
+        if not source_artifacts:
             return None
 
-        source_artifacts = [
-            task_database.lookup_single_artifact(artifact)
-            for artifact in self.data.input.source_artifacts
-        ]
-
         package = None
         artifacts_ids = []
-
         for source_artifact in source_artifacts:
-            # TODO: improve configuration_key including the index?
-            self.ensure_artifact_categories(
-                configuration_key="input.source_artifacts",
-                category=source_artifact.category,
-                expected=(ArtifactCategory.SOURCE_PACKAGE,),
-            )
-
             if package is None:
                 assert isinstance(source_artifact.data, DebianSourcePackage)
                 package = get_source_package_name(source_artifact.data)
-
             artifacts_ids.append(source_artifact.id)
 
         assert isinstance(source_artifacts[0].data, DebianSourcePackage)
@@ -141,18 +134,13 @@ class DebDiff(
             ids=artifacts_ids,
         )
 
-    def _summarize_binary_artifacts(
-        self, task_database: TaskDatabaseInterface
-    ) -> BinaryArtifactsSummary | None:
-        if not self.data.input.binary_artifacts:
+    def _summarize_binary_artifacts(self) -> BinaryArtifactsSummary | None:
+        binary_artifacts = self.binary_artifacts
+        if not binary_artifacts:
             return None
 
-        binary_artifacts_original = task_database.lookup_multiple_artifacts(
-            self.data.input.binary_artifacts[0]
-        )
-        binary_artifacts_new = task_database.lookup_multiple_artifacts(
-            self.data.input.binary_artifacts[1]
-        )
+        binary_artifacts_original = binary_artifacts[0]
+        binary_artifacts_new = binary_artifacts[1]
 
         self._validate_binary_original_new_count(
             len(binary_artifacts_original), len(binary_artifacts_new)
@@ -169,16 +157,6 @@ class DebDiff(
             ids.append(multiple_artifacts.get_ids())
 
             for artifact in multiple_artifacts:
-                # TODO: improve configuration_key including the index?
-                self.ensure_artifact_categories(
-                    configuration_key="input.binary_artifacts",
-                    category=artifact.category,
-                    expected=(
-                        ArtifactCategory.UPLOAD,
-                        ArtifactCategory.BINARY_PACKAGE,
-                    ),
-                )
-
                 assert isinstance(
                     artifact.data,
                     (DebianUpload, DebianBinaryPackage),
@@ -236,9 +214,7 @@ class DebDiff(
         )
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> DebDiffDynamicData:
+    def build_dynamic_data(self) -> DebDiffDynamicData:
         """
         Resolve artifact lookups for this task.
 
@@ -247,10 +223,10 @@ class DebDiff(
           in ``input.binary_artifacts`` originate from the same source package
         :parameter_summary: subject: original_version → new_version (arch)
         """
-        source_summary = self._summarize_source_artifacts(task_database)
-        binary_summary = self._summarize_binary_artifacts(task_database)
+        source_summary = self._summarize_source_artifacts()
+        binary_summary = self._summarize_binary_artifacts()
 
-        relevant_summary = source_summary if source_summary else binary_summary
+        relevant_summary = source_summary or binary_summary
         assert relevant_summary is not None
 
         original_version = relevant_summary.original_version
@@ -275,11 +251,7 @@ class DebDiff(
                 parameter_summary += f" ({self.build_architecture()})"
 
         return DebDiffDynamicData(
-            environment_id=self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ).id,
+            environment_id=self.environment.id,
             input_source_artifacts_ids=(
                 source_summary.ids if source_summary else None
             ),
@@ -290,17 +262,6 @@ class DebDiff(
             parameter_summary=parameter_summary,
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if self.dynamic_data is None:
-            return []
-
-        input_ids = self.dynamic_data.input_source_artifacts_ids or []
-        binary_ids = self.dynamic_data.input_binary_artifacts_ids or []
-
-        return list(chain(input_ids, *binary_ids))
-
     @override
     def _cmdline(self) -> list[str]:
         """
@@ -450,7 +411,10 @@ class DebDiff(
         original = " ".join(path.name for path in self._original_targets)
         new = " ".join(path.name for path in self._new_targets)
         debdiff_artifact = DebDiffArtifact.create(
-            debdiff_output=debdiff_file, original=original, new=new
+            debdiff_output=debdiff_file,
+            original=original,
+            new=new,
+            label=f"debdiff {self.dynamic_data.parameter_summary}",
         )
 
         uploaded = self.debusine.upload_artifact(
