@@ -21,6 +21,7 @@ from debusine.tasks.executors.base import (
     analyze_worker_all_executors,
     executor_backends,
     executor_class,
+    executor_provided_tags,
 )
 from debusine.tasks.executors.incus import (
     IncusInstance,
@@ -44,4 +45,5 @@ __all__ = [
     "analyze_worker_all_executors",
     "executor_backends",
     "executor_class",
+    "executor_provided_tags",
 ]
