@@ -586,3 +586,19 @@ def analyze_worker_all_executors() -> dict[str, Any]:
         f"executor:{backend}:available": executor_class.available()
         for backend, executor_class in _backends.items()
     }
+
+
+def executor_provided_tags() -> set[str]:
+    """
+    Return a set of scheduler tags provided by executors.
+
+    This is used to signal executors' availability and feature sets.
+    """
+    # Prevent circular import
+    import debusine.worker.tags as wtags
+
+    return {
+        wtags.EXECUTOR_PREFIX + backend
+        for backend, executor_class in _backends.items()
+        if executor_class.available()
+    }
