@@ -14,7 +14,7 @@ import logging
 from pathlib import Path
 from shutil import rmtree
 
-from fasteners import InterProcessLock  # type: ignore
+from fasteners import InterProcessLock  # type: ignore[import-untyped]
 
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse
@@ -104,7 +104,6 @@ class ImageCache:
 
         This only needs to be done once, after the image is first downloaded.
         """
-        pass
 
     def image_in_cache(self, artifact: ArtifactResponse) -> bool:
         """Return whether or not artifact is available in the local cache."""
