@@ -15,6 +15,7 @@ from subprocess import CalledProcessError, CompletedProcess, PIPE
 from typing import Any, AnyStr, Literal
 from unittest.mock import Mock, call, patch
 
+import debusine.worker.tags as wtags
 from debusine.tasks.executors.base import (
     ExecutorInterface,
     ExecutorStatistics,
@@ -25,6 +26,7 @@ from debusine.tasks.executors.base import (
     analyze_worker_all_executors,
     executor_backends,
     executor_class,
+    executor_provided_tags,
 )
 from debusine.test import TestCase
 
@@ -107,6 +109,16 @@ class ExecutorTests(TestCase):
         instance = executor_class(backends[0])
         self.assertTrue(issubclass(instance, ExecutorInterface))
 
+    def test_executor_provided_tags_no_backends_available(self) -> None:
+        self.mock_is_command_available({})
+        self.assertCountEqual(executor_provided_tags(), [])
+
+    def test_executor_provided_tags_unshare_available(self) -> None:
+        self.mock_is_command_available({"unshare": True, "ip": True})
+        self.assertCountEqual(
+            executor_provided_tags(), [wtags.EXECUTOR_PREFIX + "unshare"]
+        )
+
 
 class InstanceTests(TestCase):
     """Unit tests for the Instance basic interface."""
