@@ -31,7 +31,7 @@ from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
 from debusine.test import TestCase
 
 
-class ImageCacheTest(ExternalTaskHelperMixin[Noop], TestCase):
+class BaseImageCacheTests(ExternalTaskHelperMixin[Noop], TestCase):
     """Common setup for unit tests of ImageCache class."""
 
     def setUp(self) -> None:
@@ -47,7 +47,7 @@ class ImageCacheTest(ExternalTaskHelperMixin[Noop], TestCase):
         self.debusine_api.artifact_get.return_value = self.artifact
 
 
-class ImageCacheMetadataTests(ImageCacheTest):
+class ImageCacheMetadataTests(BaseImageCacheTests):
     """Unit tests for ImageCache class that don't work with images."""
 
     def test_cache_artifact_path(self) -> None:
@@ -97,7 +97,7 @@ class ImageCacheMetadataTests(ImageCacheTest):
             self.image_cache.image_artifact(42)
 
 
-class ImageCacheTests(ImageCacheTest):
+class ImageCacheTests(BaseImageCacheTests):
     """Unit tests for ImageCache class that work with images."""
 
     def setUp(self) -> None:
