@@ -48,7 +48,7 @@ from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
 from debusine.test import TestCase
 
 
-class RunIncusCmdTest(TestCase):
+class RunIncusCmdTests(TestCase):
     """Unit tests for run_incus_cmd."""
 
     @patch("debusine.tasks.executors.incus.check_output")
@@ -113,7 +113,7 @@ class IncusImageCacheCommonTests(ExternalTaskHelperMixin[Noop], TestCase):
         """
         if path is None:
             path = Path(
-                NamedTemporaryFile(
+                NamedTemporaryFile(  # noqa: SIM115
                     prefix="debusine-test-", suffix=".tar", delete=False
                 ).name
             )
@@ -400,11 +400,13 @@ class IncusLXCImageCacheTests(IncusImageCacheCommonTests):
         """Test that prepare only accepts bootable images."""
         source = self.create_tar(files=())
         dest_buf = BytesIO()
-        with tarfile.TarFile(fileobj=dest_buf, mode="w") as dest_tar:
-            with self.assertRaises(ImageImportError):
-                self.image_cache.prepare_incus_image(
-                    source, dest_tar, self.artifact
-                )
+        with (
+            tarfile.TarFile(fileobj=dest_buf, mode="w") as dest_tar,
+            self.assertRaises(ImageImportError),
+        ):
+            self.image_cache.prepare_incus_image(
+                source, dest_tar, self.artifact
+            )
 
     def test_prepare_incus_image_accepts_usrmerged_init(self) -> None:
         """Test that prepare accepts a /usr/sbin/init."""
