@@ -427,8 +427,7 @@ class UnshareInstanceTests(TestCase):
     def patch_unshare_command_args(self, command: str, args: list[str]) -> None:
         """Patch UnshareExecutor._unshare_command and _unshare_default_args."""
         unshare_command = mock.patch(
-            "debusine.tasks.executors.unshare."
-            "UnshareInstance.unshare_command",
+            "debusine.tasks.executors.unshare.UnshareInstance.unshare_command",
             new=command,
         )
         unshare_command.start()
@@ -465,16 +464,18 @@ class UnshareInstanceTests(TestCase):
         stderr_file_path = self.create_temporary_file()
         stdout_file_path = self.create_temporary_file()
 
-        with stderr_file_path.open("wb") as stderr_file:
-            with stdout_file_path.open("wb") as stdout_file:
-                UnshareInstance._run_unshare_command(
-                    [
-                        f"printf %s '{stderr_contents}' 1>&2; "
-                        f"printf %s '{stdout_contents}'",
-                    ],
-                    stderr=stderr_file.fileno(),
-                    stdout=stdout_file.fileno(),
-                )
+        with (
+            stderr_file_path.open("wb") as stderr_file,
+            stdout_file_path.open("wb") as stdout_file,
+        ):
+            UnshareInstance._run_unshare_command(
+                [
+                    f"printf %s '{stderr_contents}' 1>&2; "
+                    f"printf %s '{stdout_contents}'",
+                ],
+                stderr=stderr_file.fileno(),
+                stdout=stdout_file.fileno(),
+            )
 
         self.assertEqual(stderr_file_path.read_text(), stderr_contents)
         self.assertEqual(stdout_file_path.read_text(), stdout_contents)
