@@ -21,9 +21,7 @@ from typing import Any, override
 from debusine.artifacts import SigningInputArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
-    DebianUpload,
     WorkRequestResults,
     get_architectures,
     get_binary_package_name,
@@ -31,11 +29,15 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
+from debusine.tasks.inputs import (
+    EnvironmentInput,
+    SingleInput,
+    UploadArtifactsInput,
+)
 from debusine.tasks.models import (
     ExtractForSigningData,
     ExtractForSigningDynamicData,
 )
-from debusine.tasks.server import TaskDatabaseInterface
 
 # https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-f-source
 _re_package_name = re.compile(r"^[a-z0-9][a-z0-9+.-]+$")
@@ -53,14 +55,19 @@ class ExtractForSigning(
 
     TASK_VERSION = 1
 
+    template_artifact = SingleInput(
+        field="input.template_artifact",
+        categories=[ArtifactCategory.BINARY_PACKAGE],
+    )
+    binary_artifacts = UploadArtifactsInput(
+        field="input.binary_artifacts",
+        categories=[ArtifactCategory.BINARY_PACKAGE],
+    )
+    environment = EnvironmentInput()
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize the task."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         # Set by fetch_input.
         self._template_deb_name: str | None = None
@@ -90,77 +97,31 @@ class ExtractForSigning(
         return bool(worker_metadata.get(executor_available_key, False))
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> ExtractForSigningDynamicData:
+    def build_dynamic_data(self) -> ExtractForSigningDynamicData:
         """
         Resolve artifact lookups for this task.
 
         :subject: binary package name of ``template_artifact``
         :parameter_summary: same as subject (architecture)
         """
-        template_artifact = task_database.lookup_single_artifact(
-            self.data.input.template_artifact
-        )
-
-        self.ensure_artifact_categories(
-            configuration_key="input.template_artifact",
-            category=template_artifact.category,
-            expected=(ArtifactCategory.BINARY_PACKAGE,),
-        )
+        template_artifact = self.template_artifact
         assert isinstance(template_artifact.data, DebianBinaryPackage)
         binary_package_name = get_binary_package_name(template_artifact.data)
         version = get_binary_package_version(template_artifact.data)
 
-        binary_artifacts = task_database.lookup_multiple_artifacts(
-            self.data.input.binary_artifacts
-        )
+        binary_artifacts = self.binary_artifacts
         individual_binary_artifact_ids = []
-        upload_artifact_ids = []
         architectures = set()
 
-        for i, binary_artifact in enumerate(binary_artifacts):
-            self.ensure_artifact_categories(
-                configuration_key=f"input.binary_artifacts[{i}]",
-                category=binary_artifact.category,
-                expected=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.UPLOAD,
-                ),
-            )
-            match binary_artifact.category:
-                case ArtifactCategory.BINARY_PACKAGE:
-                    individual_binary_artifact_ids.append(binary_artifact.id)
-                    assert isinstance(binary_artifact.data, DebianBinaryPackage)
-                    architectures |= get_architectures(binary_artifact.data)
-
-                case ArtifactCategory.UPLOAD:
-                    upload_artifact_ids.append(binary_artifact.id)
-                    assert isinstance(binary_artifact.data, DebianUpload)
-                    architectures |= get_architectures(binary_artifact.data)
-
-                case _ as unreachable:
-                    raise AssertionError(
-                        f"Unexpected artifact category: {unreachable}"
-                    )
-
-        if upload_artifact_ids:
-            individual_binary_artifact_ids.extend(
-                task_database.find_related_artifacts(
-                    upload_artifact_ids,
-                    ArtifactCategory.BINARY_PACKAGE,
-                    relation_type=RelationType.EXTENDS,
-                ).get_ids()
-            )
+        for binary_artifact in binary_artifacts:
+            individual_binary_artifact_ids.append(binary_artifact.id)
+            assert isinstance(binary_artifact.data, DebianBinaryPackage)
+            architectures |= get_architectures(binary_artifact.data)
 
         architectures_str = ", ".join(sorted(architectures))
 
         return ExtractForSigningDynamicData(
-            environment_id=self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ).id,
+            environment_id=self.environment.id,
             input_template_artifact_id=template_artifact.id,
             input_binary_artifacts_ids=individual_binary_artifact_ids,
             subject=binary_package_name,
@@ -168,17 +129,6 @@ class ExtractForSigning(
             f"({architectures_str})",
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return [
-            self.dynamic_data.environment_id,
-            self.dynamic_data.input_template_artifact_id,
-            *self.dynamic_data.input_binary_artifacts_ids,
-        ]
-
     @override
     def fetch_input(self, destination: Path) -> bool:
         """Download the required artifacts."""
