@@ -20,7 +20,6 @@ from debusine import utils
 from debusine.artifacts import LintianArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebianLintianSummary,
     DebianSourcePackage,
@@ -33,8 +32,12 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
+from debusine.tasks.inputs import (
+    MultiInput,
+    OptionalEnvironmentInput,
+    OptionalSingleInput,
+)
 from debusine.tasks.models import LintianData, LintianDynamicData
-from debusine.tasks.server import TaskDatabaseInterface
 
 
 class Lintian(
@@ -62,14 +65,24 @@ class Lintian(
 
     CAPTURE_OUTPUT_FILENAME = "lintian.txt"
 
+    input_source_artifact = OptionalSingleInput(
+        field="input.source_artifact",
+        categories=[ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD],
+    )
+    input_binary_artifacts = MultiInput(
+        field="input.binary_artifacts",
+        categories=[ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD],
+    )
+    environment = OptionalEnvironmentInput(
+        categories=[
+            ArtifactCategory.SYSTEM_TARBALL,
+            ArtifactCategory.SYSTEM_IMAGE,
+        ]
+    )
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize object."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
 
         # In the lintian cmd: the target that is tested (local file, .dsc
         # or .changes)
@@ -92,9 +105,7 @@ class Lintian(
             self._severities_to_level[severity] = ranking
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> LintianDynamicData:
+    def build_dynamic_data(self) -> LintianDynamicData:
         """
         Resolve artifact lookups for this task.
 
@@ -106,23 +117,9 @@ class Lintian(
           ``distribution`` is ``codename`` of ``environment``.
         :configuration_context: ``codename`` of ``environment``
         """
-        input_source_artifact = task_database.lookup_single_artifact(
-            self.data.input.source_artifact
-        )
-        input_binary_artifacts = task_database.lookup_multiple_artifacts(
-            self.data.input.binary_artifacts
-        )
-
         subject = None
+        input_source_artifact = self.input_source_artifact
         if input_source_artifact is not None:
-            self.ensure_artifact_categories(
-                configuration_key="input.source_artifact",
-                category=input_source_artifact.category,
-                expected=(
-                    ArtifactCategory.SOURCE_PACKAGE,
-                    ArtifactCategory.UPLOAD,
-                ),
-            )
             assert isinstance(
                 input_source_artifact.data, (DebianSourcePackage, DebianUpload)
             )
@@ -130,20 +127,11 @@ class Lintian(
             version = get_source_package_version(input_source_artifact.data)
 
         # Validate input_binary_artifacts and get subject
-        for i, binary_artifact in enumerate(input_binary_artifacts):
-            self.ensure_artifact_categories(
-                configuration_key=f"input.binary_artifacts[{i}]",
-                category=binary_artifact.category,
-                expected=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.UPLOAD,
-                ),
-            )
+        for binary_artifact in self.input_binary_artifacts:
             assert isinstance(
                 binary_artifact.data,
                 (DebianBinaryPackage, DebianUpload),
             )
-
             if subject is None:
                 subject = get_source_package_name(binary_artifact.data)
                 version = get_source_package_version(binary_artifact.data)
@@ -163,22 +151,8 @@ class Lintian(
         runtime_context = None
         configuration_context = None
         environment_id = None
-        if self.data.environment is not None:
-            environment = self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            )
+        if (environment := self.environment) is not None:
             environment_id = environment.id
-
-            self.ensure_artifact_categories(
-                configuration_key="environment",
-                category=environment.category,
-                expected=(
-                    ArtifactCategory.SYSTEM_TARBALL,
-                    ArtifactCategory.SYSTEM_IMAGE,
-                ),
-            )
             assert isinstance(
                 environment.data, (DebianSystemTarball, DebianSystemImage)
             )
@@ -193,26 +167,13 @@ class Lintian(
                 if input_source_artifact is None
                 else input_source_artifact.id
             ),
-            input_binary_artifacts_ids=input_binary_artifacts.get_ids(),
+            input_binary_artifacts_ids=self.input_binary_artifacts.get_ids(),
             subject=subject,
             parameter_summary=f"{subject}_{version}",
             runtime_context=runtime_context,
             configuration_context=configuration_context,
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        result = []
-        if val := self.dynamic_data.input_source_artifact_id:
-            result.append(val)
-        if val := self.dynamic_data.environment_id:
-            result.append(val)
-        result += self.dynamic_data.input_binary_artifacts_ids
-        return result
-
     @classmethod
     def generate_severity_count_zero(cls) -> dict[str, int]:
         """Return dictionary. Keys: severities. Value: 0."""
