@@ -13,7 +13,7 @@ import re
 from collections import defaultdict
 from collections.abc import Sequence
 from pathlib import Path
-from typing import Any, override
+from typing import override
 
 from debian.deb822 import Changes
 
@@ -21,13 +21,14 @@ from debusine.artifacts import Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
     DebianUpload,
+    WorkRequestResults,
     get_architectures,
     get_source_package_name,
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseExternalTask
+from debusine.tasks.inputs import MultiInput
 from debusine.tasks.models import MergeUploadsData, MergeUploadsDynamicData
-from debusine.tasks.server import TaskDatabaseInterface
 from debusine.utils import find_files_suffixes
 
 
@@ -44,39 +45,27 @@ class MergeUploads(BaseExternalTask[MergeUploadsData, MergeUploadsDynamicData]):
 
     TASK_VERSION = 1
 
+    uploads = MultiInput(
+        field="input.uploads", categories=[ArtifactCategory.UPLOAD]
+    )
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize (constructor)."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
         self._changes_paths: list[Path] = []
         self._upload_artifact: Upload | None = None
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> MergeUploadsDynamicData:
+    def build_dynamic_data(self) -> MergeUploadsDynamicData:
         """
         Resolve artifact lookups for this task.
 
         :subject: shared source package name across all uploads
         :parameter_summary: subject (list of architectures)
         """
-        upload_artifacts = task_database.lookup_multiple_artifacts(
-            self.data.input.uploads
-        )
-
         source_package_names = set()
         architectures = set()
-        for upload_artifact in upload_artifacts:
-            self.ensure_artifact_categories(
-                configuration_key="input.uploads",
-                category=upload_artifact.category,
-                expected=(ArtifactCategory.UPLOAD,),
-            )
+        for upload_artifact in self.uploads:
             assert isinstance(upload_artifact.data, DebianUpload)
 
             source_package_names.add(
@@ -99,17 +88,11 @@ class MergeUploads(BaseExternalTask[MergeUploadsData, MergeUploadsDynamicData]):
             parameter_summary = None
 
         return MergeUploadsDynamicData(
-            input_uploads_ids=upload_artifacts.get_ids(),
+            input_uploads_ids=self.uploads.get_ids(),
             subject=subject,
             parameter_summary=parameter_summary,
         )
 
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return self.dynamic_data.input_uploads_ids
-
     @override
     def fetch_input(self, destination: Path) -> bool:
         """Populate work directory with user-specified binary artifact(s)."""
@@ -311,19 +294,19 @@ class MergeUploads(BaseExternalTask[MergeUploadsData, MergeUploadsDynamicData]):
         return Upload.create(changes_file=merged_changes_path)
 
     @override
-    def run(self, execute_directory: Path) -> bool:
+    def run(self, execute_directory: Path) -> WorkRequestResults:
         """Do the main work of the task."""
         all_changes = [self._read_changes(path) for path in self._changes_paths]
         merged_changes_path = self.merge_changes(execute_directory, all_changes)
         self._upload_artifact = self.make_upload_artifact(merged_changes_path)
-        return True
+        return WorkRequestResults.SUCCESS
 
     @override
     def upload_artifacts(
         self,
         execute_directory: Path,
         *,
-        execution_success: bool,  # noqa: U100
+        execution_result: WorkRequestResults,  # noqa: U100
     ) -> None:
         """Create DebianUpload artifact and relationships."""
         if not self.debusine:
