@@ -13,6 +13,7 @@ import shlex
 from pathlib import Path
 from typing import Any, override
 
+import debusine.worker.tags as wtags
 from debusine import utils
 from debusine.artifacts.local_artifact import DebianSystemTarballArtifact
 from debusine.artifacts.models import WorkRequestResults
@@ -38,6 +39,20 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
 
         return metadata
 
+    @override
+    @classmethod
+    def get_provided_worker_tags(cls) -> set[str]:
+        tags = super().get_provided_worker_tags()
+        if utils.is_command_available("mmdebstrap"):
+            tags.add(wtags.CAP_PREFIX + "mmdebstrap")
+        return tags
+
+    @override
+    def compute_system_required_tags(self) -> set[str]:
+        tags = super().compute_system_required_tags()
+        tags.add(wtags.CAP_PREFIX + "mmdebstrap")
+        return tags
+
     @override
     def build_architecture(self) -> str:
         """Return architecture."""
@@ -231,8 +246,8 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
                 # the devices in /dev are always created
                 "with_dev": True,
                 "with_init": with_init,
-                # TODO / XXX: is "mirror" meant to be the first repository?
-                # or "mirrors" and list all of them?
+                # TODO: Is "mirror" meant to be the first repository?
+                # Or "mirrors" and list all of them?
                 # Or we could duplicate all the bootstrap_repositories...
                 "mirror": main_bootstrap_repository.mirror,
             },
