@@ -16,9 +16,18 @@ from collections.abc import Iterator
 from dataclasses import dataclass
 from enum import StrEnum
 from functools import partial
-from itertools import groupby
+from itertools import chain, groupby
 from pathlib import Path
-from typing import Annotated, Any, ClassVar, Literal, Self, TypeAlias, Union
+from typing import (
+    Annotated,
+    Any,
+    ClassVar,
+    Literal,
+    Self,
+    TypeAlias,
+    Union,
+    override,
+)
 
 import debian.deb822 as deb822
 import pydantic
@@ -155,7 +164,7 @@ class BaseExtraRepository(BaseTaskDataModel, ABC):
         cls, value: list[str] | None
     ) -> list[str] | None:
         """Rewrite an empty components list to None."""
-        return value if value else None
+        return value or None
 
 
 class ExtraDebusineRepository(BaseExtraRepository):
@@ -258,6 +267,7 @@ class LookupDict(BaseTaskDataModel):
         if not isinstance(data, dict):
             raise ValueError(f"Expected dict, got {type(data)!r}")
         data = data.copy()
+        assert isinstance(data, dict)
 
         data_matchers: dict[str, CollectionItemMatcher] = dict(
             data.get("data_matchers", ())
@@ -633,6 +643,22 @@ class BaseDynamicTaskData(BaseTaskDataModel):
     # same task, and is used to look up configuration parameters.
     configuration_context: str | None = None
 
+    def get_source_package_name(self) -> str | None:
+        """
+        Extract the source package name.
+
+        :returns: the source package name, if applicable, and None otherwise.
+        """
+        return None
+
+    def get_input_artifacts_ids(self) -> list[int]:
+        """
+        Return the list of input artifact IDs used by this task.
+
+        This is used by views to show what artifacts were used by a task.
+        """
+        return []
+
 
 class BaseDynamicTaskDataWithExecutor(BaseDynamicTaskData):
     """Dynamic task data for executors."""
@@ -786,6 +812,19 @@ class AutopkgtestDynamicData(
         default_factory=list
     )
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        return self.subject
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [
+            self.environment_id,
+            self.input_source_artifact_id,
+            *self.input_binary_artifacts_ids,
+            *self.input_context_artifacts_ids,
+        ]
+
 
 class LintianInput(BaseTaskDataModel):
     """Input for a lintian task."""
@@ -834,6 +873,20 @@ class LintianDynamicData(BaseDynamicTaskDataWithExecutor):
     input_source_artifact_id: int | None = None
     input_binary_artifacts_ids: list[int] = pydantic.Field(default_factory=list)
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        return self.subject
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        result = []
+        if val := self.input_source_artifact_id:
+            result.append(val)
+        if val := self.environment_id:
+            result.append(val)
+        result += self.input_binary_artifacts_ids
+        return result
+
 
 class BlhcInput(BaseTaskDataModel):
     """Input for a blhc task."""
@@ -881,6 +934,14 @@ class BlhcDynamicData(BaseDynamicTaskDataWithExecutor):
     # required here.
     environment_id: int
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        return self.subject
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [self.input_artifact_id, self.environment_id]
+
 
 class DebDiffInput(BaseTaskDataModel):
     """Input for a debdiff task."""
@@ -963,6 +1024,18 @@ class DebDiffDynamicData(BaseDynamicTaskDataWithExecutor):
             )
         return self
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        if self.subject and self.subject.startswith("source:"):
+            return self.subject.removeprefix("source:")
+        return None
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        input_ids = self.input_source_artifacts_ids or []
+        binary_ids = self.input_binary_artifacts_ids or []
+        return list(chain(input_ids, *binary_ids))
+
 
 class MmDebstrapVariant(StrEnum):
     """Variants supported by `mmdebstrap`."""
@@ -1175,6 +1248,14 @@ class PiupartsDynamicData(
     input_binary_artifacts_ids: list[int]
     base_tgz_id: int
 
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [
+            self.environment_id,
+            self.base_tgz_id,
+            *self.input_binary_artifacts_ids,
+        ]
+
 
 class SbuildInput(BaseTaskDataModel):
     """Input for a sbuild task."""
@@ -1256,6 +1337,19 @@ class SbuildDynamicData(
     )
     binnmu_maintainer: str | None = None
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        return self.subject
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        result: list[int] = []
+        if val := self.environment_id:
+            result.append(val)
+        result.append(self.input_source_artifact_id)
+        result.extend(self.input_extra_binary_artifacts_ids)
+        return result
+
 
 class ImageCacheUsageLogEntry(BaseTaskDataModel):
     """Entry in ImageCacheUsageLog for cached executor images."""
@@ -1328,6 +1422,14 @@ class ExtractForSigningDynamicData(BaseDynamicTaskDataWithExecutor):
     input_template_artifact_id: int
     input_binary_artifacts_ids: list[int]
 
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [
+            self.environment_id,
+            self.input_template_artifact_id,
+            *self.input_binary_artifacts_ids,
+        ]
+
 
 class AssembleSignedSourceData(BaseTaskDataWithExecutor):
     """In-memory task data for the AssembleSignedSource task."""
@@ -1361,6 +1463,10 @@ class AssembleSignedSourceDynamicData(BaseDynamicTaskDataWithExecutor):
     template_id: int
     signed_ids: list[int]
 
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [self.environment_id, self.template_id, *self.signed_ids]
+
 
 class MakeSourcePackageUploadInput(BaseTaskDataModel):
     """Input for a MakeSourcePackageUpload task."""
@@ -1389,6 +1495,14 @@ class MakeSourcePackageUploadDynamicData(BaseDynamicTaskDataWithExecutor):
 
     input_source_artifact_id: int
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        return self.subject
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return [self.environment_id, self.input_source_artifact_id]
+
 
 class MergeUploadsInput(BaseTaskDataModel):
     """Input for a MergeUploads task."""
@@ -1415,6 +1529,14 @@ class MergeUploadsDynamicData(BaseDynamicTaskData):
 
     input_uploads_ids: list[int]
 
+    @override
+    def get_source_package_name(self) -> str | None:
+        return self.subject
+
+    @override
+    def get_input_artifacts_ids(self) -> list[int]:
+        return self.input_uploads_ids
+
 
 @dataclass(frozen=True)
 class InputArtifactSingle:
