@@ -23,7 +23,7 @@ from collections.abc import Collection
 from io import BytesIO
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Any, override
+from typing import override
 
 from debian.debian_support import Version
 
@@ -32,7 +32,6 @@ from debian import debfile
 from debusine.artifacts import PiupartsArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebianPiuparts,
     DebianSystemTarball,
@@ -49,8 +48,8 @@ from debusine.tasks import (
     TaskConfigError,
 )
 from debusine.tasks.executors import ExecutorImageCategory
+from debusine.tasks.inputs import EnvironmentInput, MultiInput
 from debusine.tasks.models import BackendType, PiupartsData, PiupartsDynamicData
-from debusine.tasks.server import TaskDatabaseInterface
 
 
 class Piuparts(
@@ -64,14 +63,22 @@ class Piuparts(
 
     CAPTURE_OUTPUT_FILENAME = "piuparts.txt"
 
+    base_tgz = EnvironmentInput(
+        image_category=ExecutorImageCategory.TARBALL,
+        set_backend=False,
+        try_variant=False,
+        categories=[ArtifactCategory.SYSTEM_TARBALL],
+    )
+    environment = EnvironmentInput()
+    input_binary_artifacts = MultiInput(
+        field="input.binary_artifacts",
+        categories=[ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD],
+    )
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize (constructor)."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
+
         # *.deb paths and versions. Set by self.configure_for_execution()
         self._deb_files: list[Path] = []
         self._deb_versions: dict[str, str] | None = None
@@ -87,9 +94,7 @@ class Piuparts(
         self._scripts_dir: Path | None = None
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> PiupartsDynamicData:
+    def build_dynamic_data(self) -> PiupartsDynamicData:
         """
         Resolve artifact lookups for this task.
 
@@ -100,36 +105,12 @@ class Piuparts(
           is from ``base_tgz``
         :configuration_context: ``codename`` from ``base_tgz``
         """
-        input_binary_artifacts = task_database.lookup_multiple_artifacts(
-            self.data.input.binary_artifacts
-        )
-        base_tgz = self.get_environment(
-            task_database,
-            self.data.base_tgz,
-            default_category=CollectionCategory.ENVIRONMENTS,
-            image_category=ExecutorImageCategory.TARBALL,
-            set_backend=False,
-            try_variant=False,
-        )
-
-        self.ensure_artifact_categories(
-            configuration_key="base_tgz",
-            category=base_tgz.category,
-            expected=(ArtifactCategory.SYSTEM_TARBALL,),
-        )
+        base_tgz = self.base_tgz
         assert isinstance(base_tgz.data, DebianSystemTarball)
 
         package_names = []
         source_versions = set()
-        for i, binary_artifact in enumerate(input_binary_artifacts):
-            self.ensure_artifact_categories(
-                configuration_key=f"input.binary_artifacts[{i}]",
-                category=binary_artifact.category,
-                expected=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.UPLOAD,
-                ),
-            )
+        for i, binary_artifact in enumerate(self.input_binary_artifacts):
             assert isinstance(
                 binary_artifact.data,
                 (DebianBinaryPackage, DebianUpload),
@@ -156,13 +137,9 @@ class Piuparts(
         parameter_summary += f" ({self.build_architecture()})"
 
         return PiupartsDynamicData(
-            environment_id=self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ).id,
-            extra_repositories=self.resolve_extra_repositories(task_database),
-            input_binary_artifacts_ids=input_binary_artifacts.get_ids(),
+            environment_id=self.environment.id,
+            extra_repositories=self.extra_repositories,
+            input_binary_artifacts_ids=self.input_binary_artifacts.get_ids(),
             base_tgz_id=base_tgz.id,
             subject=" ".join(package_names_sorted),
             parameter_summary=parameter_summary,
@@ -170,17 +147,6 @@ class Piuparts(
             configuration_context=configuration_context,
         )
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        if not self.dynamic_data:
-            return []
-        return [
-            self.dynamic_data.environment_id,
-            self.dynamic_data.base_tgz_id,
-            *self.dynamic_data.input_binary_artifacts_ids,
-        ]
-
     @override
     def fetch_input(self, destination: Path) -> bool:
         """Populate work directory with user-specified binary artifact(s)."""
@@ -557,10 +523,12 @@ class Piuparts(
             data=DebianPiuparts(
                 piuparts_version=str(self._piuparts_version),
                 binary_packages=self._deb_versions,
-                # TODO: Should this be the architecture of the binary
-                # packages instead?  If we're testing `Architecture: all`
-                # binary packages on amd64, "all" seems like a better value
-                # here.
+                # It may seem as though this should be the architecture of
+                # the binary packages instead (for example, we might be
+                # testing `Architecture: all` binary packages on amd64).
+                # However, the results of running `piuparts` inherently
+                # depend on the build architecture in other ways even if the
+                # input artifacts all happen to be architecture-independent.
                 architecture=self.data.build_architecture,
                 distribution=f"{base_tar_data.vendor}:{base_tar_data.codename}",
             ),
