@@ -23,6 +23,7 @@ import pydantic
 import yaml
 
 import debusine.utils
+import debusine.worker.tags as wtags
 from debusine.artifacts import (
     BinaryPackage,
     PackageBuildLog,
@@ -31,7 +32,6 @@ from debusine.artifacts import (
 )
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianSourcePackage,
     DebianSystemImage,
     DebianSystemTarball,
@@ -44,6 +44,13 @@ from debusine.tasks import (
     ExtraRepositoryMixin,
     RunCommandTask,
 )
+from debusine.tasks.inputs import (
+    DebusineFQDNInput,
+    EnvironmentInput,
+    ExtraRepositoriesInput,
+    MultiInput,
+    SingleInput,
+)
 from debusine.tasks.models import (
     BackendType,
     SbuildBuildComponent,
@@ -51,7 +58,6 @@ from debusine.tasks.models import (
     SbuildDynamicData,
 )
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
-from debusine.tasks.server import TaskDatabaseInterface
 from debusine.utils import read_dsc
 
 
@@ -65,14 +71,26 @@ class Sbuild(
 
     TASK_VERSION = 1
 
+    input_source_artifact = SingleInput(
+        field="input.source_artifact",
+        categories=[ArtifactCategory.SOURCE_PACKAGE],
+    )
+    input_extra_binary_artifacts = MultiInput(
+        field="input.extra_binary_artifacts"
+    )
+    environment = EnvironmentInput(
+        categories=[
+            ArtifactCategory.SYSTEM_TARBALL,
+            ArtifactCategory.SYSTEM_IMAGE,
+        ]
+    )
+    extra_repositories = ExtraRepositoriesInput()
+    debusine_fqdn = DebusineFQDNInput()
+
     @override
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize the sbuild task."""
-        super().__init__(task_data, dynamic_task_data)
+    def post_init(self) -> None:
+        super().post_init()
+
         self.chroots: list[str] | None = None
         self.builder = "sbuild"
 
@@ -81,9 +99,7 @@ class Sbuild(
         self._extra_packages: list[Path] = []
 
     @override
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface
-    ) -> SbuildDynamicData:
+    def build_dynamic_data(self) -> SbuildDynamicData:
         r"""
         Resolve artifact lookups for this task.
 
@@ -95,35 +111,14 @@ class Sbuild(
           None if ``build_options`` is provided
         :configuration_context: codename of ``environment``
         """
-        input_source_artifact = task_database.lookup_single_artifact(
-            self.data.input.source_artifact
-        )
-        self.ensure_artifact_categories(
-            configuration_key="input.source_artifact",
-            category=input_source_artifact.category,
-            expected=(ArtifactCategory.SOURCE_PACKAGE,),
-        )
+        input_source_artifact = self.input_source_artifact
         assert isinstance(input_source_artifact.data, DebianSourcePackage)
 
-        environment = self.get_environment(
-            task_database,
-            self.data.environment,
-            default_category=CollectionCategory.ENVIRONMENTS,
-        )
-        self.ensure_artifact_categories(
-            configuration_key="environment",
-            category=environment.category,
-            expected=(
-                ArtifactCategory.SYSTEM_TARBALL,
-                ArtifactCategory.SYSTEM_IMAGE,
-            ),
-        )
+        environment = self.environment
         assert isinstance(
             environment.data, (DebianSystemTarball, DebianSystemImage)
         )
 
-        DEBUSINE_FQDN = task_database.get_server_setting("DEBUSINE_FQDN")
-
         if self._build_options():
             # Do not record stats for non-standard builds
             runtime_context = None
@@ -141,14 +136,12 @@ class Sbuild(
 
         return SbuildDynamicData(
             environment_id=environment.id,
-            extra_repositories=self.resolve_extra_repositories(task_database),
+            extra_repositories=self.extra_repositories,
             input_source_artifact_id=input_source_artifact.id,
             input_extra_binary_artifacts_ids=(
-                task_database.lookup_multiple_artifacts(
-                    self.data.input.extra_binary_artifacts
-                ).get_ids()
+                self.input_extra_binary_artifacts.get_ids()
             ),
-            binnmu_maintainer=f"Debusine <noreply@{DEBUSINE_FQDN}>",
+            binnmu_maintainer=f"Debusine <noreply@{self.debusine_fqdn}>",
             subject=source_package_name,
             parameter_summary=f"{source_package_name}_{source_version} "
             f"({self._host_architecture})",
@@ -168,23 +161,6 @@ class Sbuild(
             case _:
                 return self.data.build_architecture
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """
-        Return the list of input artifact IDs used by this task.
-
-        This refers to the artifacts actually used by the task. If
-        dynamic_data is empty, this returns the empty list.
-        """
-        if not self.dynamic_data:
-            return []
-        result: list[int] = []
-        if val := self.dynamic_data.environment_id:
-            result.append(val)
-        result.append(self.dynamic_data.input_source_artifact_id)
-        result.extend(self.dynamic_data.input_extra_binary_artifacts_ids)
-        return result
-
     @override
     @classmethod
     def analyze_worker(cls) -> dict[str, Any]:
@@ -196,6 +172,20 @@ class Sbuild(
 
         return metadata
 
+    @override
+    @classmethod
+    def get_provided_worker_tags(cls) -> set[str]:
+        tags = super().get_provided_worker_tags()
+        if debusine.utils.is_command_available("sbuild"):
+            tags.add(wtags.CAP_PREFIX + "sbuild")
+        return tags
+
+    @override
+    def compute_system_required_tags(self) -> set[str]:
+        tags = super().compute_system_required_tags()
+        tags.add(wtags.CAP_PREFIX + "sbuild")
+        return tags
+
     @override
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
         """Check the specified worker can run the requested task."""
@@ -662,6 +652,12 @@ class Sbuild(
                     remote_binary_package.id,
                     RelationType.RELATES_TO,
                 )
+                if remote_build_log is not None:
+                    self.debusine.relation_create(
+                        remote_build_log.id,
+                        remote_binary_upload.id,
+                        RelationType.RELATES_TO,
+                    )
 
         if (
             remote_binary_upload is not None
@@ -696,13 +692,12 @@ class Sbuild(
     def upload_artifacts(
         self, directory: Path, *, execution_result: WorkRequestResults
     ) -> None:
-        """
-        Upload the artifacts from directory.
+        """Upload artifacts for the task.
 
         :param directory: directory containing the files that
           will be uploaded.
-        :param execution_success: if False skip uploading .changes and
-          *.deb/*.udeb
+        :param execution_result: if this is not ``WorkRequestResults.SUCCESS``,
+          skip uploading ``.changes`` and ``*.deb``/``*.udeb``.
         """
         if not self.debusine:
             raise AssertionError("self.debusine not set")
