@@ -8,13 +8,14 @@
 # contained in the LICENSE file.
 
 """Ontology definition of SystemBootstrap."""
+
 import abc
 import re
 import shlex
 import subprocess
 import tempfile
 from pathlib import Path
-from typing import Any, Generic, TypeVar, cast
+from typing import cast, override
 from urllib.parse import urljoin
 
 import requests
@@ -30,16 +31,10 @@ from debusine.tasks.models import (
     SystemBootstrapRepositoryCheckSignatureWith,
     SystemBootstrapRepositoryKeyring,
 )
-from debusine.tasks.server import TaskDatabaseInterface
-
-SBD = TypeVar("SBD", bound=SystemBootstrapData)
 
 
-class SystemBootstrap(
-    abc.ABC,
-    RunCommandTask[SBD, BaseDynamicTaskData],
-    DefaultDynamicData[SBD],
-    Generic[SBD],
+class SystemBootstrap[SBD: SystemBootstrapData](
+    abc.ABC, RunCommandTask[SBD, BaseDynamicTaskData], DefaultDynamicData[SBD]
 ):
     """Implement ontology SystemBootstrap."""
 
@@ -47,13 +42,10 @@ class SystemBootstrap(
 
     _OS_RELEASE_FILE = "os-release"
 
-    def __init__(
-        self,
-        task_data: dict[str, Any],
-        dynamic_task_data: dict[str, Any] | None = None,
-    ) -> None:
-        """Initialize SystemBootstrap."""
-        super().__init__(task_data, dynamic_task_data)
+    @override
+    def post_init(self) -> None:
+        super().post_init()
+
         # keyrings that will be uploaded into the chroot
         self._upload_keyrings: list[Path] = []
 
@@ -78,7 +70,7 @@ class SystemBootstrap(
             SystemBootstrapRepositoryKeyring, repository.keyring
         )
         keyring, actual_sha256sum = get_url_contents_sha256sum(
-            repo_keyring.url, 100 * 1024 * 1024, allow_file=True
+            str(repo_keyring.url), 100 * 1024 * 1024, allow_file=True
         )
 
         if expected := repo_keyring.sha256sum:
@@ -91,7 +83,7 @@ class SystemBootstrap(
 
         # Disable auto-deletion: the file will be deleted together
         # with keyring_directory when the RunCommandTask finishes
-        file = tempfile.NamedTemporaryFile(
+        file = tempfile.NamedTemporaryFile(  # noqa: SIM115
             dir=keyring_directory,
             prefix="keyring-repo-",
             # Detect ASCII armoring:
@@ -208,13 +200,13 @@ class SystemBootstrap(
 
         return components
 
-    def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    @override
+    def build_dynamic_data(self) -> BaseDynamicTaskData:
         """
         Compute dynamic data.
 
         :subject: suite name in the first ``bootstrap_repositories``
+        :parameter_summary: subject (architecture-variant)
         :runtime_context: ``$architecture:$variant:$extra_packages`` of
           ``bootstrap_options``
         :configuration_context: ``architecture`` of ``bootstrap_options``
@@ -226,12 +218,19 @@ class SystemBootstrap(
         extra_packages = ",".join(
             sorted(self.data.bootstrap_options.extra_packages)
         )
+        if variant is None:
+            parameter_summary = f"{suite_name} ({architecture})"
+        else:
+            parameter_summary = f"{suite_name} ({architecture}-{variant})"
+
         return BaseDynamicTaskData(
             subject=suite_name,
+            parameter_summary=parameter_summary,
             runtime_context=f"{architecture}:{variant}:{extra_packages}",
             configuration_context=architecture,
         )
 
+    @override
     def configure_for_execution(self, download_dir: Path) -> bool:
         """Create file.sources and add it into the log."""
         # Create file.sources (keyrings are downloaded, if needed)
@@ -305,10 +304,6 @@ class SystemBootstrap(
 
         return True
 
-    def get_label(self) -> str:
-        """Return the task label."""
-        return "bootstrap a system tarball"
-
     @staticmethod
     def _get_source_components(apt_sources: Path) -> list[str]:
         """Return the components for the first APT source in apt_sources."""
