@@ -9,6 +9,7 @@
 
 """Common test-helper code involving Tasks."""
 
+import abc
 import datetime as dt
 import hashlib
 import shlex
@@ -60,8 +61,103 @@ class TaskTestProtocol[TaskClass: BaseTask[Any, Any]](Protocol):
     task: TaskClass
 
 
-class FakeTaskDatabaseBase(TaskDatabaseInterface):
-    """Fake implementation of database interaction in worker tests."""
+class MockTaskDatabase(TaskDatabaseInterface):
+    """
+    Task database implementation that resolves fields by direct injection.
+
+    This is sufficient for testing task implementations.
+
+    Lookups raise NotImplementedError.
+    """
+
+    def __init__(self, **field_values: Any) -> None:
+        """Construct a :py:class:`MockTaskDatabase`."""
+        self.field_values: dict[str, Any] = {**field_values}
+
+    @overload
+    def lookup_single_artifact(
+        self,
+        lookup: LookupSingle,
+        default_category: CollectionCategory | None = None,
+    ) -> ArtifactInfo: ...
+
+    @overload
+    def lookup_single_artifact(
+        self, lookup: None, default_category: CollectionCategory | None = None
+    ) -> None: ...
+
+    @override
+    def lookup_single_artifact(
+        self,
+        lookup: LookupSingle | None,
+        default_category: CollectionCategory | None = None,
+    ) -> ArtifactInfo | None:
+        raise NotImplementedError("lookup_single_artifact")
+
+    @override
+    def lookup_multiple_artifacts(
+        self,
+        lookup: LookupMultiple | None,
+        default_category: CollectionCategory | None = None,
+    ) -> MultipleArtifactInfo:
+        raise NotImplementedError("lookup_multiple_artifact")
+
+    @override
+    def find_related_artifacts(
+        self,
+        artifact_ids: AbcCollection[int],
+        target_category: ArtifactCategory,
+        relation_type: RelationType = RelationType.RELATES_TO,
+    ) -> MultipleArtifactInfo:
+        raise NotImplementedError("find_related_artifacts")
+
+    @overload
+    def lookup_single_collection(
+        self,
+        lookup: LookupSingle,
+        default_category: CollectionCategory | None = None,
+    ) -> CollectionInfo: ...
+
+    @overload
+    def lookup_single_collection(
+        self, lookup: None, default_category: CollectionCategory | None = None
+    ) -> None: ...
+
+    @override
+    def lookup_single_collection(
+        self,
+        lookup: LookupSingle | None,
+        default_category: CollectionCategory | None = None,
+    ) -> CollectionInfo | None:
+        raise NotImplementedError("lookup_single_collection")
+
+    @override
+    def get_server_setting(self, setting: str) -> str:
+        raise NotImplementedError("get_server_setting")
+
+    @override
+    def export_suite_signing_keys(self, suite_id: int) -> str:
+        raise NotImplementedError("export_suite_signing_keys")
+
+    @override
+    def resolve_inputs(self, task: "BaseTask[Any, Any]") -> None:
+        for name, inp in task.inputs.items():
+            assert name in self.field_values, (
+                f"Value for field {task.__class__.__name__}.{name}"
+                " was not provided"
+            )
+            task.__dict__[name] = self.field_values[name]
+
+
+class FakeTaskDatabaseBase(MockTaskDatabase):
+    """
+    Fake implementation of database interaction in worker tests.
+
+    Fields not provided for injection are looked up normally.
+
+    This is useful to test more complex dependencies from
+    TaskDatabaseInterface, such as input field resolution.
+    """
 
     def __init__(
         self,
@@ -96,8 +192,10 @@ class FakeTaskDatabaseBase(TaskDatabaseInterface):
         ) = None,
         settings: dict[str, str] | None = None,
         suite_signing_keys: dict[int, str] | None = None,
+        **field_values: Any,
     ) -> None:
         """Construct a :py:class:`FakeTaskDatabase`."""
+        super().__init__(**field_values)
         self.single_lookups = dict(single_lookups or {})
         self.multiple_lookups = dict(multiple_lookups or {})
         self.relations = dict(relations or {})
@@ -117,37 +215,38 @@ class FakeTaskDatabaseBase(TaskDatabaseInterface):
         self, lookup: None, default_category: CollectionCategory | None = None
     ) -> None: ...
 
+    @override
     def lookup_single_artifact(
         self,
         lookup: LookupSingle | None,
         default_category: CollectionCategory | None = None,
     ) -> ArtifactInfo | None:
-        """Pretend to look up a single artifact."""
         return (
             None
             if lookup is None
             else self.single_lookups[(lookup, default_category)]
         )
 
+    @override
     def lookup_multiple_artifacts(
         self,
         lookup: LookupMultiple | None,
         default_category: CollectionCategory | None = None,
     ) -> MultipleArtifactInfo:
-        """Pretend to look up multiple artifacts."""
-        if not lookup:
-            return MultipleArtifactInfo()
-        return MultipleArtifactInfo(
-            self.multiple_lookups[(lookup, default_category)]
-        )
+        # if not lookup:
+        #     return MultipleArtifactInfo()
+        # return MultipleArtifactInfo(
+        #     self.multiple_lookups[(lookup, default_category)]
+        # )
+        raise NotImplementedError("this method is not currently used by tests")
 
+    @override
     def find_related_artifacts(
         self,
         artifact_ids: AbcCollection[int],
         target_category: ArtifactCategory,
         relation_type: RelationType = RelationType.RELATES_TO,
     ) -> MultipleArtifactInfo:
-        """Pretend to find artifacts via relations."""
         return MultipleArtifactInfo(
             self.relations[
                 (tuple(artifact_ids), target_category, relation_type)
@@ -204,7 +303,7 @@ class TaskHelperMixin[TaskClass: BaseTask[Any, Any]]:
 
     task_class: type[TaskClass]
     task: TaskClass
-    task_db: FakeTaskDatabase
+    task_db: MockTaskDatabase
 
     def __init_subclass__(cls, **kwargs: Any) -> None:
         """Compute cls.task_class."""
@@ -214,8 +313,17 @@ class TaskHelperMixin[TaskClass: BaseTask[Any, Any]]:
         # the type argument used to specialize this generic class.
         [cls.task_class] = extract_generic_type_arguments(cls, TaskHelperMixin)
 
+    def _create_task(self, task_data: dict[str, Any]) -> TaskClass:
+        """
+        Create a task with the given task data.
+
+        This should not be used directly by tests, but may be overridden by
+        variants of this mixin.
+        """
+        return self.task_class(task_data)
+
     def configure_task(
-        self: TaskTestProtocol[TaskClass],
+        self,
         task_data: dict[str, Any] | None = None,
         override: dict[str, Any] | None = None,
         remove: list[str] | None = None,
@@ -233,7 +341,9 @@ class TaskHelperMixin[TaskClass: BaseTask[Any, Any]]:
         :param remove: list of keys to remove.
         """
         if task_data is None:  # pragma: no cover
-            task_data = self.SAMPLE_TASK_DATA.copy()
+            task_data = cast(
+                TaskTestProtocol[TaskClass], self
+            ).SAMPLE_TASK_DATA.copy()
         if override:
             for key, value in override.items():
                 task_data[key] = value
@@ -241,7 +351,15 @@ class TaskHelperMixin[TaskClass: BaseTask[Any, Any]]:
             for key in remove:
                 task_data.pop(key, None)
 
-        self.task = self.task_class(task_data)
+        self.task = self._create_task(task_data)
+
+    def compute_dynamic_data[DTD: BaseDynamicTaskData](
+        self, task: BaseTask[Any, DTD]
+    ) -> DTD:
+        """Compute dynamic data for this task."""
+        assert self.task_db is not None
+        self.task_db.resolve_inputs(task)
+        return task.build_dynamic_data()
 
     def set_dynamic_data(self) -> None:
         """
@@ -250,8 +368,7 @@ class TaskHelperMixin[TaskClass: BaseTask[Any, Any]]:
         ``task_db`` must be set up in advance with enough information to
         allow computing dynamic data.
         """
-        assert self.task_db is not None
-        self.task.dynamic_data = self.task.compute_dynamic_data(self.task_db)
+        self.task.dynamic_data = self.compute_dynamic_data(self.task)
 
     def write_os_release(
         self, filename: Path, codename: str = "bookworm", version: int = 12
@@ -551,26 +668,18 @@ class ExternalTaskHelperMixin[ExternalTaskClass: BaseExternalTask[Any, Any]](
 
 
 class SampleBaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
-    BaseTask[TD, DTD]
+    BaseTask[TD, DTD], metaclass=abc.ABCMeta
 ):
     """Common test implementation of abstract task methods."""
 
     @override
-    def build_dynamic_data(
-        self,
-        task_database: TaskDatabaseInterface,  # noqa: U100
-    ) -> DTD:
+    def build_dynamic_data(self) -> DTD:
         """Resolve artifact lookups for this task."""
         return self.dynamic_task_data_type()
 
-    @override
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        return []
-
 
 class SampleBaseExternalTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
-    SampleBaseTask[TD, DTD], BaseExternalTask[TD, DTD]
+    SampleBaseTask[TD, DTD], BaseExternalTask[TD, DTD], metaclass=abc.ABCMeta
 ):
     """Common test implementation of BaseExternalTask methods."""
 
