@@ -21,14 +21,13 @@ from textwrap import dedent
 from typing import Any, AnyStr
 from unittest import mock
 
+import debusine.worker.tags as wtags
 from debian import deb822
 from debusine.artifacts import LocalArtifact, SourcePackage, Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebusineSigningOutput,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.artifacts.playground import ArtifactPlayground
@@ -38,17 +37,16 @@ from debusine.client.models import (
     RelationsResponse,
     RemoteArtifact,
 )
-from debusine.tasks import AssembleSignedSource, TaskConfigError
+from debusine.tasks import AssembleSignedSource
 from debusine.tasks.assemble_signed_source import AssembleError
 from debusine.tasks.models import (
     AssembleSignedSourceDynamicData,
-    LookupMultiple,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -97,46 +95,37 @@ class AssembleSignedSourceTests(
         self.task.work_request_id = 147
         self.task.workspace_name = "testing"
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            template=ArtifactInfo(
+                id=11,
+                category=ArtifactCategory.BINARY_PACKAGE,
+                data=DebianBinaryPackage(
+                    srcpkg_name="template",
+                    srcpkg_version="1.0-1",
+                    deb_fields={
+                        "Package": "hello",
+                        "Version": "1.0-1",
+                        "Architecture": "amd64",
+                    },
+                    deb_control_files=[],
+                ),
+            ),
+            signed=MultipleArtifactInfo(
                 (
-                    "debian/match:codename=bookworm:format=tarball:"
-                    "backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # template
-                (11, None): ArtifactInfo(
-                    id=11,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="template",
-                        srcpkg_version="1.0-1",
-                        deb_fields={
-                            "Package": "hello",
-                            "Version": "1.0-1",
-                            "Architecture": "amd64",
-                        },
-                        deb_control_files=[],
-                    ),
-                ),
-            },
-            multiple_lookups={
-                # signed
-                (LookupMultiple((12,)), None): [
                     ArtifactInfo(
                         id=12,
                         category=ArtifactCategory.SIGNING_OUTPUT,
                         data=DebusineSigningOutput(
                             purpose=KeyPurpose.UEFI, fingerprint="", results=[]
                         ),
-                    )
-                ]
-            },
+                    ),
+                )
+            ),
         )
 
     def test_can_run_on(self) -> None:
@@ -175,10 +164,16 @@ class AssembleSignedSourceTests(
             )
         )
 
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             AssembleSignedSourceDynamicData(
                 environment_id=1,
                 template_id=11,
@@ -189,21 +184,12 @@ class AssembleSignedSourceTests(
             ),
         )
 
-    def test_compute_dynamic_data_subject_raise_task_config_error(self) -> None:
-        self.task_db.single_lookups[(11, None)] = ArtifactInfo(
-            id=11,
-            category=ArtifactCategory.SOURCE_PACKAGE,
-            data=EmptyArtifactData(),
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            AssembleSignedSource.inputs["template"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.BINARY_PACKAGE,),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^template: unexpected artifact category: "
-            r"'debian:source-package'. Valid categories: "
-            r"\['debian:binary-package'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
         self.assertEqual(self.task.get_input_artifacts_ids(), [])
@@ -588,11 +574,13 @@ class AssembleSignedSourceTests(
             "debian/rules",
         ):
             (source_template_path / file).unlink()
-            with self.subTest(file=file):
-                with self.assertRaisesRegex(
+            with (
+                self.subTest(file=file),
+                self.assertRaisesRegex(
                     AssembleError, f"Required file {file} missing from template"
-                ):
-                    self.task._check_template(code_signing_path)
+                ),
+            ):
+                self.task._check_template(code_signing_path)
             (source_template_path / file).touch()
 
     def test_check_template_wrong_source_format(self) -> None:
@@ -619,11 +607,13 @@ class AssembleSignedSourceTests(
         self._create_source_template(source_template_path)
         for file in ("debian/source/options", "debian/source/local-options"):
             (source_template_path / file).touch()
-            with self.subTest(file=file):
-                with self.assertRaisesRegex(
+            with (
+                self.subTest(file=file),
+                self.assertRaisesRegex(
                     AssembleError, f"Template may not contain {file}"
-                ):
-                    self.task._check_template(code_signing_path)
+                ),
+            ):
+                self.task._check_template(code_signing_path)
             (source_template_path / file).unlink()
 
     def test_check_template_ok(self) -> None:
