@@ -19,10 +19,10 @@ from unittest.mock import MagicMock, call, create_autospec, patch
 
 import pydantic
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.local_artifact import AutopkgtestArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianAutopkgtest,
     DebianAutopkgtestResult,
     DebianAutopkgtestSource,
@@ -41,13 +41,17 @@ from debusine.tasks.models import (
     AutopkgtestNeedsInternet,
     AutopkgtestTimeout,
     BackendType,
+    ExtraExternalRepository,
     LookupMultiple,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo, CollectionInfo
+from debusine.tasks.server import (
+    ArtifactInfo,
+    MultipleArtifactInfo,
+)
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -93,30 +97,21 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         self.task.workspace_name = "testing"
         self.task.work_request_id = 170
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            input_source_artifact=ArtifactInfo(
+                id=5,
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name="hello", version="1.0", type="dpkg", dsc_fields={}
+                ),
+            ),
+            input_binary_artifacts=MultipleArtifactInfo(
                 (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=autopkgtest",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (LookupMultiple((10, 11)), None): [
                     ArtifactInfo(
                         id=10,
                         category=ArtifactCategory.BINARY_PACKAGE,
@@ -127,8 +122,10 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                         category=ArtifactCategory.UPLOAD,
                         data=self.create_upload_data("hello"),
                     ),
-                ],
-            },
+                )
+            ),
+            input_context_artifacts=MultipleArtifactInfo(),
+            extra_repositories=None,
         )
 
     def mock_executor(self) -> None:
@@ -495,6 +492,23 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         with self.assertRaises(TaskConfigError):
             self.configure_task(override={"timeout": "something"})
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.mock_is_command_available({"autopkgtest": True})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:autopkgtest:version:1",
+                wtags.CAP_PREFIX + "autopkgtest",
+            ],
+        )
+
+    def test_get_provided_worker_tags_autopkgtest_not_available(self) -> None:
+        self.mock_is_command_available({"autopkgtest": False})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [wtags.TASK_PREFIX + "worker:autopkgtest:version:1"],
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"autopkgtest": True})
@@ -586,57 +600,66 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             },
         )
 
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [
+                wtags.EXECUTOR_PREFIX + "unshare",
+                wtags.CAP_PREFIX + "autopkgtest",
+            ],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
-            AutopkgtestDynamicData(
-                environment_id=1,
-                input_source_artifact_id=5,
-                input_binary_artifacts_ids=[10, 11],
-                input_context_artifacts_ids=[],
-                subject="hello",
-                parameter_summary="hello_1.0 (amd64)",
-                runtime_context="amd64:unshare",
-                configuration_context="sid",
-            ),
+        assert self.compute_dynamic_data(self.task) == AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=5,
+            input_binary_artifacts_ids=[10, 11],
+            input_context_artifacts_ids=[],
+            subject="hello",
+            parameter_summary="hello_1.0 (amd64)",
+            runtime_context="amd64:unshare",
+            configuration_context="sid",
         )
 
     def test_compute_dynamic_data_context(self) -> None:
         """Dynamic data includes input context artifact IDs if relevant."""
-        binary_artifacts_lookup = LookupMultiple((10, 11))
-        self.task.data.input.context_artifacts = LookupMultiple.model_validate(
-            {"collection": "bookworm@debian:suite"}
+        self.task_db.field_values["input_binary_artifacts"] = (
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=self.create_binary_package_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=12,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                )
+            )
+        )
+        self.task_db.field_values["input_context_artifacts"] = (
+            MultipleArtifactInfo(
+                MultipleArtifactInfo(
+                    (
+                        ArtifactInfo(
+                            id=2,
+                            category=ArtifactCategory.BINARY_PACKAGE,
+                            data=self.create_binary_package_data("context"),
+                        ),
+                        ArtifactInfo(
+                            id=4,
+                            category=ArtifactCategory.UPLOAD,
+                            data=self.create_upload_data("context"),
+                        ),
+                    )
+                )
+            )
         )
-        self.task_db.multiple_lookups[(binary_artifacts_lookup, None)] = [
-            ArtifactInfo(
-                id=10,
-                category=ArtifactCategory.BINARY_PACKAGE,
-                data=self.create_binary_package_data("hello"),
-            ),
-            ArtifactInfo(
-                id=12,
-                category=ArtifactCategory.UPLOAD,
-                data=self.create_upload_data("hello"),
-            ),
-        ]
-        self.task_db.multiple_lookups[
-            (self.task.data.input.context_artifacts, None)
-        ] = [
-            ArtifactInfo(
-                id=2,
-                category=ArtifactCategory.BINARY_PACKAGE,
-                data=self.create_binary_package_data("context"),
-            ),
-            ArtifactInfo(
-                id=4,
-                category=ArtifactCategory.UPLOAD,
-                data=self.create_upload_data("context"),
-            ),
-        ]
-
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             AutopkgtestDynamicData(
                 environment_id=1,
                 input_source_artifact_id=5,
@@ -649,47 +672,19 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             ),
         )
 
-    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
-        """
-        Test compute_dynamic_data raise TaskConfigError.
-
-        source_artifact's category is unexpected.
-        """
-        self.task.data.input.context_artifacts = LookupMultiple.model_validate(
-            {"collection": "bookworm@debian:suite"}
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            Autopkgtest.inputs["input_source_artifact"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD),
         )
-        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
-            id=5,
-            category=ArtifactCategory.BINARY_PACKAGE,
-            data=DebianBinaryPackage(
-                srcpkg_name="hello",
-                srcpkg_version="1.0",
-                deb_fields={},
-                deb_control_files=[],
-            ),
+        self.assertEqual(
+            Autopkgtest.inputs["input_binary_artifacts"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD),
+        )
+        self.assertEqual(
+            Autopkgtest.inputs["input_context_artifacts"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.BINARY_PACKAGE, ArtifactCategory.UPLOAD),
         )
-        self.task_db.multiple_lookups[
-            (self.task.data.input.context_artifacts, None)
-        ] = [
-            ArtifactInfo(
-                id=2,
-                category=ArtifactCategory.BINARY_PACKAGE,
-                data=self.create_binary_package_data("context"),
-            ),
-            ArtifactInfo(
-                id=3,
-                category=ArtifactCategory.UPLOAD,
-                data=self.create_upload_data("context"),
-            ),
-        ]
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            "^input.source_artifact: unexpected artifact "
-            "category: 'debian:binary-package'. Valid categories: "
-            r"\['debian:source-package', 'debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
 
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
@@ -783,25 +778,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_cmdline_extra_debusine_repositories(self) -> None:
         """Extra repositories can be hosted by Debusine."""
-        self.configure_task(
-            override={
-                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
-            }
-        )
-        self.task_db.single_collection_lookups[
-            ("trixie-local@debian:suite", None)
-        ] = CollectionInfo(
-            id=1,
-            scope_name="scope",
-            workspace_name="workspace",
-            category=CollectionCategory.SUITE,
-            name="trixie-local",
-            data={"components": ["main"]},
-        )
-        self.task_db.settings["DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"] = (
-            "deb.example.com"
-        )
-        self.task_db.suite_signing_keys[1] = "\n".join(
+        signing_key = "\n".join(
             (
                 "-----BEGIN PGP PUBLIC KEY BLOCK-----",
                 "",
@@ -809,7 +786,16 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                 "-----END PGP PUBLIC KEY BLOCK-----",
             )
         )
+        self.task_db.field_values["extra_repositories"] = [
+            ExtraExternalRepository(
+                url="http://deb.example.com/scope/workspace",
+                suite="trixie-local",
+                components=["main"],
+                signing_key=signing_key,
+            )
+        ]
         self.set_dynamic_data()
+
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
         self.task._prepare_executor()
@@ -1018,7 +1004,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             id=10,
             workspace=workspace,
             category="Test",
-            created_at=dt.datetime.now(),
+            created_at=dt.datetime.now(tz=dt.UTC),
             data={},
             download_tar_gz_url="https://example.com/not-used",
             files=FilesResponseType({}),
@@ -1046,7 +1032,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             id=10,
             workspace=workspace,
             category="Test",
-            created_at=dt.datetime.now(),
+            created_at=dt.datetime.now(tz=dt.UTC),
             data={},
             download_tar_gz_url="https://example.com/not-used",
             files=FilesResponseType(files),
