@@ -16,12 +16,11 @@ from unittest.mock import call, patch
 
 from debian.debian_support import Version
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import BlhcArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianPackageBuildLog,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.client.models import LookupResultType, LookupSingleResponse
@@ -30,7 +29,7 @@ from debusine.tasks.models import BlhcDynamicData
 from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -74,25 +73,17 @@ class BlhcTests(ExternalTaskHelperMixin[Blhc], TestCase):
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.artifact
-                (421, None): ArtifactInfo(
-                    id=421,
-                    category=ArtifactCategory.PACKAGE_BUILD_LOG,
-                    data=self.build_log,
-                ),
-            }
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            input_artifact=ArtifactInfo(
+                id=421,
+                category=ArtifactCategory.PACKAGE_BUILD_LOG,
+                data=self.build_log,
+            ),
         )
 
     def test_configure_fails_with_missing_required_data(
@@ -139,10 +130,24 @@ class BlhcTests(ExternalTaskHelperMixin[Blhc], TestCase):
                 }
             )
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:blhc:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             BlhcDynamicData(
                 environment_id=1,
                 input_artifact_id=421,
@@ -153,22 +158,12 @@ class BlhcTests(ExternalTaskHelperMixin[Blhc], TestCase):
             ),
         )
 
-    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
-        """Artifact input.artifact has unexpected category."""
-        self.task_db.single_lookups[(421, None)] = ArtifactInfo(
-            id=421,
-            category=ArtifactCategory.SOURCE_PACKAGE,
-            data=EmptyArtifactData(),
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            Blhc.inputs["input_artifact"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.PACKAGE_BUILD_LOG,),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.artifact: unexpected artifact category: "
-            r"'debian:source-package'. "
-            r"Valid categories: \['debian:package-build-log'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
         self.assertEqual(self.task.get_input_artifacts_ids(), [])
