@@ -14,23 +14,22 @@ import re
 from typing import Any
 from unittest.mock import call
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import DebDiffArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebianSourcePackage,
     DebianUpload,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.client.models import LookupResultType, LookupSingleResponse
 from debusine.tasks import DebDiff, TaskConfigError
-from debusine.tasks.models import DebDiffDynamicData, LookupMultiple
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.models import DebDiffDynamicData
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -61,34 +60,29 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifacts
-                (421, None): ArtifactInfo(
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            source_artifacts=[
+                ArtifactInfo(
                     id=421,
                     category=ArtifactCategory.SOURCE_PACKAGE,
                     data=DebianSourcePackage(
                         name="hello", version="1.0", type="dpkg", dsc_fields={}
                     ),
                 ),
-                (123, None): ArtifactInfo(
+                ArtifactInfo(
                     id=123,
                     category=ArtifactCategory.SOURCE_PACKAGE,
                     data=DebianSourcePackage(
                         name="hello", version="1.1", type="dpkg", dsc_fields={}
                     ),
                 ),
-            }
+            ],
+            binary_artifacts=[],
         )
 
     def test_configure_fails_with_missing_required_data(
@@ -125,10 +119,24 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                 }
             )
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:debdiff:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             DebDiffDynamicData(
                 environment_id=1,
                 input_source_artifacts_ids=[421, 123],
@@ -139,45 +147,45 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         )
 
     def test_compute_dynamic_subject_binary_artifacts(self) -> None:
-        self.configure_task(
-            override={"input": {"binary_artifacts": [[500], [501]]}}
-        )
-        self.task_db.multiple_lookups = {
-            # binary artifacts
-            (LookupMultiple((500,)), None): [
-                ArtifactInfo(
-                    id=500,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Version": "1.0-1",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+        self.task_db.field_values["source_artifacts"] = []
+        self.task_db.field_values["binary_artifacts"] = [
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Version": "1.0-1",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-            (LookupMultiple((501,)), None): [
-                ArtifactInfo(
-                    id=501,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Version": "1.1-1",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+                )
+            ),
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Version": "1.1-1",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-        }
-
+                )
+            ),
+        ]
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             DebDiffDynamicData(
                 environment_id=1,
                 input_source_artifacts_ids=None,
@@ -194,45 +202,46 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         There is no source package, and binary packages are from different
         source packages.
         """
-        self.configure_task(
-            override={"input": {"binary_artifacts": [[500], [501]]}}
-        )
-        self.task_db.multiple_lookups = {
-            # binary artifacts
-            (LookupMultiple((500,)), None): [
-                ArtifactInfo(
-                    id=500,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Version": "1.0-1",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+        self.task_db.field_values["source_artifacts"] = []
+        self.task_db.field_values["binary_artifacts"] = [
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Version": "1.0-1",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-            (LookupMultiple((501,)), None): [
-                ArtifactInfo(
-                    id=501,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "linux-base",
-                            "Version": "1.1-1",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+                )
+            ),
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "linux-base",
+                                "Version": "1.1-1",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-        }
+                )
+            ),
+        ]
 
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             DebDiffDynamicData(
                 environment_id=1,
                 input_source_artifacts_ids=None,
@@ -241,162 +250,125 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
             ),
         )
 
-    def test_compute_dynamic_data_raise_task_config_error_source(self) -> None:
-        self.task_db.single_lookups[(421, None)] = ArtifactInfo(
-            id=421,
-            category=ArtifactCategory.BINARY_PACKAGE,
-            data=DebianSourcePackage(
-                name="hello", version="1.0", type="dpkg", dsc_fields={}
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            DebDiff.inputs["source_artifacts"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.SOURCE_PACKAGE,),
+        )
+        self.assertEqual(
+            DebDiff.inputs["binary_artifacts"].categories,  # type: ignore[attr-defined]
+            (
+                ArtifactCategory.UPLOAD,
+                ArtifactCategory.BINARY_PACKAGE,
             ),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.source_artifacts: unexpected artifact category: "
-            r"'debian:binary-package'. "
-            r"Valid categories: \['debian:source-package'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
-    def test_compute_dynamic_data_raise_task_config_error_binary(self) -> None:
-        self.configure_task(
-            override={"input": {"binary_artifacts": [[500], [501]]}}
-        )
-        self.task_db.multiple_lookups = {
-            # binary artifacts
-            (LookupMultiple((500,)), None): [
-                ArtifactInfo(
-                    id=500,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=EmptyArtifactData(),
-                ),
-            ],
-            (LookupMultiple((501,)), None): [
-                ArtifactInfo(
-                    id=501,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="linux-base",
-                        srcpkg_version="1.0",
-                        deb_fields={},
-                        deb_control_files=[],
-                    ),
-                ),
-            ],
-        }
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.binary_artifacts: unexpected artifact category: "
-            r"'debian:source-package'. Valid categories: "
-            r"\['debian:binary-package', 'debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_compute_dynamic_data_raise_task_config_error_categories(
         self,
     ) -> None:
         """Raise TaskConfigError: invalid categories binary artifacts."""
-        self.configure_task(
-            override={"input": {"binary_artifacts": [[500], [501]]}}
-        )
-        self.task_db.multiple_lookups = {
-            # binary artifacts
-            (LookupMultiple((500,)), None): [
-                ArtifactInfo(
-                    id=500,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+        self.task_db.field_values["source_artifacts"] = []
+        self.task_db.field_values["binary_artifacts"] = [
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-            (LookupMultiple((501,)), None): [
-                ArtifactInfo(
-                    id=501,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="linux-base",
-                        srcpkg_version="1.0",
-                        deb_fields={},
-                        deb_control_files=[],
+                )
+            ),
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="linux-base",
+                            srcpkg_version="1.0",
+                            deb_fields={},
+                            deb_control_files=[],
+                        ),
                     ),
-                ),
-            ],
-        }
+                )
+            ),
+        ]
 
         with self.assertRaisesRegex(
             TaskConfigError,
             r'^All binary artifacts must have the same category. '
             r'Found "debian:upload" and "debian:binary-package"$',
         ):
-            self.task.compute_dynamic_data(self.task_db)
+            self.compute_dynamic_data(self.task)
 
     def test_compute_dynamic_data_raise_invalid_category(self) -> None:
         """Raise TaskConfigError: invalid categories binary artifacts."""
-        self.configure_task(
-            override={"input": {"binary_artifacts": [[500, 501], [502, 503]]}}
-        )
-        self.task_db.multiple_lookups = {
-            # binary artifacts
-            (LookupMultiple((500, 501)), None): [
-                ArtifactInfo(
-                    id=500,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+        self.task_db.field_values["source_artifacts"] = []
+        self.task_db.field_values["binary_artifacts"] = [
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-                ArtifactInfo(
-                    id=501,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-            (LookupMultiple((502, 503)), None): [
-                ArtifactInfo(
-                    id=502,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+                )
+            ),
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=502,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-                ArtifactInfo(
-                    id=503,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Source": "hello",
-                            "Files": [{"name": "hello.deb"}],
-                        },
+                    ArtifactInfo(
+                        id=503,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
                     ),
-                ),
-            ],
-        }
+                )
+            ),
+        ]
 
         with self.assertRaisesRegex(
             TaskConfigError,
@@ -404,14 +376,14 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
             r"than one artifact all must be of category debian:binary-package. "
             r"Found: debian:upload$",
         ):
-            self.task.compute_dynamic_data(self.task_db)
+            self.compute_dynamic_data(self.task)
 
     def test_compute_dynamic_data_raise_binary_cannot_be_zero(self) -> None:
-        self.configure_task(override={"input": {"binary_artifacts": [[], []]}})
-        self.task_db.multiple_lookups = {
-            # binary artifacts
-            (LookupMultiple(()), None): [],
-        }
+        self.task_db.field_values["source_artifacts"] = []
+        self.task_db.field_values["binary_artifacts"] = [
+            MultipleArtifactInfo(),
+            MultipleArtifactInfo(),
+        ]
 
         with self.assertRaisesRegex(
             TaskConfigError,
@@ -420,7 +392,7 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                 "cannot have zero artifacts (got 0 and 0)"
             ),
         ):
-            self.task.compute_dynamic_data(self.task_db)
+            self.compute_dynamic_data(self.task)
 
     def test_get_input_artifacts_ids_dynamic_dynamic_data_none(self) -> None:
         self.task.dynamic_data = None
@@ -622,6 +594,7 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                 [binary_artifact_id_1],
                 [binary_artifact_id_2],
             ],
+            parameter_summary="A → B",
         )
 
         # Debusine.upload_artifact is mocked to verify the call only
@@ -651,7 +624,10 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         # Debusine Mock upload_artifact expected calls
         upload_artifact_calls = []
         debdiff_artifact = DebDiffArtifact.create(
-            debdiff_output=debdiff_output, original="", new=""
+            debdiff_output=debdiff_output,
+            original="",
+            new="",
+            label="debdiff A → B",
         )
 
         upload_artifact_calls.append(
@@ -689,7 +665,9 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         self.task.work_request_id = 2
         self.task.workspace_name = "testing"
         self.task.dynamic_data = DebDiffDynamicData(
-            environment_id=1, input_source_artifacts_ids=[1, 2]
+            environment_id=1,
+            input_source_artifacts_ids=[1, 2],
+            parameter_summary="A → B",
         )
         download_directory = self.create_temporary_directory()
 
@@ -746,6 +724,7 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                 / DebDiff.CAPTURE_OUTPUT_FILENAME,
                 original=file1.name,
                 new=file2.name,
+                label="debdiff A → B",
             ),
             workspace=self.task.workspace_name,
             work_request=self.task.work_request_id,
