@@ -18,33 +18,25 @@ from pathlib import Path, PurePath
 from typing import Any, AnyStr
 from unittest import mock
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import LocalArtifact, SigningInputArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
-    DebianBinaryPackages,
-    DebianUpload,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.artifacts.playground import ArtifactPlayground
-from debusine.client.models import (
-    ArtifactResponse,
-    RelationType,
-    RemoteArtifact,
-)
-from debusine.tasks import ExtractForSigning, TaskConfigError
+from debusine.client.models import ArtifactResponse, RemoteArtifact
+from debusine.tasks import ExtractForSigning
 from debusine.tasks.extract_for_signing import ExtractError
 from debusine.tasks.models import (
     ExtractForSigningDynamicData,
-    LookupMultiple,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -101,33 +93,24 @@ class ExtractForSigningTests(
         self.task.work_request_id = 147
         self.task.workspace_name = "testing"
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            template_artifact=ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.BINARY_PACKAGE,
+                data=DebianBinaryPackage(
+                    srcpkg_name="template",
+                    srcpkg_version="1.0",
+                    deb_fields={"Package": "hello", "Version": "1.0-1"},
+                    deb_control_files=[],
+                ),
+            ),
+            binary_artifacts=MultipleArtifactInfo(
                 (
-                    "debian/match:codename=bookworm:format=tarball:"
-                    "backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.template_artifact
-                (2, None): ArtifactInfo(
-                    id=2,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="template",
-                        srcpkg_version="1.0",
-                        deb_fields={"Package": "hello", "Version": "1.0-1"},
-                        deb_control_files=[],
-                    ),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (LookupMultiple((3, 4)), None): [
                     ArtifactInfo(
                         id=3,
                         category=ArtifactCategory.BINARY_PACKAGE,
@@ -156,8 +139,8 @@ class ExtractForSigningTests(
                             deb_control_files=[],
                         ),
                     ),
-                ]
-            },
+                )
+            ),
         )
 
     def test_can_run_on(self) -> None:
@@ -196,10 +179,24 @@ class ExtractForSigningTests(
             )
         )
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:extractforsigning:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             ExtractForSigningDynamicData(
                 environment_id=1,
                 input_template_artifact_id=2,
@@ -209,134 +206,16 @@ class ExtractForSigningTests(
             ),
         )
 
-    def test_compute_dynamic_data_from_upload(self) -> None:
-        """compute_dynamic_data follows relations from upload artifacts."""
-        self.task_db.multiple_lookups = {
-            # input.binary_artifacts
-            (LookupMultiple((3, 4)), None): [
-                ArtifactInfo(
-                    id=3,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "all",
-                            "Files": [{"name": "unsigned-doc_1.0_all.deb"}],
-                        },
-                    ),
-                ),
-                ArtifactInfo(
-                    id=4,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Files": [
-                                {"name": "unsigned-1_1.0_amd64.deb"},
-                                {"name": "unsigned-2_1.0_amd64.deb"},
-                            ],
-                        },
-                    ),
-                ),
-            ]
-        }
-        self.task_db.relations = {
-            (
-                (3, 4),
-                ArtifactCategory.BINARY_PACKAGE,
-                RelationType.EXTENDS,
-            ): [
-                ArtifactInfo(
-                    id=artifact_id,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="unsigned",
-                        srcpkg_version="1.0",
-                        deb_fields={
-                            "Package": binary_package_name,
-                            "Version": "1.0-1",
-                        },
-                        deb_control_files=[],
-                    ),
-                )
-                for artifact_id, binary_package_name in (
-                    (5, "unsigned-doc"),
-                    (6, "unsigned-1"),
-                    (7, "unsigned-2"),
-                )
-            ]
-        }
-
+    def test_acceptable_input_categories(self) -> None:
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
-            ExtractForSigningDynamicData(
-                environment_id=1,
-                input_template_artifact_id=2,
-                input_binary_artifacts_ids=[5, 6, 7],
-                subject="hello",
-                parameter_summary="hello_1.0-1 (all, amd64)",
-            ),
+            ExtractForSigning.inputs["template_artifact"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.BINARY_PACKAGE,),
         )
-
-    def test_compute_dynamic_data_raise_task_config_error_template(
-        self,
-    ) -> None:
-        self.task_db.single_lookups[(2, None)] = ArtifactInfo(
-            id=2,
-            category=ArtifactCategory.SOURCE_PACKAGE,
-            data=EmptyArtifactData(),
+        self.assertEqual(
+            ExtractForSigning.inputs["binary_artifacts"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.UPLOAD, ArtifactCategory.BINARY_PACKAGE),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.template_artifact: unexpected artifact category: "
-            r"'debian:source-package'. "
-            r"Valid categories: \['debian:binary-package'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
-    def test_compute_dynamic_data_raise_task_config_error_binary(
-        self,
-    ) -> None:
-        self.task_db.multiple_lookups = {
-            # input.binary_artifacts
-            (LookupMultiple((3, 4)), None): [
-                ArtifactInfo(
-                    id=3,
-                    category=ArtifactCategory.BINARY_PACKAGES,
-                    data=DebianBinaryPackages(
-                        srcpkg_name="unsigned",
-                        srcpkg_version="1.0",
-                        version="1.0",
-                        architecture="amd64",
-                        packages=["unsigned-1"],
-                    ),
-                ),
-                ArtifactInfo(
-                    id=4,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="unsigned",
-                        srcpkg_version="1.0",
-                        deb_fields={
-                            "Package": "unsigned-2",
-                            "Version": "1.0-1",
-                        },
-                        deb_control_files=[],
-                    ),
-                ),
-            ]
-        }
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.binary_artifacts\[0\]: unexpected artifact category: "
-            r"'debian:binary-packages'. "
-            r"Valid categories: \['debian:binary-package', 'debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
         self.assertEqual(self.task.get_input_artifacts_ids(), [])
