@@ -19,15 +19,14 @@ from typing import Any, cast
 from unittest import mock
 from unittest.mock import MagicMock, call
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import LintianArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebianLintianSummary,
     DebianSourcePackage,
     DebianUpload,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.client.models import FileResponse, FilesResponseType
@@ -37,12 +36,11 @@ from debusine.tasks.models import (
     BackendType,
     LintianDynamicData,
     LintianFailOnSeverity,
-    LookupMultiple,
 )
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -89,30 +87,23 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=lintian",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            input_source_artifact=ArtifactInfo(
+                id=5,
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name="hello",
+                    version="1.0",
+                    type="dpkg",
+                    dsc_fields={},
                 ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello",
-                        version="1.0",
-                        type="dpkg",
-                        dsc_fields={},
-                    ),
-                ),
-            }
+            ),
+            input_binary_artifacts=MultipleArtifactInfo(),
         )
 
     def assert_parse_output(
@@ -744,9 +735,23 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
         """Configure valid: does not raise any exception."""
         self.configure_task(override={"input": {"binary_artifacts": [6, 7]}})
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:lintian:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             LintianDynamicData(
                 environment_id=1,
                 input_source_artifact_id=5,
@@ -759,7 +764,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
 
     def test_compute_dynamic_data_subject_debian_upload(self) -> None:
         """compute_dynamic_data subject is package name of source artifact."""
-        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
+        self.task_db.field_values["input_source_artifact"] = ArtifactInfo(
             id=5,
             category=ArtifactCategory.UPLOAD,
             data=DebianUpload(
@@ -773,9 +778,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
             ),
         )
 
-        self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db).subject, "hello"
-        )
+        self.assertEqual(self.compute_dynamic_data(self.task).subject, "hello")
 
     def test_compute_dynamic_data_subject_runtime_context(self) -> None:
         """compute_dynamic_data runtime_context source only."""
@@ -788,7 +791,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
                 }
             }
         )
-        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
+        self.task_db.field_values["input_source_artifact"] = ArtifactInfo(
             id=5,
             category=ArtifactCategory.UPLOAD,
             data=DebianUpload(
@@ -802,7 +805,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
             ),
         )
 
-        dynamic_data = self.task.compute_dynamic_data(self.task_db)
+        dynamic_data = self.compute_dynamic_data(self.task)
 
         self.assertEqual(dynamic_data.subject, "hello")
         self.assertEqual(dynamic_data.runtime_context, "source:sid")
@@ -819,7 +822,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
                 "output": {"source_analysis": False},
             }
         )
-        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
+        self.task_db.field_values["input_source_artifact"] = ArtifactInfo(
             id=5,
             category=ArtifactCategory.UPLOAD,
             data=DebianUpload(
@@ -832,27 +835,28 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
                 },
             ),
         )
-        self.task_db.multiple_lookups = {
-            # input.binary_artifacts
-            (LookupMultiple((6,)), None): [
-                ArtifactInfo(
-                    id=6,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="hello-bin",
-                        srcpkg_version="1.0",
-                        deb_fields={
-                            "Package": "hello-bin",
-                            "Version": "1.0",
-                            "Architecture": "amd64",
-                        },
-                        deb_control_files=[],
+        self.task_db.field_values["input_binary_artifacts"] = (
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=6,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="hello-bin",
+                            srcpkg_version="1.0",
+                            deb_fields={
+                                "Package": "hello-bin",
+                                "Version": "1.0",
+                                "Architecture": "amd64",
+                            },
+                            deb_control_files=[],
+                        ),
                     ),
-                ),
-            ],
-        }
+                )
+            )
+        )
 
-        dynamic_data = self.task.compute_dynamic_data(self.task_db)
+        dynamic_data = self.compute_dynamic_data(self.task)
         self.assertEqual(dynamic_data.subject, "hello")
         self.assertEqual(dynamic_data.parameter_summary, "hello_1.0")
         self.assertEqual(
@@ -864,24 +868,22 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
     ) -> None:
         """compute_dynamic_data set to source artifact without binaries."""
         self.configure_task(remove=["environment"])
-        self.task_db.single_lookups = {
-            # input.source_artifact
-            (5, None): ArtifactInfo(
-                id=5,
-                category=ArtifactCategory.UPLOAD,
-                data=DebianUpload(
-                    type="dpkg",
-                    changes_fields={
-                        "Source": "hello",
-                        "Version": "1.0",
-                        "Architecture": "amd64",
-                        "Files": [{"name": "hello_1.0_amd64.deb"}],
-                    },
-                ),
+        self.task_db.field_values["environment"] = None
+        self.task_db.field_values["input_source_artifact"] = ArtifactInfo(
+            id=5,
+            category=ArtifactCategory.UPLOAD,
+            data=DebianUpload(
+                type="dpkg",
+                changes_fields={
+                    "Source": "hello",
+                    "Version": "1.0",
+                    "Architecture": "amd64",
+                    "Files": [{"name": "hello_1.0_amd64.deb"}],
+                },
             ),
-        }
+        )
 
-        dynamic_data = self.task.compute_dynamic_data(self.task_db)
+        dynamic_data = self.compute_dynamic_data(self.task)
         self.assertEqual(dynamic_data.subject, "hello")
         self.assertIsNone(dynamic_data.runtime_context)
 
@@ -892,110 +894,52 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
         There are no source packages.
         """
         self.configure_task(override={"input": {"binary_artifacts": [6]}})
-        self.task_db.multiple_lookups = {
-            # input.binary_artifacts
-            (LookupMultiple((6,)), None): [
-                ArtifactInfo(
-                    id=6,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="hello",
-                        srcpkg_version="1.0",
-                        deb_fields={
-                            "Package": "hello",
-                            "Version": "1.0",
-                            "Architecture": "amd64",
-                        },
-                        deb_control_files=[],
+        self.task_db.field_values["input_source_artifact"] = None
+        self.task_db.field_values["input_binary_artifacts"] = (
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=6,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="hello",
+                            srcpkg_version="1.0",
+                            deb_fields={
+                                "Package": "hello",
+                                "Version": "1.0",
+                                "Architecture": "amd64",
+                            },
+                            deb_control_files=[],
+                        ),
                     ),
                 )
-            ],
-        }
-
-        self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db).subject, "hello"
-        )
-
-    def test_compute_dynamic_raise_config_task_error_wrong_environment(
-        self,
-    ) -> None:
-        """
-        Test compute_dynamic_data raise TaskConfigError.
-
-        environment artifact category is unexpected.
-        """
-        self.task_db.single_lookups[
-            (
-                "debian/match:codename=bookworm:architecture=amd64:"
-                "format=tarball:backend=unshare:variant=lintian",
-                CollectionCategory.ENVIRONMENTS,
             )
-        ] = ArtifactInfo(
-            id=1,
-            category=ArtifactCategory.SOURCE_PACKAGE,
-            data=EmptyArtifactData(),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            "^environment: unexpected artifact category: "
-            "'debian:source-package'. Valid categories: "
-            r"\['debian:system-image', 'debian:system-tarball'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
+        self.assertEqual(self.compute_dynamic_data(self.task).subject, "hello")
 
-    def test_compute_dynamic_raise_config_task_error_wrong_source_artifact(
-        self,
-    ) -> None:
-        """
-        Test compute_dynamic_data raise TaskConfigError.
-
-        source artifact category is unexpected.
-        """
-        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
-            id=5,
-            category=ArtifactCategory.BINARY_PACKAGE,
-            data=EmptyArtifactData(),
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            Lintian.inputs["input_source_artifact"].categories,  # type: ignore[attr-defined]
+            (
+                ArtifactCategory.SOURCE_PACKAGE,
+                ArtifactCategory.UPLOAD,
+            ),
         )
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            "^input.source_artifact: unexpected artifact category: "
-            "'debian:binary-package'. "
-            r"Valid categories: \['debian:source-package', 'debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
-    def test_compute_dynamic_data_raise_task_config_error(
-        self,
-    ) -> None:
-        """
-        Test compute_dynamic_data raise TaskConfigError.
-
-        binary_artifact's data is a wrong type.
-        """
-        self.configure_task(
-            override={"input": {"binary_artifacts": [6]}},
-            remove=["environment"],
+        self.assertEqual(
+            Lintian.inputs["input_binary_artifacts"].categories,  # type: ignore[attr-defined]
+            (
+                ArtifactCategory.BINARY_PACKAGE,
+                ArtifactCategory.UPLOAD,
+            ),
+        )
+        self.assertEqual(
+            Lintian.inputs["environment"].categories,  # type: ignore[attr-defined]
+            (
+                ArtifactCategory.SYSTEM_TARBALL,
+                ArtifactCategory.SYSTEM_IMAGE,
+            ),
         )
-        self.task_db.multiple_lookups = {
-            # input.binary_artifacts
-            (LookupMultiple((6,)), None): [
-                ArtifactInfo(
-                    id=6,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=EmptyArtifactData(),
-                )
-            ],
-        }
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.binary_artifacts\[0\]: unexpected artifact category: "
-            r"'debian:source-package'. Valid categories: "
-            r"\['debian:binary-package', 'debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
 
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
@@ -1174,8 +1118,8 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
             binary_pkg_file.name: (binary_pkg_name, "amd64"),
         }
 
-        extract_file_name_mocked.side_effect = (
-            lambda file: filename_to_package_name_arch.get(file.name)
+        extract_file_name_mocked.side_effect = lambda file: (
+            filename_to_package_name_arch.get(file.name)
         )
         self.addCleanup(extract_package_name_arch_patcher.stop)
 
@@ -1513,7 +1457,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
                     id=artifact_id,
                     workspace=workspace,
                     category="Test",
-                    created_at=dt.datetime.now(),
+                    created_at=dt.datetime.now(tz=dt.UTC),
                     data={},
                     download_tar_gz_url="https://example.com/not-used",
                     files=FilesResponseType({filename: file_model}),
