@@ -14,10 +14,10 @@ from typing import Any
 from unittest import mock
 from unittest.mock import call
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianSourcePackage,
     WorkRequestResults,
 )
@@ -27,7 +27,7 @@ from debusine.tasks.models import MakeSourcePackageUploadDynamicData
 from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -66,33 +66,39 @@ class MakeSourcePackageUploadTests(
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:format=tarball:"
-                    "backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            input_source_artifact=ArtifactInfo(
+                id=421,
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name="hello", version="1.0", type="dpkg", dsc_fields={}
                 ),
-                # input.source_artifact
-                (421, None): ArtifactInfo(
-                    id=421,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            }
+            ),
+        )
+
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:makesourcepackageupload:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
         )
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             MakeSourcePackageUploadDynamicData(
                 environment_id=1,
                 input_source_artifact_id=421,
@@ -101,23 +107,12 @@ class MakeSourcePackageUploadTests(
             ),
         )
 
-    def test_compute_dynamic_data_raise_task_config_error_source(self) -> None:
-        self.task_db.single_lookups[(421, None)] = ArtifactInfo(
-            id=421,
-            category=ArtifactCategory.BINARY_PACKAGE,
-            data=DebianSourcePackage(
-                name="hello", version="1.0", type="dpkg", dsc_fields={}
-            ),
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            MakeSourcePackageUpload.inputs["input_source_artifact"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.SOURCE_PACKAGE,),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"input.source_artifact: unexpected artifact category: "
-            r"'debian:binary-package'. Valid categories: "
-            r"\['debian:source-package'\]",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
         self.assertEqual(self.task.get_input_artifacts_ids(), [])
