@@ -16,23 +16,23 @@ from typing import Any
 from unittest import mock
 from unittest.mock import call
 
+import debusine.worker.tags as wtags
 from debian import deb822
 from debusine.artifacts import Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
     DebianUpload,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.artifacts.playground import ArtifactPlayground
 from debusine.client.models import LookupResultType, LookupSingleResponse
 from debusine.tasks import MergeUploads, TaskConfigError
 from debusine.tasks.mergeuploads import MergeUploadsError
-from debusine.tasks.models import LookupMultiple, MergeUploadsDynamicData
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.models import MergeUploadsDynamicData
+from debusine.tasks.server import ArtifactInfo, MultipleArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -65,10 +65,9 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
-        self.task_db = FakeTaskDatabase(
-            multiple_lookups={
-                # input.uploads
-                (LookupMultiple((421, 666)), None): [
+        self.task_db = MockTaskDatabase(
+            uploads=MultipleArtifactInfo(
+                (
                     ArtifactInfo(
                         id=421,
                         category=ArtifactCategory.UPLOAD,
@@ -93,8 +92,8 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                             },
                         ),
                     ),
-                ]
-            },
+                )
+            )
         )
 
     @staticmethod
@@ -127,10 +126,21 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             for path in paths
         ]
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:mergeuploads:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(self.task.compute_system_required_tags(), [])
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             MergeUploadsDynamicData(
                 input_uploads_ids=[421, 666],
                 subject="hello",
@@ -140,24 +150,37 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
 
     def test_compute_dynamic_data_multiple_packages(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        binary_artifacts_lookup = LookupMultiple((421, 666))
-        self.task_db.multiple_lookups[(binary_artifacts_lookup, None)][1] = (
-            ArtifactInfo(
-                id=666,
-                category=ArtifactCategory.UPLOAD,
-                data=DebianUpload(
-                    type="dpkg",
-                    changes_fields={
-                        "Architecture": "s390x",
-                        "Files": [{"name": "bye_1.0-1_s390x.deb"}],
-                        "Source": "bye",
-                    },
+        self.task_db.field_values["uploads"] = MultipleArtifactInfo(
+            (
+                ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Files": [{"name": "hello_1.0_amd64.deb"}],
+                            "Source": "hello",
+                        },
+                    ),
+                ),
+                ArtifactInfo(
+                    id=666,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "s390x",
+                            "Files": [{"name": "bye_1.0-1_s390x.deb"}],
+                            "Source": "bye",
+                        },
+                    ),
                 ),
             )
         )
 
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             MergeUploadsDynamicData(
                 input_uploads_ids=[421, 666],
                 subject=None,
@@ -165,23 +188,12 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             ),
         )
 
-    def test_compute_dynamic_raise_task_error_wrong_category(self) -> None:
-        binary_artifacts_lookup = LookupMultiple((421, 666))
-        self.task_db.multiple_lookups[(binary_artifacts_lookup, None)][0] = (
-            ArtifactInfo(
-                id=421,
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=EmptyArtifactData(),
-            )
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            MergeUploads.inputs["uploads"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.UPLOAD,),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.uploads: unexpected artifact category: "
-            r"'debian:system-tarball'. Valid categories: \['debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
         self.assertEqual(self.task.get_input_artifacts_ids(), [])
