@@ -16,6 +16,7 @@ from typing import Any, cast
 from unittest import mock
 from unittest.mock import call
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.local_artifact import DebianSystemTarballArtifact
 from debusine.artifacts.models import WorkRequestResults
 from debusine.tasks import MmDebstrap
@@ -27,7 +28,7 @@ from debusine.tasks.models import (
 )
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 
@@ -78,12 +79,18 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
     ) -> None:
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
-        self.task_db = FakeTaskDatabase()
+        self.task_db = MockTaskDatabase()
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.CAP_PREFIX + "mmdebstrap"],
+        )
 
     def test_compute_dynamic_data(self) -> None:
         """Test compute_dynamic_data."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             BaseDynamicTaskData(
                 subject="bookworm",
                 parameter_summary="bookworm (amd64)",
@@ -108,6 +115,23 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
 
         self.assertTrue(self.task.data.bootstrap_options.use_signed_by)
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.mock_is_command_available({"mmdebstrap": True})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:mmdebstrap:version:1",
+                wtags.CAP_PREFIX + "mmdebstrap",
+            ],
+        )
+
+    def test_get_provided_worker_tags_mmdebstrap_not_available(self) -> None:
+        self.mock_is_command_available({"mmdebstrap": False})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [wtags.TASK_PREFIX + "worker:mmdebstrap:version:1"],
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"mmdebstrap": True})
