@@ -458,6 +458,20 @@ class LintianDataTests(TestCase):
             )
 
 
+class LintianDynamicDataTests(TestCase):
+    """Tests for :py:class:`LintianDynamicData`."""
+
+    def test_get_source_package_name(self) -> None:
+        for subject in ("foo", None):
+            with self.subTest(subject=subject):
+                d = task_models.LintianDynamicData(
+                    input_source_artifact_id=1,
+                    input_binary_artifacts_ids=[1],
+                    subject=subject,
+                )
+                self.assertEqual(d.get_source_package_name(), subject)
+
+
 class DebDiffDataTests(TestCase):
     """Tests for DebDiff task data."""
 
@@ -510,6 +524,30 @@ class DebDiffDataTests(TestCase):
             )
 
 
+class MakeSourcePackageUploadDynamicDataTests(TestCase):
+    """Tests for :py:class:`MakeSourcePackageUploadDynamicData`."""
+
+    def test_get_source_package_name(self) -> None:
+        d = task_models.MakeSourcePackageUploadDynamicData(
+            environment_id=1,
+            input_source_artifact_id=1,
+            subject="foo",
+        )
+        self.assertEqual(d.get_source_package_name(), "foo")
+
+
+class MergeUploadsDynamicDataTests(TestCase):
+    """Tests for :py:class:`MergeUploadsDynamicData`."""
+
+    def test_get_source_package_name(self) -> None:
+        d = task_models.MergeUploadsDynamicData(
+            environment_id=1,
+            input_uploads_ids=[1],
+            subject="foo",
+        )
+        self.assertEqual(d.get_source_package_name(), "foo")
+
+
 class SystemBootstrapDataTests(TestCase):
     """Tests for SystemBootstrap task data."""
 
@@ -638,6 +676,31 @@ class AutopkgtestDataTests(TestCase):
                     )
 
 
+class AutopkgtestDynamicDataTests(TestCase):
+    """Tests for :py:class:`AutopkgtestDynamicData`."""
+
+    def test_get_source_package_name(self) -> None:
+        d = task_models.AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=1,
+            input_binary_artifacts_ids=[1],
+            subject="foo",
+        )
+        self.assertEqual(d.get_source_package_name(), "foo")
+
+
+class BlhcDynamicDataTests(TestCase):
+    """Tests for :py:class:`BlhcDynamicData`."""
+
+    def test_get_source_package_name(self) -> None:
+        d = task_models.BlhcDynamicData(
+            environment_id=1,
+            input_artifact_id=1,
+            subject="foo",
+        )
+        self.assertEqual(d.get_source_package_name(), "foo")
+
+
 class ExtraExternalRepositoryTests(TestCase):
     """Tests for ExtraExternalRepository."""
 
@@ -801,6 +864,14 @@ class SbuildDynamicDataTests(TestCase):
             binnmu_maintainer="Debusine <noreply@debusine-dev>",
         )
 
+    def test_get_source_package_name(self) -> None:
+        d = task_models.SbuildDynamicData(
+            input_source_artifact_id=1,
+            binnmu_maintainer="Debusine <noreply@debusine-dev>",
+            subject="foo",
+        )
+        self.assertEqual(d.get_source_package_name(), "foo")
+
 
 class ImageCacheUsageLogEntryTests(TestCase):
     """Tests for ImageCacheUsageLogEntry."""
@@ -815,7 +886,8 @@ class ImageCacheUsageLogEntryTests(TestCase):
         """Test ImageCacheUsageLogEntry rejects naive timestamp."""
         with self.assertRaisesRegex(ValueError, "timestamp is TZ-naive"):
             task_models.ImageCacheUsageLogEntry(
-                filename="foo", timestamp=dt.datetime.now()
+                filename="foo",
+                timestamp=dt.datetime.now(),  # noqa: DTZ005
             )
 
 
@@ -860,6 +932,19 @@ class ExtractForSigningDataTests(TestCase):
             )
 
 
+class ExtractForSigningDynamicDataTests(TestCase):
+    """Tests for :py:class:`ExtractForSigningDynamicData`."""
+
+    def test_get_source_package_name(self) -> None:
+        d = task_models.ExtractForSigningDynamicData(
+            environment_id=1,
+            input_template_artifact_id=1,
+            input_binary_artifacts_ids=[1],
+            subject="foo",
+        )
+        self.assertEqual(d.get_source_package_name(), None)
+
+
 class AssembleSignedSourceDataTests(TestCase):
     """Tests for AssembleSignedSourceData."""
 
@@ -889,6 +974,21 @@ class AssembleSignedSourceDataTests(TestCase):
             )
 
 
+class AssembleSignedSourceDynamicDataTests(TestCase):
+    """Tests for :py:class:`AssembleSignedSourceDynamicData."""
+
+    def test_get_source_package_name(self) -> None:
+        d = task_models.AssembleSignedSourceDynamicData(
+            environment_id=1,
+            template_id=1,
+            signed_ids=[1],
+            subject="foo",
+        )
+        # In this case, subject is a binary package name, not a source package
+        # name
+        self.assertIsNone(d.get_source_package_name())
+
+
 class EventReactionsTests(TestCase):
     """Tests for EventReactions."""
 
@@ -1018,3 +1118,16 @@ class DebDiffDynamicDataTests(TestCase):
             input_binary_artifacts_ids=[[1, 2, 3], [10, 20, 30]],
             environment_id=3,
         )
+
+    def test_get_source_package_name(self) -> None:
+        for subject, expected in (
+            ("foo", None),
+            ("source:foo", "foo"),
+            ("binary:foo", None),
+        ):
+            with self.subTest(subject=subject):
+                d = task_models.DebDiffDynamicData(
+                    environment_id=1,
+                    subject=subject,
+                )
+                self.assertEqual(d.get_source_package_name(), expected)
