@@ -12,6 +12,7 @@
 from tempfile import TemporaryDirectory
 from unittest import TestCase, mock
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import WorkRequestResults
 from debusine.tasks import Noop, TaskConfigError
 
@@ -19,6 +20,19 @@ from debusine.tasks import Noop, TaskConfigError
 class NoopTests(TestCase):
     """Test the Noop class."""
 
+    def test_get_provided_worker_tags(self) -> None:
+        task = Noop({"result": WorkRequestResults.SUCCESS})
+        self.assertCountEqual(
+            task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:noop:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        task = Noop({"result": WorkRequestResults.SUCCESS})
+        self.assertCountEqual(task.compute_system_required_tags(), [])
+
     def test_execute_true(self) -> None:
         """Assert execute() return SUCCESS (result is SUCCESS)."""
         noop = Noop({"result": WorkRequestResults.SUCCESS})
