@@ -22,6 +22,7 @@ from unittest.mock import MagicMock, call
 
 from debian.debian_support import Version
 
+import debusine.worker.tags as wtags
 from debusine.artifacts import PiupartsArtifact
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -29,7 +30,6 @@ from debusine.artifacts.models import (
     DebianPiuparts,
     DebianSystemTarball,
     DebianUpload,
-    EmptyArtifactData,
     WorkRequestResults,
 )
 from debusine.client.models import (
@@ -41,13 +41,16 @@ from debusine.client.models import (
 from debusine.tasks import Piuparts, TaskConfigError
 from debusine.tasks.models import (
     BackendType,
-    LookupMultiple,
+    ExtraExternalRepository,
     PiupartsDynamicData,
 )
-from debusine.tasks.server import ArtifactInfo, CollectionInfo
+from debusine.tasks.server import (
+    ArtifactInfo,
+    MultipleArtifactInfo,
+)
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -96,35 +99,27 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            base_tgz=ArtifactInfo(
+                id=44,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            input_binary_artifacts=MultipleArtifactInfo(
                 (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=piuparts",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # base_tgz
-                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
-                    id=44,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (LookupMultiple((421,)), None): [
                     ArtifactInfo(
                         id=421,
                         category=ArtifactCategory.UPLOAD,
                         data=self._binary_upload_data,
-                    )
-                ]
-            },
+                    ),
+                )
+            ),
+            extra_repositories=None,
         )
 
     def test_configure_fails_with_missing_required_data(
@@ -148,10 +143,24 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         with self.assertRaises(TaskConfigError):
             self.configure_task(remove=["build_architecture"])
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:piuparts:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             PiupartsDynamicData(
                 environment_id=1,
                 input_binary_artifacts_ids=[421],
@@ -168,31 +177,19 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         self.configure_task(
             override={"base_tgz": "debian/match:codename=bookworm"}
         )
-        self.task_db.single_lookups = {
-            # environment
-            (
-                "debian/match:codename=bookworm:architecture=amd64:"
-                "format=tarball:backend=unshare:variant=piuparts",
-                CollectionCategory.ENVIRONMENTS,
-            ): ArtifactInfo(
-                id=1,
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=create_system_tarball_data(),
-            ),
-            # base_tgz
-            (
-                "debian/match:codename=bookworm:architecture=amd64:"
-                "format=tarball",
-                CollectionCategory.ENVIRONMENTS,
-            ): ArtifactInfo(
-                id=44,
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=create_system_tarball_data(codename="bookworm"),
-            ),
-        }
+        self.task_db.field_values["environment"] = ArtifactInfo(
+            id=1,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        self.task_db.field_values["base_tgz"] = ArtifactInfo(
+            id=44,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(codename="bookworm"),
+        )
 
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             PiupartsDynamicData(
                 environment_id=1,
                 input_binary_artifacts_ids=[421],
@@ -209,98 +206,57 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             override={"input": {"binary_artifacts": [421, 422]}}
         )
 
-        self.task_db.single_lookups = {
-            # environment
-            (
-                44,
-                CollectionCategory.ENVIRONMENTS,
-            ): ArtifactInfo(
-                id=44,
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=create_system_tarball_data(),
-            ),
-            # base_tgz
-            (
-                "debian/match:codename=bookworm:architecture=amd64:"
-                "format=tarball:backend=unshare:variant=",
-                CollectionCategory.ENVIRONMENTS,
-            ): ArtifactInfo(
-                id=45,
-                category=ArtifactCategory.SYSTEM_TARBALL,
-                data=create_system_tarball_data(codename="bookworm"),
-            ),
-        }
-        self.task_db.multiple_lookups = {
-            # input.binary_artifacts
-            (LookupMultiple((421, 422)), None): [
-                ArtifactInfo(
-                    id=421,
-                    category=ArtifactCategory.UPLOAD,
-                    data=self._binary_upload_data,
-                ),
-                ArtifactInfo(
-                    id=422,
-                    category=ArtifactCategory.UPLOAD,
-                    data=DebianUpload(
-                        type="dpkg",
-                        changes_fields={
-                            "Architecture": "amd64",
-                            "Version": "2.0",
-                            "Files": [{"name": "bye_1.0-1_amd64.deb"}],
-                            "Source": "bye",
-                        },
+        self.task_db.field_values["environment"] = ArtifactInfo(
+            id=44,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        self.task_db.field_values["base_tgz"] = ArtifactInfo(
+            id=45,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(codename="bookworm"),
+        )
+        self.task_db.field_values["input_binary_artifacts"] = (
+            MultipleArtifactInfo(
+                (
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self._binary_upload_data,
                     ),
-                ),
-            ]
-        }
-
-        dynamic_data = self.task.compute_dynamic_data(self.task_db)
+                    ArtifactInfo(
+                        id=422,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Version": "2.0",
+                                "Files": [{"name": "bye_1.0-1_amd64.deb"}],
+                                "Source": "bye",
+                            },
+                        ),
+                    ),
+                )
+            )
+        )
+        dynamic_data = self.compute_dynamic_data(self.task)
 
         self.assertEqual(dynamic_data.subject, "bye hello")
         self.assertEqual(dynamic_data.parameter_summary, "bye hello (amd64)")
 
-    def test_compute_dynamic_raise_config_task_error_wrong_environment(
-        self,
-    ) -> None:
-        """
-        Test compute_dynamic_data raise TaskConfigError.
-
-        environment artifact category is unexpected.
-        """
-        self.task_db.single_lookups[(44, CollectionCategory.ENVIRONMENTS)] = (
-            ArtifactInfo(
-                id=44,
-                category=ArtifactCategory.SOURCE_PACKAGE,
-                data=EmptyArtifactData(),
-            )
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            Piuparts.inputs["base_tgz"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.SYSTEM_TARBALL,),
+        )
+        self.assertEqual(
+            Piuparts.inputs["input_binary_artifacts"].categories,  # type: ignore[attr-defined]
+            (
+                ArtifactCategory.BINARY_PACKAGE,
+                ArtifactCategory.UPLOAD,
+            ),
         )
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            "^base_tgz: unexpected artifact category: 'debian:source-package'. "
-            r"Valid categories: \['debian:system-tarball'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
-    def test_compute_dynamic_raise_config_task_error_wrong_binary_artifact(
-        self,
-    ) -> None:
-        """
-        Test compute_dynamic_data raise TaskConfigError.
-
-        binary_artifact artifact category is unexpected.
-        """
-        self.task_db.multiple_lookups[(LookupMultiple((421,)), None)][
-            0
-        ].category = ArtifactCategory.SYSTEM_TARBALL
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^input.binary_artifacts\[0\]: unexpected artifact category: "
-            r"'debian:system-tarball'. Valid categories: "
-            r"\['debian:binary-package', 'debian:upload'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
 
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
@@ -423,7 +379,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 id=artifact_id,
                 workspace="System",
                 category=ArtifactCategory.SYSTEM_TARBALL,
-                created_at=dt.datetime.now(),
+                created_at=dt.datetime.now(tz=dt.UTC),
                 data=data.model_dump(),
                 download_tar_gz_url=(
                     f"http://localhost/artifact/{artifact_id}?archive=tar.gz"
@@ -759,6 +715,14 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 ]
             }
         )
+        self.task_db.field_values["extra_repositories"] = [
+            ExtraExternalRepository(
+                url="http://example.net",
+                suite="bookworm",
+                components=["main"],
+                signing_key="KEY A",
+            )
+        ]
         self.set_dynamic_data()
         script = self.prepare_extra_repo_script("buster")
         self.assertIn("mkdir -p /etc/apt/keyrings\n", script)
@@ -800,6 +764,14 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 ]
             }
         )
+        self.task_db.field_values["extra_repositories"] = [
+            ExtraExternalRepository(
+                url="http://example.net",
+                suite="bookworm",
+                components=["main"],
+                signing_key="KEY A",
+            )
+        ]
         self.set_dynamic_data()
         script = self.prepare_extra_repo_script("bookworm")
         self.assertNotIn("/etc/apt/keyrings", script)
@@ -825,25 +797,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     def test_prepare_scripts_extra_debusine_repositories(self) -> None:
         """_prepare_scripts with extra repositories hosted by Debusine."""
-        self.configure_task(
-            override={
-                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
-            }
-        )
-        self.task_db.single_collection_lookups[
-            ("trixie-local@debian:suite", None)
-        ] = CollectionInfo(
-            id=1,
-            scope_name="scope",
-            workspace_name="workspace",
-            category=CollectionCategory.SUITE,
-            name="trixie-local",
-            data={"components": ["main"]},
-        )
-        self.task_db.settings["DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"] = (
-            "deb.example.com"
-        )
-        self.task_db.suite_signing_keys[1] = "\n".join(
+        signing_key = "\n".join(
             (
                 "-----BEGIN PGP PUBLIC KEY BLOCK-----",
                 "",
@@ -851,6 +805,19 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 "-----END PGP PUBLIC KEY BLOCK-----",
             )
         )
+        self.configure_task(
+            override={
+                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
+            }
+        )
+        self.task_db.field_values["extra_repositories"] = [
+            ExtraExternalRepository(
+                url="http://deb.example.com/scope/workspace",
+                suite="trixie-local",
+                components=["main"],
+                signing_key=signing_key,
+            )
+        ]
         self.set_dynamic_data()
         script = self.prepare_extra_repo_script("trixie")
         self.assertNotIn("/etc/apt/keyrings", script)
