@@ -17,6 +17,7 @@ from typing import Any
 from unittest import mock
 from unittest.mock import MagicMock, call
 
+import debusine.worker.tags as wtags
 from debusine import utils
 from debusine.artifacts import (
     BinaryPackage,
@@ -26,11 +27,10 @@ from debusine.artifacts import (
 )
 from debusine.artifacts.models import (
     ArtifactCategory,
-    CollectionCategory,
     DebianBinaryPackage,
     DebianSourcePackage,
     DoseDistCheck,
-    EmptyArtifactData,
+    WorkRequestResults,
 )
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse, RemoteArtifact
@@ -38,14 +38,18 @@ from debusine.tasks import Sbuild, TaskConfigError
 from debusine.tasks.executors import ExecutorInterface
 from debusine.tasks.models import (
     BackendType,
+    ExtraExternalRepository,
     SbuildBuildComponent,
     SbuildDynamicData,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo, CollectionInfo
+from debusine.tasks.server import (
+    ArtifactInfo,
+    MultipleArtifactInfo,
+)
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -97,30 +101,22 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.task.work_request_id = 5
         self.task.workspace_name = "testing"
 
-        self.task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=sbuild",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
+        self.task_db = MockTaskDatabase(
+            environment=ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            input_source_artifact=ArtifactInfo(
+                id=5,
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name="hello", version="1.0", type="dpkg", dsc_fields={}
                 ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            },
-            settings={
-                "DEBUSINE_FQDN": "debusine.example.com",
-            },
+            ),
+            input_extra_binary_artifacts=MultipleArtifactInfo(),
+            extra_repositories=None,
+            debusine_fqdn="debusine.example.com",
         )
 
         self.task.logger.disabled = True
@@ -138,12 +134,18 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.cmdline_mock.return_value = cmdline
         self.addCleanup(patcher.stop)
 
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.EXECUTOR_PREFIX + "unshare", wtags.CAP_PREFIX + "sbuild"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         self.configure_task(override={"backend": BackendType.UNSHARE})
 
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             SbuildDynamicData(
                 environment_id=1,
                 input_source_artifact_id=5,
@@ -181,26 +183,14 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             },
         )
 
-        self.assertIsNone(
-            self.task.compute_dynamic_data(self.task_db).runtime_context
-        )
+        self.assertIsNone(self.compute_dynamic_data(self.task).runtime_context)
 
-    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
-        """Raise TaskConfigError: invalid input_source_artifact."""
-        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
-            id=5,
-            category=ArtifactCategory.BINARY_PACKAGE,
-            data=EmptyArtifactData(),
+    def test_acceptable_input_categories(self) -> None:
+        self.assertEqual(
+            Sbuild.inputs["input_source_artifact"].categories,  # type: ignore[attr-defined]
+            (ArtifactCategory.SOURCE_PACKAGE,),
         )
 
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            "^input.source_artifact: unexpected artifact category: "
-            r"'debian:binary-package'. Valid categories: "
-            r"\['debian:source-package'\]$",
-        ):
-            self.task.compute_dynamic_data(self.task_db)
-
     def test_configure_with_unknown_data(self) -> None:
         """Configure fails if a non-recognised key is in task_data."""
         with self.assertRaises(TaskConfigError):
@@ -209,9 +199,11 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_configure_fails_with_missing_required_data(self) -> None:
         """Configure fails with missing required keys in task_data."""
         for key in ("input", "build_architecture"):
-            with self.subTest(f"Configure with key {key} missing"):
-                with self.assertRaises(TaskConfigError):
-                    self.configure_task(remove=[key])
+            with (
+                self.subTest(f"Configure with key {key} missing"),
+                self.assertRaises(TaskConfigError),
+            ):
+                self.configure_task(remove=[key])
 
     def test_configure_fails_required_input(self) -> None:
         """Configure fails: missing source_artifact in task_data["input"]."""
@@ -521,13 +513,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_contains_arch_parameter(self) -> None:
         """Ensure --arch parameter is computed from data."""
         self.configure_task(override={"build_architecture": "mipsel"})
-        self.task_db.single_lookups[
-            (
-                "debian/match:codename=bookworm:architecture=mipsel:"
-                "format=tarball:backend=unshare:variant=sbuild",
-                CollectionCategory.ENVIRONMENTS,
-            )
-        ] = ArtifactInfo(
+        self.task_db.field_values["environment"] = ArtifactInfo(
             id=1,
             category=ArtifactCategory.SYSTEM_TARBALL,
             data=create_system_tarball_data(),
@@ -621,25 +607,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_extra_debusine_repositories(self) -> None:
         """Extra repositories can be hosted by Debusine."""
-        self.configure_task(
-            override={
-                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
-            }
-        )
-        self.task_db.single_collection_lookups[
-            ("trixie-local@debian:suite", None)
-        ] = CollectionInfo(
-            id=1,
-            scope_name="scope",
-            workspace_name="workspace",
-            category=CollectionCategory.SUITE,
-            name="trixie-local",
-            data={"components": ["main"]},
-        )
-        self.task_db.settings["DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"] = (
-            "deb.example.com"
-        )
-        self.task_db.suite_signing_keys[1] = "\n".join(
+        signing_key = "\n".join(
             (
                 "-----BEGIN PGP PUBLIC KEY BLOCK-----",
                 "",
@@ -647,6 +615,20 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 "-----END PGP PUBLIC KEY BLOCK-----",
             )
         )
+        self.configure_task(
+            override={
+                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
+            }
+        )
+        self.task_db.field_values["extra_repositories"] = [
+            ExtraExternalRepository(
+                url="http://deb.example.com/scope/workspace",
+                suite="trixie-local",
+                components=["main"],
+                signing_key=signing_key,
+            )
+        ]
+        self.task_db.field_values["debusine_fqdn"] = "deb.example.com"
         self.set_dynamic_data()
         self.patch_executor(codename="trixie")
         destination = self.create_temporary_directory()
@@ -886,13 +868,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         data = self.configuration("amd64", ["any"])
         data["backend"] = BackendType.INCUS_LXC
         self.configure_task(data)
-        self.task_db.single_lookups[
-            (
-                "debian/match:codename=bookworm:architecture=amd64:"
-                "format=tarball:backend=incus-lxc:variant=sbuild",
-                CollectionCategory.ENVIRONMENTS,
-            )
-        ] = ArtifactInfo(
+        self.task_db.field_values["environment"] = ArtifactInfo(
             id=1,
             category=ArtifactCategory.SYSTEM_TARBALL,
             data=create_system_tarball_data(),
@@ -945,7 +921,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
           returned.
         """
         temporary_directories = []
-        for _ in range(0, directories):
+        for _ in range(directories):
             temporary_directories.append(self.create_temporary_directory())
 
         class MockedPathContextManager:
@@ -962,7 +938,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 exc_tb: types.TracebackType | None,
             ) -> None:
                 exc_type, exc_val, exc_tb  # fake usage for vulture
-                pass
 
         patcher = mock.patch.object(
             self.task, "_temporary_directory", autospec=True
@@ -978,7 +953,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def run_execute(
         self,
         cmdline: list[str],
-    ) -> bool:
+    ) -> WorkRequestResults:
         """Run Sbuild.execute() mocking cmdline. Return execute()."""
         self.mock_cmdline(cmdline)
 
@@ -991,9 +966,9 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         return self.task.execute()
 
-    def test_execute_returns_true(self) -> None:
+    def test_execute_returns_success(self) -> None:
         """
-        The execute method returns True when cmd returns 0.
+        The execute method returns SUCCESS when cmd returns 0.
 
         Also check that check_directory_for_consistency_errors() didn't
         return errors.
@@ -1003,7 +978,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.patch_sbuild_debusine()
         self.patch_fetch_input(return_value=True)
         self.patch_configure_for_execution(return_value=True)
-        self.patch_task_succeeded(return_value=True)
+        self.patch_task_result(return_value=WorkRequestResults.SUCCESS)
 
         check_directory_for_consistency_errors_mocked = (
             self.patch_check_directory_for_consistency_errors()
@@ -1017,7 +992,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             "echo stderr output of the command >&2'"
         )
 
-        self.assertTrue(self.run_execute(cmd))
+        self.assertEqual(self.run_execute(cmd), WorkRequestResults.SUCCESS)
 
         expected = textwrap.dedent(
             f"""\
@@ -1048,14 +1023,14 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         check_directory_for_consistency_errors_mocked.assert_called()
 
-    def test_execute_return_false_no_fetch_required_input_files(self) -> None:
-        """execute() method returns False: cannot fetch required input files."""
+    def test_execute_return_failure_no_fetch_required_input_files(self) -> None:
+        """execute() returns FAILURE: cannot fetch required input files."""
         self.patch_fetch_input(return_value=False)
-        self.assertFalse(self.run_execute(["true"]))
+        self.assertEqual(self.run_execute(["true"]), WorkRequestResults.FAILURE)
 
     def test_execute_check_build_consistency_error_returns_errors(self) -> None:
         """
-        execute() method returns False: check_build_consistency returned errors.
+        execute() method returns FAILURE: check_build_consistency errors.
 
         Assert check_build_consistency() was called.
         """
@@ -1070,13 +1045,13 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             "some error"
         ]
 
-        self.assertFalse(self.run_execute(["true"]))
+        self.assertEqual(self.run_execute(["true"]), WorkRequestResults.FAILURE)
 
         check_directory_for_consistency_errors_mocked.assert_called()
 
-    def test_execute_returns_false(self) -> None:
+    def test_execute_returns_failure(self) -> None:
         """
-        The execute method returns False when cmd returns 1.
+        The execute method returns FAILURE when cmd returns 1.
 
         Assert check_build_consistency() was not called.
         """
@@ -1086,7 +1061,9 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         self.patch_fetch_input(return_value=True)
 
-        self.assertFalse(self.run_execute(["false"]))
+        self.assertEqual(
+            self.run_execute(["false"]), WorkRequestResults.FAILURE
+        )
 
         check_build_consistency_mocked.assert_not_called()
 
@@ -1164,14 +1141,16 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.task.executor.system_image.data = {"codename": codename}
         return mocked
 
-    def patch_task_succeeded(self, *, return_value: bool) -> mock.Mock:
+    def patch_task_result(
+        self, *, return_value: WorkRequestResults
+    ) -> mock.Mock:
         """
-        Patch self.task.task_succeeded.
+        Patch self.task.task_result.
 
         :param return_value: set mocked.return_value = return_value
         :return: the mock
         """
-        patcher = mock.patch.object(self.task, "task_succeeded", autospec=True)
+        patcher = mock.patch.object(self.task, "task_result", autospec=True)
         mocked = patcher.start()
         mocked.return_value = return_value
         self.addCleanup(patcher.stop)
@@ -1201,7 +1180,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.patch_sbuild_debusine()
         self.patch_fetch_input(return_value=True)
         self.patch_configure_for_execution(return_value=True)
-        self.patch_task_succeeded(return_value=True)
+        self.patch_task_result(return_value=WorkRequestResults.SUCCESS)
 
         execute_directory, download_directory = self.patch_temporary_directory(
             2
@@ -1223,7 +1202,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         self.assertEqual(upload_artifacts_mocked.call_count, 1)
         upload_artifacts_mocked.assert_called_with(
-            execute_directory, execution_success=True
+            execute_directory, execution_result=WorkRequestResults.SUCCESS
         )
 
         self.assertEqual(upload_work_request_debug_logs_mocked.call_count, 1)
@@ -1270,10 +1249,20 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         run_cmd = self.patch_run_cmd()
 
-        self.assertFalse(self.task.execute())
+        self.assertEqual(self.task.execute(), WorkRequestResults.FAILURE)
 
         run_cmd.assert_not_called()
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.mock_is_command_available({"sbuild": True})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:sbuild:version:1",
+                wtags.CAP_PREFIX + "sbuild",
+            ],
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"sbuild": True})
@@ -1356,20 +1345,24 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         self.assertFalse(self.task.can_run_on(worker_metadata))
 
-    def test_task_succeeded_returncode_aborted(self) -> None:
-        """task_succeeded returns False if sbuild was aborted."""
+    def test_task_result_returncode_aborted(self) -> None:
+        """task_result returns FAILURE if sbuild was aborted."""
         directory = self.create_temporary_directory()
 
-        self.assertFalse(self.task.task_succeeded(None, directory))
+        self.assertEqual(
+            self.task.task_result(None, directory), WorkRequestResults.FAILURE
+        )
 
-    def test_task_succeeded_returncode_non_zero(self) -> None:
-        """task_succeeded returns False if sbuild returns non-zero."""
+    def test_task_result_returncode_non_zero(self) -> None:
+        """task_result returns FAILURE if sbuild returns non-zero."""
         directory = self.create_temporary_directory()
 
-        self.assertFalse(self.task.task_succeeded(1, directory))
+        self.assertEqual(
+            self.task.task_result(1, directory), WorkRequestResults.FAILURE
+        )
 
-    def test_task_succeeded_status_skipped(self) -> None:
-        """task_succeeded returns False for "Status: skipped"."""
+    def test_task_result_status_skipped(self) -> None:
+        """task_result returns FAILURE for "Status: skipped"."""
         directory = self.create_temporary_directory()
         (directory / "log.build").write_text(
             textwrap.dedent(
@@ -1383,10 +1376,12 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             )
         )
 
-        self.assertFalse(self.task.task_succeeded(0, directory))
+        self.assertEqual(
+            self.task.task_result(0, directory), WorkRequestResults.FAILURE
+        )
 
-    def test_task_succeeded_status_successful(self) -> None:
-        """task_succeeded returns True for "Status: successful"."""
+    def test_task_result_status_successful(self) -> None:
+        """task_result returns SUCCESS for "Status: successful"."""
         directory = self.create_temporary_directory()
         (directory / "log.build").write_text(
             textwrap.dedent(
@@ -1400,7 +1395,9 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             )
         )
 
-        self.assertTrue(self.task.task_succeeded(0, directory))
+        self.assertEqual(
+            self.task.task_result(0, directory), WorkRequestResults.SUCCESS
+        )
 
     def test_upload_artifacts_no_deb_files(self) -> None:
         """upload_artifacts() does not try to upload deb files."""
@@ -1420,7 +1417,9 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         self.write_changes_file(changes_file, [file1])
 
-        self.task.upload_artifacts(temp_directory, execution_success=True)
+        self.task.upload_artifacts(
+            temp_directory, execution_result=WorkRequestResults.SUCCESS
+        )
 
         self.assertEqual(debusine_mock.upload_artifact.call_count, 3)
 
@@ -1460,10 +1459,12 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         dsc = self.write_dsc_example_file(self.task._dsc_file)
         (log_file := temp_directory / "log.build").write_text("log file")
 
-        self.task.upload_artifacts(temp_directory, execution_success=False)
+        self.task.upload_artifacts(
+            temp_directory, execution_result=WorkRequestResults.FAILURE
+        )
 
         # Only one artifact is uploaded: PackageBuildLog
-        # (because execution_success=False)
+        # (because execution_result=FAILURE)
         self.assertEqual(debusine_mock.upload_artifact.call_count, 1)
 
         debusine_mock.upload_artifact.assert_has_calls(
@@ -1506,7 +1507,9 @@ broken-packages: 4
 """
         )
 
-        self.task.upload_artifacts(temp_directory, execution_success=False)
+        self.task.upload_artifacts(
+            temp_directory, execution_result=WorkRequestResults.FAILURE
+        )
 
         debusine_mock.upload_artifact.assert_has_calls(
             [
@@ -1555,7 +1558,9 @@ broken-packages: 4
             autospec=True,
             return_value=None,
         ) as dose3_extract_mock:
-            self.task.upload_artifacts(temp_directory, execution_success=True)
+            self.task.upload_artifacts(
+                temp_directory, execution_result=WorkRequestResults.SUCCESS
+            )
 
         dose3_extract_mock.assert_not_called()
 
@@ -1868,7 +1873,9 @@ broken-packages: 4
         )
 
         # Upload the artifacts
-        self.task.upload_artifacts(build_directory, execution_success=True)
+        self.task.upload_artifacts(
+            build_directory, execution_result=WorkRequestResults.SUCCESS
+        )
 
         source = dsc_data["Source"]
         version = dsc_data["Version"]
@@ -1965,7 +1972,7 @@ broken-packages: 4
         self.patch_configure_for_execution(return_value=True)
         self.patch_upload_work_request_debug_logs()
         self.patch_executor()
-        self.assertFalse(self.task.execute())
+        self.assertEqual(self.task.execute(), WorkRequestResults.FAILURE)
 
     def patch_upload_package_build_log(self) -> MagicMock:
         """Patch self.task._upload_package_build_log. Return its mock."""
@@ -1996,7 +2003,9 @@ broken-packages: 4
 
         debusine_mock = self.patch_sbuild_debusine()
 
-        self.task.upload_artifacts(temp_directory, execution_success=False)
+        self.task.upload_artifacts(
+            temp_directory, execution_result=WorkRequestResults.FAILURE
+        )
 
         debusine_mock.relation_create.assert_called_with(
             upload_package_build_log_mocked.return_value.id,
@@ -2021,14 +2030,16 @@ broken-packages: 4
 
         upload_package_build_log_mocked = self.patch_upload_package_build_log()
 
-        self.task.upload_artifacts(build_directory, execution_success=False)
+        self.task.upload_artifacts(
+            build_directory, execution_result=WorkRequestResults.FAILURE
+        )
 
         upload_package_build_log_mocked.assert_called_with(
             build_directory,
             dsc_data["Source"],
             dsc_data["Version"],
             self.task._host_architecture,
-            execution_success=False,
+            execution_result=WorkRequestResults.FAILURE,
         )
 
     def test_upload_artifacts_no_dsc_file(self) -> None:
@@ -2037,7 +2048,9 @@ broken-packages: 4
 
         debusine_mock = self.patch_sbuild_debusine()
 
-        self.task.upload_artifacts(build_directory, execution_success=False)
+        self.task.upload_artifacts(
+            build_directory, execution_result=WorkRequestResults.FAILURE
+        )
 
         debusine_mock.upload_artifact.assert_not_called()
 
@@ -2048,7 +2061,9 @@ broken-packages: 4
         self.write_dsc_example_file(self.task._dsc_file)
         debusine_mock = self.patch_sbuild_debusine()
 
-        self.task.upload_artifacts(build_directory, execution_success=False)
+        self.task.upload_artifacts(
+            build_directory, execution_result=WorkRequestResults.FAILURE
+        )
 
         debusine_mock.upload_artifact.assert_not_called()
 
@@ -2060,7 +2075,9 @@ broken-packages: 4
         self.write_deb_file(build_directory / "hello_1.0_amd64.deb")
         debusine_mock = self.patch_sbuild_debusine()
 
-        self.task.upload_artifacts(build_directory, execution_success=True)
+        self.task.upload_artifacts(
+            build_directory, execution_result=WorkRequestResults.SUCCESS
+        )
 
         workspace_name = self.task.workspace_name
         expected_upload_artifact_calls = [
@@ -2080,6 +2097,52 @@ broken-packages: 4
             expected_upload_artifact_calls
         )
 
+    def test_upload_artifacts_changes_but_no_build_log(self) -> None:
+        """upload_artifacts() called with a .changes file but no build log."""
+        temp_directory = self.create_temporary_directory()
+        self.task._dsc_file = self.create_temporary_file(suffix=".dsc")
+        self.write_dsc_example_file(self.task._dsc_file)
+        self.write_deb_file(deb_file := temp_directory / "hello_1.0_amd64.deb")
+        changes_file = temp_directory / "hello_1.0_amd64.changes"
+        self.write_changes_file(changes_file, [deb_file])
+        debusine_mock = self.patch_sbuild_debusine()
+
+        self.task.upload_artifacts(
+            temp_directory, execution_result=WorkRequestResults.SUCCESS
+        )
+
+        workspace_name = self.task.workspace_name
+
+        expected_upload_artifact_calls = [
+            call(
+                BinaryPackage.create(file=deb_file),
+                workspace=workspace_name,
+                work_request=self.task.work_request_id,
+            ),
+            call(
+                Upload.create(changes_file=changes_file),
+                workspace=workspace_name,
+                work_request=self.task.work_request_id,
+            ),
+            call(
+                SigningInputArtifact.create([changes_file], temp_directory),
+                workspace=workspace_name,
+                work_request=self.task.work_request_id,
+            ),
+        ]
+        self.assertEqual(
+            debusine_mock.upload_artifact.call_count,
+            len(expected_upload_artifact_calls),
+        )
+        debusine_mock.upload_artifact.assert_has_calls(
+            expected_upload_artifact_calls
+        )
+
+        # Ensure that upload_artifacts tried to create relations
+        # (the exact creation of relations is tested in
+        # test_create_relations).
+        self.assertGreaterEqual(debusine_mock.relation_create.call_count, 1)
+
     def test_upload_artifacts(self) -> None:
         """upload_artifacts() call upload_artifact() with the artifacts."""
         temp_directory = self.create_temporary_directory()
@@ -2103,7 +2166,9 @@ broken-packages: 4
         changes_file = temp_directory / "python-network.changes"
         self.write_changes_file(changes_file, [file1, file2])
 
-        self.task.upload_artifacts(temp_directory, execution_success=True)
+        self.task.upload_artifacts(
+            temp_directory, execution_result=WorkRequestResults.SUCCESS
+        )
 
         workspace_name = self.task.workspace_name
 
@@ -2149,7 +2214,7 @@ broken-packages: 4
             expected_upload_artifact_calls
         )
 
-        # Ensure that _upload_artifacts tried to create relations
+        # Ensure that upload_artifacts tried to create relations
         # (the exact creation of relations is tested in
         # test_create_relations).
         self.assertGreaterEqual(debusine_mock.relation_create.call_count, 1)
@@ -2204,6 +2269,11 @@ broken-packages: 4
                 artifacts["binary_package"].id,
                 "relates-to",
             ),
+            call(
+                artifacts["build_log"].id,
+                artifacts["binary_upload"].id,
+                "relates-to",
+            ),
             call(
                 artifacts["signing_input"].id,
                 artifacts["binary_upload"].id,
