@@ -17,6 +17,7 @@ from unittest.mock import call, patch
 
 import pydantic
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.local_artifact import DebianSystemImageArtifact
 from debusine.artifacts.models import WorkRequestResults
 from debusine.tasks import SimpleSystemImageBuild
@@ -29,7 +30,7 @@ from debusine.tasks.models import (
 )
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 
@@ -129,7 +130,7 @@ class SimpleSystemImageBuildTests(
     ) -> None:
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
-        self.task_db = FakeTaskDatabase()
+        self.task_db = MockTaskDatabase()
 
     def tearDown(self) -> None:
         """Delete objects."""
@@ -190,9 +191,15 @@ class SimpleSystemImageBuildTests(
             },
         )
 
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.compute_system_required_tags(),
+            [wtags.CAP_PREFIX + "mmdebstrap"],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             BaseDynamicTaskData(
                 subject="stable",
                 parameter_summary="stable (amd64)",
@@ -201,6 +208,23 @@ class SimpleSystemImageBuildTests(
             ),
         )
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.mock_is_command_available({"mmdebstrap": True})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:simplesystemimagebuild:version:1",
+                wtags.CAP_PREFIX + "mmdebstrap",
+            ],
+        )
+
+    def test_get_provided_worker_tags_mmdebstrap_not_available(self) -> None:
+        self.mock_is_command_available({"mmdebstrap": False})
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [wtags.TASK_PREFIX + "worker:simplesystemimagebuild:version:1"],
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"mmdebstrap": True})
