@@ -11,11 +11,12 @@
 
 from typing import Any
 
+import debusine.worker.tags as wtags
 from debusine.tasks.models import BaseDynamicTaskData, DebootstrapVariant
 from debusine.tasks.systemimagebuild import SystemImageBuild
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 
@@ -78,12 +79,23 @@ class SystemImageBuildTests(
     ) -> None:
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
-        self.task_db = FakeTaskDatabase()
+        self.task_db = MockTaskDatabase()
+
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:systemimagebuildimpl:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(self.task.compute_system_required_tags(), [])
 
     def test_compute_dynamic_data(self) -> None:
         """Test compute_dynamic_data."""
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             BaseDynamicTaskData(
                 subject="stable",
                 parameter_summary="stable (amd64)",
@@ -98,6 +110,6 @@ class SystemImageBuildTests(
         self.configure_task(task_data=task_data)
 
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db).parameter_summary,
+            self.compute_dynamic_data(self.task).parameter_summary,
             "stable (amd64-buildd)",
         )
