@@ -24,7 +24,7 @@ from collections import defaultdict
 from collections.abc import Generator, Mapping
 from configparser import ConfigParser
 from pathlib import Path
-from typing import Any, ClassVar, TYPE_CHECKING, cast
+from typing import Any, ClassVar, Optional, TYPE_CHECKING, cast
 from unittest import mock
 from unittest.util import safe_repr
 
@@ -42,6 +42,10 @@ from debusine.artifacts.playground import ArtifactPlayground
 if TYPE_CHECKING:
     from unittest._log import _LoggingWatcher
 
+    from _typeshed import ReadableBuffer
+
+    ReadableBuffer  # fake usage for vulture
+
 
 class TestCase(unittest.TestCase):
     """
@@ -183,7 +187,7 @@ class TestCase(unittest.TestCase):
         *,
         prefix: str | None = None,
         suffix: str | None = None,
-        contents: bytes | None = None,
+        contents: Optional["ReadableBuffer"] = None,
         directory: Path | str | None = None,
     ) -> Path:
         """
@@ -202,15 +206,12 @@ class TestCase(unittest.TestCase):
 
         suffix = suffix or ""
 
-        file = tempfile.NamedTemporaryFile(
+        with tempfile.NamedTemporaryFile(
             prefix=f"{prefix}-", suffix=suffix, delete=False, dir=directory
-        )
+        ) as file:
+            if contents is not None:
+                file.write(contents)
 
-        if contents is not None:
-            file.write(contents)
-            file.close()
-
-        file.close()
         file_path = Path(file.name)
 
         self.addCleanup(file_path.unlink, missing_ok=True)
