@@ -31,6 +31,7 @@ from typing import (
     TYPE_CHECKING,
     Union,
     cast,
+    override,
     runtime_checkable,
 )
 from unittest import mock
@@ -76,9 +77,15 @@ from debusine.db.models.permissions import (
     get_resource_workspace,
     resolve_roles_list,
 )
+from debusine.db.models.tasks import DBTask, DBWorkerTask
 from debusine.db.models.work_requests import WorkRequestQuerySet
 from debusine.db.playground.scenarios import Scenario
-from debusine.tasks.models import BaseTaskDataModel
+from debusine.tasks.models import (
+    BaseDynamicTaskData,
+    BaseTaskData,
+    BaseTaskDataModel,
+)
+from debusine.tasks.server import TaskDatabaseInterface
 from debusine.test.playground import Playground
 
 # Relative import needed to break an import loop
@@ -97,6 +104,31 @@ type TestResponseType = "_MonkeyPatchedWSGIResponse"
 type AnyResponseType = Union[HttpResponseBase, "_MonkeyPatchedWSGIResponse"]
 
 
+@contextlib.contextmanager
+def preserve_db_task_registry() -> Generator[None]:
+    """Make task registry changes ephemeral."""
+    db_orig = DBTask._sub_tasks
+    DBTask._sub_tasks = copy.deepcopy(DBTask._sub_tasks)
+    try:
+        yield
+    finally:
+        DBTask._sub_tasks = db_orig
+
+
+class BaseTestWorkerTask[TD: BaseTaskData](
+    DBWorkerTask[TD, BaseDynamicTaskData], metaclass=abc.ABCMeta
+):
+    """Base class for simple worker tasks used in test code."""
+
+    @override
+    def build_dynamic_data(
+        self,
+        task_database: TaskDatabaseInterface,  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Return default dynamic data."""
+        raise NotImplementedError()
+
+
 class PermissionOverride[M: Model](abc.ABC):
     """Base class for configurable permission overrides for tests."""
 
