@@ -91,9 +91,11 @@ class TestCaseTests(TestCase):
             r'SystemExit not raised : Did not raise '
             r'SystemExit with exit_code=\^3\$'
         )
-        with self.assertRaisesRegex(self.failureException, expected_message):
-            with self.assertRaisesSystemExit(3):
-                pass
+        with (
+            self.assertRaisesRegex(self.failureException, expected_message),
+            self.assertRaisesSystemExit(3),
+        ):
+            pass
 
     def test_assert_raises_system_exit_unexpected_exit_code(self) -> None:
         """Raise self.failureException because wrong exit_code in SystemExit."""
@@ -101,9 +103,11 @@ class TestCaseTests(TestCase):
             r'\^3\$" does not match "7" : Did not raise '
             r'SystemExit with exit_code=\^3\$'
         )
-        with self.assertRaisesRegex(self.failureException, expected_message):
-            with self.assertRaisesSystemExit(3):
-                raise SystemExit(7)
+        with (
+            self.assertRaisesRegex(self.failureException, expected_message),
+            self.assertRaisesSystemExit(3),
+        ):
+            raise SystemExit(7)
 
     def test_assert_raises_system_exit_success(self) -> None:
         """Do not raise self.failureException: expected SystemExit is raised."""
@@ -192,9 +196,11 @@ class TestCaseTests(TestCase):
             'Expected msg found 1 times, expected 2 times$'
         )
 
-        with self.assertRaisesRegex(self.failureException, expected_message):
-            with self.assertLogsContains('required-log', expected_count=2):
-                logging.warning('required-log')
+        with (
+            self.assertRaisesRegex(self.failureException, expected_message),
+            self.assertLogsContains('required-log', expected_count=2),
+        ):
+            logging.warning('required-log')
 
     def test_assertLogsContains_log_expected_not_found(self) -> None:
         """assertLogsContains() raise self.failureException (wrong count)."""
@@ -204,9 +210,11 @@ class TestCaseTests(TestCase):
             'Expected msg found 0 times, expected 1 times$'
         )
 
-        with self.assertRaisesRegex(self.failureException, expected_message):
-            with self.assertLogsContains('required-log'):
-                logging.warning('some-log')
+        with (
+            self.assertRaisesRegex(self.failureException, expected_message),
+            self.assertLogsContains('required-log'),
+        ):
+            logging.warning('some-log')
 
     def test_assertLogsContains_log_expected_not_found_wrong_level(
         self,
@@ -216,9 +224,11 @@ class TestCaseTests(TestCase):
             'no logs of level WARNING or higher triggered on root'
         )
 
-        with self.assertRaisesRegex(self.failureException, expected_message):
-            with self.assertLogsContains('required-log', level=logging.WARNING):
-                logging.debug('some-log')
+        with (
+            self.assertRaisesRegex(self.failureException, expected_message),
+            self.assertLogsContains('required-log', level=logging.WARNING),
+        ):
+            logging.debug('some-log')
 
     def test_assertLogsContains_log_not_found_in_raise_exception(self) -> None:
         """
@@ -232,13 +242,13 @@ class TestCaseTests(TestCase):
             'Expected msg found 0 times, expected 1 times$'
         )
 
-        with self.assertRaisesRegex(self.failureException, expected_message):
-            with (
-                self.assertRaisesRegex(SystemExit, '3'),
-                self.assertLogsContains('The wanted message'),
-            ):
-                logging.warning('Unrelated message')
-                raise SystemExit(3)
+        with (
+            self.assertRaisesRegex(self.failureException, expected_message),
+            self.assertRaisesRegex(SystemExit, '3'),
+            self.assertLogsContains('The wanted message'),
+        ):
+            logging.warning('Unrelated message')
+            raise SystemExit(3)
 
     def test_dump_html(self) -> None:
         """Test dumping HTML to stdout."""
@@ -294,12 +304,14 @@ class TestCaseTests(TestCase):
         content = b"<!DOCTYPE html><html>\n</nothtml>"
 
         stdout = io.StringIO()
-        with contextlib.redirect_stdout(stdout):
-            with self.assertRaisesRegex(
+        with (
+            contextlib.redirect_stdout(stdout),
+            self.assertRaisesRegex(
                 AssertionError,
                 r"2:ERR_TAG_NAME_MISMATCH:Unexpected end tag : nothtml",
-            ):
-                self.assertHTMLValid(content, dump_on_error=True)
+            ),
+        ):
+            self.assertHTMLValid(content, dump_on_error=True)
 
         lines = stdout.getvalue().splitlines()
         self.assertEqual(lines[1], "    HTML parser errors")
@@ -332,13 +344,13 @@ class TestCaseTests(TestCase):
             ("//b", "'//b' matched 2 elements instead of one"),
         ):
             with self.subTest(xpath=xpath):
-                with mock.patch(
-                    "debusine.test.base.TestCase.dump_element"
-                ) as dump:
-                    with self.assertRaisesRegex(
-                        AssertionError, re.escape(message)
-                    ):
-                        self.assertHasElement(tree, xpath, dump_on_error=True)
+                with (
+                    mock.patch(
+                        "debusine.test.base.TestCase.dump_element"
+                    ) as dump,
+                    self.assertRaisesRegex(AssertionError, re.escape(message)),
+                ):
+                    self.assertHasElement(tree, xpath, dump_on_error=True)
                 dump.assert_called_with(tree, title=message)
 
     def test_assertElementHasClass(self) -> None:
@@ -393,13 +405,13 @@ class TestCaseTests(TestCase):
         """Test assertTextContentEqual with dump_on_error."""
         tree = lxml.objectify.fromstring("<p><b>this</b> is a <b>test</b></p>")
 
-        with mock.patch("debusine.test.base.TestCase.dump_element") as dump:
-            with self.assertRaisesRegex(
+        with (
+            mock.patch("debusine.test.base.TestCase.dump_element") as dump,
+            self.assertRaisesRegex(
                 AssertionError, r"'this is a test' != 'mismatch'"
-            ):
-                self.assertTextContentEqual(
-                    tree, "mismatch", dump_on_error=True
-                )
+            ),
+        ):
+            self.assertTextContentEqual(tree, "mismatch", dump_on_error=True)
         dump.assert_called_with(tree, title="Text content mismatch")
 
     def test_assertDatetimeContentEqual(self) -> None:
@@ -463,7 +475,7 @@ class TestCaseTests(TestCase):
 
     def test_assertYAMLContentEqual(self) -> None:
         """Test :py:meth:`assertYAMLContentEqual`."""
-        from debusine.web.views.view_utils import format_yaml
+        from debusine.web.views.view_utils import highlight_as_yaml
 
         for data in (
             None,
@@ -476,7 +488,7 @@ class TestCaseTests(TestCase):
             {1: "one", "two": 2},
         ):
             with self.subTest(data=data):
-                html = format_yaml(data)
+                html = highlight_as_yaml(data)
                 tree = self.assertHTMLValid(html)
                 self.assertYAMLContentEqual(tree, data)
 
