@@ -445,20 +445,20 @@ class TestCaseTests(TestCase):
                 '"detail" not found in response',
             ),
         ):
-            with self.subTest(
-                response=repr(response),
-                title=title,
-                detail_pattern=detail_pattern,
+            with (
+                self.subTest(
+                    response=repr(response),
+                    title=title,
+                    detail_pattern=detail_pattern,
+                ),
+                self.assertRaisesRegex(self.failureException, expected_regex),
             ):
-                with self.assertRaisesRegex(
-                    self.failureException, expected_regex
-                ):
-                    response = response
-                    self._process_json_response(response)
+                response = response
+                self._process_json_response(response)
 
-                    self.assertResponseProblem(
-                        response, title, detail_pattern=detail_pattern
-                    )
+                self.assertResponseProblem(
+                    response, title, detail_pattern=detail_pattern
+                )
 
     @staticmethod
     def _process_json_response(response: HttpResponse) -> None:
