@@ -26,7 +26,7 @@ from debusine.test.test_utils import (
 )
 
 
-class UtilsTest(TestCase):
+class UtilsTests(TestCase):
     """Tests for functions in the utils module."""
 
     def test_data_generator(self) -> None:
@@ -102,21 +102,25 @@ class UtilsTest(TestCase):
         )
 
 
-class UtilsDatabaseTest(DjangoTestCase):
+class UtilsDatabaseTests(DjangoTestCase):
     """Tests for functions in the utils module that need a database."""
 
     def test_print_sql_and_stack(self) -> None:
         """Test print_sql_and_stack."""
         # Nothing is printed if no queries are made
-        with contextlib.redirect_stderr(io.StringIO()) as output:
-            with print_sql_and_stack():
-                pass
+        with (
+            contextlib.redirect_stderr(io.StringIO()) as output,
+            print_sql_and_stack(),
+        ):
+            pass
         self.assertEqual(output.getvalue(), "")
 
         # Output happens with queries
-        with contextlib.redirect_stderr(io.StringIO()) as output:
-            with print_sql_and_stack():
-                list(Scope.objects.all())
+        with (
+            contextlib.redirect_stderr(io.StringIO()) as output,
+            print_sql_and_stack(),
+        ):
+            list(Scope.objects.all())
         self.assertEqual(output.getvalue()[:4], "#1: ")
         self.assertIn("test_print_sql_and_stack", output.getvalue())
         self.assertIn("SELECT", output.getvalue())
