@@ -143,10 +143,7 @@ class ArchiveViewTests(TestCase, ABC):
             response.headers["Cache-Control"],
             "max-age=1800, proxy-revalidate" if mutable else "max-age=31536000",
         )
-        # https://github.com/typeddjango/django-stubs/pull/2704
-        self.assertTrue(
-            has_vary_header(response, "Authorization")  # type: ignore[arg-type]
-        )
+        self.assertTrue(has_vary_header(response, "Authorization"))
 
     def assertResponse404(
         self, response: TestResponseType, message: str
