@@ -63,7 +63,7 @@ from debusine.db.models import (
     WorkRequest,
     Workspace,
 )
-from debusine.tasks import BaseTask
+from debusine.db.models.tasks import DBTask
 
 if TYPE_CHECKING:
     from debusine.web.views.ui.workspaces import WorkspaceUI
@@ -333,7 +333,7 @@ class WorkRequestForm(BootstrapMixin, ModelFormBase[WorkRequest]):
         self.workspace = kwargs.pop("workspace")
         super().__init__(*args, **kwargs)
 
-        tasks = BaseTask.task_names(TaskTypes.WORKER)
+        tasks = DBTask.task_names(TaskTypes.WORKER)
         self.fields["task_name"] = ChoiceField(
             choices=lambda: [(name, name) for name in sorted(tasks)]
         )
