@@ -13,11 +13,12 @@ import abc
 import datetime as dt
 import logging
 from collections.abc import Callable
-from typing import Any, Never, TypeVar, override
+from typing import Any, Never, override
 
 from django import template
 from django.conf import settings
 from django.contrib import messages
+from django.contrib.messages.storage.base import Message
 from django.db.models import Model
 from django.template import Context, Node, NodeList, TemplateSyntaxError
 from django.template.base import FilterExpression, Parser, Token
@@ -114,7 +115,7 @@ class DebusineNode(Node, abc.ABC):
         return format_html(
             "<span data-role='debusine-template-error'"
             " class='bg-danger text-white'>{ts} UTC: {message}</span>",
-            ts=dt.datetime.now(dt.timezone.utc).isoformat(),
+            ts=dt.datetime.now(dt.UTC).isoformat(),
             message=user_message,
         )
 
@@ -225,7 +226,7 @@ def widget(parser: Parser, token: Token) -> WidgetNode:
     """Parser for the {% widget %} tag."""
     bits = token.split_contents()
     if len(bits) == 2:
-        tag, value = bits
+        _, value = bits
     else:
         raise TemplateSyntaxError("{% widget %} requires exactly one argument")
     return WidgetNode(parser.compile_filter(value))
@@ -283,7 +284,7 @@ def withscope(parser: Parser, token: Token) -> WithScopeNode:
     """Parser for the {% withscope %} tag."""
     bits = token.split_contents()
     if len(bits) == 2:
-        tag, value = bits
+        _, value = bits
     else:
         raise TemplateSyntaxError("withscope requires exactly one argument")
 
@@ -311,10 +312,12 @@ def roles(resource: Model, user: PermissionUser = None) -> list[str]:
 
 
 @register.filter
-def format_yaml(value: Any, flags: str = "") -> str:
-    """Format a data structure as YAML."""
+def as_yaml(value: Any, flags: str = "") -> SafeString:
+    """Return syntax-highlighted HTML for value serialized as YAML."""
     sort_keys = "unsorted" not in flags.split(",")
-    return debusine.web.views.view_utils.format_yaml(value, sort_keys=sort_keys)
+    return debusine.web.views.view_utils.highlight_as_yaml(
+        value, sort_keys=sort_keys
+    )
 
 
 @register.simple_tag
@@ -341,13 +344,9 @@ def is_dict_like(value: Any) -> bool:
     return isinstance(value, dict)
 
 
-_KT = TypeVar("_KT")
-_VT = TypeVar("_VT")
-
-
 # https://code.djangoproject.com/ticket/12486
 @register.filter
-def lookup(d: dict[_KT, _VT], key: _KT) -> _VT:
+def lookup[KT, VT](d: dict[KT, VT], key: KT) -> VT:
     """Get a value from a dictionary."""
     return d[key]
 
@@ -576,7 +575,7 @@ def place(parser: Parser, token: Token) -> PlaceNode:
 
 
 @register.filter
-def message_toast_color_class(message: messages.storage.base.Message) -> str:
+def message_toast_color_class(message: Message) -> str:
     """Get the bootstrap color class to use for showing Django messages."""
     match message.level:
         case messages.SUCCESS:
@@ -590,7 +589,7 @@ def message_toast_color_class(message: messages.storage.base.Message) -> str:
 
 
 @register.filter
-def message_toast_header_icon(message: messages.storage.base.Message) -> str:
+def message_toast_header_icon(message: Message) -> str:
     """Get toast header classes to use for showing Django messages."""
     match message.level:
         case messages.WARNING | messages.ERROR:
