@@ -15,6 +15,7 @@ from unittest import mock
 
 from django.contrib import messages
 from django.contrib.auth.models import AnonymousUser
+from django.contrib.messages.storage.base import Message
 from django.db.models import Model
 from django.template import TemplateSyntaxError, engines
 from django.template.backends.django import Template as DjangoTemplate
@@ -37,7 +38,7 @@ from debusine.web.templatetags.debusine import (
     UINode,
     WidgetNode,
     _help,
-    format_yaml,
+    as_yaml,
     has_perm,
     icon,
     message_toast_color_class,
@@ -67,12 +68,11 @@ class DebusineNodeTests(TestCase):
             with (
                 self.subTest(setting=setting),
                 override_settings(**{setting: True}),
+                self.assertRaisesRegex(ValueError, r"devel message"),
             ):
-                with self.assertRaisesRegex(ValueError, r"devel message"):
-                    node._render_error(
-                        user_message="user message",
-                        devel_message="devel message",
-                    )
+                node._render_error(
+                    user_message="user message", devel_message="devel message"
+                )
 
     @override_settings(TEST_MODE=False)
     def test_render_error_exception_debug_or_tests(self) -> None:
@@ -504,24 +504,26 @@ class SortedTests(TestCase):
         self.assertEqual(sorted_([1, 5, 2, 4, 3]), [1, 2, 3, 4, 5])
 
 
-class FormatYamlTests(TestCase):
-    """Test the format_yaml filter."""
+class HighlightAsYamlTests(TestCase):
+    """Test the highlight_as_yaml filter."""
 
-    def test_format(self) -> None:
-        tree = self.assertHTMLValid(format_yaml(42))
-        div = self.assertHasElement(tree, "body/div")
-        self.assertTextContentEqual(div, "42\n...")
-        self.assertEqual(div.get("class"), "file-contents")
+    def test_highlight_as_yaml(self) -> None:
+        tree = self.assertHTMLValid(as_yaml(42))
 
-    def test_format_sorted(self) -> None:
-        tree = self.assertHTMLValid(format_yaml({"b": 2, "a": 1}))
         div = self.assertHasElement(tree, "body/div")
-        self.assertTextContentEqual(div, "a: 1 b: 2")
+        self.assertEqual(div.get("class"), "formatted-content")
+        formatted = div.xpath("string(pre)")
+        self.assertEqual(formatted, "42\n...\n")
 
-    def test_format_unsorted(self) -> None:
-        tree = self.assertHTMLValid(format_yaml({"b": 2, "a": 1}, "unsorted"))
-        div = self.assertHasElement(tree, "body/div")
-        self.assertTextContentEqual(div, "b: 2 a: 1")
+    def test_highlight_as_yaml_sorted(self) -> None:
+        tree = self.assertHTMLValid(as_yaml({"b": 2, "a": 1}))
+        formatted = tree.xpath("string(body/div/pre)")
+        self.assertEqual(formatted, "a: 1\nb: 2\n")
+
+    def test_highlight_as_yaml_unsorted(self) -> None:
+        tree = self.assertHTMLValid(as_yaml({"b": 2, "a": 1}, "unsorted"))
+        formatted = tree.xpath("string(body/div/pre)")
+        self.assertEqual(formatted, "b: 2\na: 1\n")
 
 
 class IconTests(TestCase):
@@ -931,7 +933,7 @@ class MessageToastTests(TestCase):
             (12345, "light"),
         ):
             with self.subTest(level=level):
-                msg = messages.storage.base.Message(level, "test")
+                msg = Message(level, "test")
                 self.assertEqual(message_toast_color_class(msg), expected_class)
 
     def test_header_icon(self) -> None:
@@ -944,5 +946,5 @@ class MessageToastTests(TestCase):
             (12345, "bi bi-info-circle"),
         ):
             with self.subTest(level=level):
-                msg = messages.storage.base.Message(level, "test")
+                msg = Message(level, "test")
                 self.assertEqual(message_toast_header_icon(msg), expected_icon)
