@@ -61,7 +61,7 @@ class WorkflowRuntimeStatusSmallTests(TestCase):
 
         self.assertHTMLEqual(
             actual,
-            '<span class="badge text-bg-secondary" title="Running">R</span>'
+            '<span class="badge text-bg-info" title="Running">R</span>'
             '<span class="badge text-bg-dark" title="Blocked">B</span>',
         )
 
@@ -117,9 +117,9 @@ class WorkflowRuntimeStatusSmallTests(TestCase):
 
         self.assertHTMLEqual(
             actual,
-            '<span title="Running" class="badge text-bg-secondary">R</span> '
+            '<span title="Running" class="badge text-bg-info">R</span> '
             f'<a href="{children_needs_input.get_absolute_url()}">'
-            '<span title="Needs Input"class="badge text-bg-secondary">I</span>'
+            '<span title="Needs Input"class="badge text-bg-info">I</span>'
             '</a>',
         )
 
@@ -146,7 +146,7 @@ class WorkflowRuntimeStatusSmallTests(TestCase):
 
         self.assertHTMLEqual(
             actual,
-            '<span title="Running" class="badge text-bg-secondary">R</span> '
-            '<span title="Needs Input" class="badge text-bg-secondary">I'
+            '<span title="Running" class="badge text-bg-info">R</span> '
+            '<span title="Needs Input" class="badge text-bg-info">I'
             '</span>',
         )
