@@ -42,8 +42,8 @@ from debusine.db.models import (
     User,
     Workspace,
 )
+from debusine.db.models.tasks import DBTask
 from debusine.db.playground import scenarios
-from debusine.tasks import BaseTask
 from debusine.test.django import TestCase
 from debusine.web.forms import (
     ArtifactForm,
@@ -521,7 +521,7 @@ class WorkRequestFormTests(TestCase):
 
         task_names = [
             (name, name)
-            for name in sorted(BaseTask.worker_task_names())
+            for name in sorted(DBTask.worker_task_names())
             if "internal" not in name
         ]
 
@@ -846,7 +846,7 @@ class WorkspaceChoiceFieldTests(TestCase):
 
         # Blank choice and public workspace
         self.assertEqual(
-            list(workspace_choice_field.choices),
+            list(workspace_choice_field.choices),  # type: ignore[arg-type]
             [
                 ("", workspace_choice_field.empty_label),
                 (workspace.id, workspace.name),
@@ -862,7 +862,7 @@ class WorkspaceChoiceFieldTests(TestCase):
         workspace_choice_field = WorkspaceChoiceField(user=self.user)
 
         self.assertEqual(
-            list(workspace_choice_field.choices),
+            list(workspace_choice_field.choices),  # type: ignore[arg-type]
             [
                 ("", workspace_choice_field.empty_label),
                 (workspace.pk, workspace.name),
@@ -938,7 +938,7 @@ class WorkflowFilterFormTests(TestCase):
         self.assertIn("with_failed_work_requests", self.form.fields)
 
 
-class DaysFieldTest(TestCase):
+class DaysFieldTests(TestCase):
     """Tests for DaysField."""
 
     def test_prepare_value(self) -> None:
