@@ -14,6 +14,7 @@ The majority of template code is tested via test_views.py. Any other specific
 code for templates is tested in this file.
 """
 
+from django.contrib.auth import get_user_model
 from django.core.handlers import exception
 from django.template.loader import get_template
 from django.urls import Resolver404
@@ -27,7 +28,7 @@ def _html_work_request_status(status: str) -> str:
         text_bg = "text-bg-secondary"
         text = "Pending"
     elif status == "running":
-        text_bg = "text-bg-secondary"
+        text_bg = "text-bg-info"
         text = "Running"
     elif status == "completed":
         text_bg = "text-bg-primary"
@@ -153,6 +154,23 @@ class WorkRequestStatusTests(TestCase):
                     rendered_template, _html_work_request_status(status)
                 )
 
+    def test_work_request_status_aborted_with_user(self) -> None:
+        """Aborted status shows aborting user when available."""
+        User = get_user_model()
+        user = User.objects.create_user(username="testuser")
+
+        rendered_template = self.template.render(
+            {
+                "status": "aborted",
+                "aborted_by": user,
+            }
+        )
+
+        self.assertIn(
+            'title="Aborted by testuser"',
+            rendered_template,
+        )
+
     def test_work_request_unexpected(self) -> None:
         """Test rendering function raise assertion."""
         with self.assertRaises(AssertionError):
