@@ -15,7 +15,6 @@ from enum import StrEnum
 from functools import cached_property
 from typing import Any, assert_never
 
-from django.contrib import messages
 from django.core.exceptions import PermissionDenied
 from django.db.models import F, QuerySet
 from django.db.models.functions import Lower
@@ -48,11 +47,7 @@ from debusine.web.views.files import (
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
 from debusine.web.views.mixins import UIShortcutsMixin
 from debusine.web.views.places import Place
-from debusine.web.views.view_utils import format_yaml
-
-
-class IncompleteArtifactError(Exception):
-    """We attempted to read an incomplete artifact."""
+from debusine.web.views.view_utils import highlight_as_yaml
 
 
 class ArtifactView(WorkspaceView, UIShortcutsMixin, abc.ABC):
@@ -150,13 +145,10 @@ class ArtifactDetailView(ArtifactView, DetailViewBase[Artifact]):
 
         if plugin_view is not None:
             # Add specific specialized plugin information
-            try:
-                ctx.update(
-                    **plugin_view(self.artifact).get_context_data(),
-                    **{"specialized_view": True},
-                )
-            except IncompleteArtifactError as e:
-                messages.add_message(self.request, messages.ERROR, str(e))
+            ctx.update(
+                **plugin_view(self.artifact).get_context_data(),
+                specialized_view=True,
+            )
 
         for file_in_artifact in self.file_list:
             self.add_object_ui_shortcuts(
@@ -168,7 +160,7 @@ class ArtifactDetailView(ArtifactView, DetailViewBase[Artifact]):
 
         ctx["artifact_label"] = self.object.get_label()
         ctx["file_list"] = self.file_list
-        ctx["artifact_data"] = format_yaml(self.object.data)
+        ctx["artifact_data"] = highlight_as_yaml(self.object.data)
 
         ctx["forward"] = (
             Artifact.objects.filter(targeted_by__artifact=self.object)
@@ -501,25 +493,6 @@ class ArtifactPlugin(abc.ABC):
         """Return ArtifactPlugin for artifact_category or None."""
         return cls._artifact_plugins.get(artifact_category)
 
-    @staticmethod
-    def read_file(file_in_artifact: FileInArtifact) -> bytes:
-        """Read contents of the file."""
-        artifact = file_in_artifact.artifact
-        scope = artifact.workspace.scope
-        file = file_in_artifact.file
-
-        if not file_in_artifact.complete:
-            raise IncompleteArtifactError(
-                f"File {file_in_artifact.path!r} has not completed uploading."
-            )
-
-        with (
-            scope.download_file_backend(file)
-            .get_entry(file)
-            .get_temporary_local_path() as path
-        ):
-            return path.read_bytes()
-
     @abc.abstractmethod
     def get_context_data(self) -> dict[str, Any]:
         """Return context data for rendering the artifact view."""
