@@ -61,13 +61,11 @@ from debusine.web.views.tables import (
 class AuthenticationBootstrapForm(BootstrapMixin, AuthenticationForm):
     """Django's AuthenticationForm using BootstrapMixin from Debusine."""
 
-    pass
-
 
 class LoginView(BaseUIView, auth_views.LoginView):
     """Class for the login view."""
 
-    base_template = "web/_base_unscoped.html"
+    base_template = "web/_base_debusine.html"
     template_name = "account/login.html"
     form_class = AuthenticationBootstrapForm
     title = "Log in"
@@ -93,7 +91,7 @@ class LogoutView(
 ):
     """Class for the logout view."""
 
-    base_template = "web/_base_unscoped.html"
+    base_template = "web/_base_debusine.html"
     template_name = "account/logged_out.html"
     title = "Logged out"
 
@@ -108,7 +106,7 @@ class UserDetailView(BaseUIView, DetailViewBase[User]):
     """Show user details."""
 
     model = User
-    base_template = "web/_base_unscoped.html"
+    base_template = "web/_base_debusine.html"
     template_name = "web/user-detail.html"
     # We need something that is not "user" not to confuse it with the
     # current user
@@ -164,7 +162,7 @@ class UserDetailView(BaseUIView, DetailViewBase[User]):
 class TokenView(BaseUIView):
     """Base view for token management."""
 
-    base_template = "web/_base_unscoped.html"
+    base_template = "web/_base_debusine.html"
 
     @cached_property
     def user(self) -> User:
@@ -339,7 +337,7 @@ class UserGroupListView(ListViewBase[GroupMembership], BaseUIView):
     """List groups for a user."""
 
     model = GroupMembership
-    base_template = "web/_base_unscoped.html"
+    base_template = "web/_base_debusine.html"
     template_name = "web/user_group-list.html"
     context_object_name = "membership_list"
     ordering = "group__name"
@@ -374,10 +372,20 @@ class UserGroupListView(ListViewBase[GroupMembership], BaseUIView):
         return ctx
 
 
+class BaseGroupUIView(BaseUIView):
+    """Base class for group UI views."""
+
+    def init_view(self) -> None:
+        """Set the current workspace, if given."""
+        super().init_view()
+        if "wname" in self.kwargs:
+            self.set_current_workspace(self.kwargs["wname"])
+
+
 class GroupDetailView(
     FormMixinBase[GroupAddUserForm],
     DetailViewBase[Group],
-    BaseUIView,
+    BaseGroupUIView,
 ):
     """List groups for a user."""
 
@@ -401,6 +409,7 @@ class GroupDetailView(
                 context.require_user()
             ),
             scope=context.require_scope(),
+            workspace=context.workspace,
             name=self.kwargs["group"],
         )
         return cast(Group, group)
@@ -451,7 +460,10 @@ class GroupDetailView(
     # Adapted from BaseDeleteView, since ProcessFormView does not set
     # self.object
     def post(
-        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+        self,
+        request: HttpRequest,  # noqa: ARG002, U100
+        *args: Any,
+        **kwargs: Any,
     ) -> HttpResponse:
         """Handle POST requests."""
         self.object = self.get_object()
@@ -474,7 +486,7 @@ class GroupDetailView(
         return self.object.get_absolute_url()
 
 
-class MembershipView(SingleObjectMixinBase[GroupMembership], BaseUIView):
+class MembershipView(SingleObjectMixinBase[GroupMembership], BaseGroupUIView):
     """Base view for group management for a user."""
 
     def get_object(
@@ -488,6 +500,7 @@ class MembershipView(SingleObjectMixinBase[GroupMembership], BaseUIView):
                 group__in=Group.objects.can_manage(context.require_user()),
                 group__scope=context.require_scope(),
                 group__name=self.kwargs["group"],
+                group__workspace=context.workspace,
                 user__username=self.kwargs["user"],
             ).select_related("group", "user"),
         )
@@ -516,6 +529,10 @@ class MembershipUpdateView(
     template_name = "web/membership-edit.html"
     title = "Update role"
 
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place_update
+
     def form_valid(self, form: ModelFormBase[GroupMembership]) -> HttpResponse:
         """Change the member's role."""
         self.object = self.object.group.set_user_role(
@@ -541,20 +558,11 @@ class MembershipDeleteView(
 
     def get_place(self) -> Place:
         """Return the Place for this page."""
-        return ResourcePlace(
-            title=f"Remove user from {self.object.group}",
-            breadcrumb=self.object.group.name,
-            url=reverse(
-                "groups:remove-member",
-                kwargs={
-                    "group": self.object.group.name,
-                    "user": self.object.user.username,
-                },
-            ),
-        )
+        return self.object.ui(self.request).place_remove
 
     def form_valid(
-        self, form: ModelFormBase[GroupMembership]  # noqa: U100
+        self,
+        form: ModelFormBase[GroupMembership],  # noqa: ARG002, U100
     ) -> HttpResponse:
         """Remove the group member."""
         success_url = self.get_success_url()
@@ -567,7 +575,7 @@ class MembershipDeleteView(
 
 
 class GroupAuditLogView(
-    TableMixin[GroupAuditLog], ListViewBase[GroupAuditLog], BaseUIView
+    TableMixin[GroupAuditLog], ListViewBase[GroupAuditLog], BaseGroupUIView
 ):
     """View for showing the full group audit log."""
 
@@ -590,6 +598,7 @@ class GroupAuditLogView(
                     ).filter(scope=context.require_scope()),
                 ),
                 name=self.kwargs["group"],
+                workspace=context.workspace,
             ),
         )
 
@@ -597,10 +606,12 @@ class GroupAuditLogView(
         """All audit log entries for the current group."""
         return self.group.audit_log.select_related("actor")
 
-    def get_place(self) -> IndexPlace:
+    def get_place(self) -> ResourcePlace:
         """Return the Place for this page."""
-        return IndexPlace(
-            title=f"Audit log for {self.group}",
-            url=reverse("groups:audit-log", kwargs={"group": self.group.name}),
-            parent=self.group.ui(self.request).place,
-        )
+        return self.group.ui(self.request).place_audit_log
+
+    def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
+        """Add bits needed by the template."""
+        ctx = super().get_context_data(**kwargs)
+        ctx["group"] = self.group
+        return ctx
