@@ -8,34 +8,36 @@
 # contained in the LICENSE file.
 
 """View for Autopkgtest work request."""
+
 import itertools
 import re
-from enum import Enum
-from typing import Any, Optional, assert_never
+from enum import StrEnum
+from typing import Any, Optional, TypeAlias, assert_never, override
 
+import pydantic
 from more_itertools import peekable
 
-from debusine.web.views.files import LogFileWidget
-
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic  # type: ignore
-
 from debusine.artifacts import AutopkgtestArtifact
 from debusine.artifacts.models import ArtifactCategory, TaskTypes
-from debusine.db.models import Artifact, FileInArtifact
-from debusine.tasks import Autopkgtest
+from debusine.db.models import Artifact, FileInArtifact, WorkRequest
+from debusine.db.models.tasks import DBTask
+from debusine.tasks.models import AutopkgtestData, AutopkgtestDynamicData
+from debusine.web.views import view_utils
 from debusine.web.views.artifacts import ArtifactPlugin
-from debusine.web.views.work_request import WorkRequestPlugin
+from debusine.web.views.files import LogFileWidget
+from debusine.web.views.work_request import Tab, WorkRequestPlugin
+
+DBAutopkgtest: TypeAlias = DBTask[AutopkgtestData, AutopkgtestDynamicData]
 
 
-class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
-    """View for Autopkgtest work request."""
+class AutopkgtestTab(Tab):
+    """Autopkgtest specialized tab."""
 
-    task_type = TaskTypes.WORKER
-    task_name = "autopkgtest"
-    task: Autopkgtest
+    def __init__(self, work_request: WorkRequest, task: DBAutopkgtest) -> None:
+        """Define this tab."""
+        super().__init__(name="autopkgtest", label="Autopkgtest")
+        self.work_request = work_request
+        self.task = task
 
     @staticmethod
     def _get_fail_on_scenarios(fail_on: dict[str, bool]) -> list[str]:
@@ -117,7 +119,7 @@ class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
             "extra_environment": task_data.extra_environment,
             "needs_internet": task_data.needs_internet,
             "fail_on_scenarios": self._get_fail_on_scenarios(
-                task_data.fail_on.dict()
+                task_data.fail_on.model_dump()
             ),
             "timeout": task_data.timeout,
         }
@@ -192,9 +194,9 @@ class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
             f"{request_data['build_architecture']}"
         )
         if request_data["vendor"] and request_data["codename"]:
-            request_data[
-                "task_description"
-            ] += f" in {request_data['vendor']}:{request_data['codename']}"
+            request_data["task_description"] += (
+                f" in {request_data['vendor']}:{request_data['codename']}"
+            )
 
         return request_data
 
@@ -208,8 +210,8 @@ class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
             result_data.append((name, result["status"], result.get("details")))
         return result_data
 
+    @override
     def get_context_data(self) -> dict[str, Any]:
-        """Return the context."""
         result_artifact = (
             self.work_request.artifact_set.filter(
                 category=AutopkgtestArtifact._category
@@ -219,11 +221,6 @@ class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
         )
 
         context_data: dict[str, Any] = {
-            "specialized_tab": {
-                "label": "Autopkgtest",
-                "slug": "autopkgtest",
-                "template": "web/_autopkgtest-work_request-detail.html",
-            },
             "request_data": self._get_request_data(),
             "result": self.work_request.result,
         }
@@ -234,10 +231,26 @@ class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
         return context_data
 
 
+class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
+    """View for Autopkgtest work request."""
+
+    task_type = TaskTypes.WORKER
+    task_name = "autopkgtest"
+    task: DBAutopkgtest
+
+    @override
+    def add_tabs(self, tabs: list[Tab]) -> None:
+        tabs.insert(0, AutopkgtestTab(self.work_request, self.task))
+
+    @override
+    def get_context_data(self) -> dict[str, Any]:
+        return {}
+
+
 class AutopkgtestLogParser:
     """Parse Autopkgtest logs."""
 
-    class Result(str, Enum):
+    class Result(StrEnum):
         PASS = "PASS"
         FAIL = "FAIL"
         SKIP = "SKIP"
@@ -261,17 +274,17 @@ class AutopkgtestLogParser:
         tooltip: str | None = None
 
     _PREFIX = r"\s*\d+s\s+autopkgtest\s+\[\d\d:\d\d:\d\d\]:"
-    _RE_STARTING = re.compile(fr"{_PREFIX}\s+starting date and time")
-    _RE_APT_SOURCE = re.compile(fr"{_PREFIX}\s+@{{20}}\s+apt-source.*")
-    _RE_TESTBED_SETUP = re.compile(fr"{_PREFIX}\s+@{{20}} test bed setup")
+    _RE_STARTING = re.compile(rf"{_PREFIX}\s+starting date and time")
+    _RE_APT_SOURCE = re.compile(rf"{_PREFIX}\s+@{{20}}\s+apt-source.*")
+    _RE_TESTBED_SETUP = re.compile(rf"{_PREFIX}\s+@{{20}} test bed setup")
     _RE_TEST_PREPARING_TESTBED = re.compile(
-        fr"{_PREFIX}\s+test\s+(.*):\s+preparing testbed"
+        rf"{_PREFIX}\s+test\s+(.*):\s+preparing testbed"
     )
-    _RE_TEST_STARTS = re.compile(fr"{_PREFIX}\s+test\s+(.*): ")
+    _RE_TEST_STARTS = re.compile(rf"{_PREFIX}\s+test\s+(.*): ")
     _RE_TEST_RUN = re.compile(
-        fr"{_PREFIX}\s+test\s+(.*): \[-----------------------"
+        rf"{_PREFIX}\s+test\s+(.*): \[-----------------------"
     )
-    _RE_SUMMARY = re.compile(fr"{_PREFIX}\s+@{{20}}\s+(summary)")
+    _RE_SUMMARY = re.compile(rf"{_PREFIX}\s+@{{20}}\s+(summary)")
     _RE_RESULT_LINE = re.compile(r"^\s*\d+s\s+\S+\s*(PASS|FAIL|SKIP)")
 
     def __init__(self, log: bytes) -> None:
@@ -421,7 +434,16 @@ class AutopkgtestArtifactPlugin(ArtifactPlugin):
                 "autopkgtest_artifact_error": "Artifact does not contain log",
             }
 
-        autopkgtest_contents = self.read_file(log_file_in_artifact)
+        try:
+            autopkgtest_contents = view_utils.read_file(
+                log_file_in_artifact, max_file_size=50 * 1024 * 1024
+            )
+        except view_utils.FileInArtifactError as exc:
+            return {
+                **specialized_tab,
+                "autopkgtest_artifact_error": exc.to_html(),
+            }
+
         sections, total_lines = AutopkgtestLogParser(
             autopkgtest_contents
         ).parse()
@@ -459,6 +481,7 @@ class AutopkgtestArtifactPlugin(ArtifactPlugin):
         log_widget = LogFileWidget(
             log_file_in_artifact,
             sections=file_sections,
+            file_contents=autopkgtest_contents,
         )
 
         return {
