@@ -174,7 +174,9 @@ class BaseUIView(ContextMixin, View):
             ctx["workspace_collections"] = (
                 context.workspace.collections.exclude(
                     category=CollectionCategory.WORKFLOW_INTERNAL
-                ).can_display(context.user)
+                )
+                .can_display(context.user)
+                .order_by("name", "category")
             )
             ctx["workflow_templates"] = list(
                 context.workspace.workflowtemplate_set.can_display(
