@@ -10,27 +10,35 @@
 """debusine collection views."""
 
 from functools import cached_property
-from typing import Any
+from typing import Any, override
 
 from django.db.models import Count, Q, QuerySet
-from django.http import Http404
+from django.http import Http404, HttpResponse, HttpResponseRedirect
 from django.urls import reverse
 
 from debusine.db.context import context
 from debusine.db.models import Collection, CollectionItem
-from debusine.web.forms import CollectionSearchForm
+from debusine.web.forms import (
+    CollectionCreateForm,
+    CollectionSearchForm,
+    CollectionUpdateForm,
+    ModelFormBase,
+)
 from debusine.web.views import sidebar, ui_shortcuts
 from debusine.web.views.base import (
+    CreateViewBase,
+    DeleteViewBase,
     DetailViewBase,
     FormMixinBase,
     ListViewBase,
+    UpdateViewBase,
     WorkspaceView,
 )
 from debusine.web.views.mixins import RightbarUIMixin, UIShortcutsMixin
 from debusine.web.views.places import Place, ResourcePlace
 from debusine.web.views.table import TableMixin
 from debusine.web.views.tables import CollectionItemTable
-from debusine.web.views.view_utils import format_yaml
+from debusine.web.views.view_utils import highlight_as_yaml
 
 
 class CollectionListView(WorkspaceView, ListViewBase[Collection]):
@@ -91,6 +99,27 @@ class CollectionCategoryListView(WorkspaceView, ListViewBase[Collection]):
         return context
 
 
+class CollectionCreateView(
+    WorkspaceView, CreateViewBase[Collection, CollectionCreateForm]
+):
+    """Create a collection."""
+
+    form_class = CollectionCreateForm
+    template_name = "web/collection-create.html"
+
+    @override
+    def init_view(self) -> None:
+        """Enforce permissions."""
+        super().init_view()
+        self.enforce(context.require_workspace().can_create_collections)
+
+    @override
+    def get_place(self) -> Place:
+        return (
+            context.require_workspace().ui(self.request).place_collection_create
+        )
+
+
 class CollectionViewMixin(WorkspaceView, UIShortcutsMixin, RightbarUIMixin):
     """Common functions for collection-specific views."""
 
@@ -98,7 +127,7 @@ class CollectionViewMixin(WorkspaceView, UIShortcutsMixin, RightbarUIMixin):
     def collection(self) -> Collection:
         """Collection for this request."""
         try:
-            return Collection.objects.get(
+            return Collection.objects.can_display(context.user).get(
                 workspace=context.workspace,
                 name=self.kwargs["cname"],
                 category=self.kwargs["ccat"],
@@ -136,7 +165,8 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
         return self.object.ui(self.request).place
 
     def get_object(
-        self, queryset: QuerySet[Collection] | None = None  # noqa: U100
+        self,
+        queryset: QuerySet[Collection] | None = None,  # noqa: ARG002, U100
     ) -> Collection:
         """Return the collection object to show."""
         return self.collection
@@ -145,18 +175,16 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
         """Return context_data with work_request_list and workspace_list."""
         context = super().get_context_data(**kwargs)
         if self.object.data:
-            context["data"] = format_yaml(self.object.data)
+            context["data"] = highlight_as_yaml(self.object.data)
 
         context["collections"] = (
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.object, collection__isnull=False)
+            self.object.child_items.active()
+            .filter(collection__isnull=False)
             .order_by("category", "name")
         )
 
         context["artifacts"] = (
-            CollectionItem.objects.filter(
-                parent_collection=self.object, artifact__isnull=False
-            )
+            self.object.child_items.filter(artifact__isnull=False)
             .values("category")
             .annotate(
                 count=Count("category", filter=Q(removed_at__isnull=True)),
@@ -168,10 +196,8 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
         )
 
         context["bare"] = (
-            CollectionItem.objects.filter(
-                parent_collection=self.object,
-                collection__isnull=True,
-                artifact__isnull=True,
+            self.object.child_items.filter(
+                collection__isnull=True, artifact__isnull=True
             )
             .values("category")
             .annotate(
@@ -186,6 +212,70 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
         return context
 
 
+class CollectionUpdateView(
+    CollectionViewMixin, UpdateViewBase[Collection, CollectionUpdateForm]
+):
+    """Edit configuration for a collection."""
+
+    form_class = CollectionUpdateForm
+    template_name = "web/collection-update.html"
+
+    @override
+    def get_place(self) -> Place:
+        return self.object.ui(self.request).place_update
+
+    def get_object(
+        self,
+        queryset: QuerySet[Collection] | None = None,  # noqa: ARG002, U100
+    ) -> Collection:
+        """Return the collection object to update."""
+        self.enforce(self.collection.can_configure)
+        return self.collection
+
+
+class CollectionDeleteView(
+    CollectionViewMixin, DeleteViewBase[Collection, ModelFormBase[Collection]]
+):
+    """Delete a collection."""
+
+    object: Collection
+    model = Collection
+    template_name = "web/collection-confirm_delete.html"
+
+    @override
+    def get_place(self) -> Place:
+        return self.object.ui(self.request).place_delete
+
+    def get_object(
+        self,
+        queryset: QuerySet[Collection] | None = None,  # noqa: ARG002, U100
+    ) -> Collection:
+        """Return the collection object to delete."""
+        self.enforce(self.collection.can_delete)
+        return self.collection
+
+    @override
+    def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
+        ctx = super().get_context_data(**kwargs)
+        item_stats = list(self.object.get_item_stats())
+        ctx["items_total"] = sum(x["count"] for x in item_stats)
+        ctx["item_stats"] = item_stats
+        return ctx
+
+    @override
+    def form_valid(
+        self,
+        form: ModelFormBase[Collection],  # noqa: U100
+    ) -> HttpResponse:
+        success_url = self.get_success_url()
+        self.object.delete_collection()
+        return HttpResponseRedirect(success_url)
+
+    @override
+    def get_success_url(self) -> str:
+        return context.require_workspace().get_absolute_url()
+
+
 class CollectionSearchView(
     TableMixin[CollectionItem],
     CollectionViewMixin,
@@ -308,5 +398,5 @@ class CollectionItemDetailView(
         """Return context_data."""
         context = super().get_context_data(**kwargs)
         if self.object.data:
-            context["data"] = format_yaml(self.object.data)
+            context["data"] = highlight_as_yaml(self.object.data)
         return context
