@@ -26,6 +26,7 @@ from debusine.artifacts.models import (
 )
 from debusine.db.models import Artifact, ArtifactRelation, FileInArtifact
 from debusine.tasks import DebDiff
+from debusine.web.views import view_utils
 from debusine.web.views.artifacts import ArtifactPlugin
 from debusine.web.views.files import FileWidget
 from debusine.web.views.work_request import WorkRequestPlugin
@@ -298,9 +299,18 @@ class DebDiffViewArtifactPlugin(ArtifactPlugin):
                 f"{DebDiff.CAPTURE_OUTPUT_FILENAME} ",
             }
 
-        debdiff_contents = self.read_file(debdiff_txt_file_in_artifact).decode(
-            errors="replace"
-        )
+        try:
+            debdiff_bytes = view_utils.read_file(
+                debdiff_txt_file_in_artifact, max_file_size=50 * 1024 * 1024
+            )
+        except view_utils.FileInArtifactError as exc:
+            return {
+                **specialized_tab,
+                "debdiff_artifact_error": exc.to_html(),
+            }
+
+        debdiff_text = debdiff_bytes.decode(errors="replace")
+
         debdiff_absolute_path = debdiff_txt_file_in_artifact.get_absolute_url()
 
         related_to = self.artifact.relations.filter(
@@ -319,7 +329,7 @@ class DebDiffViewArtifactPlugin(ArtifactPlugin):
             debdiff_parser_error = None
             try:
                 debdiff_source_summary = self._summarise_debdiff_source(
-                    debdiff_contents
+                    debdiff_text
                 )
             except ValueError as exc:
                 debdiff_source_summary = None
@@ -328,6 +338,7 @@ class DebDiffViewArtifactPlugin(ArtifactPlugin):
             debdiff_source_file_widget = FileWidget.create(
                 debdiff_txt_file_in_artifact,
                 file_tag=slug,
+                file_contents=debdiff_bytes,
             )
 
             return {
@@ -346,9 +357,7 @@ class DebDiffViewArtifactPlugin(ArtifactPlugin):
             # The original and new artifacts were binary artifacts
             debdiff_parser_error = None
             try:
-                debdiff_binary_parsed = self._parse_debdiff_binary(
-                    debdiff_contents
-                )
+                debdiff_binary_parsed = self._parse_debdiff_binary(debdiff_text)
 
             except ValueError as exc:
                 debdiff_binary_parsed = None
