@@ -43,10 +43,7 @@ class DebianPipelineViewWorkRequestPlugin(WorkRequestPlugin):
         except WorkRequest.DoesNotExist:
             return None
 
-        qa_task = QAWorkflow(
-            task_data=qa_workflow.used_task_data,
-            dynamic_task_data=qa_workflow.dynamic_task_data,
-        )
+        qa_task = QAWorkflow(qa_workflow)
 
         tab = QATab(qa_workflow, qa_task)
         if not tab.has_data:
