@@ -9,7 +9,6 @@
 
 """View extension to display contents of files."""
 
-import contextlib
 import io
 import mimetypes
 import mmap
@@ -37,6 +36,7 @@ from rest_framework import status
 from debusine.db.models import FileInArtifact
 from debusine.server.views import ProblemResponse
 from debusine.utils import parse_range_header
+from debusine.web.views import view_utils
 from debusine.web.views.base import Widget
 
 if TYPE_CHECKING:
@@ -176,7 +176,9 @@ class LinenoHtmlFormatter(HtmlFormatter[Any]):
     ) -> Generator[tuple[int, str]]:
         """Format each line with the line number."""
         for formatted, line in source:
-            # f can be 1 (line of formatted code) or 0 (Pygments wrapping)
+            # formatted can be:
+            # - 1: line of formatted code
+            # - 0: Pygments wrapping
             if formatted:
                 self.linenumber_count += 1
                 yield (
@@ -184,7 +186,7 @@ class LinenoHtmlFormatter(HtmlFormatter[Any]):
                     add_line_number(
                         line_number=self.linenumber_count,
                         suffix=self.suffix,
-                        line=SafeString(line),
+                        line=SafeString(line.rstrip("\n")),
                     ),
                 )
             else:
@@ -202,19 +204,27 @@ class FileWidget(Widget):
         *,
         file_ui: FileUI | None = None,
         file_tag: str | None = None,
+        file_contents: bytes | None = None,
     ) -> None:
         """
         Initialize object with a file to display.
 
-        @param file_tag: there might be pages with more
+        :param file_tag: there might be pages with more
           than one FileWidget (even referring to the same file). In order
           to create unique anchors to the lines, it's possible to pass a
           file_tag. The anchors will contain it, currently in the form of:
           "#L{line_number}-{file_tag}"
+        :param file_contents: if set and smaller than MAX_FILE_SIZE it will
+          be used by read_file() method. Otherwise, it will be read on demand
+           by the method read_file().
         """
         self.file_in_artifact = file_in_artifact
         self.file_ui = file_ui or FileUI.from_file_in_artifact(file_in_artifact)
         self.file_tag = file_tag
+        self.file_contents = None
+
+        if file_contents is not None and len(file_contents) <= MAX_FILE_SIZE:
+            self.file_contents = file_contents
 
     @classmethod
     def create(
@@ -223,6 +233,7 @@ class FileWidget(Widget):
         *,
         file_ui: FileUI | None = None,
         file_tag: str | None = None,
+        file_contents: bytes | None = None,
     ) -> "FileWidget":
         """Create a file widget for the given file."""
         file_ui = file_ui or FileUI.from_file_in_artifact(file_in_artifact)
@@ -230,16 +241,9 @@ class FileWidget(Widget):
             file_in_artifact,
             file_ui=file_ui,
             file_tag=file_tag,
+            file_contents=file_contents,
         )
 
-    @contextlib.contextmanager
-    def _open_file(self) -> Generator["SupportsRead[bytes]"]:
-        assert self.file_in_artifact is not None
-        scope = self.file_in_artifact.artifact.workspace.scope
-        file_backend = scope.download_file_backend(self.file_in_artifact.file)
-        with file_backend.get_stream(self.file_in_artifact.file) as stream:
-            yield stream
-
     def suffix(self) -> str:
         """Return suffix for the ids: "-{self.file_tag}" or ""."""
         return f"-{self.file_tag}" if self.file_tag else ""
@@ -251,6 +255,16 @@ class FileWidget(Widget):
             "file_ui": self.file_ui,
         }
 
+    def read_file(self) -> bytes:
+        """Return self.file_contents or read it."""
+        if self.file_contents is not None:
+            return self.file_contents
+
+        self.file_contents = view_utils.read_file(
+            self.file_in_artifact, max_file_size=MAX_FILE_SIZE
+        )
+        return self.file_contents
+
     def render(self, context: Context) -> str:
         """Render the widget."""
         assert context.template is not None
@@ -372,16 +386,14 @@ class LogFileWidget(FileWidget):
         """Add context data for text files."""
         context = super().get_context_data()
 
-        context["file_contents_div_id"] = f"file-contents{self.suffix()}"
+        context["formatted_content_div_id"] = (
+            f"formatted-content{self.suffix()}"
+        )
 
         if not self.file_in_artifact.complete:
             return context
 
-        with self._open_file() as stream:
-            # The maximum file size is MAX_FILE_SIZE. It is ok to
-            # load it in memory
-            raw_bytes = stream.read()
-
+        raw_bytes = self.read_file()
         raw_str = raw_bytes.decode("utf-8", errors="replace")
         file_contents = raw_str.replace("\r", "^M")
         file_contents = file_contents.rstrip("\n")
@@ -406,7 +418,9 @@ class TextFileWidget(FileWidget):
         """Add context data for text files."""
         context = super().get_context_data()
 
-        context["file_contents_div_id"] = f"file-contents{self.suffix()}"
+        context["formatted_content_div_id"] = (
+            f"formatted-content{self.suffix()}"
+        )
 
         if self.file_in_artifact.complete:
             # Generate CSS with:
@@ -439,13 +453,16 @@ class TextFileWidget(FileWidget):
                     lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
 
             formatter = LinenoHtmlFormatter(
-                cssclass="file-contents",
+                cssclass="formatted-content",
                 linenos=False,
                 suffix=self.suffix(),
             )
 
-            with self._open_file() as stream:
-                formatted = pygments.highlight(stream.read(), lexer, formatter)
+            formatted = pygments.highlight(
+                self.read_file(),
+                lexer,
+                formatter,
+            )
 
             context["file_content"] = SafeString(formatted)
             context["file_lineno_width"] = max(
