@@ -12,14 +12,18 @@
 import json
 import uuid
 from collections.abc import Iterable
-from typing import Any, override
+from typing import Any, TypeAlias, override
 
 from debusine.artifacts import LintianArtifact
 from debusine.artifacts.models import BaseArtifactDataModel, TaskTypes
 from debusine.db.models import Artifact, FileInArtifact, WorkRequest
+from debusine.db.models.tasks import DBTask
 from debusine.tasks import Lintian
+from debusine.tasks.models import LintianData, LintianDynamicData
 from debusine.web.views.work_request import Tab, WorkRequestPlugin
 
+DBLintian: TypeAlias = DBTask[LintianData, LintianDynamicData]
+
 
 class LintianAnalysisOccurrence(BaseArtifactDataModel):
     """A location where a Lintian tag occurred."""
@@ -41,7 +45,7 @@ class LintianAnalysisTag(BaseArtifactDataModel):
 class LintianTab(Tab):
     """Lintian specialized tab."""
 
-    def __init__(self, work_request: WorkRequest, task: Lintian) -> None:
+    def __init__(self, work_request: WorkRequest, task: DBLintian) -> None:
         """Define this tab."""
         super().__init__(name="lintian", label="Lintian")
         self.work_request = work_request
@@ -210,7 +214,7 @@ class LintianTab(Tab):
             return []
 
     @classmethod
-    def _get_request_data(cls, task: Lintian) -> dict[str, Any]:
+    def _get_request_data(cls, task: DBLintian) -> dict[str, Any]:
         """Prepare the requested data to be displayed."""
         dynamic_data = task.dynamic_data
 
@@ -264,7 +268,6 @@ class LintianTab(Tab):
     @override
     def get_context_data(self) -> dict[str, Any]:
         task = self.task
-        assert isinstance(task, Lintian)
 
         artifacts = self.work_request.artifact_set.filter(
             category=LintianArtifact._category
@@ -287,7 +290,7 @@ class LintianViewWorkRequestPlugin(WorkRequestPlugin):
 
     task_type = TaskTypes.WORKER
     task_name = "lintian"
-    task: Lintian
+    task: DBLintian
 
     @override
     def add_tabs(self, tabs: list[Tab]) -> None:
