@@ -209,8 +209,10 @@ def create_work_request_status(work_request: WorkRequest) -> SidebarItem:
     }
 
     match work_request.status:
-        case WorkRequest.Statuses.PENDING | WorkRequest.Statuses.RUNNING:
+        case WorkRequest.Statuses.PENDING:
             status_bg = "secondary"
+        case WorkRequest.Statuses.RUNNING:
+            status_bg = "info"
         case WorkRequest.Statuses.COMPLETED:
             status_bg = "primary"
         case WorkRequest.Statuses.ABORTED | WorkRequest.Statuses.BLOCKED:
