@@ -9,9 +9,11 @@
 
 """debusine list view paginators."""
 
-from typing import Any, TypeVar
+from collections import defaultdict
+from typing import Any
 
 from django.db.models import Model, Q, QuerySet
+from django_cte import With
 
 from debusine.artifacts.models import TaskTypes
 from debusine.db.models import (
@@ -34,12 +36,11 @@ from debusine.web.views.table import (
     FilterToggle,
     NumberColumn,
     Ordering,
+    Paginator,
     StringColumn,
     Table,
 )
 
-M = TypeVar("M", bound=Model)
-
 
 class GroupMembershipTable(Table[GroupMembership]):
     """Table set up for GroupMembership."""
@@ -60,6 +61,7 @@ class GroupMembershipTable(Table[GroupMembership]):
 
     template_name = "web/_group_membership-list.html"
     default_order = "role"
+    related_fields = ("user",)
 
 
 class GroupMembershipAdminTable(GroupMembershipTable):
@@ -175,12 +177,16 @@ class WorkRequestTable(Table[WorkRequest]):
 
     template_name = "web/_work_request-list.html"
     default_order = "-created_at"
+    related_fields = (
+        "workspace__scope",
+        "aborted_by",
+    )
 
 
 class FilterStatuses(FilterField):
     """Custom queryset filtering for workflow statuses."""
 
-    def filter_queryset(
+    def filter_queryset[M: Model](
         self, queryset: QuerySet[M, M], value: Any
     ) -> QuerySet[M, M]:
         """Filter a queryset."""
@@ -206,7 +212,7 @@ class FilterFailedWorkRequests(FilterToggle):
         """Pass an empty Q, since we ignore it for filtering."""
         super().__init__(label, q=Q())
 
-    def filter_queryset(
+    def filter_queryset[M: Model](
         self, queryset: QuerySet[M, M], value: Any
     ) -> QuerySet[M, M]:
         """Filter the queryset."""
@@ -219,15 +225,59 @@ class FilterFailedWorkRequests(FilterToggle):
         # this can only be used to filter top-level workflows.
         assert not queryset.filter(parent__isnull=False).exists()
 
-        return queryset.filter(
-            id__in=(
-                WorkRequest.objects.filter(workflow_root_id__in=queryset)
-                .exclude(task_type=TaskTypes.WORKFLOW)
-                .visible_in_workflow()
-                .filter(result=WorkRequest.Results.FAILURE)
-                .values("workflow_root_id")
+        workflows = With(queryset, name="workflows")
+        filtered = (
+            workflows.join(WorkRequest, id=workflows.col.id)
+            .with_cte(workflows)
+            .filter(
+                id__in=(
+                    workflows.join(
+                        WorkRequest, workflow_root_id=workflows.col.id
+                    )
+                    .exclude(task_type=TaskTypes.WORKFLOW)
+                    .visible_in_workflow()
+                    .filter(result=WorkRequest.Results.FAILURE)
+                    .values("workflow_root_id")
+                )
             )
         )
+        assert isinstance(filtered, QuerySet)
+        return filtered
+
+
+class WorkflowPaginator(Paginator[WorkRequest]):
+    """Paginator for workflow WorkRequests."""
+
+    def get_context_data(self) -> dict[str, Any]:
+        """Return context data for this paginator."""
+        ctx = super().get_context_data()
+
+        object_list = self.page_obj.object_list
+        assert isinstance(object_list, WorkRequestQuerySet)
+        workflows_flattened: dict[int, list[WorkRequest]] = defaultdict(list)
+        work_request_status_counts: dict[
+            int, dict[WorkRequest.Statuses, int]
+        ] = defaultdict(lambda: defaultdict(int))
+        work_request_result_counts: dict[
+            int, dict[WorkRequest.Results, int]
+        ] = defaultdict(lambda: defaultdict(int))
+        if object_list.exists():
+            for work_request in object_list.workflow_flattened():
+                workflows_flattened[work_request.ancestor_id].append(
+                    work_request
+                )
+                work_request_status_counts[work_request.ancestor_id][
+                    work_request.status
+                ] += 1
+                if work_request.result:
+                    work_request_result_counts[work_request.ancestor_id][
+                        work_request.result
+                    ] += 1
+        ctx["workflows_flattened"] = workflows_flattened
+        ctx["work_request_status_counts"] = work_request_status_counts
+        ctx["work_request_result_counts"] = work_request_result_counts
+
+        return ctx
 
 
 class WorkflowTable(Table[WorkRequest]):
@@ -260,6 +310,7 @@ class WorkflowTable(Table[WorkRequest]):
     template_name = "web/_workflow-list.html"
     default_order = "id.desc"
     filter_form_class = WorkflowFilterForm
+    related_fields = ("workspace__scope", "created_by")
 
     def init_filters(self) -> None:
         """Add filter options based on database contents."""
@@ -284,6 +335,21 @@ class WorkflowTable(Table[WorkRequest]):
                 user.username, user.get_full_name(), query=Q(created_by=user)
             )
 
+    def get_paginator(
+        self,
+        per_page: int,
+        *,
+        orphans: int = 0,
+        allow_empty_first_page: bool = True,
+    ) -> WorkflowPaginator:
+        """Return the Paginator for this table."""
+        return WorkflowPaginator(
+            self,
+            per_page=per_page,
+            orphans=orphans,
+            allow_empty_first_page=allow_empty_first_page,
+        )
+
 
 class CollectionItemTable(Table[CollectionItem]):
     """Table set up for CollectionItem."""
