@@ -986,7 +986,7 @@ class AutopkgtestArtifactPluginTests(TestCase):
             "Cannot display autopkgtest: Artifact does not contain log",
         )
 
-    def test_autopkgtest_with_incomplete_log(self) -> None:
+    def test_autopkgtest_with_file_in_artifact_error(self) -> None:
         data = DebianAutopkgtest(
             results={},
             cmdline="autopkgtest",
@@ -1008,11 +1008,13 @@ class AutopkgtestArtifactPluginTests(TestCase):
         response = self.client.get(artifact.get_absolute_url())
         tree = self.assertResponseHTML(response)
 
-        error_toast = self.assertHasElement(
+        error = self.assertHasElement(
             tree,
-            "//div[@id='user-message-container']//div[@class='toast-body']",
+            "//div[@id='error-artifact']",
         )
 
         self.assertTextContentEqual(
-            error_toast, "File 'log' has not completed uploading."
+            error,
+            'Cannot display autopkgtest: '
+            'File "log" has not completed uploading.',
         )
