@@ -144,6 +144,10 @@ class TestBaseUI(ViewTestMixin, TestCase):
             name="testing", category=CollectionCategory.WORKFLOW_INTERNAL
         )
 
+        self.playground.create_collection(
+            name="bookworm", category=CollectionCategory.SUITE
+        )
+
         view = BaseUIView()
         view.request = RequestFactory().get("/")
         ctx = view.get_context_data()
@@ -156,9 +160,8 @@ class TestBaseUI(ViewTestMixin, TestCase):
             list(
                 self.scenario.workspace.collections.exclude(
                     category=CollectionCategory.WORKFLOW_INTERNAL
-                )
+                ).order_by("name", "category")
             ),
-            ordered=False,
         )
 
     def test_get_context_data_with_version(self) -> None:
