@@ -743,6 +743,29 @@ class DebDiffViewArtifactPluginTests(ViewTestMixin, TestCase):
         # Assert text report no differences
         self.assertFalse(tree.xpath("//p[@id='debdiff-report-no-differences']"))
 
+    def test_get_context_data_file_in_artifact_error(self) -> None:
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.DEBDIFF,
+            paths={DebDiff.CAPTURE_OUTPUT_FILENAME: b""},
+            create_files=True,
+            data={"original": "hello-1.dsc", "new": "hello-2.dsc"},
+            skip_add_files_in_store=True,
+        )
+
+        response = self.client.get(artifact.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+
+        error = self.assertHasElement(
+            tree,
+            "//div[@id='error-artifact']",
+        )
+
+        self.assertTextContentEqual(
+            error,
+            'Cannot display debdiff: '
+            'File "debdiff.txt" has not completed uploading.',
+        )
+
 
 class DebDiffViewWorkRequestPluginTests(TestCase):
     """Tests for DebDiffViewWorkRequestPlugin."""
