@@ -7,7 +7,7 @@
 # modified, propagated, or distributed except according to the terms
 # contained in the LICENSE file.
 
-"""Tests for the artifact views."""
+"""Tests for the file widgets."""
 
 import json
 import os.path
@@ -589,7 +589,7 @@ class FileWidgetTests(ViewTestMixin, TestCase):
                         "file_in_artifact": file,
                         "file_lineno_width": 2,
                         "file_ui": FileUI.from_file_in_artifact(file),
-                        "file_contents_div_id": "file-contents",
+                        "formatted_content_div_id": "formatted-content",
                     },
                 )
 
@@ -607,7 +607,7 @@ class FileWidgetTests(ViewTestMixin, TestCase):
             {
                 "file_in_artifact": file,
                 "file_ui": FileUI.from_file_in_artifact(file),
-                "file_contents_div_id": "file-contents",
+                "formatted_content_div_id": "formatted-content",
             },
         )
 
@@ -631,7 +631,7 @@ class FileWidgetTests(ViewTestMixin, TestCase):
             context["file_content"],
             textwrap.dedent(
                 """\
-                <div class="file-contents">
+                <div class="formatted-content">
                 <a id="L1" href="#L1">1</a><i># Title\n</i>
                 <a id="L2" href="#L2">2</a><i>Text\n</i>
                 </div>\n
@@ -664,6 +664,34 @@ class FileWidgetTests(ViewTestMixin, TestCase):
             },
         )
 
+    def test_read_file_from_fileinartifact(self) -> None:
+        file = self.binary
+        widget = FileWidget.create(file)
+
+        self.assertEqual(widget.read_file(), self.contents["file.bin"])
+
+        # File contents is cached
+        self.assertEqual(widget.file_contents, self.contents["file.bin"])
+
+    def test_read_file_from_contents(self) -> None:
+        file = self.binary
+        widget = FileWidget.create(file, file_contents=b"already-read-contents")
+
+        # File contents is used instead of reading self.binary
+        self.assertEqual(widget.read_file(), b"already-read-contents")
+
+    def test_initial_contents_discarded(self) -> None:
+        file = self.binary
+        big_contents = b"x" * (MAX_FILE_SIZE + 1)
+
+        widget = FileWidget.create(file, file_contents=big_contents)
+
+        # Contents is too big: discarded
+        self.assertIsNone(widget.file_contents)
+
+        # Contents was discarded: read_file() tries to read and succeeds
+        self.assertEqual(widget.read_file(), self.contents["file.bin"])
+
 
 class LogFileWidgetTests(ViewTestMixin, TestCase):
     """Tests for LogFileWidget."""
@@ -672,7 +700,7 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
     file: ClassVar[FileInArtifact]
 
     before_contents = (
-        '<div class="file-contents"><pre class="m-0 p-0"><span></span>'
+        '<div class="formatted-content"><pre class="m-0 p-0"><span></span>'
     )
     after_contents = '</pre></div>'
 
@@ -756,11 +784,13 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
 
         self.assertIsInstance(context["file_ui"], FileUI)
         self.assertEqual(context["file_in_artifact"], self.file)
-        self.assertEqual(context["file_contents_div_id"], "file-contents")
+        self.assertEqual(
+            context["formatted_content_div_id"], "formatted-content"
+        )
         self.assertCountEqual(
             context.keys(),
             {
-                "file_contents_div_id",
+                "formatted_content_div_id",
                 "file_in_artifact",
                 "file_lineno_width",
                 "file_ui",
@@ -847,11 +877,13 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
         self.assertEqual(context["suffix"], "-tag")
         self.assertIsInstance(context["file_ui"], FileUI)
         self.assertEqual(context["file_in_artifact"], self.file)
-        self.assertEqual(context["file_contents_div_id"], "file-contents-tag")
+        self.assertEqual(
+            context["formatted_content_div_id"], "formatted-content-tag"
+        )
         self.assertCountEqual(
             context.keys(),
             {
-                "file_contents_div_id",
+                "formatted_content_div_id",
                 "file_in_artifact",
                 "file_lineno_width",
                 "file_ui",
@@ -875,7 +907,54 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
             )
         )
 
-        self.assertIn("first line", rendered)
+        log_contents = (
+            self.contents["log"].decode().replace("\r", "^M").split("\n")
+        )
+
+        tree = self.assertHTMLValid(rendered)
+
+        file_contents = self.assertHasElement(
+            tree, '//div[@id="formatted-content"]'
+        )
+
+        card_body = self.assertHasElement(
+            file_contents, './/div[contains(@class, "card-body")]'
+        )
+
+        # Assert two untitled sections (lines that are not collapsible)
+        first_untitled_section = self.assertHasElement(
+            card_body, './pre[@class="formatted-content"][1]'
+        )
+        self.assertEqual(len(first_untitled_section.xpath("./i")), 1)
+        self.assertEqual(
+            first_untitled_section.xpath("./i")[0].text, log_contents[0]
+        )
+
+        second_untitled_section = self.assertHasElement(
+            card_body, './pre[@class="formatted-content"][2]'
+        )
+        self.assertEqual(len(second_untitled_section.xpath("./i")), 1)
+        self.assertEqual(
+            second_untitled_section.xpath("./i")[0].text, log_contents[3]
+        )
+
+        # Assert lines in the section "A title"
+        details = self.assertHasElement(card_body, "./details")
+        summary = self.assertHasElement(details, "./summary")
+        label = self.assertHasElement(
+            summary, './/span[@class="log-section-label"]'
+        )
+        self.assertTextContentEqual(label, "A title")
+
+        section_pre = self.assertHasElement(
+            details, './pre[@class="formatted-content"]'
+        )
+
+        section_lines = section_pre.xpath("./i")
+        self.assertEqual(len(section_lines), 2)
+
+        self.assertEqual(section_lines[0].text, log_contents[1])
+        self.assertEqual(section_lines[1].text, log_contents[2])
 
     def test_incomplete(self) -> None:
         self.file.complete = False
@@ -885,7 +964,7 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
         self.assertEqual(
             context,
             {
-                "file_contents_div_id": "file-contents",
+                "formatted_content_div_id": "formatted-content",
                 "file_ui": FileUI.from_file_in_artifact(self.file),
                 "file_in_artifact": self.file,
             },
@@ -922,22 +1001,27 @@ class LinenoHtmlFormatterTests(TestCase):
 
         with mock.patch(
             "debusine.web.views.files.add_line_number"
-        ) as add_line_numbers:
+        ) as add_line_number_mock:
             self.assertEqual(
                 list(formatter.wrap([(0, "<pre>")])), [(0, "<pre>")]
             )
 
-        add_line_numbers.assert_not_called()
+        add_line_number_mock.assert_not_called()
 
         with mock.patch(
             "debusine.web.views.files.add_line_number", return_value="CHANGED"
-        ) as add_line_numbers:
+        ) as add_line_number_mock:
             self.assertEqual(
-                list(formatter.wrap([(1, "foo")])), [(1, "CHANGED")]
+                list(formatter.wrap([(1, "foo"), (1, "bar\n")])),
+                [(1, "CHANGED"), (1, "CHANGED")],
             )
 
-        add_line_numbers.assert_called_once_with(
-            line_number=1, suffix="-suffix", line="foo"
+        self.assertEqual(
+            add_line_number_mock.mock_calls,
+            [
+                mock.call(line_number=1, suffix="-suffix", line="foo"),
+                mock.call(line_number=2, suffix="-suffix", line="bar"),
+            ],
         )
 
 
