@@ -14,6 +14,7 @@ from unittest import mock
 
 from django.conf import settings
 from django.contrib import messages
+from django.contrib.messages.storage.base import Message
 from django.urls import reverse
 from rest_framework import status
 
@@ -463,9 +464,7 @@ class ToastViewTests(ViewTestMixin, TestCase):
 
     title = "Test showing django messages"
 
-    def get(
-        self, type_arg: str | None = None
-    ) -> list[messages.storage.base.Message]:
+    def get(self, type_arg: str | None = None) -> list[Message]:
         data: dict[str, Any] = {}
         if type_arg is not None:
             data["type"] = type_arg
