@@ -19,6 +19,7 @@ import lxml.objectify
 from django.contrib import messages
 from django.db.models import Max
 from django.template.response import SimpleTemplateResponse
+from django.test import override_settings
 from django.urls import reverse
 from rest_framework import status
 
@@ -75,9 +76,48 @@ class WorkspaceDetailTests(ViewTestMixin, TestCase):
         details = self.assertHasElement(
             tree, "//table[@id='workspace-details']"
         )
-        self.assertHasElement(details, "//tr[@id='workspace-details-public']")
+        self.assertHasElement(details, ".//tr[@id='workspace-details-public']")
         self.assertHasElement(
-            details, "//tr[@id='workspace-details-default-expiration']"
+            details, ".//tr[@id='workspace-details-default-expiration']"
+        )
+
+    @override_settings(
+        DEBUSINE_DEBIAN_ARCHIVE_URL_SCHEME="https",
+        DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN="deb.example.org",
+    )
+    def test_debian_archive_url(self) -> None:
+        Collection.objects.create(
+            workspace=self.workspace_public,
+            category=CollectionCategory.ARCHIVE,
+            name="_",
+        )
+        response = self.client.get(self.workspace_public.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+        a = self.assertHasElement(
+            tree,
+            "//table[@id='workspace-details']"
+            "//tr[@id='workspace-debian-archive']/td/a",
+        )
+        url = (
+            f"https://deb.example.org/{self.scenario.scope.name}/"
+            f"{self.workspace_public.name}/"
+        )
+        self.assertEqual(a.get("href"), url)
+        self.assertTextContentEqual(a, url)
+
+    @override_settings(DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN="deb.example.org")
+    def test_debian_archive_url_archive_not_present(self) -> None:
+        Collection.objects.filter(
+            workspace=self.workspace_public,
+            category=CollectionCategory.ARCHIVE,
+        ).delete()
+        response = self.client.get(self.workspace_public.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+        self.assertFalse(
+            tree.xpath(
+                "//table[@id='workspace-details']"
+                "//tr[@id='workspace-debian-archive']"
+            )
         )
 
     def test_permissions(self) -> None:
@@ -584,7 +624,7 @@ class TaskConfigurationInspectorTests(ViewTestMixin, TestCase):
             configuration_items.div[0].div[0], item.name()
         )
         self.assertYAMLContentEqual(
-            configuration_items.div[0].div[1], item.dict()
+            configuration_items.div[0].div[1], item.model_dump()
         )
 
     def test_form_invalid(self) -> None:
