@@ -83,7 +83,10 @@ class WorkRequestUI(UI[WorkRequest]):
             .filter(parent__isnull=False)
             .visible_in_workflow()
             .order_by("id")
-            .select_related("workspace__scope")
+            .select_related(
+                "workspace__scope",
+                "aborted_by",
+            )
         ):
             assert wr.parent_id is not None
             children[wr.parent_id].append(wr)
