@@ -8,11 +8,13 @@
 # contained in the LICENSE file.
 
 """debusine WorkRequest view."""
+
 import abc
 import datetime as dt
 import functools
 import itertools
-from typing import Any, cast, override
+from dataclasses import dataclass
+from typing import Any, TYPE_CHECKING, cast, override
 
 from django.contrib import messages
 from django.contrib.auth import get_user_model
@@ -28,7 +30,7 @@ from rest_framework import status
 from debusine.artifacts.models import TaskTypes
 from debusine.db.context import context
 from debusine.db.models import Artifact, TaskDatabase, User, WorkRequest
-from debusine.db.models.artifacts import ArtifactQuerySet
+from debusine.db.models.tasks import DBTask
 from debusine.db.models.work_requests import (
     CannotAbort,
     CannotRetry,
@@ -37,9 +39,14 @@ from debusine.db.models.work_requests import (
     WorkRequestQuerySet,
 )
 from debusine.server.tasks.wait import Confirm
+from debusine.server.workflows import Workflow
 from debusine.server.workflows.models import WorkRequestManualUnblockAction
-from debusine.tasks import BaseTask, TaskConfigError
-from debusine.tasks.models import OutputData
+from debusine.tasks import TaskConfigError
+from debusine.tasks.models import (
+    LookupMultiple,
+    LookupSingle,
+    OutputData,
+)
 from debusine.web.forms import (
     WorkRequestConfirmForm,
     WorkRequestForm,
@@ -58,9 +65,21 @@ from debusine.web.views.base import (
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
 from debusine.web.views.mixins import RightbarUIMixin, UIShortcutsMixin
 from debusine.web.views.places import Place
-from debusine.web.views.table import TableMixin
+from debusine.web.views.table import Paginator, TableMixin
 from debusine.web.views.tables import ArtifactTable, WorkRequestTable
-from debusine.web.views.view_utils import format_json, format_yaml
+from debusine.web.views.view_utils import (
+    highlight_as_json,
+    highlight_as_yaml,
+    syntax_highlight,
+)
+
+if TYPE_CHECKING:
+    from debusine.db.models.artifacts import (
+        ArtifactQuerySet,
+        ArtifactWithComplete,
+    )
+
+    ArtifactWithComplete  # fake usage for vulture
 
 
 class Tab(Widget):
@@ -110,6 +129,15 @@ class WorkRequestObjectMixin(WorkspaceView):
         return qs.in_current_workspace().can_display(context.user)
 
 
+@dataclass(frozen=True)
+class InputArtifactsRow:
+    """Represents a InputArtifact row for the template."""
+
+    input_label: str
+    lookup_display: LookupSingle | LookupMultiple
+    artifacts: list[Artifact]
+
+
 class WorkRequestDetailView(
     RightbarUIMixin,
     WorkRequestObjectMixin,
@@ -203,7 +231,7 @@ class WorkRequestDetailView(
     def get_default_description(self) -> str | None:
         """Return default description for the Task."""
         try:
-            Task = BaseTask.class_from_name(
+            Task = DBTask.class_from_name(
                 TaskTypes(self.object.task_type), self.object.task_name
             )
         except ValueError:
@@ -230,6 +258,119 @@ class WorkRequestDetailView(
             self.plugin.add_tabs(tabs)
         return tabs
 
+    @staticmethod
+    def _get_workflow_input_artifacts(
+        task: Workflow[Any, Any],
+    ) -> tuple[list[InputArtifactsRow], list["ArtifactWithComplete"]]:
+        """
+        Build context data describing the input artifacts of a workflow.
+
+        Return tuple:
+        * list of InputArtifactsRows (containing information for rendering)
+        * list of all referenced Artifact objects
+        """
+        input_artifacts = task.get_input_artifacts()
+
+        # Artifact ids for each InputArtifact
+        input_artifacts_artifact_ids: list[list[int]] = []
+
+        all_artifact_ids: set[int] = set()
+
+        # Collect all the artifact_ids
+        for input_artifact in input_artifacts:
+            artifact_ids = input_artifact.resolved_ids()
+            input_artifacts_artifact_ids.append(artifact_ids)
+            all_artifact_ids.update(artifact_ids)
+
+        # Query all the artifacts
+        artifacts_qs = (
+            Artifact.objects.filter(pk__in=all_artifact_ids)
+            .order_by("pk")
+            .annotate_complete()
+        )
+
+        all_artifacts = list(artifacts_qs)
+        artifacts_by_id: dict[int, Artifact] = {a.id: a for a in all_artifacts}
+
+        input_artifact_rows: list[InputArtifactsRow] = []
+
+        for input_artifact, artifact_ids in zip(
+            input_artifacts, input_artifacts_artifact_ids, strict=True
+        ):
+            artifacts_for_row = [
+                artifacts_by_id[artifact_id]
+                for artifact_id in sorted(set(artifact_ids))
+                if artifact_id in artifacts_by_id
+            ]
+
+            match input_artifact.lookup:
+                case str() | int():
+                    lookup_display = syntax_highlight(
+                        "text", str(input_artifact.lookup)
+                    )
+                case _:
+                    lookup_display = highlight_as_yaml(
+                        input_artifact.lookup.export()
+                    )
+
+            input_artifact_rows.append(
+                InputArtifactsRow(
+                    input_label=input_artifact.label,
+                    lookup_display=lookup_display,
+                    artifacts=artifacts_for_row,
+                )
+            )
+
+        return input_artifact_rows, all_artifacts
+
+    def _get_task_input_artifacts(
+        self, task: DBTask[Any, Any] | None
+    ) -> tuple[Paginator[Artifact], list["ArtifactWithComplete"]]:
+        """
+        Build context data describing the input artifacts of a task.
+
+        Return a tuple containing:
+        * paginator over the task input artifacts (empty if the task is None).
+        * list of all referenced Artifact objects
+        """
+        input_artifacts: "ArtifactQuerySet[ArtifactWithComplete]" = (
+            Artifact.objects.none().annotate_complete()
+        )
+        all_artifacts: list["ArtifactWithComplete"] = []
+
+        if task is not None and (
+            input_artifact_ids := task.get_input_artifacts_ids()
+        ):
+            input_artifacts = (
+                Artifact.objects.filter(pk__in=input_artifact_ids)
+                .order_by("pk")
+                .annotate_complete()
+            )
+            all_artifacts.extend(input_artifacts)
+
+        input_artifacts_table = ArtifactTable(
+            self.request,
+            input_artifacts,
+            prefix="input_artifacts",
+            fragment="work_request",
+        )
+        input_artifacts_table_paginator = input_artifacts_table.get_paginator(
+            per_page=20
+        )
+
+        return input_artifacts_table_paginator, all_artifacts
+
+    @staticmethod
+    def _get_workflow_parents(work_request: WorkRequest) -> list[WorkRequest]:
+        workflow_parents = []
+
+        parent = work_request.parent
+        while parent is not None:
+            workflow_parents.append(parent)
+            parent = parent.parent
+
+        return list(reversed(workflow_parents))
+
     def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
         """Return context for this view's tab."""
         ctx = super().get_context_data(**kwargs)
@@ -255,6 +396,9 @@ class WorkRequestDetailView(
 
         try:
             task = self.object.get_task()
+
+            if self.object.is_workflow:
+                assert isinstance(task, Workflow)
         except (TaskConfigError, InternalTaskError):
             task = None
 
@@ -264,22 +408,14 @@ class WorkRequestDetailView(
             ctx["confirm_label"] = task.data.confirm_label
             ctx["deny_label"] = task.data.deny_label
 
-        input_artifacts: ArtifactQuerySet[Any] = Artifact.objects.none()
-        if task and (input_artifact_ids := task.get_input_artifacts_ids()):
-            input_artifacts = Artifact.objects.filter(
-                pk__in=input_artifact_ids
-            ).annotate_complete()
-
-        input_artifacts_table = ArtifactTable(
-            self.request,
-            input_artifacts,
-            prefix="input_artifacts",
-            fragment="work_request",
-        )
-        input_artifacts_table_paginator = input_artifacts_table.get_paginator(
-            per_page=20
-        )
-        ctx["input_artifacts_table"] = input_artifacts_table_paginator
+        if isinstance(task, Workflow):
+            ctx["input_artifacts_list"], input_artifacts = (
+                self._get_workflow_input_artifacts(task)
+            )
+        else:
+            ctx["input_artifacts_table"], input_artifacts = (
+                self._get_task_input_artifacts(task)
+            )
 
         built_artifacts_table = ArtifactTable(
             self.request,
@@ -296,7 +432,7 @@ class WorkRequestDetailView(
 
         # Generate UI shortcuts for artifacts
         for artifact in itertools.chain(
-            input_artifacts_table_paginator.page_obj.object_list,
+            input_artifacts,
             built_artifacts_table_paginator.page_obj.object_list,
         ):
             self.add_object_ui_shortcuts(
@@ -305,11 +441,11 @@ class WorkRequestDetailView(
                 ui_shortcuts.create_artifact_download(artifact),
             )
 
-        ctx["task_data"] = format_yaml(self.object.task_data)
+        ctx["task_data"] = highlight_as_yaml(self.object.task_data)
 
         if (dynamic_task_data := self.object.dynamic_task_data) is not None:
             ctx["show_task_data_dynamic"] = True
-            ctx["task_data_dynamic"] = format_yaml(dynamic_task_data)
+            ctx["task_data_dynamic"] = highlight_as_yaml(dynamic_task_data)
         else:
             ctx["show_task_data_dynamic"] = False
 
@@ -317,20 +453,28 @@ class WorkRequestDetailView(
             configured_task_data := self.object.configured_task_data
         ) is not None:
             ctx["show_task_data_configured"] = True
-            ctx["task_data_configured"] = format_yaml(configured_task_data)
+            ctx["task_data_configured"] = highlight_as_yaml(
+                configured_task_data
+            )
+            ctx["scheduler_tags"] = highlight_as_yaml(
+                {
+                    "provided": sorted(self.object.scheduler_tags_provided),
+                    "required": sorted(self.object.scheduler_tags_required),
+                }
+            )
         else:
             ctx["show_task_data_configured"] = False
 
         ctx["work_request_output_data"] = self._format_output_data(
             self.object.output_data
         )
-        ctx["work_request_event_reactions_json"] = format_json(
+        ctx["work_request_event_reactions_json"] = highlight_as_json(
             self.object.event_reactions_json
         )
-        ctx["work_request_output_data_json"] = format_json(
+        ctx["work_request_output_data_json"] = highlight_as_json(
             self.object.output_data_json
         )
-        ctx["work_request_workflow_data_json"] = format_json(
+        ctx["work_request_workflow_data_json"] = highlight_as_json(
             self.object.workflow_data_json
         )
 
@@ -343,13 +487,7 @@ class WorkRequestDetailView(
             prefix="reverse_dependencies",
         ).get_paginator(per_page=10)
 
-        workflow_parents = []
-        parent = self.object.parent
-        while parent is not None:
-            workflow_parents.append(parent)
-            parent = parent.parent
-
-        ctx["workflow_parents"] = reversed(workflow_parents)
+        ctx["workflow_parents"] = self._get_workflow_parents(self.object)
 
         # Let the plugin override parts of the context, if needed
         if self.plugin:
@@ -420,7 +558,7 @@ class WorkRequestPlugin(abc.ABC):
     model = WorkRequest
     task_type: str
     task_name: str
-    task: BaseTask[Any, Any]
+    task: DBTask[Any, Any]
     template_name: str
 
     #: Maps (task_type, task_name) tuples to their associated plugin class.
@@ -434,14 +572,7 @@ class WorkRequestPlugin(abc.ABC):
     def __init__(self, work_request: WorkRequest) -> None:
         """Store the work_request to display."""
         self.work_request = work_request
-
-        task_cls = BaseTask.class_from_name(
-            TaskTypes(self.task_type), self.task_name
-        )
-        self.task = task_cls(
-            task_data=self.work_request.used_task_data,
-            dynamic_task_data=self.work_request.dynamic_task_data,
-        )
+        self.task = self.work_request.get_task()
 
     @classmethod
     def plugin_for(cls, instance: WorkRequest) -> "WorkRequestPlugin | None":
@@ -462,7 +593,6 @@ class WorkRequestPlugin(abc.ABC):
 
     def add_tabs(self, tabs: list[Tab]) -> None:
         """Add extra tabs to the tab list."""
-        pass
 
     def get_description_data(self) -> dict[str, Any]:
         """Return data for the description or {} if dynamic_data is None."""
@@ -506,7 +636,7 @@ class WorkRequestListView(
             task_data__build_architecture__isnull=True
         )
 
-        # TODO when WorkRequests have tags: have a build_architecture
+        # TODO: When WorkRequests have tags: have a build_architecture
         # tag and filter in the database. This will allow:
         # -simplify logic filtering (instead of using
         # task_data__build_architecture and for the ones without this
@@ -647,7 +777,10 @@ class WorkRequestRetryView(
         return redirect(new_work_request.get_absolute_url())
 
     def post(
-        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+        self,
+        request: HttpRequest,  # noqa: ARG002, U100
+        *args: Any,
+        **kwargs: Any,
     ) -> HttpResponse:
         """Handle POST requests."""
         work_request = self.get_object()
@@ -737,7 +870,10 @@ class WorkRequestUnblockView(
 
     @catch_http_errors
     def post(
-        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+        self,
+        request: HttpRequest,  # noqa: ARG002, U100
+        *args: Any,
+        **kwargs: Any,
     ) -> HttpResponse:
         """Handle POST requests."""
         work_request = self.get_object()
@@ -808,7 +944,10 @@ class WorkRequestConfirmView(
 
     @catch_http_errors
     def post(
-        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+        self,
+        request: HttpRequest,  # noqa: ARG002, U100
+        *args: Any,
+        **kwargs: Any,
     ) -> HttpResponse:
         """Handle POST requests."""
         work_request = self.get_object()
