@@ -73,8 +73,13 @@ from debusine.tasks import (
     BaseTask,
     BaseTaskWithExecutor,
     TaskConfigError,
+    analyze_external_worker_tasks,
+    get_provided_external_worker_tags,
+)
+from debusine.tasks.executors import (
+    analyze_worker_all_executors,
+    executor_provided_tags,
 )
-from debusine.tasks.executors import analyze_worker_all_executors
 from debusine.tasks.models import OutputData, OutputDataError, WorkerType
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
@@ -82,6 +87,7 @@ from debusine.worker.system_information import (
     cpu_time,
     native_architecture,
     system_metadata,
+    system_provided_tags,
 )
 
 
@@ -497,17 +503,30 @@ class Worker:
 
         return await inner()
 
+    def _get_provided_tags(self) -> set[str]:
+        """Return a set with tags provided by this worker."""
+        tags: set[str] = set()
+        tags |= system_provided_tags(self._worker_type)
+        tags |= executor_provided_tags()
+        tags |= get_provided_external_worker_tags()
+        return tags
+
+    def _get_worker_metadata(self) -> dict[str, Any]:
+        """Compute the worker metadata."""
+        return {
+            **system_metadata(self._worker_type),
+            **analyze_worker_all_executors(),
+            **analyze_external_worker_tasks(),
+            "provided_tags": sorted(self._get_provided_tags()),
+        }
+
     async def _send_dynamic_metadata(
         self,
         msg_content: dict[Any, Any],  # noqa: ARG002, U100
     ) -> None:
         dynamic_metadata_path = '/1.0/worker/dynamic-metadata/'
 
-        metadata = {
-            **system_metadata(self._worker_type),
-            **analyze_worker_all_executors(),
-            **BaseTask.analyze_worker_all_tasks(),
-        }
+        metadata = self._get_worker_metadata()
 
         response = await self._async_http_client.put(
             dynamic_metadata_path, json=metadata
