@@ -15,6 +15,7 @@ from typing import Any
 
 import psutil
 
+import debusine.worker.tags as wtags
 from debusine.tasks.models import WorkerType
 
 
@@ -83,3 +84,14 @@ def system_metadata(worker_type: WorkerType) -> dict[str, Any]:
         metadata["system:architectures"] = kernel_supported_archs
 
     return metadata
+
+
+def system_provided_tags(worker_type: WorkerType) -> set[str]:
+    """Return the tags provided by the worker system."""
+    tags: set[str] = set()
+    tags.add(wtags.WORKER_TYPE_PREFIX + worker_type)
+    if native_arch := native_architecture():
+        tags.add(wtags.BUILD_ARCH_PREFIX + native_arch)
+    for arch in kernel_supported_architectures() or ():
+        tags.add(wtags.BUILD_ARCH_PREFIX + arch)
+    return tags
