@@ -136,14 +136,14 @@ class ConfigHandlerTests(TestCase):
             f"Configuration directory cannot be found in: ['{directory}']"
         )
 
-        with mock.patch.object(
-            ConfigHandler, 'default_directories', new=[directory]
+        with (
+            mock.patch.object(
+                ConfigHandler, 'default_directories', new=[directory]
+            ),
+            self.assertLogsContains(log_message),
+            self.assertRaisesSystemExit(3),
         ):
-            with (
-                self.assertLogsContains(log_message),
-                self.assertRaisesSystemExit(3),
-            ):
-                ConfigHandler()
+            ConfigHandler()
 
     def test_choose_from_default_directories_succeeds(self) -> None:
         """
