@@ -606,8 +606,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         # The server sends a 'request_dynamic_metadata' and the worker
         # sends system_metadata(task_models.WorkerType.EXTERNAL),
-        # analyze_worker_all_executors, and
-        # BaseTask.analyze_worker_all_tasks
+        # analyze_worker_all_executors, and analyze_external_worker_tasks
         send_dynamic_metadata.assert_called_once()
 
     async def test_process_message_dynamic_metadata_request_signing(
@@ -628,9 +627,14 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
                 return_value={},
             ),
             mock.patch(
-                "debusine.tasks.BaseTask.analyze_worker_all_tasks",
+                "debusine.worker._worker.analyze_external_worker_tasks",
                 return_value={},
             ),
+            mock.patch(
+                "debusine.worker._worker.Worker._get_provided_tags",
+                autospec=True,
+                return_value=set(),
+            ),
         ):
             await self.worker.connect()
 
@@ -643,13 +647,13 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             self.server_latest_request.json_content,
             {
                 **system_metadata(task_models.WorkerType.SIGNING),
+                "provided_tags": [],
             },
         )
 
     async def test_send_dynamic_metadata(self) -> None:
         """Test a successful dynamic metadata update."""
-        executors_metadata = {"executor:unshare:available": True}
-        tasks_metadata = {"sbuild:version": 1, "sbuild:available": True}
+        worker_metadata = {"test": True}
 
         self.setup_worker()
         self.setup_valid_token()
@@ -657,14 +661,9 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         with (
             mock.patch(
-                "debusine.worker._worker.analyze_worker_all_executors",
+                "debusine.worker._worker.Worker._get_worker_metadata",
                 autospec=True,
-                return_value=executors_metadata,
-            ),
-            mock.patch(
-                'debusine.tasks.BaseTask.analyze_worker_all_tasks',
-                autospec=True,
-                return_value=tasks_metadata,
+                return_value=worker_metadata,
             ),
             self.assertLogs(level=logging.DEBUG) as logs,
         ):
@@ -684,12 +683,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         self.assertTrue(self.server_latest_request.is_authenticated)
 
         self.assertEqual(
-            self.server_latest_request.json_content,
-            {
-                **system_metadata(task_models.WorkerType.EXTERNAL),
-                **executors_metadata,
-                **tasks_metadata,
-            },
+            self.server_latest_request.json_content, worker_metadata
         )
 
     async def test_send_dynamic_metadata_failed(self) -> None:
@@ -712,7 +706,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
                 return_value=executors_metadata,
             ),
             mock.patch(
-                'debusine.tasks.BaseTask.analyze_worker_all_tasks',
+                'debusine.worker._worker.analyze_external_worker_tasks',
                 autospec=True,
                 return_value=tasks_metadata,
             ),
